Package: BioIndex
Type: Package
Title: Biological Indicators and Indices for MEDITS Survey Data
Version: 0.6.4
Date: 2026-04-29
Authors@R: c(
    person("Walter", "Zupa", email = "zupa@fondazionecoispa.org", role = c("aut", "cre")),
    person("Loredana", "Casciaro", email = "casciaro@coispa.eu", role = "rev"),
    person("Cosmidano", "Neglia", email = "neglia@coispa.eu", role = "rev"),
    person("Isabella", "Bitetto", email = "bitetto@fondazionecoispa.org", role = "rev"),
    person("Maria Teresa", "Spedicato", email = "spedicato@fondazionecoispa.org", role = "aut")
  )
Author: Walter Zupa [aut, cre],
  Loredana Casciaro [rev],
  Cosmidano Neglia [rev],
  Isabella Bitetto [rev],
  Maria Teresa Spedicato [aut]
Maintainer: Walter Zupa <zupa@fondazionecoispa.org>
Description: Supports the standardized analysis of Mediterranean 
    International Bottom Trawl Survey (MEDITS) data and the calculation 
    of biological indicators for selected species and population 
    components. The package provides functions to estimate abundance 
    and biomass indices, analyse size structure and length frequency 
    distributions, derive sex ratio and maturity related metrics, 
    explore spatial patterns, and assess temporal trends across 
    surveys. Developed for integration within the Regional Database 
    for Fisheries (RDBFIS) framework, it is intended to work on 
    quality checked input data and to produce reproducible outputs 
    that can support monitoring, comparative analyses among 
    Geographical Sub-Areas (GSAs) and countries, and fishery management.
License: GPL-3
Encoding: UTF-8
LazyData: TRUE
LazyDataCompression: xz
RoxygenNote: 7.3.3
Depends: R (>= 4.1)
Imports: dplyr, ggplot2, gridExtra, hms, magrittr, marmap, methods,
        mgcv, reshape2, shiny, shinyjs, stringr, terra, tidyterra, zip
Suggests: knitr, mapproj, maps (>= 3.4.1), rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-25 16:18:21 UTC; Walter
Repository: CRAN
Date/Publication: 2026-05-29 10:30:02 UTC
Built: R 4.6.0; ; 2026-05-29 11:21:49 UTC; unix

# Generated by roxygen2: do not edit by hand

export(BioIndex)
export(LFD)
export(MEDITS.distance)
export(MEDITS.to.dd)
export(MIW)
export(aggregate_gsas)
export(bubble_plot_by_haul_indexes)
export(bubbleplot_RS_by_hauls)
export(check_date_haul)
export(check_dictionary)
export(check_hauls_TBTA)
export(check_numeric_range)
export(convert_coordinates)
export(dd.distance)
export(dd.to.MEDITS)
export(hauls_position)
export(index_on_grid)
export(index_recr)
export(index_spawn)
export(index_ts_F)
export(index_ts_M)
export(indices_ts)
export(merge_TATB)
export(merge_TATBTC)
export(merge_TATC)
export(overlayGrid)
export(run_BioIndex_app)
export(sex_ratio)
export(sex_ratio_on_grid)
export(spear)
export(spearman)
import(ggplot2)
import(grDevices)
import(graphics)
import(mgcv)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_map)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_contour)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,map_data)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,dev.copy)
importFrom(grDevices,dev.off)
importFrom(grDevices,heat.colors)
importFrom(grDevices,jpeg)
importFrom(grDevices,tiff)
importFrom(graphics,box)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(gridExtra,grid.arrange)
importFrom(hms,hms)
importFrom(magrittr,"%>%")
importFrom(marmap,as.xyz)
importFrom(marmap,getNOAA.bathy)
importFrom(methods,is)
importFrom(mgcv,gam)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(shiny,runApp)
importFrom(shinyjs,useShinyjs)
importFrom(stats,AIC)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,cor.test)
importFrom(stats,dchisq)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,lm)
importFrom(stats,na.exclude)
importFrom(stats,nls)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,time)
importFrom(stringr,str_split)
importFrom(terra,extract)
importFrom(terra,unwrap)
importFrom(terra,vect)
importFrom(terra,wrap)
importFrom(tidyterra,geom_spatvector)
importFrom(utils,data)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(zip,zip)

# BioIndex

**BioIndex** supports the standardized analysis of MEDITS trawl survey data and the calculation of biological indicators for selected species and population components. The package provides functions to estimate abundance and biomass indices, analyse size structure and length frequency distributions, derive sex ratio and maturity related metrics, explore spatial patterns, and assess temporal trends across surveys. 

Developed for integration within the RDBFIS framework, it is intended to work on quality checked input data (via the **RoME** package) and to produce reproducible outputs that can support monitoring, comparative analyses among GSAs and countries, and fishery management.

The package allows to perform the analysis at GSA level or country level, estimating the time series and the spatial distribution of a wide pool of population state-indicators. It also offers statistical trend analysis of both abundance and biomass time series based on the Hotelling-Pabst test (Cotter, 2009) and spatial analysis using the 0.5*0.5° GFCM grid or bubble plots.

# Shiny Application

BioIndex includes an embedded Shiny application providing a user-friendly graphical interface for all analyses. To launch the application, use:

```r
library(BioIndex)
run_BioIndex_app()
```

# Installation

Install the latest BioIndex release from CRAN (when available):

```r
install.packages("BioIndex")
```

Or install the development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("COISPA/BioIndex")
```

# Use

The main function `BioIndex()` can be used to launch a complete analysis:

```r
BioIndex(ta=TA, tb=TB, tc=TC, sspp="MERLMER",rec_threshold=200,
         spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10, 
         country="all", map_lim=c(13.3,15.2,39.9,41.3),
         depth_lines=c(50,200,800), strata=BioIndex::strata_scheme, 
         stratification_tab = BioIndex::stratification, resolution=1, 
         buffer=0.1, wd=tempdir(), zip=FALSE, save=FALSE, verbose=TRUE)
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE, fig.height=8, fig.width=8------------------------------------
# BioIndex::BioIndex(ta=BioIndex::TA, tb=BioIndex::TB, tc=BioIndex::TC,
#                    sspp="MERLMER",rec_threshold=200,
#                    spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10,
#                    country="all", map_lim=c(13.3,15.2,39.9,41.3),
#                    depth_lines=c(50,200,800), strata=BioIndex::strata_scheme,
#                    stratification_tab = BioIndex::stratification, resolution=1,
#                    buffer=0.1, wd=tempdir(), zip=FALSE, save=FALSE, verbose=TRUE)

library(shiny)
library(shinyjs)
library(BioIndex)

invisible(options(shiny.maxRequestSize = 500 * 1024^2))

# Safely retrieve aggregate_gsas whether it is exported or internal in BioIndex
aggregate_gsas_safe <- function(...) {
  if ("aggregate_gsas" %in% getNamespaceExports("BioIndex")) {
    BioIndex::aggregate_gsas(...)
  } else {
    get("aggregate_gsas", envir = asNamespace("BioIndex"))(...)
  }
}

# Normalize the output structure returned by aggregate_gsas
parse_aggregate_result <- function(x, gsas) {
  agg_gsa <- paste(sort(unique(as.character(gsas))), collapse = "")

  if (is.null(x) || !is.list(x) || length(x) < 5) {
    stop("aggregate_gsas returned an unexpected object.")
  }

  nms <- names(x)

  if (!is.null(nms) && all(c("ta", "tb", "tc") %in% nms)) {
    ta_out <- x$ta
    tb_out <- x$tb
    tc_out <- x$tc
    stratification_out <- if ("stratification" %in% nms) x$stratification else x[[4]]
    strata_scheme_out <- if ("strata_scheme" %in% nms) x$strata_scheme else x[[5]]
    aggregated_gsa_out <- if ("aggregated_gsa" %in% nms) x$aggregated_gsa else agg_gsa
  } else {
    ta_out <- x[[1]]
    tb_out <- x[[2]]
    tc_out <- x[[3]]
    stratification_out <- x[[4]]
    strata_scheme_out <- x[[5]]
    aggregated_gsa_out <- agg_gsa
  }

  list(
    ta = ta_out,
    tb = tb_out,
    tc = tc_out,
    stratification = stratification_out,
    strata_scheme = strata_scheme_out,
    aggregated_gsa = aggregated_gsa_out
  )
}

ui <- fluidPage(
  useShinyjs(),

  tags$head(
    tags$style(HTML("
      body {
        background-color: #f7f7f7;
      }

      .app-header {
        text-align: center;
        padding: 15px 0 10px 0;
      }

      .app-subtitle {
        color: #444444;
        margin-top: -5px;
        margin-bottom: 20px;
      }

      .section-box {
        background-color: white;
        border: 1px solid #dddddd;
        border-radius: 10px;
        padding: 20px;
        margin-bottom: 20px;
        box-shadow: 0 1px 4px rgba(0, 0, 0, 0.05);
      }

      .section-title {
        font-size: 22px;
        font-weight: 600;
        margin-bottom: 15px;
        color: #1f2d3d;
      }

      .btn-main {
        min-width: 180px;
        font-size: 16px;
        font-weight: 600;
      }

      .log-box pre {
        background-color: #fafafa;
        border: 1px solid #d9d9d9;
        border-radius: 6px;
        padding: 12px;
        white-space: pre-wrap;
        word-wrap: break-word;
        max-height: 500px;
        overflow-y: auto;
      }

      .status-box pre {
        background-color: #fafafa;
        border: 1px solid #d9d9d9;
        border-radius: 6px;
        padding: 12px;
        white-space: pre-wrap;
      }
    "))
  ),

  div(
    class = "app-header",
    tags$img(src = "BioIndex_Logo.png", height = "170px"),
    h1("BioIndex"),
    p("Analysis of trawl survey data using MEDITS file format", class = "app-subtitle")
  ),

  fluidRow(
    column(
      width = 12,
      div(
        class = "section-box",
        div(class = "section-title", "Input files"),
        fluidRow(
          column(
            width = 3,
            fileInput("ta_file", "Upload TA file", accept = c(".csv", ".txt"))
          ),
          column(
            width = 3,
            fileInput("tb_file", "Upload TB file", accept = c(".csv", ".txt"))
          ),
          column(
            width = 3,
            fileInput("tc_file", "Upload TC file", accept = c(".csv", ".txt"))
          ),
          column(
            width = 3,
            selectInput(
              "sep",
              "Main file separator",
              choices = c(";" = ";", "," = ","),
              selected = ";"
            )
          )
        ),
        br(),
        fluidRow(
          column(
            width = 4,
            fileInput("strata_file", "Optional custom strata CSV", accept = c(".csv", ".txt"))
          ),
          column(
            width = 4,
            fileInput("stratification_file", "Optional custom stratification CSV", accept = c(".csv", ".txt"))
          ),
          column(
            width = 4,
            selectInput(
              "aux_sep",
              "Custom table separator",
              choices = c(";" = ";", "," = ","),
              selected = ";"
            )
          )
        )
      )
    )
  ),

  fluidRow(
    column(
      width = 12,
      div(
        class = "section-box",
        div(class = "section-title", "Analysis settings"),

        fluidRow(
          column(
            width = 3,
            selectizeInput(
              "sspp",
              "Species code (sspp)",
              choices = NULL,
              selected = NULL,
              options = list(placeholder = "Upload TA/TB/TC files first")
            )
          ),
          column(
            width = 3,
            checkboxInput(
              "use_gsa_aggregation",
              "Aggregate multiple GSAs",
              value = FALSE
            ),
            uiOutput("gsa_selector_ui")
          ),
          column(
            width = 3,
            selectInput(
              "country",
              "Country",
              choices = NULL,
              selected = NULL
            )
          ),
          column(
            width = 3,
            selectInput(
              "sexes",
              "Sexes",
              choices = c("all", "M", "F"),
              selected = "all"
            )
          )
        ),

        fluidRow(
          column(
            width = 3,
            numericInput("rec_threshold", "Recruit threshold (mm)", value = 200, min = 0)
          ),
          column(
            width = 3,
            numericInput("spaw_threshold", "Spawner threshold (mm)", value = 210, min = 0)
          ),
          column(
            width = 3,
            numericInput("haul_threshold", "Haul threshold", value = 30, min = 1, step = 1)
          ),
          column(
            width = 3,
            numericInput("buffer", "Buffer", value = 0.1, min = 0, step = 0.01)
          )
        ),

        fluidRow(
          column(
            width = 3,
            selectInput("depth_min", "Depth lower", choices = NULL)
          ),
          column(
            width = 3,
            selectInput("depth_max", "Depth upper", choices = NULL)
          ),
          column(
            width = 3,
            textInput("depth_lines", "Depth lines", value = "")
          ),
          column(
            width = 3,
            numericInput("resolution", "Resolution", value = NA)
          )
        ),

        fluidRow(
          column(
            width = 3,
            numericInput("xmin", "Longitude lower limit", value = NA)
          ),
          column(
            width = 3,
            numericInput("xmax", "Longitude upper limit", value = NA)
          ),
          column(
            width = 3,
            numericInput("ymin", "Latitude lower limit", value = NA)
          ),
          column(
            width = 3,
            numericInput("ymax", "Latitude upper limit", value = NA)
          )
        ),

        fluidRow(
          style = "margin-top: 10px;",
          column(
            width = 12,
            div(
              style = "text-align: center;",
              checkboxInput("verbose", "Verbose", value = TRUE)
            )
          )
        ),

        fluidRow(
          column(
            width = 12,
            style = "text-align: center; margin-top: 10px;",
            actionButton("run_btn", "Run BioIndex", class = "btn-primary btn-main"),
            tags$span(style = "display:inline-block; width:15px;"),
            downloadButton("download_results", "Download results zip", class = "btn-default btn-main")
          )
        )
      )
    )
  ),

  fluidRow(
    column(
      width = 12,
      div(
        class = "section-box status-box",
        div(class = "section-title", "Status"),
        verbatimTextOutput("status")
      )
    )
  ),

  fluidRow(
    column(
      width = 12,
      div(
        class = "section-box log-box",
        div(class = "section-title", "Run log"),
        verbatimTextOutput("run_log")
      )
    )
  )
)

server <- function(input, output, session) {

  gsa_defaults <- data.frame(
    GSA  = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
             16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30),
    xmin = c(-5.600000, -3.333333, -5.600000, -2.216667,  0.500000,
             -1.000000,  2.951167,  7.627755,  7.525000, 11.000000,
             6.000000,  8.500000, 10.467833, 10.015361, 13.500000,
             11.000000, 12.141194, 15.169798, 15.082944, 19.166666,
             11.536992, 22.527028, 23.000000, 29.000000, 32.000000,
             25.139999, 34.349998, 26.324205, 27.306167, 34.465444),
    xmax = c(-1.000000, -2.666667, -2.216667,  8.806623,  6.000000,
             6.000000,  8.000000,  9.750000, 13.004581, 16.221250,
             11.000000, 12.000000, 13.500000, 15.300000, 15.300000,
             15.300000, 18.765417, 20.186167, 19.166666, 23.225389,
             25.139999, 29.000000, 26.500000, 36.209250, 35.000000,
             34.349998, 35.991306, 29.939583, 41.779528, 39.305611),
    ymin = c(36.00000, 35.75000, 35.08453, 35.07286, 38.00000,
             37.00000, 41.33000, 41.30000, 41.30000, 37.03458,
             38.00000, 36.71289, 35.00000, 33.16779, 35.00000,
             35.00000, 41.92224, 39.60821, 35.00000, 35.00000,
             30.26622, 34.00000, 34.00000, 34.00000, 34.00000,
             30.82203, 31.44039, 40.07335, 40.90875, 45.23872),
    ymax = c(37.58622, 36.08333, 36.00000, 38.00000, 40.50000,
             42.42947, 43.78458, 43.25000, 44.42711, 41.30000,
             41.78455, 38.50000, 37.00000, 35.00000, 36.50000,
             38.00000, 45.79456, 42.47188, 40.52042, 39.88215,
             35.00000, 41.01206, 36.00000, 36.91292, 35.78333,
             34.00000, 35.88234, 41.15125, 46.88956, 47.28956)
  )

  get_gsa_map_limits <- function(gsa_values, gsa_defaults) {
    gsa_values <- suppressWarnings(as.numeric(gsa_values))
    gsa_values <- gsa_values[!is.na(gsa_values)]

    if (length(gsa_values) == 0) {
      return(NULL)
    }

    rows <- gsa_defaults[gsa_defaults$GSA %in% gsa_values, , drop = FALSE]

    if (nrow(rows) == 0) {
      return(NULL)
    }

    c(
      xmin = min(rows$xmin, na.rm = TRUE),
      xmax = max(rows$xmax, na.rm = TRUE),
      ymin = min(rows$ymin, na.rm = TRUE),
      ymax = max(rows$ymax, na.rm = TRUE)
    )
  }

  rv <- reactiveValues(
    status = "Waiting for input.",
    log = "",
    zipfile = NULL,
    run_dir = NULL,
    running = FALSE,
    ta = NULL,
    tb = NULL,
    tc = NULL,
    strata = BioIndex::strata_scheme,
    stratification_tab = BioIndex::stratification
  )

  output$status <- renderText({
    rv$status
  })

  output$run_log <- renderText({
    rv$log
  })

  output$gsa_selector_ui <- renderUI({
    req(rv$ta)

    gsa_choices <- sort(unique(suppressWarnings(as.numeric(as.character(rv$ta$AREA)))))
    gsa_choices <- gsa_choices[!is.na(gsa_choices)]

    if (isTRUE(input$use_gsa_aggregation)) {
      selectizeInput(
        "GSA_multi",
        "GSAs to aggregate",
        choices = gsa_choices,
        selected = NULL,
        multiple = TRUE,
        options = list(
          placeholder = "Select two or more GSAs",
          plugins = list("remove_button")
        )
      )
    } else {
      selectInput(
        "GSA",
        "GSA",
        choices = gsa_choices,
        selected = if (length(gsa_choices) > 0) gsa_choices[1] else NULL
      )
    }
  })

  read_medits_file <- function(fileinfo, sep) {
    req(fileinfo)
    read.table(
      fileinfo$datapath,
      sep = sep,
      header = TRUE,
      stringsAsFactors = FALSE
    )
  }

  observe({
    if (isTRUE(rv$running)) {
      disable("run_btn")
    } else {
      enable("run_btn")
    }
  })

  observe({
    req(input$strata_file)

    strata_custom <- try(read_medits_file(input$strata_file, input$aux_sep), silent = TRUE)

    if (inherits(strata_custom, "try-error")) {
      rv$status <- "Error while reading custom strata file. Default strata will be used."
      rv$strata <- BioIndex::strata_scheme
    } else {
      rv$strata <- strata_custom
      rv$status <- "Custom strata file loaded."
    }
  })

  observe({
    if (is.null(input$strata_file)) {
      rv$strata <- BioIndex::strata_scheme
    }
  })

  observe({
    req(input$stratification_file)

    strat_custom <- try(read_medits_file(input$stratification_file, input$aux_sep), silent = TRUE)

    if (inherits(strat_custom, "try-error")) {
      rv$status <- "Error while reading custom stratification file. Default stratification will be used."
      rv$stratification_tab <- BioIndex::stratification
    } else {
      rv$stratification_tab <- strat_custom
      rv$status <- "Custom stratification file loaded."
    }
  })

  observe({
    if (is.null(input$stratification_file)) {
      rv$stratification_tab <- BioIndex::stratification
    }
  })

  observe({
    req(input$ta_file, input$tb_file, input$tc_file)

    ta <- try(read_medits_file(input$ta_file, input$sep), silent = TRUE)
    tb <- try(read_medits_file(input$tb_file, input$sep), silent = TRUE)
    tc <- try(read_medits_file(input$tc_file, input$sep), silent = TRUE)

    if (inherits(ta, "try-error") || inherits(tb, "try-error") || inherits(tc, "try-error")) {
      rv$status <- "Error while reading one or more input files for selector update."
      return(NULL)
    }

    rv$ta <- ta
    rv$tb <- tb
    rv$tc <- tc

    if (!all(c("GENUS", "SPECIES") %in% names(tb))) {
      rv$status <- "TB file must contain GENUS and SPECIES columns."
      return(NULL)
    }

    if (!"AREA" %in% names(ta)) {
      rv$status <- "TA file must contain AREA column."
      return(NULL)
    }

    if (!"COUNTRY" %in% names(ta)) {
      rv$status <- "TA file must contain COUNTRY column."
      return(NULL)
    }

    sspp_choices <- sort(unique(paste0(
      trimws(as.character(tb$GENUS)),
      trimws(as.character(tb$SPECIES))
    )))
    sspp_choices <- sspp_choices[!is.na(sspp_choices) & nzchar(sspp_choices)]

    COUNTRY_choices <- sort(unique(trimws(as.character(ta$COUNTRY))))
    COUNTRY_choices <- COUNTRY_choices[!is.na(COUNTRY_choices) & nzchar(COUNTRY_choices)]

    updateSelectizeInput(
      session,
      "sspp",
      choices = sspp_choices,
      selected = if (length(sspp_choices) > 0) sspp_choices[1] else NULL,
      server = TRUE
    )

    updateSelectInput(
      session,
      "country",
      choices = c("all", COUNTRY_choices),
      selected = "all"
    )

    rv$status <- "Files loaded. Select species, GSA and country."
  })

  observe({
    selected_gsas <- if (isTRUE(input$use_gsa_aggregation)) input$GSA_multi else input$GSA
    selected_country <- input$country

    req(selected_gsas)

    # Update map limits
    map_vals <- get_gsa_map_limits(selected_gsas, gsa_defaults)

    if (!is.null(map_vals)) {
      updateNumericInput(session, "xmin", value = unname(map_vals["xmin"]))
      updateNumericInput(session, "xmax", value = unname(map_vals["xmax"]))
      updateNumericInput(session, "ymin", value = unname(map_vals["ymin"]))
      updateNumericInput(session, "ymax", value = unname(map_vals["ymax"]))
    } else {
      updateNumericInput(session, "xmin", value = NA_real_)
      updateNumericInput(session, "xmax", value = NA_real_)
      updateNumericInput(session, "ymin", value = NA_real_)
      updateNumericInput(session, "ymax", value = NA_real_)
    }

    # Update depth range based on strata
    current_strata <- rv$strata
    if (!is.null(current_strata) && "GSA" %in% names(current_strata)) {
      relevant_strata <- current_strata[as.character(current_strata$GSA) %in% as.character(selected_gsas), ]

      if (!is.null(selected_country) && selected_country != "all" && "COUNTRY" %in% names(relevant_strata)) {
        relevant_strata <- relevant_strata[relevant_strata$COUNTRY == selected_country, ]
      }

      if (nrow(relevant_strata) > 0) {
        all_mins <- sort(unique(relevant_strata$MIN_DEPTH))
        all_maxs <- sort(unique(relevant_strata$MAX_DEPTH))

        updateSelectInput(session, "depth_min", choices = all_mins, selected = min(all_mins))
        updateSelectInput(session, "depth_max", choices = all_maxs, selected = max(all_maxs))

        # Update depth lines with sensible defaults from stratification cutoffs
        all_depths <- sort(unique(c(all_mins, all_maxs)))
        d_min <- min(all_depths)
        d_max <- max(all_depths)
        
        if (length(all_depths) >= 3) {
          inner_depths <- all_depths[-c(1, length(all_depths))]
          target <- (d_min + d_max) / 2
          mid_depth <- inner_depths[which.min(abs(inner_depths - target))]
        } else {
          mid_depth <- round((d_min + d_max) / 2)
        }
        
        updateTextInput(session, "depth_lines", value = paste(d_min, mid_depth, d_max, sep = ","))
      }
    }
  })

  observeEvent(input$run_btn, {

    req(rv$ta, rv$tb, rv$tc)
    req(input$sspp, input$country)

    sspp_val <- as.character(input$sspp)
    rec_threshold_val <- suppressWarnings(as.numeric(input$rec_threshold))
    spaw_threshold_val <- suppressWarnings(as.numeric(input$spaw_threshold))
    haul_threshold_val <- suppressWarnings(as.integer(input$haul_threshold))

    resolution_val <- input$resolution
    if (length(resolution_val) == 0 || is.null(resolution_val) || identical(resolution_val, "")) {
      resolution_val <- NA_real_
    } else {
      resolution_val <- suppressWarnings(as.numeric(resolution_val))
    }

    if (is.na(resolution_val) && !is.na(input$resolution)) {
      rv$status <- "Invalid resolution."
      rv$log <- "Resolution must be numeric or empty."
      rv$running <- FALSE
      return(NULL)
    }

    buffer_val <- suppressWarnings(as.numeric(input$buffer))
    sexes_val <- as.character(input$sexes)
    single_gsa_val <- input$GSA
    multi_gsa_val <- input$GSA_multi
    country_val <- as.character(input$country)
    strata_val <- rv$strata
    stratification_tab_val <- rv$stratification_tab
    zip_val <- TRUE
    save_val <- TRUE
    verbose_val <- isTRUE(input$verbose)

    depth_val <- c(
      suppressWarnings(as.numeric(input$depth_min)),
      suppressWarnings(as.numeric(input$depth_max))
    )

    map_lim_val <- c(
      suppressWarnings(as.numeric(input$xmin)),
      suppressWarnings(as.numeric(input$xmax)),
      suppressWarnings(as.numeric(input$ymin)),
      suppressWarnings(as.numeric(input$ymax))
    )

    if (any(is.na(depth_val))) {
      rv$status <- "Invalid depth range."
      rv$log <- "Depth lower and upper must be numeric."
      rv$running <- FALSE
      return(NULL)
    }

    if (any(is.na(map_lim_val))) {
      rv$status <- "Invalid map limits."
      rv$log <- "Map limits are missing. Select a GSA with defaults or insert xmin, xmax, ymin, ymax manually."
      rv$running <- FALSE
      return(NULL)
    }

    depth_lines_val <- suppressWarnings(as.numeric(trimws(strsplit(input$depth_lines, ",")[[1]])))

    if (any(is.na(depth_lines_val))) {
      rv$status <- "Invalid depth lines."
      rv$log <- "Depth lines must be provided as comma-separated numeric values."
      rv$running <- FALSE
      return(NULL)
    }

    if (is.na(rec_threshold_val)) {
      rv$status <- "Invalid recruit threshold."
      rv$log <- "Recruit threshold must be numeric."
      rv$running <- FALSE
      return(NULL)
    }

    if (is.na(spaw_threshold_val)) {
      rv$status <- "Invalid spawner threshold."
      rv$log <- "Spawner threshold must be numeric."
      rv$running <- FALSE
      return(NULL)
    }

    if (!isTRUE(all.equal(as.numeric(input$haul_threshold), as.numeric(haul_threshold_val)))) {
      rv$status <- "Invalid haul threshold."
      rv$log <- "Haul threshold must be an integer value."
      rv$running <- FALSE
      return(NULL)
    }

    if (is.na(buffer_val)) {
      rv$status <- "Invalid buffer."
      rv$log <- "Buffer must be numeric."
      rv$running <- FALSE
      return(NULL)
    }

    if (depth_val[1] >= depth_val[2]) {
      rv$status <- "Invalid depth range."
      rv$log <- "Depth lower must be smaller than depth upper."
      rv$running <- FALSE
      return(NULL)
    }

    if (map_lim_val[1] >= map_lim_val[2] || map_lim_val[3] >= map_lim_val[4]) {
      rv$status <- "Invalid map limits."
      rv$log <- "Longitude and latitude lower limits must be smaller than upper limits."
      rv$running <- FALSE
      return(NULL)
    }

    rv$status <- "Preparing analysis..."
    rv$log <- ""
    rv$zipfile <- NULL
    rv$run_dir <- NULL
    rv$running <- TRUE

    ta <- rv$ta
    tb <- rv$tb
    tc <- rv$tc

    if (isTRUE(input$use_gsa_aggregation)) {

      gsas_run <- sort(unique(as.character(multi_gsa_val)))

      if (length(gsas_run) < 2) {
        rv$status <- "Invalid aggregated GSA selection."
        rv$log <- "Please select at least two GSAs to aggregate."
        rv$running <- FALSE
        return(NULL)
      }

      agg_raw <- tryCatch(
        aggregate_gsas_safe(
          ta = ta,
          tb = tb,
          tc = tc,
          gsas = gsas_run,
          strata_scheme = strata_val,
          stratification = stratification_tab_val
        ),
        error = function(e) e
      )

      if (inherits(agg_raw, "error")) {
        rv$status <- "Aggregation failed."
        rv$log <- paste("ERROR:", conditionMessage(agg_raw))
        rv$running <- FALSE
        return(NULL)
      }

      agg_res <- tryCatch(
        parse_aggregate_result(agg_raw, gsas_run),
        error = function(e) e
      )

      if (inherits(agg_res, "error")) {
        rv$status <- "Aggregation failed."
        rv$log <- paste("ERROR:", conditionMessage(agg_res))
        rv$running <- FALSE
        return(NULL)
      }

      ta <- agg_res$ta
      tb <- agg_res$tb
      tc <- agg_res$tc
      stratification_tab_val <- agg_res$stratification
      strata_val <- agg_res$strata_scheme
      GSA_val <- agg_res$aggregated_gsa

    } else {

      req(single_gsa_val)

      GSA_val <- suppressWarnings(as.numeric(single_gsa_val))

      if (is.na(GSA_val)) {
        rv$status <- "Invalid GSA."
        rv$log <- "GSA must be numeric."
        rv$running <- FALSE
        return(NULL)
      }
    }

    wd_run <- file.path(
      tempdir(),
      paste0("BioIndex_", format(Sys.time(), "%Y%m%d_%H%M%S"))
    )
    dir.create(wd_run, recursive = TRUE, showWarnings = FALSE)
    rv$run_dir <- wd_run

    rv$status <- "BioIndex analysis started..."

    result <- tryCatch({
      log_output <- capture.output(
        BioIndex(
          ta = ta,
          tb = tb,
          tc = tc,
          sspp = sspp_val,
          rec_threshold = rec_threshold_val,
          spaw_threshold = spaw_threshold_val,
          haul_threshold = haul_threshold_val,
          sexes = sexes_val,
          depth = depth_val,
          GSA = GSA_val,
          country = country_val,
          map_lim = map_lim_val,
          depth_lines = depth_lines_val,
          strata = strata_val,
          stratification_tab = stratification_tab_val,
          resolution = resolution_val,
          buffer = buffer_val,
          wd = wd_run,
          zip = zip_val,
          save = save_val,
          verbose = verbose_val
        ),
        type = "output"
      )

      list(
        success = TRUE,
        log_output = log_output
      )
    }, error = function(e) {
      list(
        success = FALSE,
        error_message = conditionMessage(e)
      )
    })

    if (isTRUE(result$success)) {

      zip_candidates <- list.files(
        path = wd_run,
        pattern = "\\.zip$",
        recursive = TRUE,
        full.names = TRUE
      )

      final_log <- result$log_output

      if (length(zip_candidates) > 0) {
        zipfile_found <- zip_candidates[1]
        final_log <- c(
          final_log,
          "",
          "Run completed successfully.",
          paste("ZIP file created:", zipfile_found)
        )
      } else {
        zipfile_found <- NULL
        final_log <- c(
          final_log,
          "",
          "Run completed successfully, but no ZIP file was found."
        )
      }

      rv$zipfile <- zipfile_found
      rv$log <- paste(final_log, collapse = "\n")
      rv$status <- "Analysis completed."

    } else {
      rv$zipfile <- NULL
      rv$log <- paste("ERROR:", result$error_message)
      rv$status <- "Analysis failed."
    }

    rv$running <- FALSE
  })

  output$download_results <- downloadHandler(
    filename = function() {
      zipfile <- rv$zipfile
      if (!is.null(zipfile) && file.exists(zipfile)) {
        basename(zipfile)
      } else {
        "BioIndex_results.zip"
      }
    },
    content = function(file) {
      zipfile <- rv$zipfile
      req(!is.null(zipfile))
      req(file.exists(zipfile))
      ok <- file.copy(zipfile, file, overwrite = TRUE)
      if (!isTRUE(ok)) {
        stop("Unable to copy the ZIP file for download.")
      }
    }
  )
}

shinyApp(ui, server)

#' MEDITS coordinates in decimal degrees
#' @description
#' The function returns the data frame of the TA table with the coordinates expressed as decimal degrees.
#'
#' @param Data data frame of TA table
#'
#' @return the function return the same data frame with the coordinates converted in the decimal degrees format
#' @examples
#' data(TA)
#' convert_coordinates(TA)
#' @export convert_coordinates

convert_coordinates<-function(Data)  {

# [ORIGINALE]:
#   lat_start=Data$SHOOTING_LATITUDE
#   lon_start= Data$SHOOTING_LONGITUDE
#   lat_end=Data$HAULING_LATITUDE
#   lon_end= Data$HAULING_LONGITUDE
#   LatStartDeg = floor(floor(lat_start)/100);
#   LonStartDeg = floor(floor(lon_start)/100);
#   LatStartMin=(lat_start-LatStartDeg*100)/60
#   LonStartMin=(lon_start-LonStartDeg*100)/60
#   LatEndDeg = floor(floor(lat_end)/100);
#   LonEndDeg = floor(floor(lon_end)/100);
#   LatEndMin=(lat_end-LatEndDeg*100)/60
#   LonEndMin=(lon_end-LonEndDeg*100)/60
# 
#   lat_start2= LatStartDeg + LatStartMin
#   lon_start2 = LonStartDeg + LonStartMin
#   lat_end2 = LatEndDeg + LatEndMin
#   lon_end2 = LonEndDeg + LonEndMin
#   Data$lat_start = lat_start2
#   Data$lon_start = lon_start2
#   Data$lat_end = lat_end2
#   Data$lon_end = lon_end2
# 
#   # quadrant 3
#   Data[Data$SHOOTING_QUADRANT == 3, "lat_start"] <- -1 * Data[Data$SHOOTING_QUADRANT == 3, "lat_start"]
#   Data[Data$HAULING_QUADRANT == 3, "lat_end"] <- -1 * Data[Data$HAULING_QUADRANT == 3, "lat_end"]
# 
#   # quadrant 5
#   Data[Data$SHOOTING_QUADRANT == 5, "lat_start"] <- -1 * Data[Data$SHOOTING_QUADRANT == 5, "lat_start"]
#   Data[Data$HAULING_QUADRANT == 5, "lat_end"] <- -1 * Data[Data$HAULING_QUADRANT == 5, "lat_end"]
# 
#   Data[Data$SHOOTING_QUADRANT == 5, "lon_start"] <- -1 * Data[Data$SHOOTING_QUADRANT == 5, "lon_start"]
#   Data[Data$HAULING_QUADRANT == 5, "lon_end"] <- -1 * Data[Data$HAULING_QUADRANT == 5, "lon_end"]
# 
#   # quadrant 7
#   Data[Data$SHOOTING_QUADRANT == 7, "lon_start"] <- -1 * Data[Data$SHOOTING_QUADRANT == 7, "lon_start"]
#   Data[Data$HAULING_QUADRANT == 7, "lon_end"] <- -1 * Data[Data$HAULING_QUADRANT == 7, "lon_end"]
# [MODIFICATO]:
  Data$lat_start = NA
  Data$lon_start = NA
  Data$lat_end = NA
  Data$lon_end = NA

  non_na_idx <- !is.na(Data$SHOOTING_LATITUDE) & !is.na(Data$SHOOTING_LONGITUDE) &
                !is.na(Data$HAULING_LATITUDE) & !is.na(Data$HAULING_LONGITUDE)

  if (any(non_na_idx)) {
    lat_start=Data$SHOOTING_LATITUDE[non_na_idx]
    lon_start= Data$SHOOTING_LONGITUDE[non_na_idx]
    lat_end=Data$HAULING_LATITUDE[non_na_idx]
    lon_end= Data$HAULING_LONGITUDE[non_na_idx]
    LatStartDeg = floor(floor(lat_start)/100);
    LonStartDeg = floor(floor(lon_start)/100);
    LatStartMin=(lat_start-LatStartDeg*100)/60
    LonStartMin=(lon_start-LonStartDeg*100)/60
    LatEndDeg = floor(floor(lat_end)/100);
    LonEndDeg = floor(floor(lon_end)/100);
    LatEndMin=(lat_end-LatEndDeg*100)/60
    LonEndMin=(lon_end-LonEndDeg*100)/60

    lat_start2= LatStartDeg + LatStartMin
    lon_start2 = LonStartDeg + LonStartMin
    lat_end2 = LatEndDeg + LatEndMin
    lon_end2 = LonEndDeg + LonEndMin
    
    Data$lat_start[non_na_idx] = lat_start2
    Data$lon_start[non_na_idx] = lon_start2
    Data$lat_end[non_na_idx] = lat_end2
    Data$lon_end[non_na_idx] = lon_end2
  }

  # quadrant 3
  idx_3_shoot <- which(Data$SHOOTING_QUADRANT == 3 & !is.na(Data$lat_start))
  idx_3_haul  <- which(Data$HAULING_QUADRANT == 3 & !is.na(Data$lat_end))
  if (length(idx_3_shoot) > 0) {
    Data$lat_start[idx_3_shoot] <- -1 * Data$lat_start[idx_3_shoot]
  }
  if (length(idx_3_haul) > 0) {
    Data$lat_end[idx_3_haul] <- -1 * Data$lat_end[idx_3_haul]
  }

  # quadrant 5
  idx_5_shoot <- which(Data$SHOOTING_QUADRANT == 5 & !is.na(Data$lat_start))
  idx_5_haul  <- which(Data$HAULING_QUADRANT == 5 & !is.na(Data$lat_end))
  if (length(idx_5_shoot) > 0) {
    Data$lat_start[idx_5_shoot] <- -1 * Data$lat_start[idx_5_shoot]
    Data$lon_start[idx_5_shoot] <- -1 * Data$lon_start[idx_5_shoot]
  }
  if (length(idx_5_haul) > 0) {
    Data$lat_end[idx_5_haul] <- -1 * Data$lat_end[idx_5_haul]
    Data$lon_end[idx_5_haul] <- -1 * Data$lon_end[idx_5_haul]
  }

  # quadrant 7
  idx_7_shoot <- which(Data$SHOOTING_QUADRANT == 7 & !is.na(Data$lon_start))
  idx_7_haul  <- which(Data$HAULING_QUADRANT == 7 & !is.na(Data$lon_end))
  if (length(idx_7_shoot) > 0) {
    Data$lon_start[idx_7_shoot] <- -1 * Data$lon_start[idx_7_shoot]
  }
  if (length(idx_7_haul) > 0) {
    Data$lon_end[idx_7_haul] <- -1 * Data$lon_end[idx_7_haul]
  }

  return(Data)
}

#' Aggregate MEDITS data across multiple GSAs
#'
#' Aggregate TA, TB, and TC MEDITS tables across a user-defined set of
#' Geographical Sub-Areas (GSAs), and update the associated stratification
#' objects accordingly.
#'
#' This function filters the input TA, TB, and TC tables to retain only the
#' selected GSAs, concatenates the original \code{AREA} and \code{HAUL_NUMBER}
#' values to create unique haul identifiers across GSAs, and replaces the
#' \code{AREA} field with a single aggregated GSA code obtained by collapsing
#' the selected GSA values into one string. The same aggregation is applied to
#' the \code{stratification} and \code{strata_scheme} tables.
#'
#' @param ta A data frame containing the MEDITS \code{TA} table.
#' @param tb A data frame containing the MEDITS \code{TB} table.
#' @param tc A data frame containing the MEDITS \code{TC} table.
#' @param gsas A numeric or character vector of GSA codes to be aggregated.
#' @param strata_scheme A data frame describing the depth-strata scheme for the
#'   selected GSAs. Defaults to \code{BioIndex::strata_scheme}.
#' @param stratification A data frame containing the stratification information
#'   for the selected GSAs. Defaults to \code{BioIndex::stratification}.
#'
#' @details
#' The function performs three main operations:
#' \enumerate{
#'   \item Checks that all requested GSAs are present in the \code{AREA} column
#'   of the input \code{ta}, \code{tb}, and \code{tc} tables.
#'   \item Filters the three MEDITS tables to the selected GSAs and updates
#'   \code{HAUL_NUMBER} by prefixing it with the original \code{AREA} value,
#'   ensuring uniqueness after aggregation.
#'   \item Filters and updates the \code{stratification} and
#'   \code{strata_scheme} tables so that all selected GSAs are represented under
#'   a single aggregated GSA code.
#' }
#'
#' The aggregated GSA code is created using \code{paste(gsas, collapse = "")}.
#' For example, \code{c(17, 18)} becomes \code{"1718"}.
#'
#' @return
#' A list with five elements:
#' \describe{
#'   \item{[[1]]}{The filtered and aggregated \code{TA} table.}
#'   \item{[[2]]}{The filtered and aggregated \code{TB} table.}
#'   \item{[[3]]}{The filtered and aggregated \code{TC} table.}
#'   \item{[[4]]}{The filtered and updated \code{stratification} table.}
#'   \item{[[5]]}{The filtered and updated \code{strata_scheme} table.}
#' }
#'
#' @examples
#' # Use internal data
#' data("strata_scheme", package = "BioIndex")
#' data("stratification", package = "BioIndex")
#' data("TA", package = "BioIndex")
#' data("TB", package = "BioIndex")
#' data("TC", package = "BioIndex")
#'
#' d <- aggregate_gsas(
#'   ta = TA,
#'   tb = TB,
#'   tc = TC,
#'   gsas = 10,
#'   strata_scheme = BioIndex::strata_scheme,
#'   stratification = BioIndex::stratification
#' )
#'
#' ta_agg <- d[[1]]
#' tb_agg <- d[[2]]
#' tc_agg <- d[[3]]
#' stratification_agg <- d[[4]]
#' strata_scheme_agg <- d[[5]]
#'
#' @seealso
#' \code{\link[BioIndex:BioIndex]{strata_scheme}},
#' \code{\link[BioIndex:BioIndex]{stratification}}
#'
#' @export

aggregate_gsas <- function(ta,tb,tc,gsas, strata_scheme = BioIndex::strata_scheme, stratification = BioIndex::stratification) {

    if (FALSE) {
        ta <- read.table("D:/OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L/______ MEDITS DATA __OFFICIAL___/MEDBSsurvey/Demersal/TA_MEDITS_FORMAT_2025.csv", sep=";",header = TRUE)

        tb <- read.table("D:/OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L/______ MEDITS DATA __OFFICIAL___/MEDBSsurvey/Demersal/TB_MEDITS_FORMAT_2025.csv", sep=";",header = TRUE)

        tc <- read.table("D:/OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L/______ MEDITS DATA __OFFICIAL___/MEDBSsurvey/Demersal/TC_MEDITS_FORMAT_2025.csv", sep=";",header = TRUE)

        gsas <- c(17,18)

        strata_scheme = BioIndex::strata_scheme
        stratification = BioIndex::stratification

    }

    try(if(!all(gsas %in% unique(ta$AREA))) stop("\nOne or more GSAs in the selection is not included in the TA table"))

    try(if(!all(gsas %in% unique(tb$AREA))) stop("\nOne or more GSAs in the selection is not included in the TB table"))

    try(if(!all(gsas %in% unique(tc$AREA))) stop("\nOne or more GSAs in the selection is not included in the TC table"))

    ta <- ta[ta$AREA %in% gsas, ]
    tb <- tb[tb$AREA %in% gsas, ]
    tc <- tc[tc$AREA %in% gsas, ]
    ta$HAUL_NUMBER <- paste(ta$AREA,ta$HAUL_NUMBER,sep="_")
    tb$HAUL_NUMBER <- paste(tb$AREA,tb$HAUL_NUMBER,sep="_")
    tc$HAUL_NUMBER <- paste(tc$AREA,tc$HAUL_NUMBER,sep="_")
    ta$AREA <- paste(gsas, collapse="")
    tb$AREA <- paste(gsas, collapse="")
    tc$AREA <- paste(gsas, collapse="")

    stratification <- stratification[stratification$GSA %in% gsas, ]
    strata_scheme <- strata_scheme[strata_scheme$GSA %in% gsas, ]

    stratification$GSA <- paste(gsas, collapse="")
    strata_scheme$GSA <- paste(gsas, collapse="")

    return(list(ta,tb,tc,stratification,strata_scheme))
}

#' Check date haul (RoME)
#' @description
#' Check if in TB, TC or TE the date by haul is the same of the one reported in TA.
#'
#' @param DataTA MEDITS TA table
#' @param Data MEDITS TB, TC or TE table
#' @param year reference year for the analysis
#' @param wd working directory
#' @param suffix name of the log file
#' @note
#' This function is an internal routine based on \strong{RoME version 0.2.3}.
#' It is provided within \code{BioIndex} to ensure the package remains
#' functional and self-sufficient for data validation.
#' @return \code{TRUE} if the validation passes, \code{FALSE} otherwise.
#' @examples
#' data(TA)
#' data(TB)
#' check_date_haul(TA, TB, year = 2007)
#' @export
check_date_haul <- function(DataTA, Data, year, wd=NA, suffix=NA) {

  if (is.na(wd)) {
    wd <- tempdir()
  }

  if (!file.exists(file.path(wd, "Logfiles"))) {
    dir.create(file.path(wd, "Logfiles"), recursive = TRUE, showWarnings = FALSE)
  }
  
  if (missing(suffix) || is.na(suffix)) {
    suffix <- paste0(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"))
  }
  
  Errors <- file.path(wd, "Logfiles", paste0("Logfile_", suffix, ".dat"))
  if (!file.exists(Errors)) {
    file.create(Errors)
  }

  if (length(year) != 1 || is.na(year)) {
    stop("Argument 'year' must be a single non-NA value")
  }
  
  DataTA <- DataTA[DataTA$YEAR == year, ]
  Data <- Data[Data$YEAR == year, ]
  
  DataTA <- DataTA[!is.na(DataTA$AREA), ]
  Data <- Data[!is.na(Data$AREA), ]

  if (nrow(Data) == 0) {
    write(paste("\n----------- check correctness of date by haul - ", year), file = Errors, append = TRUE)
    write("No data found for this year", file = Errors, append = TRUE)
    return(TRUE)
  }

  tf <- as.character(Data$TYPE_OF_FILE[1])
  write(paste("\n----------- check correctness of date by haul in", tf, "-", year), file = Errors, append = TRUE)

  Data$Date <- paste(Data$AREA, Data$COUNTRY, Data$HAUL_NUMBER, Data$DAY, Data$MONTH, Data$YEAR, sep="-")
  TA_df <- DataTA
  TA_df$Date <- paste(TA_df$AREA, TA_df$COUNTRY, TA_df$HAUL_NUMBER, TA_df$DAY, TA_df$MONTH, TA_df$YEAR, sep="-")

  numberError <- 0
  unique_dataset <- Data[!duplicated(Data$Date), ]
  
  for (i in 1:nrow(unique_dataset)) {
    if (!(unique_dataset$Date[i] %in% TA_df$Date)) {
      if (tf != "TL") {
        write(paste("Haul", unique_dataset$HAUL_NUMBER[i], ", code species", unique_dataset$GENUS[i], unique_dataset$SPECIES[i], ": the date is not consistent with the date reported in TA."), file = Errors, append = TRUE)
      } else {
        write(paste("Haul", unique_dataset$HAUL_NUMBER[i], ": the date is not consistent with the date reported in TA."), file = Errors, append = TRUE)
      }
      numberError <- numberError + 1
    }
  }

  if (numberError == 0) {
    write("No error occurred", file = Errors, append = TRUE)
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' Conversion of decimal degrees coordinates in MEDITS format
#' @description Conversion of decimal degrees coordinates in MEDITS format
#' @param data data frame of the hauls data (TA, table A) in MEDITS format
#' @return The function returns the data frame of the TA (table A) reporting the coordinates in MEDITS format.
#' @examples
#' data(TA)
#' ta_dd <- MEDITS.to.dd(TA)
#' dd.to.MEDITS(ta_dd)
#' @export
dd.to.MEDITS <- function(data)  {

  deg <- floor(data$SHOOTING_LATITUDE)
  dec <- data$SHOOTING_LATITUDE - (deg)
  dd  <- (deg*100)+(dec*60)
  data$SHOOTING_LATITUDE <- abs(dd)

  ##
  deg <- floor(data$SHOOTING_LONGITUDE)
  dec <- data$SHOOTING_LONGITUDE - (deg)
  dd  <- (deg*100)+(dec*60)
  data$SHOOTING_LONGITUDE <- abs(dd)

  ##
  deg <- floor(data$HAULING_LATITUDE)
  dec <- data$HAULING_LATITUDE - (deg)
  dd  <- (deg*100)+(dec*60)
  data$HAULING_LATITUDE <- abs(dd)

  ##
  deg <- floor(data$HAULING_LONGITUDE)
  dec <- data$HAULING_LONGITUDE - (deg)
  dd  <- (deg*100)+(dec*60)
  data$HAULING_LONGITUDE <- abs(dd)

  return(data)
}

#' @importFrom stats predict AIC nls
#' @importFrom stats pnorm cor.test rnorm dchisq deviance df.residual na.exclude sd time
#' @importFrom mgcv gam
.ChangeYears.func<-function(Year, Indic ,Sd=NULL,alpha=0.05,species,B=199,ytext,CV=0.01,lastn=5,write=FALSE,speciesname,lastligne=FALSE) {
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))

  if (FALSE) {
    Year=years
    Indic=indi
    Sd=rep(NA,length(indi))
    alpha=alpha
    species=system
    B=199
    ytext=indicators[j]
    CV=CVindicators
    lastn=lastn
    write=FALSE
    speciesname=indiname[j]
    lastligne=FALSE

    }

    utils::globalVariables(c("lab_y", "AGE", "LENGTH", "LENGTH_CLASS", "med_bathy", "strata"))




#searches for change in second derivative of index
# Year input data = year vector
# Indic input data = indicator time series estimates as vector
# Sd input data = standard deviation estimates as vector
# species = species name
# if no Sd exists, a CV of 1% is assumed; needed for incorporating uncertainty
# B number of samples for parametric bootstrap (normal distribution)
# ytext : label of indicator name for plot
# lastn : number of recent years (time horizon) for diagnosis of change



####AUXILIARY FUNCTIONS###########
.Ggam.func<-function(G,annee,first=2,df=1){
 #function for fitting Gam to G series
 #first=1 returns df for fit to raw data and p-value for chisquare test of fit
 #first=2 returns fitted values
 if(first==1){
      if(length(annee)>10) fit<-gam(G~s(annee,bs="tp"))
      else fit<-gam(G~s(annee,bs="tp",k=length(annee)-1))
      pchisq<-dchisq(deviance(fit),df.residual(fit))
 }
 else fit<-gam(G~s(annee,k=df+1,fx=TRUE,bs="tp"))
 ifelse(first==1,return(list(df=summary(fit)$edf,pchisq=pchisq)),return(fit$fitted))
}

.Deriv.func<-function(G,annee,df)
{
#uses local difference between fitted function to estimate gradients
#returns first derivative for years with observations
#and  second derivative
 delta <- 1e-7
 data<-data.frame(G=G,annee=annee)
 fit<-gam(G~s(annee,k=df+1,fx=TRUE,bs="tp"),data=data)

 #predict years-delta and years on scale of linear predictor
 data.pred<-data.frame(annee=c(annee-delta,annee))
 Xp1<-predict.gam(fit,data.pred,type="lpmatrix")

 #predict years and years+delta on scale of linear predictor
 data.pred<-data.frame(annee=c(annee,annee+delta))
 Xp2<-predict.gam(fit,data.pred,type="lpmatrix")

 #calculate finite difference Xp(annee+delta) - Xp(annee) of linear predictor
#
 ny<-length(annee)
 X1<-(Xp1[(ny+1):(2*ny),]-Xp1[1:ny,])/delta
 X2<-(Xp2[(ny+1):(2*ny),]-Xp2[1:ny,])/delta

  # GAM model Y= b X = f(years)
 # gradients dY=b dX
 deriv1<-X2%*%coef(fit) ## gradient
 #Vg <- X1%*%vcov(fit)%*%t(X1) ## cov matrix of gradiant

 #2nd derivative is difference between adjacent gradients
 deriv2<-deriv1 - X1%*%coef(fit)

 return(data.frame(deriv1=deriv1,deriv2=deriv2))
}

######
.Bootstrap.func<-function(D)
{
#bootrap using normal distribution
res<-rep(NA,nrow(D))
for(i in 1:nrow(D)) res[i]<-rnorm(1,D[i,1],D[i,2])
res
}

######ChangeSummary.func############
.ChangeSummary.func<-function(D=Smoothed,lastn,Indic,Year,write=write,alpha)
{
if(FALSE){
D=Smoothed
Indic=Indic
Year=Year
lastn=lastn
alpha=alpha
}
#for carrying out individual union tests and final test intersection test
#D= results table of smoothed trends and derivatives with sd
#Indic= raw index estimate
#lastn= time horizon in which minimum should not be, ie 5 most recent years
#lastnmin= time horizon for minimum value
#names(Smoothed)=c("Species","Year","G","Gsd","G1","G1sd","G2","G2sd")

if(is.factor(D$Year)) D$Year<-as.numeric(levels(D$Year)[D$Year])
if(is.factor(D$G)) D$G<-as.numeric(levels(D$G)[D$G])
if(is.factor(D$Gsd)) D$Gsd<-as.numeric(levels(D$Gsd)[D$Gsd])
if(is.factor(D$G1)) D$G1<-as.numeric(levels(D$G1)[D$G1])
if(is.factor(D$G1sd)) D$G1sd<-as.numeric(levels(D$G1sd)[D$G1sd])
if(is.factor(D$G2)) D$G2<-as.numeric(levels(D$G2)[D$G2])
if(is.factor(D$G2sd)) D$G2sd<-as.numeric(levels(D$G2sd)[D$G2sd])

years<-seq(min(D$Year),max(D$Year),1)
res<-matrix(NA,1,ncol=16)

#Intersection-Union test
#There are 3 union tests, ie for each group H0 is true if any one of the tests is true
#In the final test, the global H0 of no change is true if it is true for all 3 groups

#1. test for location of maximum and minimum of series
#H0 no decrease: max in final lastn years: binomial(1,theta), theta => 1-alpha
#H1 decrease: max NOT before final lastnmin years
#H0 no increase: min in final lastn years: binomial(1,theta), theta => 1-alpha
#H1 increase: min NOT before final lastnmin years
T1Dec<-ifelse(is.element(max(D$G),rev(D$G)[1:lastn]),1,0)
T1Inc<-ifelse(is.element(min(D$G),rev(D$G)[1:lastn]),1,0)

#2. test for local slopes in last lastn years
#HO no decrease: One of the annual slopes >0
#H1 decrease: ALL annual slopes <=0
#HO no increase: One of the annual slopes <0
#H1 increase: ALL annual slopes >=0
t2dec=t2inc=rep(0,lastn)
i=1
for(i in 1:lastn){
 testdec<-1-pnorm((rev(as.numeric(D$G1))[i]/rev(as.numeric(D$G1sd))[i]),lower.tail=FALSE)
 t2dec[i]=sum(testdec<=alpha) #=1 if H0 not true
}
T2Dec<-ifelse(sum(t2dec)<lastn,1,0) # =0 if H0 rejected, =1 if H0 accepted

for(i in 1:lastn){
 testinc<-1-pnorm((rev(as.numeric(D$G1))[i]/rev(as.numeric(D$G1sd))[i]),lower.tail=TRUE)
 t2inc[i]=sum(testinc<=alpha) #=1 if H0 not true
}
T2Inc<-ifelse(sum(t2inc)<lastn,1,0) #H0 not true only if ALL tests are significant!

#3. test for second derivatives in lastn years, ignoring the final year estimate,
#H0 no decrease: some second derivatives over lastn years >=0
#H1 decrease: all second derivatives over lastn years <0
#H0 no increase: some second derivatives <=0
#H1 increase: all second derivatives >0

t3dec=t3inc=rep(0,lastn)
for(i in 1:lastn){
 testdec<-pnorm((rev(as.numeric(D$G2))[i]/rev(as.numeric(D$G2sd))[i]),lower.tail=TRUE)
 t3dec[i]=sum(testdec<=alpha) #=1 if H0 not true
}
T3Dec<-ifelse(sum(t3dec,na.rm=TRUE)<lastn,1,0) # =0 if H0 rejected, =1 if H0 accepted

for(i in 1:lastn){
 testinc<-pnorm((rev(as.numeric(D$G2))[i]/rev(as.numeric(D$G2sd))[i]),lower.tail=FALSE)
 t3inc[i]=sum(testinc<=alpha) #=1 if H0 not true
}
T3Inc<-ifelse(sum(t3inc,na.rm=TRUE)<lastn,1,0) #H0 not true only if ALL tests are significant!

#######linear model test#####################
#linear slope over nlastmin years
#print(years)
id<-Year>=(rev(years)[lastn])
SlopeLinRec<- coefficients(lm(Indic[id]~Year[id]))[2]
#p-value of linear slope over lastn years
PLinRec<- coefficients(summary(lm(Indic[id]~Year[id])))[8]
#linear slope over all years
SlopeLinAll<- coefficients(lm(Indic~Year))[2]
#p-value of linear slope over all years
PLinAll<- coefficients(summary(lm(Indic~Year)))[8]

###Mann Kendall test
#print(Indic[id])
#print(Year[id])
PMKinc<-cor.test(scale(Year[id]),Indic[id],method="kendall",alternative="greater")$p.value
PMKdec<-cor.test(scale(Year[id]),Indic[id],method="kendall",alternative="less")$p.value

#create recent diagnose ###################
#code -1 for recent "decreasing" :
#codage 1 for recent "increase" :
#T1Dec etc =1 if H0 true!!!
#Global H0 rejected if ALL H0s are rejected!!!, ie T1+T2+T3=0
#Tx=1 if corresponding H0 is true!
RecentDec<-ifelse((T1Dec+T2Dec)==0,-1,0)
ChangePointDec<-ifelse(T3Dec==1,0,-1)#=0 if change point occured, =-1 if not
RecentInc<-ifelse((T1Inc+T2Inc)==0,1,0)
ChangePointInc<-ifelse(T3Inc==1,0,1)#=0 if change point occured, =1 if not
#print(c(RecentDec,RecentInc))
#print(c("all Tinc",T1Inc, T2Inc,T3Inc,"RecentInc",RecentInc))

#Create diagnose based on linear slope for last nlast (5) years
RecentDiagnosL5<-0
RecentDiagnosL5[SlopeLinRec<0&PLinRec<=alpha]<-(-1)
RecentDiagnosL5[SlopeLinRec>0&PLinRec<=0.05]<-(1)

#create diagnos based on Mann-Kendall test
RecentMannKendallDec<-0
RecentMannKendallDec[PMKdec<=alpha]<-(-1)
RecentMannKendallInc<-0
RecentMannKendallInc[PMKinc<=alpha]<-1

res=c(RecentDec,ChangePointDec,RecentInc,ChangePointInc,RecentDiagnosL5,PLinRec,
RecentMannKendallDec,RecentMannKendallInc,PMKdec,PMKinc)
return(res)
}

#################################
#main
# remove years with missing data
id<-!is.na(Indic);Indic<-Indic[id];Year<-Year[id];Sd<-Sd[id]

#create vector of standar deviations if not provided using given CV
#same if all standard deviations are NA
if(is.null(Sd)|sum(Sd)==0|is.na(sum(Sd)))Sd<-Indic*CV
ind <- scale(1:length(Year), scale = FALSE)

resmax<-res2nd<-matrix(0,1,length(Year))
reslow<-matrix(0,1,4);reshigh<-matrix(0,1,2)
reslast<-matrix(NA,1,6)
dimnames(resmax)<-list(species,as.character(Year))
dimnames(reslow)<-list(species,c("mindata","minGam","lastYear","lastyear"))
dimnames(reshigh)<-list(species,c("maxdata","maxGam"))
dimnames(reslast)<-list(species,c("minGdata","minGGam","lastGdata","lastGGam","maxGdata","maxGGam"))
i<-1

#statistics G
G<-Indic
anneesp<-Year
ida<-(1:length(Year))[is.element(Year,anneesp)]
ida<-ida[!is.na(ida)]
#dflnG<-ceiling(SecondDeriv.func(G,anneesp,first=1))

#degrees of freedom for smooth function, minimum 2
fit<-.Ggam.func(G,anneesp,first=1)
dfG<-ceiling(fit$df)
#pvalue for chisquare test of gam fit (residual deviance)
pchisq<-fit$pchisq
if(dfG==1) dfG<-2
#linear model whole time series
indsp <- scale(1:length(anneesp), scale = FALSE)
Gfit<-lm(G ~ indsp, na.action = na.exclude)
Gpvalue <- summary(Gfit)$coefficients[8]

#smooth model; get predicted values at years with observations
Ggam<-.Ggam.func(G,anneesp,first=2,df=dfG)

Db<-Db.orig<-cbind(Indic,Sd)
Db<-data.frame(Db)
Gboot<-Gderiv1<-Gderiv2<-Ggam<-matrix(NA,nrow=B,ncol=length(Year))
Gboot[1,ida]<-G
Ggam[1,ida]<-.Ggam.func(G,anneesp,first=2,df=dfG)#predicted values
Derivs<-.Deriv.func(G,anneesp,df=dfG)
Gderiv1[1,ida]<-Derivs$deriv1
Gderiv2[1,ida]<-Derivs$deriv2

#bootstrap for standard deviations of derivatives
for(j in 2:B) {
 		  Db$Estimation<-.Bootstrap.func(Db.orig)
		  Gboot[j,ida]<- Db$Estimation
		  Ggam[j,ida]<-.Ggam.func(Gboot[j,ida],anneesp,first=2,df=dfG)
		  Derivs<-.Deriv.func(Gboot[j,ida],anneesp,df=dfG)
		  Gderiv1[j,ida]<-Derivs$deriv1
		  Gderiv2[j,ida]<-Derivs$deriv2
}
G1sd<-apply(Gderiv1,2,sd,na.rm=TRUE)
G2sd<-apply(Gderiv2,2,sd,na.rm=TRUE)
Ggamsd<-apply(Ggam,2,sd,na.rm=TRUE)

#create results matrix:
yr=length(anneesp)
Smoothed<-cbind(rep(species,yr),anneesp,Ggam[1,],Ggamsd,Gderiv1[1,],G1sd,Gderiv2[1,],G2sd)
tab=cbind(rep(species,yr),anneesp,round(pchisq,3),round(Ggam[1,],2),round(Ggamsd,2),round(Gderiv1[1,],2),
round(G1sd,2),Gderiv2[1,],G2sd)
Smoothed=data.frame(Smoothed)
names(Smoothed)=c("Species","Year","G","Gsd","G1","G1sd","G2","G2sd")
tab=data.frame(tab)
names(tab)=c("Species","Year","pchisq","G","Gsd","G1","G1sd","G2","G2sd")
if(write==TRUE)
 write.table(tab,paste(species,ytext,".txt",sep=""),col.names=TRUE,row.names=FALSE,
 sep=";",quote=FALSE)

Diagnos<-.ChangeSummary.func(D=Smoothed,Indic=Indic,Year=Year,lastn=lastn,alpha=alpha)
Diagnos=data.frame(matrix(Diagnos,nrow=1))
names(Diagnos)=c("RecentDec","ChangePointDec","RecentInc","ChangePointInc",
"RecentDiagnoseL5","Plin","RecentMannKendallDec","RecentMannKendallInc",
"PMKdec","PMKinc")
#plot indicator time series with smooths
par(xaxs = "r")
if(Diagnos$RecentDiagnoseL5==0)textdiaglin="stable"
if(Diagnos$RecentDiagnoseL5<0)textdiaglin="decrease"
if(Diagnos$RecentDiagnoseL5>0)textdiaglin="increase"

textregle="unknown"
if(Diagnos$RecentDec==0 &Diagnos$RecentInc==0) textregle="stable"
if(Diagnos$RecentDec<0) textregle="decrease"
#if(Diagnos$ChangePointDec==0) textregle=paste(textregle,"*",sep="")
if(Diagnos$RecentInc>0) textregle="increase"
#if(Diagnos$ChangePointInc==0) textregle=paste(textregle,"*",sep="")

#par(mar=c(2,2,2,0.5))
.error.bar(as.integer(anneesp), G, lower = apply(Gboot[,ida],2,quantile,0.025,na.rm=TRUE),
		upper = apply(Gboot[,ida],2,quantile,0.975,na.rm=TRUE), incr = FALSE,
main = speciesname,xlab ="year", ylab =lab_y,cex.main=1.5,axes=TRUE,bty="o")
axis(2,labels=TRUE)
ifelse(lastligne,axis(1,labels=TRUE),axis(1,labels=TRUE))
box()
#main = paste(species,"\n","Changes in last", lastn,"years", "linear:",textdiaglin,
#"; rule:",textregle),xlab ="Year", ylab = ytext,cex.main=0.7)
#add smooth lines from gam + 2.5 and 97.5 percentiles
lines(anneesp,Ggam[1,ida],lty=1)
lines(anneesp,apply(Ggam[,ida],2,quantile,0.025,na.rm=TRUE),lty=2)
lines(anneesp,apply(Ggam[,ida],2,quantile,0.975,na.rm=TRUE),lty=2)
#savePlot(paste(ytext,species),type="wmf")
#
return(c(species,ytext,pchisq,Diagnos))

}



.error.bar<-function(x, y = NULL, lower, upper, incr = TRUE, bar.ends = FALSE, gap = FALSE, add = FALSE, horizontal = FALSE, ..., xlab = deparse(substitute(x)), xlim, ylim) {
	draw.null.warn <- function(draw, gap)
	{
		if(!any(draw)) {
			warning("Not enough room for a gap.")
			draw <- !draw
			gap <- 0
		}
		invisible(list(draw = draw, gap = gap))
	}
	if(missing(x))
		stop("no data for x or y")
	if(missing(y)) {
		if(missing(xlab))
			xlab <- "Index"
		y <- x
		x <- time(x)
	}
	n <- length(x)
	if(length(y) != n)
		stop("length of y must equal the length of x")
	center <- if(horizontal) x else y
	if(missing(lower))
		stop("you must provide lower")
	if(length(lower) > 1 && length(lower) != n)
		stop("length of lower must be 1 or equal to the length of x")
	#if incr=T lower is assumed >=0
	if(incr) lower <- center - abs(lower) else lower <- rep(lower,
			length = n)
	if(any(lower >= center))
		warning(paste(
			"There are values of 'lower' which are greater or equal to ",
			if(horizontal) "x" else "y"))
	if(missing(upper))
		upper <- 2 * center - lower
	else {
		if(length(upper) > 1 && length(upper) != n)
			stop("length of upper must be 1 or equal to the length of x"
				)
		if(incr)
			upper <- center + upper
		else upper <- rep(upper, length = n)
	}
	if(any(upper <= center))
		warning(paste(
			"There are values of 'upper' which are smaller or equal to ",
			if(horizontal) "x" else "y"))
	if(!add)
		if(horizontal) {
			if(missing(ylim))
				plot(x, y, xlim = if(missing(xlim)) range(
						c(lower, upper), na.rm = TRUE
						) else xlim, xlab = xlab,
					...)
			else plot(x, y, xlim = if(missing(xlim)) range(
						c(lower, upper), na.rm = TRUE
						) else xlim, ylim = ylim,
					xlab = xlab, ...)
		}
		else {
			if(missing(xlim))
				plot(x, y, ylim = if(missing(ylim)) range(
						c(lower, upper), na.rm = TRUE
						) else ylim, xlab = xlab,
					...)
			else plot(x, y, ylim = if(missing(ylim)) range(
						c(lower, upper), na.rm = TRUE
						) else ylim, xlim = xlim,
					xlab = xlab, ...)
		}
	if(horizontal) {
		if(gap)
			gap <- 0.75 * par("cxy")[1]
		draw <- x - lower > gap
		z <- draw.null.warn(draw, gap)
		draw <- z$draw
		gap <- z$gap
		segments(lower[draw], y[draw], x[draw] - gap, y[draw])
		draw <- upper - x > gap
		z <- draw.null.warn(draw, gap)
		draw <- z$draw
		gap <- z$gap
		segments(x[draw] + gap, y[draw], upper[draw], y[draw])
		if(bar.ends) {
			size.bar <- par("cxy")[2]
			segments(lower, y - size.bar, lower, y + size.bar)
			segments(upper, y - size.bar, upper, y + size.bar)
		}
	}
	else {
		if(gap)
			gap <- 0.75 * par("cxy")[2]
		draw <- upper - y > gap
		z <- draw.null.warn(draw, gap)
		draw <- z$draw
		gap <- z$gap
		segments(x[draw], y[draw] + gap, x[draw], upper[draw])
		draw <- y - lower > gap
		z <- draw.null.warn(draw, gap)
		draw <- z$draw
		gap <- z$gap
		segments(x[draw], y[draw] - gap, x[draw], lower[draw])
		if(bar.ends) {
			size.bar <- par("cxy")[1]
			segments(x - size.bar, upper, x + size.bar, upper)
			segments(x - size.bar, lower, x + size.bar, lower)
		}
	}
}

#' Estimate hauls distances (decimal degrees)
#' @description
#' Function to estimate the hauls length using TA (table A, hauls data) with coordinates in the decimal degrees format (dd.ddd). The distances could be returned expressed in meters, kilometers and nautical miles.
#'
#' @param data data frame of the hauls data (TA, table A) with coordinates reported as decimal degrees
#' @param unit string value indicating the measure unit of the distance. Allowed values: "m" for meters, "km" for kilometers and "NM" for nautical miles.
#' @param verbose give verbose output reporting in the output the selected measure unit of the distance.
#' @return The function returns the vector of the distances expressed in the selected measure unit.
#' @examples
#' data(TA)
#' ta_dd <- MEDITS.to.dd(TA)
#' dd.distance(ta_dd, unit = "km", verbose = FALSE)
#' @export
dd.distance<-function(data, unit = "m", verbose=TRUE)  {
  N1  <- (((data$SHOOTING_LATITUDE/2)+45)*pi )/180
  N2  <- (((data$HAULING_LATITUDE/2)+45)*pi )/180
  N3  <- atan((pi*(data$HAULING_LONGITUDE-data$SHOOTING_LONGITUDE))/(180*(log(tan(N2))-log(tan(N1)))))
  dist <- abs(60*(data$HAULING_LATITUDE-data$SHOOTING_LATITUDE)/cos(N3))*1852
  if (unit =="m"){
    dist = dist
    if (verbose){
    message("meters")
      }
  }
  if (unit =="km"){
    dist = dist/1000
    if (verbose){
    message("kilometers")
    }
    }
  if (unit =="NM"){
    dist = dist/1852
    if (verbose){
      message("Nautical Miles")
    }
      }
  return(dist)
}

#' Merge TAâ€“TB and TAâ€“TC tables (MEDITS protocol)
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @description
#' This function is the primary entry point for integrating MEDITS survey data
#' (e.g. TA, TB, and TC tables), producing unified datasets essential for
#' population analysis and the estimation of demographic indicators.
#' The routine:
#' \itemize{
#'   \item filters hauls by validity and by country;
#'   \item performs a full set of quality checks (times, positions, wing opening,
#'         haul/date consistency) using internal validation functions
#'         (based on \code{RoME} version 0.2.3);
#'   \item merges TA with TB and TA with TC, respectively;
#'   \item computes swept area, mean positions, depth stratum, raising
#'         factors, densities and biomasses;
#'   \item (optionally) writes the two merged tables to
#'         \code{file.path(wd, "output")}.
#' }
#'
#' @param ta      A MEDITS or MEDITS-like \strong{TA} table
#'                (columns listed in \code{BioIndex::TA_cols}).
#' @param tb      A MEDITS or MEDITS-like \strong{TB} table
#'                (columns listed in \code{BioIndex::TB_cols}).
#' @param tc      A MEDITS or MEDITS-like \strong{TC} table
#'                (columns listed in \code{BioIndex::TC_cols}).
#' @param species Character. MEDITS 7-letter RUBIN code (e.g. \code{"MERLMER"}):
#'                the first 4 letters are used as \emph{GENUS}, the last 3 as
#'                \emph{SPECIES}.
#' @param country Character vector of MEDITS country codes to keep.
#'                Use \code{"all"} (default) to include every country present
#'                in \code{ta}.
#' @param strata  A data-frame with the depth-stratification scheme adopted
#'                by the MEDITS survey. Defaults to
#'                \code{BioIndex::strata_scheme}.
#' @param wd      Working directory. When \code{save = TRUE}, the merged
#'                tables are written to \code{file.path(wd, "output")}.
#' @param save    Logical. If \code{TRUE} (default) the function writes
#'                \code{"mergeTATB_<GENUS><SPECIES>.csv"} and
#'                \code{"mergeTATC_<GENUS><SPECIES>.csv"}.
#' @param verbose Logical (default \code{TRUE}); prints progress messages.
#'
#' @return A list of two \code{data.frame}s:
#'   \describe{
#'     \item{\code{merge_TA_TB}}{One row per haul with TA metadata, TB catch
#'           totals, depth stratum, densities and biomasses.}
#'     \item{\code{merge_TA_TC}}{One row per haul/length-class/sex/maturity
#'           with TA metadata, TC counts, depth stratum, raising factor,
#'           densities and biomasses.}
#'   }
#'
#' @details
#' The implementation reproduces the official BioIndex workflow.
#'
#' \strong{Data Validation:}
#' The function automatically performs syntactic data validation using internal
#' routines based on \code{RoME} version 0.2.3 logic to ensure data integrity
#' and standardisation.
#'
#' \strong{Optimisations:}
#' Two micro-optimisations are included:
#' \enumerate{
#'   \item vectorised replacement of missing \code{"NA NA"} entries
#'         in both merges;
#'   \item a single loop over depth strata (instead of haul Ã— stratum) to
#'         assign \code{STRATUM_CODE}.
#' }
#' Numerical outputs are identical to the reference routine.
#'
#' @note
#' This version of \code{BioIndex} uses internal validation functions based on
#' \strong{RoME version 0.2.3}. The package is fully self-sufficient and does
#' not require external validation packages to be installed.
#'
#' @return A \code{list} containing two data frames: the first with merged TA-TB-TC data at the haul level, and the second with the length-frequency distribution.
#' @examples
#' # Use internal data
#'
#' data("TA", package = "BioIndex")
#' data("TB", package = "BioIndex")
#' data("TC", package = "BioIndex")
#' m <- merge_TATBTC(
#'   ta      = TA,
#'   tb      = TB,
#'   tc      = TC,
#'   species = "MERLMER",
#'   country = "all",
#'   wd      = tempdir(),
#'   verbose = TRUE
#' )
#' mTATB <- m[[1]]   # TAâ€“TB merged table
#' mTATC <- m[[2]]   # TAâ€“TC merged table
#'
#' @export
#' @importFrom hms hms
#' @importFrom utils write.table
merge_TATBTC <- function(ta, tb, tc,
                         species,
                         country = "all",
                         strata  = BioIndex::strata_scheme,
                         wd      = NA,
                         save    = TRUE,
                         verbose = TRUE) {

  if (is.na(wd)) { wd <- tempdir() }

  strata_scheme <- strata  # local alias

  ## ------------------------------------------------------------------ ##
  ##  Column templates                                                  ##
  ## ------------------------------------------------------------------ ##
  TA_cols <- BioIndex::TA_cols
  TB_cols <- BioIndex::TB_cols
  TC_cols <- BioIndex::TC_cols

  ## ------------------------------------------------------------------ ##
  ##  Species code parsing                                              ##
  ## ------------------------------------------------------------------ ##
  species[2] <- substr(species, 5, 7)
  species[1] <- substr(species[1], 1, 4)
  sspp  <- paste(species[1], species[2], sep = "")
  GENERE <- species[1]
  SPECIE <- species[2]

  ## ------------------------------------------------------------------ ##
  ##  Input data frames                                                 ##
  ## ------------------------------------------------------------------ ##
  TA <- ta
  TB <- tb
  TC <- tc

  ## ------------------------------------------------------------------ ##
  ##  COUNTRY filter                                                    ##
  ## ------------------------------------------------------------------ ##
  TA_country <- sort(unique(as.character(TA$COUNTRY)))
  TB_country <- sort(unique(as.character(TB$COUNTRY)))
  TC_country <- sort(unique(as.character(TC$COUNTRY)))
  l_country  <- length(TA_country)

  for (i in seq_len(l_country)) {
    if (!(TA_country[i] %in% TB_country))
      warning(paste("The country ", TA_country[i], " is not present in the TB file.", sep = ""))
    if (!(TA_country[i] %in% TC_country))
      warning(paste("The country ", TA_country[i], " is not present in the TC file.", sep = ""))
  }

  if (l_country == 1) {
    country_analysis <- TA_country
  } else {
    country_analysis <- if (any(country %in% "all")) TA_country else country
  }

  TA <- TA[TA$COUNTRY %in% country_analysis, ]
  TB <- TB[TB$COUNTRY %in% country_analysis, ]
  TC <- TC[TC$COUNTRY %in% country_analysis, ]

  ## ------------------------------------------------------------------ ##
  ##  Unique haul identifiers                                           ##
  ## ------------------------------------------------------------------ ##
  id_TA <- if (nrow(TA) > 0) {
      data.frame(id = paste(TA$AREA, TA$COUNTRY, TA$YEAR, "_",
                            TA$VESSEL, TA$MONTH, TA$DAY, "_",
                            TA$HAUL_NUMBER, sep = ""))
  } else {
      data.frame(id = character(0))
  }
  id_TB <- if (nrow(TB) > 0) {
      data.frame(id = paste(TB$AREA, TB$COUNTRY, TB$YEAR, "_",
                            TB$VESSEL, TB$MONTH, TB$DAY, "_",
                            TB$HAUL_NUMBER, sep = ""))
  } else {
      data.frame(id = character(0))
  }
  id_TC <- if (nrow(TC) > 0) {
      data.frame(id = paste(TC$AREA, TC$COUNTRY, TC$YEAR, "_",
                            TC$VESSEL, TC$MONTH, TC$DAY, "_",
                            TC$HAUL_NUMBER, sep = ""))
  } else {
      data.frame(id = character(0))
  }

  id_invalid <- id_TA$id[which(TA$VALIDITY == "I")]

  ## Data validation: local BioIndex routines (based on RoME 0.2.3)     ##
  suffix <- paste(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"), sep = "")
  for (yy in sort(unique(TA$YEAR))) {
      TAy <- TA[TA$YEAR == yy, ]
      TBy <- TB[TB$YEAR == yy, ]
      TCy <- TC[TC$YEAR == yy, ]

      if (!check_dictionary(TAy, "SHOOTING_TIME", 0:2400, yy, file.path(wd, "output"), suffix))
          stop("SHOOTING_TIME out of range.")
      if (!check_dictionary(TAy, "HAULING_TIME", 0:2400, yy, file.path(wd, "output"), suffix))
          stop("HAULING_TIME out of range.")
      if (!check_dictionary(TAy, "WING_OPENING", c(30, 50:250), yy, file.path(wd, "output"), suffix))
          stop("WING_OPENING out of range.")
      if (!check_numeric_range(TAy, "SHOOTING_LATITUDE", c(3020, 4730), yy, file.path(wd, "output"), suffix))
          stop("SHOOTING_LATITUDE out of range.")
      if (!check_numeric_range(TAy, "HAULING_LATITUDE", c(3020, 4730), yy, file.path(wd, "output"), suffix))
          stop("HAULING_LATITUDE out of range.")
      if (!check_numeric_range(TAy, "SHOOTING_LONGITUDE", c(0, 4200), yy, file.path(wd, "output"), suffix))
          stop("SHOOTING_LONGITUDE out of range.")
      if (!check_numeric_range(TAy, "HAULING_LONGITUDE", c(0, 4200), yy, file.path(wd, "output"), suffix))
          stop("HAULING_LONGITUDE out of range.")
      if (!check_date_haul(TAy, TBy, yy, file.path(wd, "output"), suffix))
          stop("Date in TB not consistent with TA.")
      if (!check_date_haul(TAy, TCy, yy, file.path(wd, "output"), suffix))
          stop("Date in TC not consistent with TA.")
      if (!check_hauls_TBTA(TAy, TBy, yy, file.path(wd, "output"), suffix))
          stop("Hauls in TB not consistent with TA.")
  }

  ## ------------------------------------------------------------------ ##
  ##  Data preparation for merging                                      ##
  ## ------------------------------------------------------------------ ##
  colnames(TA)[colnames(TA) == "AREA"] <- "GSA"
  colnames(TB)[colnames(TB) == "AREA"] <- "GSA"
  colnames(TC)[colnames(TC) == "AREA"] <- "GSA"

  TA_merge <- cbind(id_TA, TA)
  TA_merge <- TA_merge[!TA_merge$id %in% id_invalid, ]
  TA_merge <- TA_merge[, which(colnames(TA_merge) %in% TA_cols)]

  TB_merge <- cbind(id_TB, TB)
  TB_merge$GENUS   <- as.character(TB_merge$GENUS)
  TB_merge$SPECIES <- as.character(TB_merge$SPECIES)
  TB_merge <- TB_merge[!TB_merge$id %in% id_invalid, ]
  TB_merge <- TB_merge[TB_merge$GENUS == species[1] &
                         TB_merge$SPECIES == species[2],
                       which(colnames(TB_merge) %in% TB_cols)]

  TC_merge <- cbind(id_TC, TC)
  TC_merge$GENUS   <- as.character(TC_merge$GENUS)
  TC_merge$SPECIES <- as.character(TC_merge$SPECIES)
  TC_merge <- TC_merge[!TC_merge$id %in% id_invalid, ]
  TC_merge <- TC_merge[TC_merge$GENUS == species[1] &
                         TC_merge$SPECIES == species[2],
                       which(colnames(TC_merge) %in% TC_cols)]

  #######################################################################
  ##                 MERGE TA â€“ TB  (optimised)                        ##
  #######################################################################
  if (verbose) message("- Merging TA-TB files")

  merge_TATB <- merge(TA_merge, TB_merge, by = "id", all.x = TRUE)
  merge_TATB$MEDITS_CODE <- paste(merge_TATB$GENUS, merge_TATB$SPECIES)

  ## -------- Optimization 1: remove row-by-row loop ------------------ ##
  na_idx <- merge_TATB$MEDITS_CODE == "NA NA"
  if (any(na_idx)) {
    merge_TATB$MEDITS_CODE[na_idx]               <- "NA"
    merge_TATB$GENUS[na_idx]                     <- -1
    merge_TATB$SPECIES[na_idx]                   <- -1
    merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL[na_idx]  <- 0
    merge_TATB$TOTAL_NUMBER_IN_THE_HAUL[na_idx]  <- 0
    merge_TATB$NB_OF_FEMALES[na_idx]             <- 0
    merge_TATB$NB_OF_MALES[na_idx]               <- 0
    merge_TATB$NB_OF_UNDETERMINED[na_idx]        <- 0
  }
  ## ------------------------------------------------------------------ ##

  coord <- convert_coordinates(merge_TATB)

  merge_TATB$MEAN_LATITUDE      <- (merge_TATB$SHOOTING_LATITUDE + merge_TATB$HAULING_LATITUDE) / 2
  merge_TATB$MEAN_LATITUDE_DEC  <- (coord$lat_start + coord$lat_end) / 2
  merge_TATB$MEAN_LONGITUDE     <- (merge_TATB$SHOOTING_LONGITUDE + merge_TATB$HAULING_LONGITUDE) / 2
  merge_TATB$MEAN_LONGITUDE_DEC <- (coord$lon_start + coord$lon_end) / 2
  merge_TATB$MEAN_DEPTH         <- (merge_TATB$SHOOTING_DEPTH + merge_TATB$HAULING_DEPTH) / 2
  merge_TATB$SWEPT_AREA         <- merge_TATB$DISTANCE * merge_TATB$WING_OPENING / 10000000

  strata_subset <- strata_scheme[
    strata_scheme$GSA == unique(merge_TATB$GSA) &
      strata_scheme$COUNTRY %in% as.character(unique(merge_TATB$COUNTRY)), ]

  ## -------- Optimization 2: single loop over strata ------------------ ##
  merge_TATB$STRATUM_CODE <- NA
  for (j in seq_len(nrow(strata_subset))) {
    lower <- strata_subset[j, 4]; upper <- strata_subset[j, 5]; code <- strata_subset[j, 3]
    if (code == 1) {
      idx <- floor(merge_TATB$MEAN_DEPTH) >= lower & floor(merge_TATB$MEAN_DEPTH) <= upper
    } else {
      idx <- floor(merge_TATB$MEAN_DEPTH)  > lower & floor(merge_TATB$MEAN_DEPTH) <= upper
    }
    merge_TATB$STRATUM_CODE[idx] <- code
  }
  ## ------------------------------------------------------------------ ##

  ## Remaining TA-TB calculations unchanged ---------------------------- ##
  hour_shooting <- ifelse(nchar(merge_TATB$SHOOTING_TIME) == 4,
                          substr(merge_TATB$SHOOTING_TIME, 1, 2),
                          substr(merge_TATB$SHOOTING_TIME, 1, 1))
  min_shooting  <- ifelse(nchar(merge_TATB$SHOOTING_TIME) == 4,
                          substr(merge_TATB$SHOOTING_TIME, 3, 4),
                          substr(merge_TATB$SHOOTING_TIME, 2, 3))
  hour_hauling  <- ifelse(nchar(merge_TATB$HAULING_TIME) == 4,
                          substr(merge_TATB$HAULING_TIME, 1, 2),
                          substr(merge_TATB$HAULING_TIME, 1, 1))
  min_hauling   <- ifelse(nchar(merge_TATB$HAULING_TIME) == 4,
                          substr(merge_TATB$HAULING_TIME, 3, 4),
                          substr(merge_TATB$HAULING_TIME, 2, 3))

  hms_shooting <- hms(rep(0, length(hour_shooting)),
                      as.numeric(min_shooting), as.numeric(hour_shooting))
  hms_hauling  <- hms(rep(0, length(hour_hauling)),
                      as.numeric(min_hauling),  as.numeric(hour_hauling))
  duration     <- as.numeric(hms_hauling - hms_shooting) / 3600

  merge_TATB$SHOOTING_TIME <- hms_shooting
  merge_TATB$HAULING_TIME  <- hms_hauling
  merge_TATB$N_h    <- merge_TATB$TOTAL_NUMBER_IN_THE_HAUL / duration
  merge_TATB$N_km2  <- merge_TATB$TOTAL_NUMBER_IN_THE_HAUL / merge_TATB$SWEPT_AREA
  merge_TATB$kg_h   <- merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL / duration / 1000
  merge_TATB$kg_km2 <- merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL / merge_TATB$SWEPT_AREA / 1000
  merge_TATB$MEDITS_CODE <- as.character(merge_TATB$MEDITS_CODE)

  if (save) {
    if (!dir.exists(file.path(wd, "output"))) dir.create(file.path(wd, "output"), recursive = TRUE, showWarnings = FALSE)
    write.table(merge_TATB,
                file.path(wd, "output", paste0("mergeTATB_", species[1], species[2], ".csv")),
                sep = ";", row.names = FALSE)
  }
  if (verbose) message("TA-TB files correctly merged")

  #######################################################################
  ##                 MERGE TA â€“ TC  (optimised)                        ##
  #######################################################################
  if (verbose) message("- Merging TA-TC files")

  merge_TATC <- merge(TA_merge, TC_merge, by = "id", all.x = TRUE, all.y = TRUE)
  merge_TATC$MEDITS_CODE <- paste(merge_TATC$GENUS, merge_TATC$SPECIES)

  coord_tc <- convert_coordinates(merge_TATC)
  merge_TATC$MEAN_LATITUDE      <- (merge_TATC$SHOOTING_LATITUDE + merge_TATC$HAULING_LATITUDE) / 2
  merge_TATC$MEAN_LATITUDE_DEC  <- (coord_tc$lat_start + coord_tc$lat_end) / 2
  merge_TATC$MEAN_LONGITUDE     <- (merge_TATC$SHOOTING_LONGITUDE + merge_TATC$HAULING_LONGITUDE) / 2
  merge_TATC$MEAN_LONGITUDE_DEC <- (coord_tc$lon_start + coord_tc$lon_end) / 2
  merge_TATC$MEAN_DEPTH         <- (merge_TATC$SHOOTING_DEPTH + merge_TATC$HAULING_DEPTH) / 2

  ## Same â€œNA NAâ€ optimization ----------------------------------------- ##
  na_idx2 <- merge_TATC$MEDITS_CODE == "NA NA"
  if (any(na_idx2)) {
    merge_TATC$MEDITS_CODE[na_idx2] <- -1
    merge_TATC[na_idx2,
               c("GENUS","SPECIES","SEX","LENGTH_CLASSES_CODE",
                 "WEIGHT_OF_THE_FRACTION","WEIGHT_OF_THE_SAMPLE_MEASURED",
                 "MATURITY","MATSUB","LENGTH_CLASS")] <- -1
    merge_TATC$WEIGHT_OF_THE_FRACTION[na_idx2] <- 0
    merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED[na_idx2] <- 0
    merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE[na_idx2] <- 0
  }

  merge_TATC$SWEPT_AREA         <- merge_TATC$DISTANCE * merge_TATC$WING_OPENING / 10000000

  strata_subset2 <- strata_scheme[
    strata_scheme$GSA == unique(merge_TATB$GSA) &
      strata_scheme$COUNTRY %in% as.character(unique(merge_TATB$COUNTRY)), ]

  ## Optimization 2 also on TA-TC -------------------------------------- ##
  merge_TATC$STRATUM_CODE <- NA
  for (j in seq_len(nrow(strata_subset2))) {
    lower <- strata_subset2[j, 4]; upper <- strata_subset2[j, 5]; code <- strata_subset2[j, 3]
    idx <- merge_TATC$MEAN_DEPTH > lower & merge_TATC$MEAN_DEPTH <= upper
    if (code == 1)
      idx <- merge_TATC$MEAN_DEPTH >= lower & merge_TATC$MEAN_DEPTH <= upper
    merge_TATC$STRATUM_CODE[idx] <- code
  }
  ## ------------------------------------------------------------------ ##

  hour_shooting <- ifelse(nchar(merge_TATC$SHOOTING_TIME) == 4,
                          substr(merge_TATC$SHOOTING_TIME, 1, 2),
                          substr(merge_TATC$SHOOTING_TIME, 1, 1))
  min_shooting  <- ifelse(nchar(merge_TATC$SHOOTING_TIME) == 4,
                          substr(merge_TATC$SHOOTING_TIME, 3, 4),
                          substr(merge_TATC$SHOOTING_TIME, 2, 3))
  hour_hauling  <- ifelse(nchar(merge_TATC$HAULING_TIME) == 4,
                          substr(merge_TATC$HAULING_TIME, 1, 2),
                          substr(merge_TATC$HAULING_TIME, 1, 1))
  min_hauling   <- ifelse(nchar(merge_TATC$HAULING_TIME) == 4,
                          substr(merge_TATC$HAULING_TIME, 3, 4),
                          substr(merge_TATC$HAULING_TIME, 2, 3))

  hms_shooting <- hms(rep(0, length(hour_shooting)),
                      as.numeric(min_shooting), as.numeric(hour_shooting))
  hms_hauling  <- hms(rep(0, length(hour_hauling)),
                      as.numeric(min_hauling),  as.numeric(hour_hauling))
  duration     <- as.numeric(hms_hauling - hms_shooting) / 3600

  merge_TATC$SHOOTING_TIME <- hms_shooting
  merge_TATC$HAULING_TIME  <- hms_hauling

  merge_TATC$MATURITY_STAGE <- with(merge_TATC, ifelse(
    MATURITY %in% c(-1, 0, 1), MATURITY,
    ifelse(MATSUB %in% c("A","B","C","D","E","F"),
           paste(MATURITY, MATSUB, sep = ""), MATURITY)))

  merge_TATC$RAISING_FACTOR <- 0
  pos <- merge_TATC$WEIGHT_OF_THE_FRACTION > 0
  merge_TATC$RAISING_FACTOR[pos] <- merge_TATC$WEIGHT_OF_THE_FRACTION[pos] /
    merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED[pos]
  merge_TATC$RAISING_FACTOR[merge_TATC$RAISING_FACTOR > 0 &
                              merge_TATC$RAISING_FACTOR < 1] <- 1

  merge_TATC$N_h    <- merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE *
    merge_TATC$RAISING_FACTOR / duration
  merge_TATC$N_km2  <- merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE *
    merge_TATC$RAISING_FACTOR / merge_TATC$SWEPT_AREA
  merge_TATC$kg_h   <- merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED / duration / 1000
  merge_TATC$kg_km2 <- merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED / merge_TATC$SWEPT_AREA / 1000

  if (save) {
    if (!dir.exists(file.path(wd, "output"))) dir.create(file.path(wd, "output"), recursive = TRUE, showWarnings = FALSE)
    write.table(merge_TATC,
                file.path(wd, "output", paste0("mergeTATC_", species[1], species[2], ".csv")),
                sep = ";", row.names = FALSE)
  }
  if (verbose) message("TA-TC files correctly merged")

  ## ------------------------------------------------------------------ ##
  ##  Return                                                            ##
  ## ------------------------------------------------------------------ ##
  return(list(merge_TA_TB = merge_TATB,
              merge_TA_TC = merge_TATC))
}

#' Estimation of Mean Individual Weight (MIW) time series
#' @description
#' Calculates the Mean Individual Weight (MIW) time series, providing a
#' summary indicator of the average size within the captured population
#' and potential shifts due to fishing pressure.
#'
#' @param mTATB data frame of the merged TA and TB
#' @param GSA reference GSA for the analysis
#' @param country reference countries in the GSA for the analysis
#' @param depth_range range of depth strata to perform the analysis (min, max)
#' @param strata_scheme data frame of the stratification scheme
#' @param stratification data frame of strata surface area
#' @param wd working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @param verbose boolean. If TRUE messages are reported in the console
#' @importFrom gridExtra grid.arrange
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot geom_density ggtitle geom_boxplot xlab ylab ylim xlim ggsave theme_bw geom_line geom_point
#' @importFrom stringr str_split
#' @import grDevices
#' @import graphics
#' @importFrom magrittr %>%
#' @return A \code{data.frame} containing the Mean Individual Weight (MIW) time series.
#' @examples
#' data(TA)
#' data(TB)
#' # Create a merged dataset for GSA 10
#' mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
#' MIW(mTATB, GSA=10, country="all", depth_range=c(10,800),
#'     strata_scheme=BioIndex::strata_scheme,
#'     stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
#' @export
#'
#'
MIW <- function(mTATB, GSA, country="all", depth_range, strata_scheme, stratification, wd=NA, save=TRUE,verbose=TRUE){
    if (is.na(wd)) { wd <- tempdir() }
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))

    if (FALSE) {
        GSA <- 18
        country <- "all"
        depthr <- c(10,800)
        # strata=BioIndex::strata_scheme
        # stratification_tab = BioIndex::stratification
        strata_scheme=BioIndex::strata_scheme
        stratification = BioIndex::stratification
        # wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\Test_BioIndex_package"
        save=TRUE
        verbose=TRUE

        # MIW(mTATB, GSA, country="all", depth_range=depthr, strata_scheme=strata, stratification=stratification_tab, wd, save,verbose)
    }

    vmsg <- function(...) if (isTRUE(verbose)) message(...)

    ## ---- validate working directory when save=TRUE -----------
    if (isTRUE(save)) {
        if (is.na(wd) || !nzchar(wd)) {
            vmsg("[MIW] `save=TRUE` but `wd` not provided - disabling file output.")
            save <- FALSE
        } else {
            if (!dir.exists(wd)) dir.create(wd, recursive = TRUE, showWarnings = FALSE)
            out_dir <- file.path(wd, "output")
            if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE,
                                                 showWarnings = FALSE)
        }
    }

    #-----------------------
    # MIW INDICES
    #-----------------------
    year <- value <- NULL
    #---- COUNTRIES
    countries <- unique(mTATB[!is.na(mTATB$COUNTRY),"COUNTRY"])
    l_country <- length(unique(mTATB$COUNTRY))

    if (l_country==1){
        check_country ="Y"
        country_analysis <- countries
    } else {
        if (any(country %in% "all")) {
            country_analysis <<- countries
        } else {
            country_analysis <<- country
        }
    }
    merge_TATB <- mTATB
    merge_TATB <- merge_TATB[merge_TATB$COUNTRY %in% country_analysis, ]

    strata_scheme <- strata_scheme[strata_scheme$GSA == GSA & strata_scheme$COUNTRY %in% as.character(unique(country_analysis))[1], ]

    #---
    dependent <- "MIW"
    dep_text <-"MIW (kg)"
    GENERE <- as.character(unique(merge_TATB$GENUS)[unique(merge_TATB$GENUS) != -1])
    SPECIE <- as.character(unique(merge_TATB$SPECIES)[unique(merge_TATB$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")
    GSA <- unique(merge_TATB$GSA)
    species <- sspp
    #---
    data <- merge_TATB
    depth <- data.frame(matrix(NA,ncol=4, nrow=length(strata_scheme$CODE)))
    colnames(depth) <- c("strata", "min", "max", "bul")
    depth$strata <- strata_scheme$CODE # c(1,2,3,4,5)
    depth$min <-strata_scheme$MIN_DEPTH  # c(10,50,100,200,500)
    depth$max <-strata_scheme$MAX_DEPTH  # c(50,100,200,500,800)
    # print("Select the depth range for the analysis")
    # depth_range <- dlgInput("Depth range for the analysis: ", default="5,35",Sys.info()[""])$res
    # depth_range <- data.frame(range = strsplit(depth_range, ",")); depth_range <- as.numeric(as.character(depth_range[,1]))
    if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
    data <-  data[data$MEAN_DEPTH>depth_range[1] & data$MEAN_DEPTH<=depth_range[2],]
    idx_from <- which(depth[,"min"] == depth_range[1])
    idx_to   <- which(depth[,"max"] == depth_range[2])

    if (length(idx_from) != 1 || length(idx_to) != 1) {
        stop(paste0("Error: depth range [", depth_range[1], ", ", depth_range[2],
                    "] does not match the strata boundaries defined for this GSA. ",
                    "Available min depths: ", paste(sort(unique(depth$min)), collapse=", "),
                    "; Available max depths: ", paste(sort(unique(depth$max)), collapse=", ")))
    }
    strata_range <- seq(idx_from, idx_to, 1)
    depth[depth$strata %in% strata_range, "bul"] <- T
    depth[!(depth$strata %in% strata_range), "bul"] <- F
    #---
    ddd <- merge_TATB

    year_range <- data.frame(year = sort(unique(merge_TATB$YEAR)))

    res_table_cala2 <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    res_table_cala2$index_1 <- NA
    res_table_cala2$index_2 <- NA
    res_table_cala2$index_3 <- NA
    res_table_cala2$index_4 <- NA
    res_table_cala2$index_5 <- NA
    res_table_cala2$index_6 <- NA

    sd_strata <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    sd_strata$sd_1 <- NA
    sd_strata$sd_2 <- NA
    sd_strata$sd_3 <- NA
    sd_strata$sd_4 <- NA
    sd_strata$sd_5 <- NA
    sd_strata$sd_6 <- NA

    se_table2  <-  data.frame(year=sort(unique(merge_TATB$YEAR)))
    se_table2$sum_1 <- NA
    se_table2$sum_2 <- NA
    se_table2$sum_3 <- NA
    se_table2$sum_4 <- NA
    se_table2$sum_5 <- NA
    se_table2$sum_6 <- NA
    se_table2$sd <- NA

    analysis_stratum1 <- NA
    analysis_stratum2 <- NA
    analysis_stratum3 <- NA
    analysis_stratum4 <- NA
    analysis_stratum5 <- NA
    analysis_stratum6 <- NA

    analysis_stratum1 <- depth[1,4]
    analysis_stratum2 <- depth[2,4]
    analysis_stratum3 <- depth[3,4]
    analysis_stratum4 <- depth[4,4]
    analysis_stratum5 <- depth[5,4]
    analysis_stratum6 <- depth[6,4]

    if (is.na(analysis_stratum1)){depth[1,4]<- F; analysis_stratum1 <- F}
    if (is.na(analysis_stratum2)){depth[2,4]<- F; analysis_stratum2 <- F}
    if (is.na(analysis_stratum3)){depth[3,4]<- F; analysis_stratum3 <- F}
    if (is.na(analysis_stratum4)){depth[4,4]<- F; analysis_stratum4 <- F}
    if (is.na(analysis_stratum5)){depth[5,4]<- F; analysis_stratum5 <- F}
    if (is.na(analysis_stratum6)){depth[6,4]<- F; analysis_stratum6 <- F}


    stratification$SURF <- as.numeric(stratification$SURF)

    if (analysis_stratum1 == T){area_s1 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==1 & stratification$COUNTRY %in% country_analysis,5])} else {area_s1 <- 0}
    if (analysis_stratum2 == T){area_s2 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==2 & stratification$COUNTRY %in% country_analysis,5])} else {area_s2 <- 0}
    if (analysis_stratum3 == T){area_s3 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==3 & stratification$COUNTRY %in% country_analysis,5])} else {area_s3 <- 0}
    if (analysis_stratum4 == T){area_s4 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==4 & stratification$COUNTRY %in% country_analysis,5])} else {area_s4 <- 0}
    if (analysis_stratum5 == T){area_s5 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==5 & stratification$COUNTRY %in% country_analysis,5])} else {area_s5 <- 0}
    if (analysis_stratum6 == T){area_s6 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==6 & stratification$COUNTRY %in% country_analysis,5])} else {area_s6 <- 0}

    peso_s1 <- area_s1/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s2 <- area_s2/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s3 <- area_s3/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s4 <- area_s4/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s5 <- area_s5/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s6 <- area_s6/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)

    i=1
    for(i in 1:length(res_table_cala2[,1])) {

        data2 <- ddd[ddd$YEAR == res_table_cala2[i,1] , ]

        if (analysis_stratum1 == T) {s1 <- data2[data2$MEAN_DEPTH > depth[1,2] & data2$MEAN_DEPTH <= depth[1,3], ]}
        if (analysis_stratum2 == T) {s2 <- data2[data2$MEAN_DEPTH > depth[2,2] & data2$MEAN_DEPTH <= depth[2,3], ]}
        if (analysis_stratum3 == T) {s3 <- data2[data2$MEAN_DEPTH > depth[3,2] & data2$MEAN_DEPTH <= depth[3,3], ]}
        if (analysis_stratum4 == T) {s4 <- data2[data2$MEAN_DEPTH > depth[4,2] & data2$MEAN_DEPTH <= depth[4,3], ]}
        if (analysis_stratum5 == T) {s5 <- data2[data2$MEAN_DEPTH > depth[5,2] & data2$MEAN_DEPTH <= depth[5,3], ]}
        if (analysis_stratum6 == T) {s6 <- data2[data2$MEAN_DEPTH > depth[6,2] & data2$MEAN_DEPTH <= depth[6,3], ]}


        if (analysis_stratum1 == T){
            # [ORIGINALE]:
            # #index computation
            # s1 <- s1[!is.na(s1$TOTAL_NUMBER_IN_THE_HAUL ) & s1$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            # s1$mean_MIW <- (s1[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s1[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            # s_MIW<- sum(s1[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s1[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            # s_a <- sum(s1$SWEPT_AREA)
            # res_table_cala2[i,2] <- s_MIW
            # #se computation
            # sq <- (s1$mean_MIW - s_MIW)^2
            # sqA <- sq*s1$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s1[,1])-1)
            # f1 <- s_a/area_s1
            # se_table2[i,2] <- (((peso_s1)^2 * sum_sqA)/ s_a)* (1-f1)
            # sd_strata[i,2] <- sqrt( ((sum_sqA)/ s_a)* (1-f1))

            # [MODIFICATO]:
            s1 <- s1[!is.na(s1$TOTAL_NUMBER_IN_THE_HAUL ) & s1$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            s1$mean_MIW <- (s1[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s1[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            s_MIW <- sum(s1[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s1[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            if (is.nan(s_MIW)) s_MIW <- NA
            s_a <- sum(s1$SWEPT_AREA)
            res_table_cala2[i,2] <- s_MIW
            s1_clean <- s1[!is.na(s1$mean_MIW), ]
            s_a_clean <- sum(s1_clean$SWEPT_AREA)
            if (nrow(s1_clean) > 1) {
                sq <- (s1_clean$mean_MIW - s_MIW)^2
                sqA <- sq*s1_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s1_clean)-1)
                f1 <- s_a_clean/area_s1
                se_table2[i,2] <- (((peso_s1)^2 * sum_sqA)/ s_a_clean)* (1-f1)
                sd_strata[i,2] <- sqrt( ((sum_sqA)/ s_a_clean)* (1-f1))
            } else {
                se_table2[i,2] <- 0
                sd_strata[i,2] <- 0
            }
        } else {area_s1 = 0}

        if (analysis_stratum2 == T){
            # [ORIGINALE]:
            # #index computation
            # s2 <- s2[!is.na(s2$TOTAL_NUMBER_IN_THE_HAUL ) & s2$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            # s2$mean_MIW <- (s2[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s2[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            # s_MIW<- sum(s2[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s2[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            # s_a <- sum(s2$SWEPT_AREA)
            # res_table_cala2[i,3] <- s_MIW
            # #se computation
            # sq <- (s2$mean_MIW - s_MIW)^2
            # sqA <- sq*s2$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s2[,1])-1)
            # f2 <- s_a/area_s2
            # se_table2[i,3] <- (((peso_s2)^2 * sum_sqA)/ s_a)* (1-f2)
            # sd_strata[i,3] <- sqrt( ((sum_sqA)/ s_a)* (1-f2))

            # [MODIFICATO]:
            s2 <- s2[!is.na(s2$TOTAL_NUMBER_IN_THE_HAUL ) & s2$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            s2$mean_MIW <- (s2[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s2[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            s_MIW <- sum(s2[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s2[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            if (is.nan(s_MIW)) s_MIW <- NA
            s_a <- sum(s2$SWEPT_AREA)
            res_table_cala2[i,3] <- s_MIW
            s2_clean <- s2[!is.na(s2$mean_MIW), ]
            s_a_clean <- sum(s2_clean$SWEPT_AREA)
            if (nrow(s2_clean) > 1) {
                sq <- (s2_clean$mean_MIW - s_MIW)^2
                sqA <- sq*s2_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s2_clean)-1)
                f2 <- s_a_clean/area_s2
                se_table2[i,3] <- (((peso_s2)^2 * sum_sqA)/ s_a_clean)* (1-f2)
                sd_strata[i,3] <- sqrt( ((sum_sqA)/ s_a_clean)* (1-f2))
            } else {
                se_table2[i,3] <- 0
                sd_strata[i,3] <- 0
            }
        } else {area_s2 = 0}

        if (analysis_stratum3 == T){
            # [ORIGINALE]:
            # #index computation
            # s3 <- s3[!is.na(s3$TOTAL_NUMBER_IN_THE_HAUL ) & s3$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            # s3$mean_MIW <- (s3[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s3[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            # s_MIW<- sum(s3[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s3[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            # s_a <- sum(s3$SWEPT_AREA)
            # res_table_cala2[i,4] <- s_MIW
            # #se computation
            # sq <- (s3$mean_MIW - s_MIW)^2
            # sqA <- sq*s3$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s3[,1])-1)
            # f3 <- s_a/area_s3
            # se_table2[i,4] <- (((peso_s3)^2 * sum_sqA)/ s_a)* (1-f3)
            # sd_strata[i,4] <- sqrt( ((sum_sqA)/ s_a)* (1-f3))

            # [MODIFICATO]:
            s3 <- s3[!is.na(s3$TOTAL_NUMBER_IN_THE_HAUL ) & s3$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            s3$mean_MIW <- (s3[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s3[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            s_MIW <- sum(s3[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s3[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            if (is.nan(s_MIW)) s_MIW <- NA
            s_a <- sum(s3$SWEPT_AREA)
            res_table_cala2[i,4] <- s_MIW
            s3_clean <- s3[!is.na(s3$mean_MIW), ]
            s_a_clean <- sum(s3_clean$SWEPT_AREA)
            if (nrow(s3_clean) > 1) {
                sq <- (s3_clean$mean_MIW - s_MIW)^2
                sqA <- sq*s3_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s3_clean)-1)
                f3 <- s_a_clean/area_s3
                se_table2[i,4] <- (((peso_s3)^2 * sum_sqA)/ s_a_clean)* (1-f3)
                sd_strata[i,4] <- sqrt( ((sum_sqA)/ s_a_clean)* (1-f3))
            } else {
                se_table2[i,4] <- 0
                sd_strata[i,4] <- 0
            }
        } else {area_s3 = 0}

        if (analysis_stratum4 == T){
            # [ORIGINALE]:
            # #index computation
            # s4 <- s4[!is.na(s4$TOTAL_NUMBER_IN_THE_HAUL ) & s4$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            # s4$mean_MIW <- (s4[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s4[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            # s_MIW<- sum(s4[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s4[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            # s_a <- sum(s4$SWEPT_AREA)
            # res_table_cala2[i,5] <- s_MIW
            # #se computation
            # sq <- (s4$mean_MIW - s_MIW)^2
            # sqA <- sq*s4$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s4[,1])-1)
            # f4 <- s_a/area_s4
            # se_table2[i,5] <- (((peso_s4)^2 * sum_sqA)/ s_a)* (1-f4)
            # sd_strata[i,5] <- sqrt( ((sum_sqA)/ s_a)* (1-f4))

            # [MODIFICATO]:
            s4 <- s4[!is.na(s4$TOTAL_NUMBER_IN_THE_HAUL ) & s4$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            s4$mean_MIW <- (s4[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s4[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            s_MIW <- sum(s4[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s4[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            if (is.nan(s_MIW)) s_MIW <- NA
            s_a <- sum(s4$SWEPT_AREA)
            res_table_cala2[i,5] <- s_MIW
            s4_clean <- s4[!is.na(s4$mean_MIW), ]
            s_a_clean <- sum(s4_clean$SWEPT_AREA)
            if (nrow(s4_clean) > 1) {
                sq <- (s4_clean$mean_MIW - s_MIW)^2
                sqA <- sq*s4_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s4_clean)-1)
                f4 <- s_a_clean/area_s4
                se_table2[i,5] <- (((peso_s4)^2 * sum_sqA)/ s_a_clean)* (1-f4)
                sd_strata[i,5] <- sqrt( ((sum_sqA)/ s_a_clean)* (1-f4))
            } else {
                se_table2[i,5] <- 0
                sd_strata[i,5] <- 0
            }
        } else {area_s4 = 0}

        if (analysis_stratum5 == T){
            # [ORIGINALE]:
            # #index computation
            # s5 <- s5[!is.na(s5$TOTAL_NUMBER_IN_THE_HAUL ) & s5$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            # s5$mean_MIW <- (s5[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s5[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            # s_MIW<- sum(s5[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s5[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            # s_a <- sum(s5$SWEPT_AREA)
            # res_table_cala2[i,6] <- s_MIW
            # #se computation
            # sq <- (s5$mean_MIW - s_MIW)^2
            # sqA <- sq*s5$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s5[,1])-1)
            # f5 <- s_a/area_s5
            # se_table2[i,6] <- (((peso_s5)^2 * sum_sqA)/ s_a)* (1-f5)
            # sd_strata[i,6] <- sqrt( ((sum_sqA)/ s_a)* (1-f5))

            # [MODIFICATO]:
            s5 <- s5[!is.na(s5$TOTAL_NUMBER_IN_THE_HAUL ) & s5$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            s5$mean_MIW <- (s5[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s5[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            s_MIW <- sum(s5[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s5[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            if (is.nan(s_MIW)) s_MIW <- NA
            s_a <- sum(s5$SWEPT_AREA)
            res_table_cala2[i,6] <- s_MIW
            s5_clean <- s5[!is.na(s5$mean_MIW), ]
            s_a_clean <- sum(s5_clean$SWEPT_AREA)
            if (nrow(s5_clean) > 1) {
                sq <- (s5_clean$mean_MIW - s_MIW)^2
                sqA <- sq*s5_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s5_clean)-1)
                f5 <- s_a_clean/area_s5
                se_table2[i,6] <- (((peso_s5)^2 * sum_sqA)/ s_a_clean)* (1-f5)
                sd_strata[i,6] <- sqrt( ((sum_sqA)/ s_a_clean)* (1-f5))
            } else {
                se_table2[i,6] <- 0
                sd_strata[i,6] <- 0
            }
        } else {area_s5 = 0}

        if (analysis_stratum6 == T){
            # [ORIGINALE]:
            # #index computation
            # s6 <- s6[!is.na(s6$TOTAL_NUMBER_IN_THE_HAUL ) & s6$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            # s6$mean_MIW <- (s6[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s6[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            # s_MIW<- sum(s6[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s6[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            # s_a <- sum(s6$SWEPT_AREA)
            # res_table_cala2[i,7] <- s_MIW
            # #se computation
            # sq <- (s6$mean_MIW - s_MIW)^2
            # sqA <- sq*s6$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s6[,1])-1)
            # f6 <- s_a/area_s6
            # se_table2[i,7] <- (((peso_s6)^2 * sum_sqA)/ s_a)* (1-f6)
            # sd_strata[i,7] <- sqrt( ((sum_sqA)/ s_a)* (1-f6))

            # [MODIFICATO]:
            s6 <- s6[!is.na(s6$TOTAL_NUMBER_IN_THE_HAUL ) & s6$TOTAL_NUMBER_IN_THE_HAUL>0, ]
            s6$mean_MIW <- (s6[ , "TOTAL_WEIGHT_IN_THE_HAUL" ]/1000) / s6[ , "TOTAL_NUMBER_IN_THE_HAUL" ]
            s_MIW <- sum(s6[ ,"TOTAL_WEIGHT_IN_THE_HAUL"]/1000)/sum(s6[ ,"TOTAL_NUMBER_IN_THE_HAUL"])
            if (is.nan(s_MIW)) s_MIW <- NA
            s_a <- sum(s6$SWEPT_AREA)
            res_table_cala2[i,7] <- s_MIW
            s6_clean <- s6[!is.na(s6$mean_MIW), ]
            s_a_clean <- sum(s6_clean$SWEPT_AREA)
            if (nrow(s6_clean) > 1) {
                sq <- (s6_clean$mean_MIW - s_MIW)^2
                sqA <- sq*s6_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s6_clean)-1)
                f6 <- s_a_clean/area_s6
                se_table2[i,7] <- (((peso_s6)^2 * sum_sqA)/ s_a_clean)* (1-f6)
                sd_strata[i,7] <- sqrt( ((sum_sqA)/ s_a_clean)* (1-f6))
            } else {
                se_table2[i,7] <- 0
                sd_strata[i,7] <- 0
            }
        } else {area_s6 = 0}

        sum_res_est <- c(res_table_cala2[i,2]*peso_s1, res_table_cala2[i,3]*peso_s2,
                         res_table_cala2[i,4]*peso_s3, res_table_cala2[i,5]*peso_s4,
                         res_table_cala2[i,6]*peso_s5, res_table_cala2[i,7]*peso_s6)
        valid <- !is.na(sum_res_est) & !is.nan(sum_res_est)
        res_table_cala2[i, 8] <- if (any(valid)) sum(sum_res_est[valid]) else NA
        colnames(res_table_cala2) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices")
        se_table2[i, "sd"] <-sqrt(rowSums(se_table2[i, 2:7], na.rm = T))
    }

    timeseries <- data.frame(year = res_table_cala2[,1], MIW = res_table_cala2[,8], sd= se_table2[, "sd"])
    timeseries$CV <- timeseries[,3] / timeseries[,2]
    timeseries$invCV <- 1/timeseries$CV
    colnames(res_table_cala2) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices")



    cols.strata <- (depth[!is.na(depth$strata), 'strata'] + 1)
    n.col.strata <- length(cols.strata)
    cv_strata <- sd_strata
    col=5
    for (col in cols.strata) {
        row=1
        for (row in 1:length(year_range[,1])) {
            if (!is.na(sd_strata[row,col])) {
                val <- res_table_cala2[row,col]
                if (!is.na(val) && !is.nan(val) && val != 0){
                    cv_strata[row,col] <- cv_strata[row,col]/val
                } else {
                    cv_strata[row,col] <- 0
                }
            }
        }
    }

    TAB_SD_CV <- cbind(sd_strata[, c(1,cols.strata)],cv_strata[,c(cols.strata)])
    colnames(TAB_SD_CV) <- c("year", paste("SD", depth[!is.na(depth$strata), 'strata'],sep="_"), paste("CV", depth[!is.na(depth$strata), 'strata'],sep="_"))
    tab.strata.MIW <- data.frame(year=res_table_cala2[,"year"], res_table_cala2[, cols.strata], TAB_SD_CV[,-1])
    colnames(tab.strata.MIW) <- c("year", paste("MIW", depth[!is.na(depth$strata), 'strata'],sep="_"), paste("SD", depth[!is.na(depth$strata), 'strata'],sep="_"), paste("CV", depth[!is.na(depth$strata), 'strata'],sep="_"))
    title.MIW.haul <- paste(sspp,"_GSA",GSA,"_(",dependent,"_by_stratum)-RSS_",depth_range[1],"-",depth_range[2], " m", sep="")


    is.nan.data.frame <- function(x){do.call(cbind, lapply(x, is.nan))}
    tab.strata.MIW[is.nan.data.frame(tab.strata.MIW)] <- NA

    if(save){
        write.table(tab.strata.MIW, paste(wd,"/output/",title.MIW.haul,".csv", sep=""), sep=";", row.names = F)
    }


    # plot timeseries of mean indices 1
    main <- paste(sspp,"_GSA",GSA,"_(",dependent,")-Random_Stratified_Sampling_",depth_range[1],"-",depth_range[2], " m", sep="")
    main.lab <- paste(sspp," GSA",GSA," (",dependent,")-RSS ",depth_range[1],"-",depth_range[2], " m", sep="")
    max_index <- max(timeseries[,"MIW"]) + max(timeseries[!is.na(timeseries$sd),"sd"])*1.2
    if (save){
        write.table(timeseries, paste(wd,"/output/",main,"_Timeseries.csv", sep=""), sep=";", row.names = F)
    }

    if (save){
        jpeg(paste(wd,"/output/",main,"_Timeseries.jpg",sep=""), res = 300, width = 8, height = 7, units = 'in')
        par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
        plot(timeseries[,"year"],  timeseries[,"MIW"], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab) # ylim=c(0,max_index*1.2)
        lines(timeseries[,"year"], (timeseries[,"MIW"]-1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
        lines(timeseries[,"year"], (timeseries[,"MIW"]+1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
        legend("topright", c("time series", "CI"), lty=c(1,2), pch=c(16, NA), col=c("black","red"))
        dev.off()
    } else {
        par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
        plot(timeseries[,"year"],  timeseries[,"MIW"], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab) # ylim=c(0,max_index*1.2)
        lines(timeseries[,"year"], (timeseries[,"MIW"]-1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
        lines(timeseries[,"year"], (timeseries[,"MIW"]+1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
        legend("topright", c("time series", "CI"), lty=c(1,2), pch=c(16, NA), col=c("black","red"))
    }


    # 2
    sel.strata1 <- c(1,depth[(depth$bul), "strata"] + 1)
    sel.strata2 <- c(1, depth[(depth$bul), "strata"] + 1 + length(cols.strata))
    tab.ind <- tab.strata.MIW[, sel.strata1]
    tab.sd <- sd_strata[, sel.strata1]
    df.plot1 <- melt(tab.ind, id.vars = 1)
    df.plot2 <- melt(tab.sd, id.vars = 1)
    var1 <-  as.data.frame(do.call(rbind, str_split(as.character(df.plot1$variable),"_"))); colnames(var1) <- c("var", "strata")
    var2 <-  as.data.frame(do.call(rbind, str_split(as.character(df.plot2$variable),"_"))); colnames(var2) <- c("var", "strata")
    df.plot1 <- cbind(df.plot1,var1)
    df.plot2 <- cbind(df.plot2,var2)
    df.plot1 <- data.frame(df.plot1, sd=df.plot2$value)
    main <- paste(sspp,"_GSA",GSA,"_(",dependent,"_by_stratum)-RSS_",depth_range[1],"-",depth_range[2], " m", sep="")
    main.lab <- paste(sspp," GSA",GSA," (",dependent," by stratum)-RSS ",depth_range[1],"-",depth_range[2], " m", sep="")

    years <- sort(unique(df.plot1$year))
    if(length(years)>1){
        pm1 <- ggplot(data=df.plot1, aes(x=year, y=value, group=strata, colour=strata)) +
            geom_line(na.rm=TRUE) +
            geom_point(na.rm=TRUE) +
            xlab("year") +
            ylab(dep_text) +
            ggtitle(main.lab) +
            theme_bw()
    } else {
        pm1 <- ggplot(data=df.plot1, aes(x=year, y=value, group=strata, colour=strata)) +
            geom_point(na.rm=TRUE) +
            xlab("year") +
            ylab(dep_text) +
            ggtitle(main.lab) +
            theme_bw()
    }

        # print(pm1) # rimosso per policy CRAN

    if (save){
        if(save) ggsave(paste(wd,"/output/",main,"_Timeseries.jpg",sep=""), dpi=300 , width=6, height=5, plot = pm1)
    }
    #-------



    if (isTRUE(verbose))
        message("\n Estimation of MIW completed ")
    results <- list(timeseries,tab.strata.MIW)
    return(results)
}

#' Spearman test
#'
#' @param abundance data frame of abundance indices
#' @param biomass data frame of biomass indices
#' @param years reference years for the analysis
#' @param sspp reference species for the analysis
#' @param wd path of working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @param verbose boolean. If TRUE messages are prompted in the console
#' @importFrom utils write.table
#' @return A \code{data.frame} or \code{list} with the results of the Spearman rank correlation test.
#' @examples
#' abundance <- data.frame(year = 2010:2020, index = runif(11, 1, 10))
#' biomass <- data.frame(year = 2010:2020, index = runif(11, 5, 20))
#' spearman(abundance=abundance, biomass=biomass, years=c(2010,2020), 
#'          sspp="MERLMER", wd=tempdir(), save=FALSE)
#' @export
spearman <- function(abundance=NA, biomass=NA, years, sspp=NA, wd=NA, save=TRUE, verbose=FALSE){
    if (is.na(wd)) { wd <- tempdir() }

    if (FALSE) {

        # library(BioIndex)
        years <- c(2017, 2020)

        GSA=10
        save=TRUE
        country="all"
        depth_range <- c(10,800)
        species <- "MERLMER"

        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        verbose=TRUE
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)


        m <- merge_TATBTC(ta, tb, tc, species=species, country=country, wd=wd, verbose=TRUE)
        mTATB <- m[[1]]
        index <- indices_ts(mTATB, GSA=GSA, country=country, depth_range=depth_range, strata_scheme=BioIndex::strata_scheme, stratification=BioIndex::stratification,wd, save=TRUE)

        abundance= index[[1]]
        biomass= index[[2]]

        spearman(abundance=abundance, biomass=biomass, years, sspp=species, wd=wd, save=TRUE)
    }



    ########   START   ########

    if (is.na(wd) & save) {
        save=TRUE
        if (verbose){
            message("Missing working directory. Results are not saved in the local folder.")
        }
    }


    yrange <- data.frame(range = as.numeric(range(years)))
    rownames(yrange) <- c("start", "end")


#----------------------------
# Abundance indices
#----------------------------
    if(all(!is.na(abundance)) & class(abundance)%in% c("data.frame")){
        timeseries <- abundance
        years_sel <- (yrange[1,1]:yrange[2,1])
        timeseries <- timeseries[timeseries$year %in% years_sel , ]
        x=log(timeseries[,2]+1)
        spear(x)
        spearman_abundance <- spear(x)
        spearman_abundance <- data.frame(index="abundance", spearman_abundance)
        if(is.infinite(spearman_abundance$t)) {spearman_abundance$t <- "-Inf"}
    } else {
        spearman_abundance <- data.frame(matrix(ncol=4,nrow=1))
        colnames(spearman_abundance) <- c("index","r","t","p")
        spearman_abundance[1,] <- c("abundance",NA,NA,NA)

    }

#----------------------------
# Biomass indices
#----------------------------
    if(all(!is.na(biomass)) & class(biomass)%in% c("data.frame")){
        timeseries <- biomass
        years_sel <- (yrange[1,1]:yrange[2,1])
        timeseries <- timeseries[timeseries$year %in% years_sel , ]
        x=log(timeseries[,2]+1)
        spear(x)
        spearman_biomass <- spear(x)
        spearman_biomass <- data.frame(index="biomass", spearman_biomass)
        if(is.infinite(spearman_biomass$t)) {spearman_biomass$t <- "-Inf"}
    } else {
        spearman_biomass <- data.frame(matrix(ncol=4,nrow=1))
        colnames(spearman_biomass) <- c("index","r","t","p")
        spearman_biomass[1,] <- c("biomass",NA,NA,NA)
    }
#----------------------------
# Results in table
#----------------------------
    tab_spearman <- rbind(spearman_abundance , spearman_biomass)

    if(save & !is.na(wd)){
        write.table(tab_spearman, paste(wd, "/output/",sspp," - Spearman summary_RSS.csv", sep=""), sep=";", row.names=FALSE)
    }

    return(tab_spearman)

}

#' Main function to perform BioIndex analysis
#'
#' @description
#' BioIndex is an R package designed to support the standardized analysis of MEDITS trawl survey data and the calculation of biological indicators for selected species and population components.
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#'
#' @param ta data frame of the TA table in the MEDITS format
#' @param tb data frame of the TB table in the MEDITS format
#' @param tc data frame of the TC table in the MEDITS format
#' @param sspp reference species for the analysis
#' @param rec_threshold cutoff threshold for recruits (reported in mm)
#' @param spaw_threshold cutoff threshold for spawners  (reported in mm)
#' @param haul_threshold minimum number of individuals to be used in estimation of the spatial indicators
#' @param sexes reference sex for the analysis
#' @param depth reference depth range
#' @param GSA reference GSA for the analysis
#' @param country reference country
#' @param map_lim coordinates limits for the maps
#' @param depth_lines depth contours to be plotted in the maps (3 values allowed, e.g c(50,200,800))
#' @param strata data frame of the reference strata for the study area
#' @param stratification_tab data frame of the stratification scheme
#' @param resolution resolution of the depth line
#' @param buffer buffer around the map
#' @param wd path of the working directory
#' @param zip boolean. If TRUE the results are stored in a zip file into the working directory
#' @param save boolean. If TRUE the results are stored in the working directory
#' @param verbose boolean. If TRUE messages are promted in the console
#' @examples
#' \donttest{
#' BioIndex(ta=TA[TA$YEAR %in% c(2015, 2016), ], tb=TB[TB$YEAR %in% c(2015, 2016), ],
#' tc=TC[TC$YEAR %in% c(2015, 2016), ], sspp="MERLMER",rec_threshold=200,
#' spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10, country="all",
#' map_lim=c(13.3,15.2,39.9,41.3),depth_lines=c(50,200,800),
#' strata=BioIndex::strata_scheme, stratification_tab =
#' BioIndex::stratification, resolution=NA, buffer=0.1, wd=tempdir(),
#' zip=FALSE, save=TRUE, verbose=TRUE)
#' }
#'
#' @importFrom methods is
#' @importFrom zip zip
#' @return A \code{list} containing the results of the BioIndex workflow, including data frames and plot objects.
#' @export
BioIndex <- function(ta,
                     tb,
                     tc,
                     sspp,
                     rec_threshold,
                     spaw_threshold,
                     haul_threshold = 30,
                     sexes = "all",
                     depth,
                     GSA,
                     country = "all",
                     map_lim,
                     depth_lines = c(10, 200, 800),
                     strata = BioIndex::strata_scheme,
                     stratification_tab = BioIndex::stratification,
                     resolution = NA,
                     buffer = 0.1,
                     wd = NA,
                     zip = TRUE,
                     save = TRUE,
                     verbose = TRUE) {
    if (is.na(wd)) {
        wd <- tempdir()
    }
    if (verbose) message("No directory specified. Results will be saved in the temporary folder: ", wd)

    if (FALSE) {
        # library(BioIndex)
        # wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\Test_BioIndex_package"
        # ta <- read.table(
        #     file.path(wd, "input", "TA GSA18 2017-2020.csv"),
        #     sep = ";",
        #     header = TRUE
        # )
        # tb <- read.table(
        #     file.path(wd, "input", "TB GSA18 2017-2020.csv"),
        #     sep = ";",
        #     header = TRUE
        # )
        # tc <- read.table(
        #     file.path(wd, "input", "TC GSA18 2017-2020.csv"),
        #     sep = ";",
        #     header = TRUE
        # )
        #
        # ta <- read.table(file.path(wd, "input", "TA.csv"),
        #                  sep = ";",
        #                  header = TRUE)
        # tb <- read.table(file.path(wd, "input", "TB.csv"),
        #                  sep = ";",
        #                  header = TRUE)
        # tc <- read.table(file.path(wd, "input", "TC.csv"),
        #                  sep = ";",
        #                  header = TRUE)

        ta <- read.table(file.path(wd, "TA-2020-GRC.csv"), sep = ",", header = TRUE)[, -1]
        tb <- read.table(file.path(wd, "TB-2020-GRC.csv"), sep = ",", header = TRUE)[, -1]
        tc <- read.table(file.path(wd, "TC-2020-GRC.csv"), sep = ",", header = TRUE)[, -1]
        colnames(ta) <- colnames(BioIndex::TA)
        colnames(tb) <- colnames(BioIndex::TB)
        colnames(tc) <- colnames(BioIndex::TC)
        sspp <- "MERLMER"
        rec_threshold <- 200
        spaw_threshold <- 210
        haul_threshold <- 30
        sexes <- "all"
        depth <- c(50, 200)
        GSA <- 20
        country <- "all"
        map_lim <- c(19.166, 23.23, 35, 39.88)
        depth_lines <- c(50, 100, 200)
        buffer <- 0.1
        res <- NA
        resolution <- res
        strata <- BioIndex::strata_scheme
        stratification_tab <- BioIndex::stratification
        save <- TRUE
        verbose <- TRUE

        BioIndex(
            ta,
            tb,
            tc,
            sspp,
            rec_threshold = rec_threshold,
            spaw_threshold = spaw_threshold,
            sexes = "all",
            depth = depth,
            GSA = GSA,
            country = "all",
            map_lim = map_lim,
            depth_lines = depth_lines,
            strata = strata,
            stratification_tab = stratification_tab,
            resolution = res,
            buffer = buffer,
            wd = wd,
            zip = TRUE,
            save = TRUE,
            verbose = TRUE
        )


        # ta <- read.table(file.path(wd,"input","TA_Combine.csv"),sep=";",header=TRUE)
        # tb <- read.table(file.path(wd,"input","TB_Combine.csv"),sep=";",header=TRUE)
        # tc <- read.table(file.path(wd,"input","TC_Combine.csv"),sep=";",header=TRUE)
        # sspp <- species <- "PSETMAX"
        # rec_threshold=201
        # spaw_threshold=367
        # haul_threshold=30
        # sexes <- "all"
        # depth <- c(10,125)
        # GSA <- 29
        # country <- "TUR"
        # map_lim <- c(26.9,42.28,40.33,46.89)
        # depth_lines <- c(20,50,100)
        # buffer=0.1
        # res=1
        # stratas=read.table("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\Test_BioIndex_package\\strata_BS.csv",sep=";",header=TRUE)
        # stratification_tabs = read.table("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\Test_BioIndex_package\\stratification scheme_BS.csv",sep=";",header=TRUE)
        # strata=stratas
        # stratification = stratification_tabs
        # depth_range=depth
        # save=TRUE
        # verbose=TRUE


        # BioIndex(ta, tb, tc, sspp,rec_threshold=rec_threshold, spaw_threshold=spaw_threshold,sexes="all", depth=depth, GSA=GSA, country="all", map_lim=map_lim,depth_lines=depth_lines, strata=stratas, stratification_tab = stratification_tabs, resolution=res, buffer=buffer, wd=wd, zip=FALSE, save=TRUE, verbose=TRUE)
    }

    # Filter data by GSA if present to ensure consistency in downstream functions
    if ("AREA" %in% colnames(ta)) {
        ta <- ta[as.character(ta$AREA) %in% as.character(GSA), ]
    }
    if ("AREA" %in% colnames(tb)) {
        tb <- tb[as.character(tb$AREA) %in% as.character(GSA), ]
    }
    if ("AREA" %in% colnames(tc)) {
        tc <- tc[as.character(tc$AREA) %in% as.character(GSA), ]
    }
    if ("GSA" %in% colnames(strata)) {
        strata <- strata[as.character(strata$GSA) %in% as.character(GSA), ]
    }
    if ("GSA" %in% colnames(stratification_tab)) {
        stratification_tab <- stratification_tab[as.character(stratification_tab$GSA) %in% as.character(GSA), ]
    }

    if (dir.exists(file.path(wd, "output"))) {
        dir <- list.files(
            file.path(wd, "output"),
            recursive = TRUE,
            full.names = TRUE,
            include.dirs = TRUE
        )
        unlink(dir, recursive = T)
    }


    m <- merge_TATBTC(
        ta,
        tb,
        tc,
        species = sspp,
        country = country,
        wd = wd,
        strata = strata,
        verbose = TRUE
    )

    mTATB <- m[[1]]
    mTATC <- m[[2]]

    ms <- overlayGrid(
        mTATB = mTATB,
        mTATC = mTATC,
        GSA = GSA,
        country = country,
        wd = wd,
        save = save,
        verbose = verbose
    )

    mTATBsp <- ms[[1]]
    mTATCsp <- ms[[2]]

    hauls_position(
        mTATB = mTATB,
        country = country,
        map_lim = map_lim,
        depth_lines = depth_lines,
        buffer = buffer,
        res = resolution,
        wd = wd,
        save,
        verbose
    )


    #--------------------------------------------------------------
    # Map of abundance and biomass indices by haul
    #--------------------------------------------------------------
    if (verbose) message("\n########################")
    if (verbose) message("Plot of indices by haul")
    if (verbose) message("########################")
    if (verbose) message("")

    source.check <- try(
        bubble_plot_by_haul_indexes(
            mTATB,
            map_lim,
            depth_lines,
            buffer = 0,
            res = resolution,
            wd = wd,
            save,
            verbose
        ),
        silent = T
    )

    if (!is(source.check, "try-error")) {
        if (verbose) message("\nPlot of indices by haul - completed")
    } else {
        if (verbose) message("\nPlot of indices by haul skipped - (Run Error)\n")
    }


    #--------------------------------------------------------------
    # Abundance and biomass indices per GSA in the timeseries
    #--------------------------------------------------------------

    if (verbose) message("\n########################")
    if (verbose) message("Time series of indices")
    if (verbose) message("########################")
    if (verbose) message("")
    index <- indices_ts(
        mTATB,
        GSA = GSA,
        country = country,
        depth_range = depth,
        strata_scheme = strata,
        stratification = stratification_tab,
        wd,
        save
    )
    if (verbose) message("Time series of indices - completed")


    #--------------------------------------------------------------
    # Mean Individual Weights (MIW) per GSA in the timeseries
    #--------------------------------------------------------------

    if (verbose) message("\n########################")
    if (verbose) message("Time series of indices")
    if (verbose) message("########################")
    if (verbose) message("")
    MIW(
        mTATB,
        GSA,
        country,
        depth_range = depth,
        strata_scheme = strata,
        stratification = stratification_tab,
        wd,
        save,
        verbose
    )
    if (verbose) message("Time series of MIW - completed")


    #--------------------------------------------------------------
    # Sex ratio per GSA in the timeseries
    #--------------------------------------------------------------
    if (verbose) message("\n########################")
    if (verbose) message("Sex-ratio time series")
    if (verbose) message("########################")
    if (verbose) message("")

    SR_analysis <- "ok"

    if (SR_analysis == "ok") {
        if (sum(mTATB$NB_OF_FEMALES + mTATB$NB_OF_MALES, na.rm = TRUE) == 0) {
            if (verbose) message("Not enough sex data for sex-ratio estimation")
            if (verbose) message("\nSex-ratio analysis skipped")
        } else {
            sex_ratio(
                mTATB,
                GSA,
                country = country,
                depth_range = depth,
                stratas = strata,
                stratification = stratification_tab,
                wd,
                save
            )
            if (verbose) message("\nSex-ratio analysis - completed")
        }
    } else {
        if (verbose) message("\nSex-ratio analysis skipped")
    }


    # #--------------------------------------------------------------
    # # Abundance indices of spawners in the time series
    # #--------------------------------------------------------------
    # message("\n############################")
    # message("Spawners' abundance indices")
    # message("############################")
    # message("")
    # #------> check the threshold in the file "~/input/maturity_sizes.csv"
    # df_cutoff <- spaw_threshold
    # if (is.na(df_cutoff)) {
    #     message(
    #         "The SPAWNERS' threshold value not provided. Please, define a value in the 'spaw_threshold' parameter."
    #     )
    # }
    # skip_spawners <- FALSE
    # spaw_analysis <- "ok"
    #
    # if (spaw_analysis == "ok") {
    #     source.check <- try(index_spawn(
    #         mTATB,
    #         mTATC,
    #         GSA,
    #         country,
    #         depth_range = depth,
    #         cutoff = spaw_threshold,
    #         stratification = stratification_tab,
    #         wd,
    #         save
    #     )
    #     ,
    #     silent = T)
    #
    #     if (!is(source.check, "try-error"))  {
    #         index_spawn(
    #             mTATB,
    #             mTATC,
    #             GSA,
    #             country,
    #             depth_range = depth,
    #             cutoff = spaw_threshold,
    #             stratification = stratification_tab,
    #             wd,
    #             save
    #         )
    #         if (skip_spawners) {
    #             message("\nSpawners' indices analysis skipped")
    #         } else {
    #             message("\nSpawners' indices analysis - completed")
    #         }
    #
    #     } else {
    #         message("\nSpawners' indices analysis skipped - (Run Error)\n")
    #     }
    #
    # }  else {
    #     message("\nSpawners' indices analysis skipped")
    # }

    #--------------------------------------------------------------
    # Abundance indices of spawners in the time series
    #--------------------------------------------------------------
    if (verbose) message("\n############################")
    if (verbose) message("Spawners' abundance indices")
    if (verbose) message("############################")
    if (verbose) message("")

    df_cutoff <- spaw_threshold
    if (is.na(df_cutoff)) {
        if (verbose) {
            message(
                "The SPAWNERS' threshold value not provided. Please, define a value in the 'spaw_threshold' parameter."
            )
        }
    }

    skip_spawners <- FALSE
    spaw_analysis <- "ok"

    if (spaw_analysis == "ok") {
        source.check <- try(
            index_spawn(
                mTATB,
                mTATC,
                GSA,
                country,
                depth_range = depth,
                cutoff = spaw_threshold,
                stratification = stratification_tab,
                wd,
                save
            ),
            silent = TRUE
        )

        if (!is(source.check, "try-error")) {
            if (skip_spawners) {
                if (verbose) message("\nSpawners' indices analysis skipped")
            } else {
                if (verbose) message("\nSpawners' indices analysis - completed")
            }
        } else {
            if (verbose) message("\nSpawners' indices analysis skipped - (Run Error)\n")
        }
    } else {
        if (verbose) message("\nSpawners' indices analysis skipped")
    }


    # #--------------------------------------------------------------
    # # Abundance indices of recruits in the time series
    # #--------------------------------------------------------------
    # message("\n############################")
    # message("Recruits' abundance indices")
    # message("############################")
    # message("")
    # #------> check the threshold in the file "~/input/maturity_sizes.csv"
    # df_cutoff <- rec_threshold
    # if (is.na(df_cutoff)) {
    #     message(
    #         "The RECRUITS' threshold value not provided. Please, define a value in the 'rec_threshold' parameter."
    #     )
    # }
    # skip_recruits <- FALSE
    # rec_analysis <- "ok"
    #
    # if (rec_analysis == "ok") {
    #     source.check <- try(index_recr(
    #         mTATB,
    #         mTATC,
    #         GSA,
    #         country,
    #         depth_range = depth,
    #         cutoff = rec_threshold,
    #         stratification = stratification_tab,
    #         wd,
    #         save
    #     )
    #     ,
    #     silent = T)
    #
    #     if (!is(source.check, "try-error"))  {
    #         index_recr(
    #             mTATB,
    #             mTATC,
    #             GSA,
    #             country,
    #             depth_range = depth,
    #             cutoff = rec_threshold,
    #             stratification = stratification_tab,
    #             wd,
    #             save
    #         )
    #         if (skip_recruits) {
    #             message("\nRecruits' indices analysis skipped")
    #         } else {
    #             message("\nRecruits' indices analysis - completed")
    #         }
    #
    #     } else {
    #         message("\nRecruits' indices analysis skipped - (Run Error)\n")
    #     }
    #
    # }  else {
    #     message("\nRecruits' indices analysis skipped")
    # }


    #--------------------------------------------------------------
    # Abundance indices of recruits in the time series
    #--------------------------------------------------------------
    if (verbose) message("\n############################")
    if (verbose) message("Recruits' abundance indices")
    if (verbose) message("############################")
    if (verbose) message("")

    df_cutoff <- rec_threshold
    if (is.na(df_cutoff)) {
        if (verbose) {
            message(
                "The RECRUITS' threshold value not provided. Please, define a value in the 'rec_threshold' parameter."
            )
        }
    }

    skip_recruits <- FALSE
    rec_analysis <- "ok"

    if (rec_analysis == "ok") {
        source.check <- try(
            index_recr(
                mTATB,
                mTATC,
                GSA,
                country,
                depth_range = depth,
                cutoff = rec_threshold,
                stratification = stratification_tab,
                wd,
                save
            ),
            silent = TRUE
        )

        if (!is(source.check, "try-error")) {
            if (skip_recruits) {
                if (verbose) message("\nRecruits' indices analysis skipped")
            } else {
                if (verbose) message("\nRecruits' indices analysis - completed")
            }
        } else {
            if (verbose) message("\nRecruits' indices analysis skipped - (Run Error)\n")
        }
    } else {
        if (verbose) message("\nRecruits' indices analysis skipped")
    }


    #--------------------------------------------------------------
    # LFD & L0.95
    #--------------------------------------------------------------
    if (verbose) message("\n############################")
    if (verbose) message("LFD, L0.50 & L0.95")
    if (verbose) message("############################")
    if (verbose) message("")

    LFD_analysis <- "ok"

    if (LFD_analysis == "ok") {
        if (sexes == "all") {
            lfd <- LFD(
                mTATC,
                sex = "all",
                GSA = GSA,
                country = country,
                depth_range = depth,
                strata_scheme = strata,
                stratification = stratification_tab,
                wd,
                save
            )
            Lquant(lfd[[1]], wd, sspp, GSA, save, verbose)
        }

        if (sexes == "M") {
            lfd <- LFD(
                mTATC,
                sex = "M",
                GSA = GSA,
                country = country,
                depth_range = depth,
                strata_scheme = strata,
                stratification = stratification_tab,
                wd,
                save
            )
            Lquant(lfd[[1]], wd, sspp, GSA, save, verbose)
        }
        if (sexes == "F") {
            lfd <- LFD(
                mTATC,
                sex = "F",
                GSA = GSA,
                country = country,
                depth_range = depth,
                strata_scheme = strata,
                stratification = stratification_tab,
                wd,
                save
            )
            Lquant(lfd[[1]], wd, sspp, GSA, save, verbose)
        }
        if (verbose) message("\nLFD & L0.95 analysis - completed")
    } else {
        if (verbose) message("\nLFD & L0.95 analysis skipped")
    }


    #--------------------------------------------------------------
    # Spearman test of trends on short timeseries
    #--------------------------------------------------------------
    if (verbose) message("\n###########################################")
    if (verbose) message("Spearman test of trends on short timeseries")
    if (verbose) message("###########################################")
    if (verbose) message("")
    years <- sort(unique(mTATB$YEAR))
    if (length(years) >= 3) {
        Trend_analysis <- "ok"
        if (Trend_analysis == "ok") {
            spearman(
                abundance = index[[1]],
                biomass = index[[2]],
                years,
                sspp,
                wd,
                save
            )
            if (verbose) message("\nSpearman test - completed")
        } else {
            if (verbose) message("\nSpearman test skipped")
        }
    } else {
        if (verbose) message("\nSpearman test skipped")
    }


    #--------------------------------------------------------------
    # Abundance indices for statistical squares
    # inverse of CV of abundance indices for statistical squares
    # Biomass indices for statistical squares
    # Mean individual weight for statistical squares
    #--------------------------------------------------------------
    depth_range <- paste(depth, collapse = ",")

    index_on_grid(
        mTATBsp,
        stratum = depth_range,
        wd,
        map_range = map_lim,
        threshold = haul_threshold,
        verbose = TRUE,
        save = TRUE
    )


    #--------------------------------------------------------------
    # Sex ratio for statistical squares
    #--------------------------------------------------------------
    if (verbose) message("\n#############################")
    if (verbose) message("Sex-ratio on GFCM grid")
    if (verbose) message("#############################")
    if (verbose) message("")

    #------> select the minimum number of individuals per haul to be considered in the analysis
    sexratio_grid_skip <- FALSE
    SR_GRID_analysis <- "ok"
    depth_stratum <- paste(depth, collapse = ",")
    if (SR_GRID_analysis == "ok") {
        sex_ratio_on_grid(
            mTATBsp = mTATBsp,
            depth = depth_stratum,
            wd = wd,
            map_range = map_lim,
            threshold = haul_threshold,
            verbose = TRUE,
            save = TRUE
        )

        if (sexratio_grid_skip) {
            if (verbose) message("\nSex-ratio on GFCM grid skipped")
        } else {
            if (verbose) message("\nSex-ratio on GFCM grid - completed")
        }
    } else {
        if (verbose) message("\nSex-ratio on GFCM grid skipped")
    }


    #--------------------------------------------------------------
    # Bubble plots - indices of recruits (abundance)
    # Bubble plots - indices of spawners (abundance)
    #--------------------------------------------------------------
    if (verbose) message("\n################################################")
    if (verbose) message("Bubble plots - indices of recruits and spawners")
    if (verbose) message("################################################")
    if (verbose) message("")
    #------> check the threshold in the file "~/input/maturity_sizes.csv"

    if (any(is.na(c(rec_threshold, spaw_threshold)))) {
        if (verbose) message("Missing threshold value for the selected species.")
    } else {
        bubbleplot_RS_by_hauls(
            mTATC = mTATC,
            map_range = map_lim,
            thresh_rec = rec_threshold,
            thresh_spaw = spaw_threshold,
            depths = depth_lines,
            buffer = buffer,
            res = resolution,
            wd = wd,
            save = save,
            verbose = verbose
        )
        if (verbose) message("\nBubble plots - indices of recruits and spawners - completed")
    }


    if (verbose) message("\n###################")
    if (verbose) message(" Analysis completed")
    if (verbose) message("###################")
    if (verbose) message("")


    #----------------
    # ZIP FILE
    #----------------
    # BioIndex_Report_MERLMER_GSA10_depth_10_800
    if (zip) {
        files <- list.files(
            path = file.path(wd, "output"),
            recursive = TRUE,
            full.names = TRUE,
            include.dirs = TRUE
        )
        zips <- grep(".zip", files)
        if (length(zips) > 0) {
            files <- files[-zips]
        }

        if (length(files) > 0) {
            output <- file.path(wd, "output")
            zip::zip(paste0(
                "BioIndex_results_", sspp, "_GSA", GSA, "_Depth", paste(depth, collapse = "-"), "m_",
                paste(
                    as.character(Sys.Date()),
                    format(Sys.time(), "_h%Hm%Ms%OS0"),
                    ".zip",
                    sep = ""
                )
            ), "output", root = wd)
            unlink(files, recursive = TRUE)
            unlink(file.path(wd, "output"),
                force = TRUE,
                recursive = TRUE
            )
            if (verbose) message("ZIP file generated successfully in: ", wd)
        } else {
            if (verbose) message("No files generated to be zipped.")
        }
    }
}

#' @importFrom stats AIC coef coefficients cor.test dchisq deviance
#' @importFrom stats df.residual lm na.exclude nls pnorm predict rnorm sd time quantile
#' @importFrom utils data write.table read.table
#' @importFrom magrittr %>%
#' @importFrom dplyr arrange mutate filter select group_by summarize
#' @importFrom terra unwrap
#' @importFrom graphics box
#' @importFrom grDevices dev.off jpeg heat.colors
#' @importFrom utils data write.table
#' @importFrom shinyjs useShinyjs
NULL

# Solve R CMD check notes on global variables
utils::globalVariables(c("AGE", "LENGTH", "LENGTH_CLASS", "lab_y", "med_bathy", "strata",
                         "N_km2", "kg_km2", "YEAR", "STRATUM_CODE", "SWEPT_AREA",
                         "n_raised", "num_weighted", "LENGTH_CLASSES_CODE",
                         "GENUS", "SPECIES", "COUNTRY", "MEAN_DEPTH",
                         "NB_OF_FEMALES", "NB_OF_MALES", "NB_FM", "Indices_F",
                         "Indices_FM", "sr", "variance", "stratum", "value",
                         "year", "STRATUM", "numb_per_stratum", "id", "ALL.STRATA",
                         "longitude", "latitude", "group", "V1", "V2", "V3"))

#' Plot sex ratio spatial distribution
#'
#' @description
#' This function calculates and plots the spatial distribution of sex ratio (F / (F + M)) over the GFCM grid for a given species. It uses data from merged MEDITS datasets (TA, TB, TC), previously spatialized with `overlayGrid()`. The function filters hauls by total abundance threshold and depth range, calculates sex ratio per grid cell, and plots the output using color-coded categories based on quantiles. The result is a choropleth map of sex ratio patterns, useful to highlight spatial differences in sex structure. Raw sex ratio estimates per grid cell can also be exported.
#'
#' @param mTATBsp A spatial version of the merged TA-TB dataset, processed through `overlayGrid()`, containing sex information.
#' @param depth Character string specifying the depth range used for filtering (e.g., `"10,800"`). Must correspond to available internal GFCM grid masks.
#' @param wd Working directory. Used to save output files if `save = TRUE`.
#' @param map_range numeric vector with coordinates defining the map extent (`xmin`, `xmax`, `ymin`, `ymax`).
#' @param threshold Minimum number of individuals required per haul to be included in the sex ratio estimation. Default is 30.
#' @param save Logical. If `TRUE`, the plot and sex ratio table are saved to the working directory.
#' @param verbose Logical. If `TRUE`, informative messages are printed to the console.
#'
#' @return A `data.frame` containing sex ratio (`ratio`), standard deviation (`sd`), and coefficient of variation (`CV`) for each GFCM grid cell.
#'
#' @importFrom terra extract unwrap
#' @importFrom stats quantile
#' @importFrom ggplot2 ggplot geom_polygon coord_sf scale_fill_manual labs xlab ylab aes
#' @importFrom ggplot2 theme_bw
#' @importFrom tidyterra geom_spatvector
#' @importFrom ggplot2 ggsave
#' @importFrom dplyr arrange
#' @importFrom ggplot2 map_data
#' @examples
#' data(TA)
#' data(TB)
#' data(TC)
#' m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
#'                   species = "MERLMER", country = "all", verbose = FALSE)
#' mTATBsp <- overlayGrid(m[[1]], m[[2]], GSA = 10, save = FALSE, verbose = FALSE)[[1]]
#' map_range <- c(9, 15, 39, 42)
#' sex_ratio_on_grid(mTATBsp, depth = "10,800", map_range = map_range, threshold = 5,
#'                   save = FALSE, verbose = FALSE)
#' @export
sex_ratio_on_grid <- function(mTATBsp, depth, wd=NA, map_range,threshold=30,verbose=FALSE, save=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }

    if (FALSE) {
        verbose=TRUE
        save=TRUE
        map_range <- c(15,21,39.5,42.5)
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        country="all"
        species <- "MERLMER"

        m <- merge_TATBTC(ta, tb, tc, species="MERLMER", country="all", wd=wd, verbose=TRUE)

        mTATB <- m[[1]]
        mTATC <- m[[2]]

        mTATBsp <- overlayGrid(mTATB, mTATC, GSA=NA, wd=wd, save=TRUE, verbose=FALSE)[[1]]
        threshold=30
        depth <- "10,800"
        sex_ratio_on_grid (mTATBsp, depth, wd, map_range,threshold=30,verbose=TRUE, save=TRUE)
    }

    level <- category <- long <- lat <- group <- NULL

    # depthstr <- depth
    depth_range <- as.numeric(str_split(depth,",")[[1]])

    #----------------

    resolve_depthstr <- function(depth_range) {

        dr <- sort(as.numeric(depth_range))
        if (anyNA(dr)) stop("depth_range contains NA.")
        if (dr[1] >= dr[2]) stop("depth_range must be increasing (min < max).")

        lo <- dr[1]
        hi <- dr[2]

        # 2) Map to allowed depthstr values, ensuring coverage of [lo, hi]

        # Shallow set (mapped to 0_35 / 0_45 / 0_125 in your code)
        if (hi <= 35) {
            return("5,35")      # maps to BioIndex::stratum_0_35
        }
        if (hi <= 45) {
            return("5,45")      # maps to BioIndex::stratum_0_45
        }
        if (hi <= 125) {
            return("0,125")     # canonical, maps to BioIndex::stratum_0_125
        }

        # Deep set (mapped to 0_200 / 0_800 / 200_800)
        if (hi <= 200) {
            return("10,200")    # maps to BioIndex::stratum_0_200
        }
        if (hi <= 800) {
            if (lo >= 200) {
                return("200,800") # maps to BioIndex::stratum_200_800
            } else {
                return("10,800")  # maps to BioIndex::stratum_0_800
            }
        }
    }

    #------------------------------

    depthstr <- resolve_depthstr(depth_range)

    if (depthstr == "5,45") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_45)
    }

    if (depthstr == "5,35") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_35)
    }

    if (depthstr == "10,125") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_125)
    }

    if (depthstr == "5,125") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_125)
    }

    if (depthstr == "0,125") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_125)
    }

    if (depthstr == "10,200") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_200)
    }

    if (depthstr == "10,800") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_0_800)
    }

    if (depthstr == "200,800") {
        cgpmgrid <- terra::unwrap(BioIndex::stratum_200_800)
    }
    cgpmgrid_bkp <- cgpmgrid
    continent <- terra::unwrap(BioIndex::continent)



    metaDBnew <- mTATBsp
    years <- sort(unique(metaDBnew$YEAR))
    GSA <- unique(metaDBnew$GSA)[1]
    GENERE <- as.character(unique(metaDBnew$GENUS)[unique(metaDBnew$GENUS) != -1])
    SPECIE <- as.character(unique(metaDBnew$SPECIES)[unique(metaDBnew$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")

    if (length(years) < 10) {
        n_years <- length(years)
        firstyear <- years[length(years)-(n_years-1)]
        last_10year <- years[years >= firstyear]
    } else {
        firstyear <- years[length(years)-9]
        last_10year <- years[years >= firstyear]
    }

    metaDBnew <- metaDBnew[metaDBnew$YEAR %in% last_10year & metaDBnew$TOTAL_NUMBER_IN_THE_HAUL > threshold, ]  #
    metaDBnew <- metaDBnew[metaDBnew$MEAN_DEPTH > depth_range[1] & metaDBnew$MEAN_DEPTH <= depth_range[2], ]  #


    if (nrow(metaDBnew)==0){
        if (verbose){
        message("Not enough data for the sex ratio estimation.\nTry again reducing the threshold value.\n")
            }
        sexratio_grid_skip <- TRUE
    } else {

        if ( all(metaDBnew$NB_OF_FEMALES == 0) & all(metaDBnew$NB_OF_MALES == 0) ) {
            if(verbose){
            message("No sex information available in TC for the selected species. Sex-ratio estimation is not possible")
                }
            sexratio_grid_skip <- TRUE
        } else {

            grid_id <- unique(metaDBnew$cgpmgrid_id)
            sex_ratio <- data.frame(matrix(NA, nrow=length(grid_id), ncol=4))
            colnames(sex_ratio) <- c("cgpmgrid_id", "ratio", "sd", "CV")
            sex_ratio[,1] <-grid_id

            i=3
            for (i in 1:length(grid_id)){
                dd <- metaDBnew[metaDBnew$cgpmgrid_id == grid_id[i], ]
                dd$nkm2_F <- dd$NB_OF_FEMALES/dd$SWEPT_AREA
                dd$nkm2_M <- dd$NB_OF_MALES/dd$SWEPT_AREA
                dd$nkm_FM <- (dd$NB_OF_MALES + dd$NB_OF_FEMALES)/dd$SWEPT_AREA
                dd$sr_haul<- dd$nkm2_F / dd$nkm_FM

                nkm_F <- sum(dd$nkm2_F)
                nkm_M <- sum(dd$nkm2_M)
                nkm_FM <- sum(dd$nkm_FM)
                sex_ratio[i,2] <- nkm_F/nkm_FM
                sex_ratio[i,3] <- sqrt(sum((sex_ratio[i,2] - dd$sr_haul)^2)/length(dd$sr_haul))
                sex_ratio[i,4] <- sex_ratio[i,3]/sex_ratio[i,2]
            }

            if (verbose) message("CSV of sex ratio by grid cell successfully created.")
            if(save){
            write.table(sex_ratio, paste(wd, "/output/",sspp," - GFCM SEX RATIO.csv", sep=""), sep=";", row.names=FALSE)
                if (verbose) message("CSV of sex ratio by grid cell successfully saved.")
            }

            names(map_range) <- c("xmin","xmax","ymin","ymax")
            x1 <- min(map_range[1])
            x2 <- max(map_range[2])
            y1 <- min(map_range[3])
            y2 <- max(map_range[4])

            #----------------------------------------
            # Saving maps of the sex-ratio by grid
            #----------------------------------------
            cat_lim <- quantile(sex_ratio$ratio[!is.na(sex_ratio$ratio)], probs = seq(0, 1,0.20) )
            cate_label <-  round(cat_lim,3)

            cate_label_min <- cate_label
            cate_label_max <- cate_label[2:length(cate_label)]

            cate_label <- paste("(", cate_label_min, ", ", cate_label_max, "]", sep="")
            cate_label[1] <- paste("[", cate_label_min[1], ", ", cate_label_max[1], "]", sep="")

            cate_label <- cate_label[1: (length(cate_label)-1)]

            palette_cate <- heat.colors(length(cate_label))
            palette_cate <- palette_cate[length(palette_cate):1]

            # continent <- unwrap(BioIndex::continent)
            cgpmgrid <- cgpmgrid_bkp
            cgpmgrid$SR <- NA
            cgpmgrid$color <- "#FFFFFF"

            i=1
            for (i in 1:length(sex_ratio[,1])){
                cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- sex_ratio[i,"ratio"]

                if (all(is.na(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]]) )){
                    cgpmgrid$color[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- "#FFFFFF"
                } else {

                    if(all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <= cat_lim[2])  & all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] >= cat_lim[1])){
                        cgpmgrid$color[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- palette_cate[1]
                    }

                    if(all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <= cat_lim[3])  & all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] > cat_lim[2])){
                        cgpmgrid$color[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- palette_cate[2]
                    }

                    if(all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <= cat_lim[4])  & all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] > cat_lim[3])){
                        cgpmgrid$color[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- palette_cate[3]
                    }

                    if(all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <= cat_lim[5])  & all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] > cat_lim[4])){
                        cgpmgrid$color[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- palette_cate[4]
                    }

                    if(all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <= cat_lim[6])  & all(cgpmgrid$SR[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] > cat_lim[5])){
                        cgpmgrid$color[cgpmgrid$GFCM_ID==sex_ratio[i,"cgpmgrid_id"]] <- palette_cate[5]
                    }
                }
            }


            lx =map_range["xmax"] - map_range["xmin"]
            ly =map_range["ymax"] - map_range["ymin"]
            ratio <- ly/lx
            img_width <- 30
            img_height <- img_width*ratio


            # par( mar=c(4, 5, 4, 2)) #c(bottom, left, top, right)
            # plot(1,1,type="n",xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), xlab=expression(paste("Longitude (",degree,"E)")), ylab=expression(paste("Latitude (",degree,"N)")), main=paste("sex ratio -", sspp, " GSA", GSA),asp=1 )
            # plot(cgpmgrid, col=cgpmgrid$color, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")                                                                                 #data = centroidi_coords,
            # if (depth == "200,800" | depth == 3){
            #     plot(mask,col="white", xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")
            # }
            # plot(continent, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), border="grey", col="light grey", add=T)
            # legend("topleft", cate_label, col=palette_cate, pch = 15, pt.cex=c(2), title = "sex ratio\n(classes based on percentiles)", bty = "n")
            # box()
            # psr <- recordPlot()

            tab_color <- data.frame(category=cate_label,color=palette_cate,level=seq(1,length(cate_label),1))
            cgpmgrid <- terra::merge(cgpmgrid,tab_color,by.x="color",by.y="color")
            world <- map_data("world")
            xl <- c(x1,x2)
            yl <- c(y1,y2)
            x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
            y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))
            buff=0

            cols <- c("#FFFF80",
                      "#FFFF00",
                      "#FFAA00",
                      "#FF5500",
                      "#FF0000")

            names(cols) <- tab_color$category
            cgpmgrid$level <- is.factor(cgpmgrid$level)
            cgpmgrid$level <- factor(cgpmgrid$level, levels=seq(1,nrow(tab_color),1))
            cgpmgrid <- cgpmgrid %>% arrange(level)

            p1 <- ggplot(cgpmgrid)+
                geom_spatvector(aes(fill = category))+
                scale_fill_manual(values = cols, breaks=names(cols))+
                geom_polygon(data=world, aes(long,lat,group=group), fill="dark grey", colour="darkgrey")+
                coord_sf(xlim = xl, ylim = yl)+
                labs(fill="Sex ratio")+
                xlab("longitude")+
                ylab("latitude")



            if (verbose) message("plot of sex ratio by grid cell successfully created")
            if (save){
                ggplot2::ggsave(filename=paste(wd, "/output/",sspp," - GFCM GRID Sex Ratio.jpg", sep = ""), plot=p1, width=25, height=25, bg="white", units="cm", dpi=200)
                if (verbose) message("Plot of sex ratio by grid cell successfully saved.")
            }

            return(sex_ratio)
        }
    }

}

#' Interception Union Tets
#'
#' @param abundance dataframe of abundance time series as produced by indices_ts function
#' @param biomass dataframe of biomass time series as produced by indices_ts function
#' @param species reference species for the analysis (MEDITS code)
#' @param lastn number of recent years for diagnosis of change
#' @param GSA reference GSA for the analysis (default is 10)
#' @param wd working directory (default is NA, uses tempdir())
#' @param save boolean. If TRUE results are saved in the output folder
#' @importFrom utils write.table
#' @importFrom grDevices jpeg dev.copy dev.off
#' @return A \code{data.frame} containing the Indicator of Unfished Trends (IUT) results.
#' @import mgcv
IUT <- function(abundance, biomass, species, lastn=5, GSA=10, wd=NA, save=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }
    if (save) {
        if (!dir.exists(file.path(wd, "output"))) {
            dir.create(file.path(wd, "output"), showWarnings = FALSE, recursive = TRUE)
        }
    }
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))

    if (FALSE) {
        GSA=10
        save=TRUE
        depth_range <- c(10,800)

        verbose=TRUE
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex_app\\R_BioIndex_3.3_(in update)"
        ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=TRUE)
        tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=TRUE)
        tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=TRUE)
        country="all"

        species <- "MERLMER"

        m <- merge_TATBTC(ta, tb, tc, species="MERLMER", country="all", wd=wd, verbose=TRUE)
        mTATB <- m[[1]]
        ind <- indices_ts(mTATB, GSA=10, country="all", depth_range=c(10,800), strata_scheme=BioIndex::strata_scheme, stratification=BioIndex::stratification,wd, save=TRUE)

        IUT(abundance=ind[[1]], biomass=ind[[2]], species="MERLMER", lastn=4, save=TRUE)

    }

    write=FALSE
    alpha=0.05
    CVindicators=0.05 	#used in bootstrap for uncertainty
    system<-paste("GSA", GSA, sep="")
    sspp <- species
    #####END PART THAT HAS TO BE ADAPTED BY USER##########


    ##MAIN part for carrying out analysis###########

    #read data table
    df1 <- abundance
    df2 <- biomass
    dat <- data.frame(year = df1$year , ecosystem = paste("GSA",GSA, sep=""), abundance = df1$abundance, biomass = df2$biomass)
    response <- c("Abundance (n/km^2)", "Biomass (kg/km^2)")
    cols_1_2 <- c("year", "ecosystem")

    dat[,3] <- dat[,3]
    dat[,4] <- dat[,4]
    indicators= colnames(dat)[!(colnames(dat) %in% cols_1_2)]  # c("ARITANT")
    indiname=indicators


    #col.names(dat)=c("Year","ecosystem","delta_10","L95_10","10ARISFOLF2D","10PAPE LONF2D","10PAPE LONM2D","10ARISFOLM 2D","10NEPRNORM2D")
    nrows <- length(indicators)/2
    ncols <- 2


    # windows(width=(10),height=(5*nrows))
    par(mfrow=c(nrows,ncols),mar=c(5,5,1.5,0.5),las=1,lab=c(5,3,5))


    #for each indicator run test and store results
    #results matrix
    Res<-matrix(NA,nrow=length(indicators),ncol=14)
    dimnames(Res)<-list(NULL,c("Area","Indicator","Indicators","pchisq","RecentDecrease","ChangePointDec","RecentIncrease","ChangePointInc","RecentLinear","Plinear","DecMK","IncMK","PMKdec","PMKinc"))
    hasdata=0
    j=1
    for(j in 1:length(indicators)){
        lastligne=ifelse(j==length(indicators),TRUE,FALSE)
        indi=dat[,colnames(dat)==indicators[j]]
        id=!is.na(indi)# &dat[,1]<2006
        indi=indi[id]
        years<-dat[id,1]
        Res[j,1]<-system
        Res[j,2:3]<-indicators[j]
        lab_y <- response[j]
        if(length(indi)>=lastn){
            Res[j,2:14]<-unlist(.ChangeYears.func(Year=years,Indic=indi,Sd=rep(NA,length(indi)),species=system,ytext=indicators[j],CV=CVindicators,lastn=lastn,write=write, speciesname=indiname[j],alpha=alpha,lastligne=lastligne))
            hasdata=1
        }
    }


    if(save){
        write.table(Res,paste(wd,"/output/",system,"IUtest",lastn,"_RSS.csv",sep=""),sep=";",quote=FALSE,row.names=FALSE,append=FALSE)

       if(hasdata==1)dev.copy(jpeg,paste(wd,"/output/",sspp,"_",system,"_SmoothedIndicators_RSS.jpg",sep=""),height=5,width=10, units='in', res=300)
        dev.off()
    }
    #summarise results across indicators########

    ni=length(indicators)
    res=matrix(NA,nrow=ni*length(system),ncol=14)
    i=1
    for(i in 1:length(system)){
        res[((i-1)*ni+1):(i*ni),1:14]=as.matrix(Res)
    }
    header=names(data.frame(Res))

    dimnames(res)=dimnames(res)=list(NULL,header)
    res=data.frame(res[,c(1,3,4,5,7)])
    res[,4] <- as.factor(res[,4])
    res[,5] <- as.factor(res[,5])
    j=3
    for(j in 3:5) {res[,j]=as.numeric(as.character(res[,j]))}
    res$Change=res$RecentDecrease+res$RecentIncrease

    #create table with results, one column per indicator, one row per ecosystem
    indi=sort(unique(res[,2]))
    table=data.frame(cbind(res[res[,2]==indi[1],6],res[res[,2]==indi[2],6]))
    names(table)=indi

    table$system=res[res[,2]==indi[1],1]
    table=table[,c(3,1:2)]

    #replace -1 by - 1 by + and 0 by ""
    for (c in 2:3){
        if (table[c] == -1){table[c] = "decrease"}
        if (table[c] == 1){table[c] = "increase"}
        if (table[c] == 0){table[c] = "nochange"}

    }

    if (save){
        write.table(table,paste(wd, "/output/",sspp,"_GSA",GSA,"_IUT_results_",lastn,"years_RSS.csv",sep=""),sep=";",col.names=TRUE,row.names=FALSE,quote=FALSE)
}
return(table)
}

#' Sex ratio
#' @description
#' Assesses the proportion of sexes within the population, providing
#' critical information on demographic structure, reproductive potential,
#' and stock resilience.
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @param mTATB data frame of the merged TA and TB
#' @param GSA reference GSA for the analysis
#' @param country vector of reference countries for the analysis
#' @param depth_range range of depth strata to perform the analysis (min, max)
#' @param stratas data frame of the reference strata for the study area
#' @param stratification data frame of strata surface area
#' @param wd working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @param verbose boolean. If TRUE a message is printed
#' @return A \code{data.frame} containing the sex ratio statistics by year and stratum.
#' @examples
#' data(TA)
#' data(TB)
#' # Create a merged dataset for GSA 10
#' mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
#' sex_ratio(mTATB, GSA=10, country="all", depth_range=c(10,800),
#'           stratas=BioIndex::strata_scheme,
#'           stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
#' @export
sex_ratio <- function(mTATB, GSA, country, depth_range, stratas, stratification, wd=NA, save=TRUE,verbose=FALSE) {
    if (is.na(wd)) { wd <- tempdir() }
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))

    if (FALSE) {
        GSA=10
        save=TRUE
        merge_TATB <- mTATB
        depth_range <- c(10,800)
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        country="all"
        species <- "MERLMER"
        m <- merge_TATBTC(ta, tb, tc, species="MERLMER", country="all", wd=wd, verbose=TRUE)
        mTATB <- m[[1]]
        sex_ratio(mTATB, GSA, country="all", depth_range, stratification, wd, save)


        stratas <- strata
        stratification <- stratification_tab
    }

    if (is.na(wd) & save) {
        save=TRUE
        if (verbose){
            message("Missing working directory. Results are not saved in the local folder.")
        }
    }

    merge_TATB <- mTATB

    GENERE <- as.character(unique(merge_TATB$GENUS)[unique(merge_TATB$GENUS) != -1])
    SPECIE <- as.character(unique(merge_TATB$SPECIES)[unique(merge_TATB$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")
    GSA <- unique(merge_TATB$GSA)
    species <- sspp

    countries <- unique(mTATB[!is.na(mTATB$COUNTRY),"COUNTRY"])
    l_country <- length(unique(mTATB$COUNTRY))

    if (l_country==1){
        check_country ="Y"
        country_analysis <- countries
    } else {
        if (any(country %in% "all")) {
            country_analysis <<- countries
        } else {
            country_analysis <<- country
        }
    }

    ddd <- merge_TATB
    ddd$NB_FM <- ddd$NB_OF_FEMALES + ddd$NB_OF_MALES
    year_range <- data.frame(year = sort(unique(merge_TATB$YEAR)))

    strata_scheme <- stratas
    strata_scheme <- strata_scheme[strata_scheme$GSA == GSA & strata_scheme$COUNTRY %in% as.character(unique(country_analysis))[1], ]

    depth <- data.frame(matrix(NA,ncol=4, nrow=length(strata_scheme$CODE)))
    colnames(depth) <- c("strata", "min", "max", "bul")
    depth$strata <- strata_scheme$CODE # c(1,2,3,4,5)
    depth$min <-strata_scheme$MIN_DEPTH  # c(10,50,100,200,500)
    depth$max <-strata_scheme$MAX_DEPTH  # c(50,100,200,500,800)
    if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
    data <-  ddd[ddd$MEAN_DEPTH>depth_range[1] & ddd$MEAN_DEPTH<=depth_range[2],]
    idx_from <- which(depth[,"min"] == depth_range[1])
    idx_to   <- which(depth[,"max"] == depth_range[2])

    if (length(idx_from) != 1 || length(idx_to) != 1) {
        stop(paste0("Error: depth range [", depth_range[1], ", ", depth_range[2],
                    "] does not match the strata boundaries defined for this GSA. ",
                    "Available min depths: ", paste(sort(unique(depth$min)), collapse=", "),
                    "; Available max depths: ", paste(sort(unique(depth$max)), collapse=", ")))
    }
    strata_range <- seq(idx_from, idx_to, 1)
    depth[depth$strata %in% strata_range, "bul"] <- T
    depth[!(depth$strata %in% strata_range), "bul"] <- F

    positive_hauls<- 0

    res_table_calaF <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    res_table_calaF$index_1 <- NA
    res_table_calaF$index_2 <- NA
    res_table_calaF$index_3 <- NA
    res_table_calaF$index_4 <- NA
    res_table_calaF$index_5 <- NA
    res_table_calaF$index_6 <- NA

    res_table_calaFM <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    res_table_calaFM$index_1 <- NA
    res_table_calaFM$index_2 <- NA
    res_table_calaFM$index_3 <- NA
    res_table_calaFM$index_4 <- NA
    res_table_calaFM$index_5 <- NA
    res_table_calaFM$index_6 <- NA

    # FM <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    # FM$index_1 <- NA
    # FM$index_2 <- NA
    # FM$index_3 <- NA
    # FM$index_4 <- NA
    # FM$index_5 <- NA

    SR <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    SR$Indices_F <- NA
    SR$Indices_FM <- NA
    SR$sr <- NA
    SR$variance <- NA

    # stratification <- read.table(paste(wd,"/scripts/utilities/stratification scheme.csv", sep=""), sep=";", header=T)
    stratification$SURF <- as.numeric(stratification$SURF)

    analysis_stratum1 <- NA
    analysis_stratum2 <- NA
    analysis_stratum3 <- NA
    analysis_stratum4 <- NA
    analysis_stratum5 <- NA
    analysis_stratum6 <- NA

    analysis_stratum1 <- depth[1,4]
    analysis_stratum2 <- depth[2,4]
    analysis_stratum3 <- depth[3,4]
    analysis_stratum4 <- depth[4,4]
    analysis_stratum5 <- depth[5,4]
    analysis_stratum6 <- depth[6,4]

    if (is.na(analysis_stratum1)){depth[1,4]<- F; analysis_stratum1 <- F}
    if (is.na(analysis_stratum2)){depth[2,4]<- F; analysis_stratum2 <- F}
    if (is.na(analysis_stratum3)){depth[3,4]<- F; analysis_stratum3 <- F}
    if (is.na(analysis_stratum4)){depth[4,4]<- F; analysis_stratum4 <- F}
    if (is.na(analysis_stratum5)){depth[5,4]<- F; analysis_stratum5 <- F}
    if (is.na(analysis_stratum6)){depth[6,4]<- F; analysis_stratum6 <- F}

    res_table_cala2 <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    res_table_cala2$index_1 <- NA
    res_table_cala2$index_2 <- NA
    res_table_cala2$index_3 <- NA
    res_table_cala2$index_4 <- NA
    res_table_cala2$index_5 <- NA
    res_table_cala2$index_6 <- NA

    if (analysis_stratum1 == T){area_s1 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==1 & stratification$COUNTRY %in% country_analysis,5])} else {area_s1 <- 0}
    if (analysis_stratum2 == T){area_s2 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==2 & stratification$COUNTRY %in% country_analysis,5])} else {area_s2 <- 0}
    if (analysis_stratum3 == T){area_s3 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==3 & stratification$COUNTRY %in% country_analysis,5])} else {area_s3 <- 0}
    if (analysis_stratum4 == T){area_s4 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==4 & stratification$COUNTRY %in% country_analysis,5])} else {area_s4 <- 0}
    if (analysis_stratum5 == T){area_s5 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==5 & stratification$COUNTRY %in% country_analysis,5])} else {area_s5 <- 0}
    if (analysis_stratum6 == T){area_s6 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==6 & stratification$COUNTRY %in% country_analysis,5])} else {area_s6 <- 0}

    peso_s1 <- area_s1/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s2 <- area_s2/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s3 <- area_s3/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s4 <- area_s4/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s5 <- area_s5/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s6 <- area_s6/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)

    i=1
    for (i in 1:length(res_table_cala2[,1])) {

        data2 <- ddd[ddd$YEAR == res_table_cala2[i,1] , ]

        if (analysis_stratum1 == T) {s1 <- data2[data2$MEAN_DEPTH >= depth[1,2] & data2$MEAN_DEPTH < depth[1,3], ]}
        if (analysis_stratum2 == T) {s2 <- data2[data2$MEAN_DEPTH >= depth[2,2] & data2$MEAN_DEPTH < depth[2,3], ]}
        if (analysis_stratum3 == T) {s3 <- data2[data2$MEAN_DEPTH >= depth[3,2] & data2$MEAN_DEPTH < depth[3,3], ]}
        if (analysis_stratum4 == T) {s4 <- data2[data2$MEAN_DEPTH >= depth[4,2] & data2$MEAN_DEPTH < depth[4,3], ]}
        if (analysis_stratum5 == T) {s5 <- data2[data2$MEAN_DEPTH >= depth[5,2] & data2$MEAN_DEPTH < depth[5,3], ]}
        if (analysis_stratum6 == T) {s6 <- data2[data2$MEAN_DEPTH >= depth[6,2] & data2$MEAN_DEPTH < depth[6,3], ]}

        if (analysis_stratum1 == T){
            s_nF <- sum(s1[!is.na(s1$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_nFM <- sum(s1[!is.na(s1$NB_FM),"NB_FM"])
            s_a <- sum(s1$SWEPT_AREA)
            res_table_calaF[i,2] <- s_nF/s_a
            res_table_calaFM[i,2] <- s_nFM/s_a
        } else {area_s1 = 0}

        if (analysis_stratum2 == T){
            s_nF <- sum(s2[!is.na(s2$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_nFM <- sum(s2[!is.na(s2$NB_FM),"NB_FM"])
            s_a <- sum(s2$SWEPT_AREA)
            res_table_calaF[i,3] <- s_nF/s_a
            res_table_calaFM[i,3] <- s_nFM/s_a
        } else {area_s2 = 0}

        if (analysis_stratum3 == T){
            s_nF <- sum(s3[!is.na(s3$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_nFM <- sum(s3[!is.na(s3$NB_FM),"NB_FM"])
            s_a <- sum(s3$SWEPT_AREA)
            res_table_calaF[i,4] <- s_nF/s_a
            res_table_calaFM[i,4] <- s_nFM/s_a
        } else {area_s3 = 0}

        if (analysis_stratum4 == T){
            s_nF <- sum(s4[!is.na(s4$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_nFM <- sum(s4[!is.na(s4$NB_FM),"NB_FM"])
            s_a <- sum(s4$SWEPT_AREA)
            res_table_calaF[i,5] <- s_nF/s_a
            res_table_calaFM[i,5] <- s_nFM/s_a
        } else {area_s4 = 0}

        if (analysis_stratum5 == T){
            #index computation
            s_nF <- sum(s5[!is.na(s5$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_nFM <- sum(s5[!is.na(s5$NB_FM),"NB_FM"])
            s_a <- sum(s5$SWEPT_AREA)
            res_table_calaF[i,6] <- s_nF/s_a
            res_table_calaFM[i,6] <- s_nFM/s_a
        } else {area_s5 = 0}

        if (analysis_stratum6 == T){
            #index computation
            s_nF <- sum(s6[!is.na(s6$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_nFM <- sum(s6[!is.na(s6$NB_FM),"NB_FM"])
            s_a <- sum(s6$SWEPT_AREA)
            res_table_calaF[i,7] <- s_nF/s_a
            res_table_calaFM[i,7] <- s_nFM/s_a
        } else {area_s6 = 0}

        positive_hauls[i] <- length(merge_TATB[merge_TATB$TOTAL_NUMBER_IN_THE_HAUL>0 & merge_TATB$YEAR == res_table_cala2[i,1],"TOTAL_NUMBER_IN_THE_HAUL"])

        sum_res_est_F <- c(res_table_calaF[i,2]*peso_s1,res_table_calaF[i,3]*peso_s2,res_table_calaF[i,4]*peso_s3,res_table_calaF[i,5]*peso_s4,res_table_calaF[i,6]*peso_s5,res_table_calaF[i,7]*peso_s6)
        sum_res_est_FM <- c(res_table_calaFM[i,2]*peso_s1,res_table_calaFM[i,3]*peso_s2,res_table_calaFM[i,4]*peso_s3,
                            res_table_calaFM[i,5]*peso_s4,res_table_calaFM[i,6]*peso_s5,res_table_calaFM[i,7]*peso_s6)

        res_table_calaF[i, 8]<- sum(sum_res_est_F[!is.na(sum_res_est_F)])# /sum(area_s1,area_s2,area_s3,area_s4,area_s5)
        res_table_calaFM[i, 8]<- sum(sum_res_est_FM[!is.na(sum_res_est_FM)])# /sum(area_s1,area_s2,area_s3,area_s4,area_s5)
        colnames(res_table_calaF) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices_F")
        colnames(res_table_calaFM) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices_FM")
        SR$year[i] <- res_table_calaF$year[i]
        SR$Indices_F[i] <- res_table_calaF$Indices_F[i]
        SR$Indices_FM[i] <- res_table_calaFM[i,8]
        SR$sr[i] <- SR$Indices_F[i] / SR$Indices_FM[i]
        # [ORIGINALE]:
        # SR$variance[i] <- sqrt(SR$sr[i]*(1-SR$sr[i]))/res_table_calaFM[i,8]

        # [MODIFICATO]:
        SR$variance[i] <- (SR$sr[i]*(1-SR$sr[i]))/res_table_calaFM[i,8]
    }

    timeseries <- SR


    # plot sex-ratio
    main <- paste(sspp,"_GSA",GSA,"_(Sex ratio)-Random_Stratified_Sampling_",depth_range[1],"-",depth_range[2], " m", sep="")
    main.lab <- paste(sspp," GSA",GSA," (Sex ratio)-RSS ",depth_range[1],"-",depth_range[2], " m", sep="")
    sd_sr <- sqrt(timeseries[!is.na(timeseries$variance),"variance"])
    max_index <- 1 + (max(sd_sr[!is.na(sd_sr)])*1.2)

    if (save){
        write.table(timeseries, paste(wd,"/output/",main,"_Timeseries.csv", sep=""), sep=";", row.names = F)

        jpeg(paste(wd,"/output/",main,"_Timeseries.jpg",sep=""), res = 300, width = 8, height = 7, units = 'in')
        par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
        plot(timeseries[,1],  timeseries[,4], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab="sex ratio (FF/(FF+MM))", main=main.lab) # ylim=c(0,max_index*1.2)
        lines(timeseries[,1], (timeseries[,4]-1.96*sqrt(timeseries[,5])), type="l",lty=2, col="red" )
        lines(timeseries[,1], (timeseries[,4]+1.96*sqrt(timeseries[,5])), type="l",lty=2, col="red" )
        legend("topright", c("Sex ratio"), lty=c(1,2), pch=c(16), col=c("black"))
        dev.off()
    } else {
        par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
        plot(timeseries[,1],  timeseries[,4], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab="sex ratio (FF/(FF+MM))", main=main.lab) # ylim=c(0,max_index*1.2)
        lines(timeseries[,1], (timeseries[,4]-1.96*sqrt(timeseries[,5])), type="l",lty=2, col="red" )
        lines(timeseries[,1], (timeseries[,4]+1.96*sqrt(timeseries[,5])), type="l",lty=2, col="red" )
        legend("topright", c("Sex ratio"), lty=c(1,2), pch=c(16), col=c("black"))
    }

return(timeseries)
}

#' Check dictionary (RoME)
#' @description
#' The function checks whether the values contained in specific fields are consistent with the allowed values of the dictionaries.
#'
#' @param ResultData data frame in MEDITS tables
#' @param Field field of the table to be checked
#' @param Values vector of the allowed values
#' @param year reference year for the analysis
#' @param wd working directory
#' @param suffix name of the log file
#' @param verbose boolean. If TRUE messages are prompted in the console
#' @note
#' This function is an internal routine based on \strong{RoME version 0.2.3}.
#' It is provided within \code{BioIndex} to ensure the package remains
#' functional and self-sufficient for data validation.
#' @return \code{TRUE} if the validation passes, \code{FALSE} otherwise.
#' @examples
#' data(TA)
#' check_dictionary(TA, Field = "COUNTRY", Values = c("ITA"), year = 2007)
#' @export
check_dictionary <- function(ResultData, Field, Values, year, wd=NA, suffix=NA, verbose=FALSE) {

  if (is.na(wd)) {
    wd <- tempdir()
    if (verbose) message(paste("Missing working directory. Results are saved in the temporary folder: ", wd))
  }

  log_dir <- file.path(wd, "Logfiles")
  if (!dir.exists(log_dir)) dir.create(log_dir, recursive = TRUE)
  
  if (missing(suffix) || is.na(suffix)) {
    suffix <- paste0(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"))
  }
  
  log_file <- file.path(log_dir, paste0("Logfile_", suffix, ".dat"))
  if (!file.exists(log_file)) file.create(log_file)

  write(paste0("\n----------- check dictionary for field: ", Field, " - ", year), file = log_file, append = TRUE)

  if (length(year) != 1 || is.na(year)) {
    stop("Argument 'year' must be a single non-NA value")
  }
  
  df <- ResultData[ResultData$YEAR == year, , drop = FALSE]
  if (nrow(df) == 0) {
     write(sprintf("No data found for year %s", year), file = log_file, append = TRUE)
     return(TRUE)
  }
  tf <- df$TYPE_OF_FILE[1]

  all_vals <- as.character(df[[Field]])
  Values_char <- as.character(Values)
  allowsNA <- any(is.na(Values))
  allowed_vals <- setdiff(Values_char, NA_character_)

  msgs_na <- character()
  if (!allowsNA) {
    na_idx <- which(is.na(all_vals))
    if (length(na_idx) > 0) {
      msgs_na <- sprintf("Haul %s: value not allowed for %s in %s (actual: NA)", df$HAUL_NUMBER[na_idx], Field, tf)
    }
  }

  empty_idx <- which(!is.na(all_vals) & all_vals == "")
  msgs_empty <- character()
  if (length(empty_idx) > 0) {
    msgs_empty <- switch(tf,
                         TA = sprintf("Haul %s: the field %s is empty in %s", df$HAUL_NUMBER[empty_idx], Field, tf),
                         TB = sprintf("Haul %s %s %s: the field %s is empty in %s", df$HAUL_NUMBER[empty_idx], df$GENUS[empty_idx], df$SPECIES[empty_idx], Field, tf),
                         TC = sprintf("Haul %s %s %s %s %s: the field %s is empty in %s", df$HAUL_NUMBER[empty_idx], df$GENUS[empty_idx], df$SPECIES[empty_idx], df$SEX[empty_idx], df$LENGTH_CLASS[empty_idx], Field, tf),
                         TE = sprintf("Haul %s %s %s %s %s: the field %s is empty in %s", df$HAUL_NUMBER[empty_idx], df$GENUS[empty_idx], df$SPECIES[empty_idx], df$SEX[empty_idx], df$LENGTH_CLASS[empty_idx], Field, tf),
                         TL = sprintf("Haul %s %s %s %s %s: the field %s is empty in %s", df$HAUL_NUMBER[empty_idx], df$GENUS[empty_idx], df$SPECIES[empty_idx], df$SEX[empty_idx], df$LENGTH_CLASS[empty_idx], Field, tf),
                         character()
    )
  }

  inv_idx <- which(!is.na(all_vals) & all_vals != "" & !(all_vals %in% allowed_vals))
  if (Field == "SEX") inv_idx <- inv_idx[all_vals[inv_idx] != "FALSE"]
  msgs_inv <- character()
  if (length(inv_idx) > 0) {
    msgs_inv <- switch(tf,
                       TA = sprintf("Haul %s: value '%s' not allowed for %s in %s", df$HAUL_NUMBER[inv_idx], all_vals[inv_idx], Field, tf),
                       TB = sprintf("Haul %s %s %s: value '%s' not allowed for %s in %s", df$HAUL_NUMBER[inv_idx], df$GENUS[inv_idx], df$SPECIES[inv_idx], all_vals[inv_idx], Field, tf),
                       TC = sprintf("Haul %s %s %s %s %s: value '%s' not allowed for %s in %s", df$HAUL_NUMBER[inv_idx], df$GENUS[inv_idx], df$SPECIES[inv_idx], df$SEX[inv_idx], df$LENGTH_CLASS[inv_idx], all_vals[inv_idx], Field, tf),
                       TE = sprintf("Haul %s %s %s %s %s: value '%s' not allowed for %s in %s", df$HAUL_NUMBER[inv_idx], df$GENUS[inv_idx], df$SPECIES[inv_idx], df$SEX[inv_idx], df$LENGTH_CLASS[inv_idx], all_vals[inv_idx], Field, tf),
                       TL = sprintf("Haul %s: value '%s' not allowed for %s in %s", df$HAUL_NUMBER[inv_idx], all_vals[inv_idx], Field, tf),
                       character()
    )
  }

  msgs <- c(msgs_na, msgs_empty, msgs_inv)
  if (length(msgs) == 0) {
    write(sprintf("No error occurred for field %s in %s", Field, tf), file = log_file, append = TRUE)
    return(TRUE)
  } else {
    write(msgs, file = log_file, append = TRUE)
    return(FALSE)
  }
}

#' Overlay mTATB and mTATC on GFCM spatial grid
#' @description
#' Overlays survey haul data onto a standard spatial grid (e.g., GFCM grid),
#' harmonizing biological information with a consistent geographic structure
#' for standardized spatial analysis.
#'
#' @param mTATB data frame of the merged TA and TB
#' @param mTATC data frame of the merged TA and TC
#' @param GSA reference GSA for the analysis
#' @param country reference countries for the analysis
#' @param wd working directory used to save results
#' @param save boolean. If TRUE the outputs are saved in the local folder
#' @param verbose boolean. If TRUE messages are prompted in the console
#' @importFrom terra wrap unwrap vect
#' @return A \code{list} or \code{data.frame} containing the merged biological data mapped to the spatial grid.
#' @examples
#' # Use internal data
#' data(TA)
#' data(TB)
#' data(TC)
#' # Create a merged dataset for GSA 10
#' m <- merge_TATBTC(TA[TA$AREA==10,], TB[TB$AREA==10,], TC[TC$AREA==10,], species="MERLMER")
#' mTATB <- m[[1]]
#' mTATC <- m[[2]]
#' overlayGrid(mTATB, mTATC, GSA=10, save=FALSE)
#' @export

overlayGrid <- function(mTATB, mTATC, GSA=NA, country="all", wd=NA, save=TRUE, verbose=FALSE) {
    if (is.na(wd)) { wd <- tempdir() }

    if (FALSE) {
        verbose=TRUE
        save=TRUE
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        country="all"

        # load("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\- Pacchetto R BioIndex -\\BioIndex\\data\\TA_cols.rda")
        # load("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\- Pacchetto R BioIndex -\\BioIndex\\data\\TB_cols.rda")
        # load("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\- Pacchetto R BioIndex -\\BioIndex\\data\\TC_cols.rda")
        species <- "MERLMER"

        m <- merge_TATBTC(ta, tb, tc, species="MERLMER", wd=wd, verbose=TRUE)

        mTATB <- m[[1]]
        mTATC <- m[[2]]

        overlayGrid(mTATB, mTATC, GSA=NA, country="all", wd=wd, save=TRUE, verbose=FALSE)

    }


    if (verbose){
        message("\n########################")
        message("spatial metaDB preparation")
        message("########################")
        message("")
    }

    countries <- unique(mTATB[!is.na(mTATB$COUNTRY),"COUNTRY"])
    l_country <- length(unique(mTATB$COUNTRY))

    if (l_country==1){
        check_country ="Y"
        country_analysis <- countries
    } else {
        if (any(country %in% "all")) {
            country_analysis <<- countries
        } else {
            country_analysis <<- country
        }
    }

    mTATB <- mTATB[mTATB$COUNTRY %in% country_analysis,  ]
    mTATC <- mTATC[mTATC$COUNTRY %in% country_analysis,  ]

    cgpmgrid <- terra::unwrap(BioIndex::cgpmgrid)

    metaDB <- mTATB
    if (any(is.na(GSA))) {
        GSA <- unique(metaDB$GSA)[1]
    }
    metaDB <- metaDB[metaDB$GSA %in% GSA, ]
    GENERE <- as.character(unique(metaDB$GENUS)[unique(metaDB$GENUS) != -1])
    SPECIE <- as.character(unique(metaDB$SPECIES)[unique(metaDB$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")

    # Overlay metaDB with the grid
    dft <- data.frame(metaDB[, c("MEAN_LONGITUDE_DEC","MEAN_LATITUDE_DEC")])
    vt <- terra::vect(dft, geom=c("MEAN_LONGITUDE_DEC", "MEAN_LATITUDE_DEC"), crs="+proj=longlat", keepgeom=FALSE)
    overlay <- terra::extract(x=cgpmgrid, y=vt)
    dupl <- which(duplicated(overlay$id.y))
    if (length(dupl)>0){
        overlay <- overlay[-(which(duplicated(overlay$id.y))), ]
    }

    metaDBnew_georef <- data.frame(metaDB, x_center= overlay$gfcm.cen_5,y_center=overlay$gfcm.cen_6 ,cgpmgrid_id=overlay$GFCM_ID)

    centroidi <- terra::unwrap(BioIndex::centroidi)

    mTATB_geo <- metaDBnew_georef
    if(verbose){
        message("Catch metaDB successfully created.")
    }
    if (save){

        write.table(metaDBnew_georef, paste(wd, "/output/",sspp," - allGSAs_metaDB_catch in GRID.csv",sep=""), sep=";", row.names=F)
        if(verbose){
            message(paste("Catch metaDB saved in the output folder"))
            message("")
        }
    }
    if (any(is.na(metaDBnew_georef$cgpmgrid_id))) {
        if (verbose) {
            message("One or more hauls are out of the reference grid area. Data from these hauls should be excluded by statial analysis.")
            message("")
        }
    }

    # Read metaDB file and transform into spatial object
    metaDB <- mTATC
    metaDB <- metaDB[metaDB$GSA %in% GSA, ]
    overlay <- terra::extract(cgpmgrid, data.frame(metaDB$MEAN_LONGITUDE_DEC, metaDB$MEAN_LATITUDE_DEC))

    dupl <- NULL
    dupl <- which(duplicated(overlay$id.y))
    if (length(dupl)>0){
        overlay <- overlay[-(which(duplicated(overlay$id.y))), ]
    }
    metaDBnew_georef <- data.frame(metaDB, x_center= overlay$gfcm.cen_5,y_center=overlay$gfcm.cen_6 ,cgpmgrid_id=overlay$GFCM_ID)

    mTATC_geo <- metaDBnew_georef
    if(verbose){
        message("Biological metaDB successfully created.")
    }
    if(save){

        write.table(metaDBnew_georef, paste(wd, "/output/",sspp," - allGSAs_metaDB_biological in GRID.csv",sep=""), sep=";", row.names=F)
        if(verbose){
            message(paste("Biological metaDB saved in the output folder"))
            message("")
        }
    }
    if (any(is.na(metaDBnew_georef$cgpmgrid_id))) {
        if (verbose){
            message("One or more hauls are out of the reference grid area. Data from these hauls should be excluded by statial analysis.")
            message("")
        }
    }
    return(list(mTATB_geo,mTATC_geo))
}

#' Merge TA and TC tables (haul-level lengthâ€“frequency data)
#'
#' @description
#' Links haul metadata (TA) with length-frequency samples (TC),
#' enabling the analysis of population demographic structure and the
#' reconstruction of length-frequency distributions (LFD).
#' The function keeps only valid hauls, runs all relevant quality checks
#' (times, positions, wing opening, haul/date consistency) using internal
#' validation functions (based on \code{RoME} version 0.2.3),
#' computes swept area, depth stratum, raising factors,
#' abundance/biomass indicators, andâ€”optionallyâ€”writes the merged data.
#'
#' @param ta   A `data.frame`/`data.table` containing a full TA dataset
#'             (columns listed in `BioIndex::TA_cols`).
#' @param tc   A `data.frame`/`data.table` containing a full TC dataset
#'             (columns listed in `BioIndex::TC_cols`).
#' @param species Character, MEDITS 7-letter code (e.g. `"ARISFOL"`).
#'                The first 4 characters are interpreted as *GENUS* and
#'                the last 3 as *SPECIES*.
#' @param country Character vector of ISO-3 country codes to keep; use
#'                `"all"` (default) to keep every country present in `ta`.
#' @param strata  Depth-stratum reference table; default is
#'                `BioIndex::strata_scheme`.
#' @param wd      Working directory.  If `NA` (the default when
#'                `save=FALSE`) no files are written.
#' @param save    Logical.  If `TRUE`, writes
#'                `"mergeTATC_<GENUS><SPECIES>.csv"` to
#'                `file.path(wd, "output")`.
#' @param verbose Logical; if `TRUE` (default) prints progress messages.
#'
#' @return A `data.frame` in which each row represents one length-class
#'         (and sex/maturity) sampled in a haul, including:
#'         * **TA** metadata
#'         * **TC** length-frequency counts and weights
#'         * Calculated fields: mean lat/lon, swept area, depth stratum,
#'           raising factor, density (`N_h`, `N_km2`) and biomass
#'           (`kg_h`, `kg_km2`)
#'
#' @details
#' The code is identical to the official BioIndex routine.
#'
#' \strong{Data Validation:}
#' The function performs syntactic data validation using internal
#' implementations of the validation routines (based on \code{RoME} v0.2.3).
#'
#' \strong{Optimisations:}
#' Includes two micro-optimisations: vectorised handling of missing `"NA NA"` entries
#' and a single depth-stratum loop. Numerical results remain unchanged.
#'
#' @note
#' This version of \code{BioIndex} uses internal validation functions based on
#' \strong{RoME version 0.2.3}. The package is fully self-sufficient and does
#' not require external validation packages to be installed.
#'
#' @seealso [BioIndex] package documentation.
#' @importFrom hms hms
#' @importFrom utils write.table
#' @examples
#' # Use internal data
#' data("TA", package = "BioIndex")
#' data("TC", package = "BioIndex")
#' m_tc <- merge_TATC(
#'   ta       = TA,
#'   tc       = TC,
#'   species  = "ARISFOL",
#'   country  = "ESP",
#'   wd       = tempdir(),
#'   save=FALSE
#' )
#' head(m_tc)
#' @export
merge_TATC <- function(ta, tc,
                       species,
                       country = "all",
                       strata  = BioIndex::strata_scheme,
                       wd      = NA,
                       save    = TRUE,
                       verbose = TRUE) {

    if (is.na(wd)) { wd <- tempdir() }
    ## Working-directory check ------------------------------------------- ##
    if (is.na(wd) & save) {
        save <- FALSE
        if (verbose)
            message("Missing working directory. Results will not be saved locally.")
    }

    strata_scheme <- strata   # local alias

    ## Column templates -------------------------------------------------- ##
    TA_cols <- BioIndex::TA_cols
    TC_cols <- BioIndex::TC_cols   # TB template removed

    ## Species code parsing --------------------------------------------- ##
    species[2] <- substr(species, 5, 7)
    species[1] <- substr(species[1], 1, 4)

    ## Input data -------------------------------------------------------- ##
    TA <- ta
    TC <- tc

    ## COUNTRY filter (TB removed) -------------------------------------- ##
    TA_country <- sort(unique(TA$COUNTRY))
    TC_country <- sort(unique(TC$COUNTRY))
    for (cn in TA_country)
        if (!(cn %in% TC_country))
            warning("Country ", cn, " not present in TC file.")

    country_analysis <- if (length(TA_country) == 1) TA_country else
        if (any(country %in% "all")) TA_country else country
    TA <- TA[TA$COUNTRY %in% country_analysis, ]
    TC <- TC[TC$COUNTRY %in% country_analysis, ]

    ## Unique haul IDs --------------------------------------------------- ##
    uid <- function(df) {
        if (nrow(df) == 0) return(character(0))
        paste(df$AREA, df$COUNTRY, df$YEAR, "_",
              df$VESSEL, df$MONTH, df$DAY, "_",
              df$HAUL_NUMBER, sep = "")
    }
    TA$id <- uid(TA); TC$id <- uid(TC)
    invalid_ids <- TA$id[TA$VALIDITY == "I"]

    ## Data validation: local BioIndex routines (based on RoME 0.2.3)     ##
    suffix <- paste(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"), sep = "")
    for (yy in sort(unique(TA$YEAR))) {
        TAy <- TA[TA$YEAR == yy, ]; TCy <- TC[TC$YEAR == yy, ]
        if (!check_dictionary(TAy,"SHOOTING_TIME",0:2400,yy,paste0(wd,"/output/"),suffix))
            stop("SHOOTING_TIME out of range.")
        if (!check_dictionary(TAy,"HAULING_TIME",0:2400,yy,paste0(wd,"/output/"),suffix))
            stop("HAULING_TIME out of range.")
        if (!check_dictionary(TAy,"WING_OPENING",c(30,50:250),yy,paste0(wd,"/output/"),suffix))
            stop("WING_OPENING out of range.")
        if (!check_numeric_range(TAy,"SHOOTING_LATITUDE",c(3020,4730),yy,paste0(wd,"/output/"),suffix))
            stop("SHOOTING_LATITUDE out of range.")
        if (!check_numeric_range(TAy,"HAULING_LATITUDE",c(3020,4730),yy,paste0(wd,"/output/"),suffix))
            stop("HAULING_LATITUDE out of range.")
        if (!check_numeric_range(TAy,"SHOOTING_LONGITUDE",c(0,4200),yy,paste0(wd,"/output/"),suffix))
            stop("SHOOTING_LONGITUDE out of range.")
        if (!check_numeric_range(TAy,"HAULING_LONGITUDE",c(0,4200),yy,paste0(wd,"/output/"),suffix))
            stop("HAULING_LONGITUDE out of range.")
        if (!check_date_haul(TAy, TCy, yy, paste0(wd,"/output/"), suffix))
            stop("Date in TC not consistent with TA.")
    }

    ## Preparation for merge -------------------------------------------- ##
    names(TA)[names(TA)=="AREA"] <- "GSA"
    names(TC)[names(TC)=="AREA"] <- "GSA"

    TA_ <- TA[!TA$id %in% invalid_ids, TA_cols]

    TC_ <- TC[!TC$id %in% invalid_ids &
                  TC$GENUS == species[1] & TC$SPECIES == species[2], TC_cols]

    ## MERGE TA â€“ TC  (unchanged logic) --------------------------------- ##
    if (verbose) message("- Merging TA-TC files")
    merge_TATC <- merge(TA_, TC_, by = "id", all.x = TRUE, all.y = TRUE)
    merge_TATC$MEDITS_CODE <- paste(merge_TATC$GENUS, merge_TATC$SPECIES)

    ## Clean â€œNA NAâ€ rows (vectorised) ---------------------------------- ##
    bad <- merge_TATC$MEDITS_CODE == "NA NA"
    if (any(bad)) {
        merge_TATC$MEDITS_CODE[bad] <- -1
        merge_TATC[bad,
                   c("GENUS","SPECIES","SEX","LENGTH_CLASSES_CODE",
                     "WEIGHT_OF_THE_FRACTION","WEIGHT_OF_THE_SAMPLE_MEASURED",
                     "MATURITY","MATSUB","LENGTH_CLASS")] <- -1
        merge_TATC$WEIGHT_OF_THE_FRACTION[bad] <- 0
        merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED[bad] <- 0
        merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE[bad] <- 0
    }

    coord <- convert_coordinates(merge_TATC)
    merge_TATC$MEAN_LATITUDE     <- (merge_TATC$SHOOTING_LATITUDE +
                                         merge_TATC$HAULING_LATITUDE) / 2
    merge_TATC$MEAN_LATITUDE_DEC <- (coord$lat_start + coord$lat_end) / 2
    merge_TATC$MEAN_LONGITUDE    <- (merge_TATC$SHOOTING_LONGITUDE +
                                         merge_TATC$HAULING_LONGITUDE) / 2
    merge_TATC$MEAN_LONGITUDE_DEC<- (coord$lon_start + coord$lon_end) / 2
    merge_TATC$MEAN_DEPTH <- (merge_TATC$SHOOTING_DEPTH + merge_TATC$HAULING_DEPTH) / 2
    merge_TATC$SWEPT_AREA <- merge_TATC$DISTANCE * merge_TATC$WING_OPENING / 1e7

    strata_sub <- strata_scheme[
        strata_scheme$GSA == unique(merge_TATC$GSA) &
            strata_scheme$COUNTRY %in% unique(merge_TATC$COUNTRY), ]
    merge_TATC$STRATUM_CODE <- NA
    for (k in seq_len(nrow(strata_sub))) {
        lo <- strata_sub[k,4]; hi <- strata_sub[k,5]; cd <- strata_sub[k,3]
        sel <- if (cd==1)
            merge_TATC$MEAN_DEPTH >= lo & merge_TATC$MEAN_DEPTH <= hi
        else
            merge_TATC$MEAN_DEPTH  > lo & merge_TATC$MEAN_DEPTH <= hi
        merge_TATC$STRATUM_CODE[sel] <- cd
    }

    ## Duration & density (original lines) ------------------------------ ##
    hh  <- ifelse(nchar(merge_TATC$SHOOTING_TIME)==4,
                  substr(merge_TATC$SHOOTING_TIME,1,2),
                  substr(merge_TATC$SHOOTING_TIME,1,1))
    mm  <- ifelse(nchar(merge_TATC$SHOOTING_TIME)==4,
                  substr(merge_TATC$SHOOTING_TIME,3,4),
                  substr(merge_TATC$SHOOTING_TIME,2,3))
    hh2 <- ifelse(nchar(merge_TATC$HAULING_TIME)==4,
                  substr(merge_TATC$HAULING_TIME,1,2),
                  substr(merge_TATC$HAULING_TIME,1,1))
    mm2 <- ifelse(nchar(merge_TATC$HAULING_TIME)==4,
                  substr(merge_TATC$HAULING_TIME,3,4),
                  substr(merge_TATC$HAULING_TIME,2,3))

    h1 <- hms(rep(0, length(hh)),  as.numeric(mm),  as.numeric(hh))
    h2 <- hms(rep(0, length(hh2)), as.numeric(mm2), as.numeric(hh2))
    dur <- as.numeric(h2 - h1) / 3600

    merge_TATC$RAISING_FACTOR <- 0
    pos <- merge_TATC$WEIGHT_OF_THE_FRACTION > 0
    merge_TATC$RAISING_FACTOR[pos] <-
        pmax(merge_TATC$WEIGHT_OF_THE_FRACTION[pos] /
                 merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED[pos], 1)

    merge_TATC$N_h    <- merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE *
        merge_TATC$RAISING_FACTOR / dur
    merge_TATC$N_km2  <- merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE *
        merge_TATC$RAISING_FACTOR / merge_TATC$SWEPT_AREA
    merge_TATC$kg_h   <- merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED / 1000 / dur
    merge_TATC$kg_km2 <- merge_TATC$WEIGHT_OF_THE_SAMPLE_MEASURED / 1000 / merge_TATC$SWEPT_AREA

    ## Save & return ----------------------------------------------------- ##
    if (save)
        write.table(merge_TATC,
                    file = file.path(wd, "output",
                                     paste0("mergeTATC_", species[1], species[2], ".csv")),
                    sep = ";", row.names = FALSE)
    if (verbose) message("TA-TC merge completed")

    merge_TATC
}

#' Estimation of LW relationship
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @param TE MEDITS or MEDITS-like TE table
#' @param sp species RUBIN code (MEDITS format, e.g. "MERLMER")
#' @param GEAR type of gear reported in the corresponding TA file
#' @param GSA reference GSA for the analysis
#' @param country reference country
#' @param n_records minimum number of records to perform the analysis
#' @param wd path of the working directory
#' @param save boolean. If TRUE the outputs are saved in the local folder
#' @importFrom stats lm predict coef coefficients nls
#' @importFrom grDevices jpeg dev.off
#' @return A \code{data.frame} containing the length-weight relationship parameters specifically for females.
LWf <- function(TE, sp, GEAR, GSA, country=NA, n_records = 10, wd = NA, save = TRUE) {
    if (is.na(wd)) {
        wd <- tempdir()
    }
    
    if (FALSE) {
        sp <- "MERLMER"
        GEAR <- "GOC73"
        country <- country_analysis
        n_records <- 5

        TE=TE
        sp=sspp
        GEAR=GEAR
        GSA=GSA
        country = country_analysis
        years = n_years
        n_records=10
    }

    genus <- substr(unique(sp), 1, 4)
    species <- substr(unique(sp), 5, 7)

    sspp <- sp
    TE$COUNTRY <- as.character(TE$COUNTRY)
    if (all(is.na(country))) {
        country <- as.character(unique(TE$COUNTRY))
        TE <- TE[TE$AREA == GSA & TE$COUNTRY %in% country, ]
    } else {
        TE <- TE[TE$AREA == GSA & TE$COUNTRY %in% country, ]
    }

    if (unique(GEAR)[1] %in% c("BEAMT", "TBB")) {
        { #####  INDIVIDUAL BRUT WEIGHT - start #####
            data_iw <- TE[TE$GENUS == genus & TE$SPECIES == species &
                              as.character(TE$INDIVIDUAL_BRUT_WEIGHT) != "ND", ]
            data_iw$INDIVIDUAL_BRUT_WEIGHT <- as.numeric(data_iw$INDIVIDUAL_BRUT_WEIGHT)

            data <- data_iw
            colnames(data)[which(colnames(data) == "INDIVIDUAL_BRUT_WEIGHT")] <- "INDIVIDUAL_WEIGHT"
            var <- "Individual brut weight"

            if (nrow(data) > 1) {
                sexes <- unique(data$SEX)

                ###########   LW directory creation ###########################
                oldwd <- getwd()
                on.exit(setwd(oldwd), add = TRUE)
                
                if (save) {
                    if (!dir.exists(file.path(wd, "output"))) {
                        dir.create(file.path(wd, "output"), showWarnings = FALSE)
                    }
                    if (!dir.exists(file.path(wd, "output", "LW"))) {
                        dir.create(file.path(wd, "output", "LW"), showWarnings = FALSE)
                    }
                }
                
                LW_table_tot <- list()
                sex <- "M"
                for (sex in sexes) {
                    years <- unique(data$YEAR)
                    spdf <- data.frame(matrix(NA, ncol = 2, nrow = 1))
                    colnames(spdf) <- c("genus", "species")
                    spdf[, "genus"] <- substr(sspp, 1, 4)
                    spdf[, "species"] <- substr(sspp, 5, 7)

                    LW_table <- data.frame(matrix(nrow = length(years), ncol = 5))
                    colnames(LW_table) <- c("species", "sex", "a", "b", "year")
                    LW_table$species <- sspp
                    LW_table$sex <- sex


                    for (year in years) {

                        ###########   sex directory creation ###########################
                        if (save) {
                            if (!dir.exists(file.path(wd, "output", "LW", sex))) {
                                dir.create(file.path(wd, "output", "LW", sex), showWarnings = FALSE, recursive = TRUE)
                            }
                        }
                        
                        #################################################################
                        datay <- data[data$SEX == sex & data$YEAR == year & data$GENUS == genus & data$SPECIES == species, ]

                        # LC definition --------------------------------------------
                        LC_range <- c(min(datay$LENGTH_CLASS, na.rm = TRUE), max(datay$LENGTH_CLASS, na.rm = TRUE))

                        class_code <- unique(datay[datay$GENUS != -1, "LENGTH_CLASSES_CODE"])
                        if (class_code == "m") {
                            cl.size <- 1
                        } else {
                            if (class_code == "0") {
                                cl.size <- 5
                            } else {
                                if (class_code == "1") {
                                    cl.size <- 10
                                }
                            }
                        }

                        df <- data.frame(X = datay$LENGTH_CLASS, Y = datay$INDIVIDUAL_WEIGHT)
                        if (nrow(df) > n_records) {
                            model.lm <- lm(log(Y) ~ log(X), data = df)
                            a <- model.lm$coefficients[1]
                            b <- model.lm$coefficients[2]

                            if (!inherits(try(nls(Y ~ a * (X^b), data = df, start = list(a = exp(a), b = b)), silent = TRUE), "try-error")) {
                                mod <- nls(Y ~ a * (X^b), data = df, start = list(a = exp(a), b = b))
                                lc_list <- seq(LC_range[1], LC_range[2], cl.size)
                                prediction <- data.frame(matrix(nrow = length(lc_list), ncol = 2))
                                colnames(prediction) <- c("X", "pred")
                                prediction$X <- lc_list
                                prediction$pred <- predict(mod, newdata = prediction)

                                if (save) {
                                    jpeg(filename  = file.path(wd, "output", "LW", sex, paste0(sspp, "-", sex, "-", year, "-LW_Brut_weight.jpg")), width = 20, height = 15, bg = "white", units = "cm", res = 200)
                                    plot(df, pch = 16, cex = .5, main = paste(sspp, sex, sep = " - "), xlab = "length (mm)", ylab = "weight (g)")
                                    lines(prediction$X, prediction$pred, col = "red", lwd = 2)
                                    legend("topleft", c(paste("a = ", round(coef(mod)[[1]], 6)), paste("b = ", round(coef(mod)[[2]], 4))))
                                    dev.off()
                                }

                                LW_table[which(years == year), 3] <- coef(mod)[[1]]
                                LW_table[which(years == year), 4] <- coef(mod)[[2]]
                                LW_table[which(years == year), 5] <- year
                            } else {
                                LW_table[which(years == year), 3] <- NA
                                LW_table[which(years == year), 4] <- NA
                                LW_table[which(years == year), 5] <- year
                                warning(paste("Not enougth data to estimate the length-weigth parameters for ", sspp, " - sex = ", sex, "\n", sep = ""))
                            } # try mod
                        } else {
                            LW_table[which(years == year), 3] <- NA
                            LW_table[which(years == year), 4] <- NA
                            LW_table[which(years == year), 5] <- year
                            warning(paste("Not enougth data to estimate the length-weigth parameters for ", sspp, " - sex = ", sex, "\n", sep = ""))
                        } # nrow di df > di n_records
                    } # years

                    LW_table_tot[[sex]] <- LW_table
                    if (save) {
                        write.table(LW_table, file.path(wd, "output", "LW", sex, paste0("LW_", sspp, "_", sex, "_Brut_Weight.csv")), sep = ";", row.names = FALSE)
                    }
                }
                LW_table_final_brut <- do.call(rbind, LW_table_tot)
                LW_table_final_brut$notes <- "Brut weight"
                if (save) {
                    write.table(LW_table_final_brut, file.path(wd, "output", "LW", paste0("LW_", sspp, "_Brut_Weight.csv")), sep = ";", row.names = FALSE)
                }
            } else {
                message("Not enougth data for length-weight analysis")
            }
        } #####  INDIVIDUAL BRUT WEIGHT - end #####


        { #####  CLEAN WEIGHT - start #####

            if (is.character(TE$CLEAN_WEIGHT)) {
                data_cw <- TE[TE$GENUS == genus & TE$SPECIES == species &
                                  as.character(TE$CLEAN_WEIGHT != "ND"), ]
                data_cw$CLEAN_WEIGHT <- as.numeric(data_cw$CLEAN_WEIGHT)
            } else if (is.numeric(TE$CLEAN_WEIGHT)) {
                data_cw <- TE[TE$GENUS == genus & TE$SPECIES == species &
                                  !is.na(TE$CLEAN_WEIGHT), ]
                data_cw$CLEAN_WEIGHT <- as.numeric(data_cw$CLEAN_WEIGHT)
            }


            data <- data_cw
            colnames(data)[which(colnames(data) == "CLEAN_WEIGHT")] <- "INDIVIDUAL_WEIGHT"
            var <- "Clean weight"

            if (nrow(data) > 1) {
                sexes <- unique(data$SEX)

                ###########   LW directory creation ###########################
                if (save) {
                    if (!dir.exists(file.path(wd, "output", "LW"))) {
                        dir.create(file.path(wd, "output", "LW"), showWarnings = FALSE, recursive = TRUE)
                    }
                }
                
                sex="F"
                for (sex in sexes) {
                    years <- unique(data$YEAR)
                    spdf <- data.frame(matrix(NA, ncol = 2, nrow = 1))
                    colnames(spdf) <- c("genus", "species")
                    spdf[, "genus"] <- substr(sspp, 1, 4)
                    spdf[, "species"] <- substr(sspp, 5, 7)

                    LW_table <- data.frame(matrix(nrow = length(years), ncol = 5))
                    colnames(LW_table) <- c("species", "sex", "a", "b", "year")
                    LW_table$species <- sspp
                    LW_table$sex <- sex

                    year <- years[1]
                    for (year in years) {

                        ###########   sex directory creation ###########################
                        if (save) {
                            if (!dir.exists(file.path(wd, "output", "LW", sex))) {
                                dir.create(file.path(wd, "output", "LW", sex), showWarnings = FALSE, recursive = TRUE)
                            }
                        }
                        
                        #################################################################
                        datay <- data[data$SEX == sex & data$YEAR == year & data$GENUS == genus & data$SPECIES == species, ]

                        # LC definition --------------------------------------------
                        LC_range <- c(min(datay$LENGTH_CLASS, na.rm = TRUE), max(datay$LENGTH_CLASS, na.rm = TRUE))

                        class_code <- unique(datay[datay$GENUS != -1, "LENGTH_CLASSES_CODE"])
                        if (class_code == "m") {
                            cl.size <- 1
                        } else {
                            if (class_code == "0") {
                                cl.size <- 5
                            } else {
                                if (class_code == "1") {
                                    cl.size <- 10
                                }
                            }
                        }

                        df <- data.frame(X = datay$LENGTH_CLASS, Y = datay$INDIVIDUAL_WEIGHT)
                        if (nrow(df) > n_records) {
                            model.lm <- lm(log(Y) ~ log(X), data = df)
                            a <- model.lm$coefficients[1]
                            b <- model.lm$coefficients[2]

                            if (!inherits(try(nls(Y ~ a * (X^b), data = df, start = list(a = exp(a), b = b)), silent = TRUE), "try-error")) {
                                mod <- nls(Y ~ a * (X^b), data = df, start = list(a = exp(a), b = b))

                                lc_list <- seq(LC_range[1], LC_range[2], cl.size)
                                prediction <- data.frame(matrix(nrow = length(lc_list), ncol = 2))
                                colnames(prediction) <- c("X", "pred")
                                prediction$X <- lc_list
                                prediction$pred <- predict(mod, newdata = prediction)

                                if (save) {
                                    jpeg(filename  = file.path(wd, "output", "LW", sex, paste0(sspp, "-", sex, "-", year, "-LW_Clean_Weight.jpg")), width = 20, height = 15, bg = "white", units = "cm", res = 200)
                                    plot(df, pch = 16, cex = .5, main = paste(sspp, sex, sep = " - "), xlab = "length (mm)", ylab = "weight (g)")
                                    lines(prediction$X, prediction$pred, col = "red", lwd = 2)
                                    legend("topleft", c(paste("a = ", round(coef(mod)[[1]], 6)), paste("b = ", round(coef(mod)[[2]], 4))))
                                    dev.off()
                                }

                                LW_table[which(years == year), 3] <- coef(mod)[[1]]
                                LW_table[which(years == year), 4] <- coef(mod)[[2]]
                                LW_table[which(years == year), 5] <- year
                            } else {
                                LW_table[which(years == year), 3] <- NA
                                LW_table[which(years == year), 4] <- NA
                                LW_table[which(years == year), 5] <- year
                                warning(paste("Not enougth data to estimate the length-weigth parameters for ", sspp, " - sex = ", sex, "\n", sep = ""))
                            } # try mod
                        } else {
                            LW_table[which(years == year), 3] <- NA
                            LW_table[which(years == year), 4] <- NA
                            LW_table[which(years == year), 5] <- year
                            warning(paste("Not enougth data to estimate the length-weigth parameters for ", sspp, " - sex = ", sex, "\n", sep = ""))
                        } # nrow di df > di n_records
                    } # years

                    LW_table_tot[[sex]] <- LW_table
                    if (save) {
                        write.table(LW_table, file.path(wd, "output", "LW", sex, paste0("LW_", sspp, "_", sex, "_Clean_Weight.csv")), sep = ";", row.names = FALSE)
                    }
                }
                LW_table_final_clean <- do.call(rbind, LW_table_tot)
                LW_table_final_clean$notes <- "Clean weight"
                if (save) {
                    write.table(LW_table_final_clean, file.path(wd, "output", "LW", paste0("LW_", sspp, "_Clean_Weight.csv")), sep = ";", row.names = FALSE)
                }

                if (exists("LW_table_final_brut") & exists("LW_table_final_clean")){
                    table_final <- rbind(LW_table_final_brut,LW_table_final_clean)
                } else if (!(exists("LW_table_final_brut") & exists("LW_table_final_clean"))) {
                    table_final <- LW_table_final_clean
                } else if ((exists("LW_table_final_brut") & !exists("LW_table_final_clean"))) {
                    table_final <- LW_table_final_brut
                } else {
                    table_final <- NULL
                }
                if (save && !is.null(table_final)) {
                    write.table(table_final, file.path(wd, "output", "LW", paste0("LW_", sspp, ".csv")), sep = ";", row.names = FALSE)
                }
                return(LW_table)
            } else {
                message("Not enougth data for length-weight analysis")
            }

        } #### CLEAN WEIGHT - end
    } else if (unique(GEAR)[1] %in% c("GOC73", "TRAWL")) {
        data <- TE[TE$GENUS == genus & TE$SPECIES == species &
                       as.character(TE$INDIVIDUAL_WEIGHT) != "ND", ]

        if (nrow(data) > 1) {
            sexes <- unique(data$SEX)

            ###########   LW directory creation ###########################
            if (save) {
                if (!dir.exists(file.path(wd, "output", "LW"))) {
                    dir.create(file.path(wd, "output", "LW"), showWarnings = FALSE, recursive = TRUE)
                }
            }
            
            sex="I"
            for (sex in sexes) {
                years <- unique(data$YEAR)
                spdf <- data.frame(matrix(NA, ncol = 2, nrow = 1))
                colnames(spdf) <- c("genus", "species")
                spdf[, "genus"] <- substr(sspp, 1, 4)
                spdf[, "species"] <- substr(sspp, 5, 7)

                LW_table <- data.frame(matrix(nrow = length(years), ncol = 5))
                colnames(LW_table) <- c("species", "sex", "a", "b", "year")
                LW_table$species <- sspp
                LW_table$sex <- sex
                year <- 2012
                for (year in years) {

                    ###########   sex directory creation ###########################
                    if (save) {
                        if (!dir.exists(file.path(wd, "output", "LW", sex))) {
                            dir.create(file.path(wd, "output", "LW", sex), showWarnings = FALSE, recursive = TRUE)
                        }
                    }
                    
                    #################################################################
                    datay <- data[data$SEX == sex & data$YEAR == year & data$GENUS == genus & data$SPECIES == species, ]

                    # LC definition --------------------------------------------
                    LC_range <- c(min(datay$LENGTH_CLASS, na.rm = TRUE), max(datay$LENGTH_CLASS, na.rm = TRUE))

                    class_code <- unique(datay[datay$GENUS != -1, "LENGTH_CLASSES_CODE"])
                    if (class_code == "m") {
                        cl.size <- 1
                    } else {
                        if (class_code == "0") {
                            cl.size <- 5
                        } else {
                            if (class_code == "1") {
                                cl.size <- 10
                            }
                        }
                    }

                    df <- data.frame(X = datay$LENGTH_CLASS, Y = datay$INDIVIDUAL_WEIGHT)
                    if (nrow(df) > n_records) {
                        model.lm <- lm(log(Y) ~ log(X), data = df)
                        a <- model.lm$coefficients[1]
                        b <- model.lm$coefficients[2]

                        if (!inherits(try(nls(Y ~ a * (X^b), data = df, start = list(a = exp(a), b = b)), silent = TRUE), "try-error")) {
                            mod <- nls(Y ~ a * (X^b), data = df, start = list(a = exp(a), b = b))

                            lc_list <- seq(LC_range[1], LC_range[2], cl.size)
                            prediction <- data.frame(matrix(nrow = length(lc_list), ncol = 2))
                            colnames(prediction) <- c("X", "pred")
                            prediction$X <- lc_list
                            prediction$pred <- predict(mod, newdata = prediction)

                            if (save) {
                                jpeg(filename  = file.path(wd, "output", "LW", sex, paste0(sspp, "-", sex, "-", year, "-LW.jpg")), width = 20, height = 15, bg = "white", units = "cm", res = 200)
                                plot(df, pch = 16, cex = .5, main = paste(sspp, sex, sep = " - "), xlab = "length (mm)", ylab = "weight (g)")
                                lines(prediction$X, prediction$pred, col = "red", lwd = 2)
                                legend("topleft", c(paste("a = ", round(coef(mod)[[1]], 6)), paste("b = ", round(coef(mod)[[2]], 4))))
                                dev.off()
                            }

                            LW_table[which(years == year), 3] <- coef(mod)[[1]]
                            LW_table[which(years == year), 4] <- coef(mod)[[2]]
                            LW_table[which(years == year), 5] <- year
                        } else {
                            LW_table[which(years == year), 3] <- NA
                            LW_table[which(years == year), 4] <- NA
                            LW_table[which(years == year), 5] <- year
                            warning(paste("Not enougth data to estimate the length-weigth parameters for ", sspp, " - sex = ", sex, "\n", sep = ""))
                        } # try mod
                    } else {
                        LW_table[which(years == year), 3] <- NA
                        LW_table[which(years == year), 4] <- NA
                        LW_table[which(years == year), 5] <- year
                        warning(paste("Not enougth data to estimate the length-weigth parameters for ", sspp, " - sex = ", sex, "\n", sep = ""))
                    } # nrow di df > di n_records
                } # years
                if (save) {
                    write.table(LW_table, file.path(wd, "output", "LW", sex, paste0("LW_", sspp, "_", sex, "_Individual_Weight.csv")), sep = ";", row.names = FALSE)
                }
            } # sexes
        } else {
            message("Not enougth data for length-weight analysis")
        }
    }
}
 ### close function LW

#' Spearman test for timeseries
#'
#' @param x time series
#' @importFrom stats pt
#' @return A \code{data.frame} representing the Spearman correlation coefficient and related statistics.
#' @examples
#' x <- runif(10, 1, 100)
#' spear(x)
#' @export
spear<-function (x){

    N = length(x)
    S = rank(x,ties.method ="min") # crank(x)
    R = c(1:length(x))

    r = 1-6*sum((R-S)^2)/N/(N^2-1)
    t=r*sqrt((N-2)/(1-r^2))

    p=2*(1-pt(abs(t),N-2)) #
    results=data.frame(r=r,t=t,p=p)
    return(results)
}

#' Plot of hauls time series
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @description
#' This function generates a spatial plot of haul positions over time, displaying haul labels and bathymetric lines over a customizable map extent.
#' If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
#' @param mTATB data frame
#' @param country country code as reported in MEDITS format. "all" code to perform the analysis on all the countries of the same GSA
#' @param map_lim coordinates limits for the plotted map
#' @param depth_lines vactor of three depth bathymetrical lines to be plotted
#' @param buffer buffer to the coordinate limits in map units
#' @param res resolution of the bathymetrical lines
#' @param wd working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @param verbose boolean. If TRUE messages are reported in the console
#' @return A \code{ggplot} object representing the spatial distribution of hauls.
#' @examples
#' \donttest{
#' # Use internal data
#' data(TA)
#' data(TB)
#' # Create a merged dataset for GSA 10 and year 2016
#' mTATB <- merge_TATB(TA[TA$AREA == 10 & TA$YEAR == 2016, ],
#'                     TB[TB$AREA == 10 & TB$YEAR == 2016, ],
#'                     "MERLMER")
#' hauls_position(mTATB, country = "all", map_lim = NA, depth_lines = c(200, 500, 800), save = FALSE)
#' }
#' @export
hauls_position <- function(mTATB, country = "all", map_lim, depth_lines, buffer = 0, res = NA, wd = NA, save = TRUE, verbose = TRUE) {
  if (is.na(wd)) {
    wd <- tempdir()
  }

  Haul <- V1 <- V2 <- V3 <- Year <- group <- merge_TATB <- lat <- lon <- long <- NULL #

  if (FALSE) {
    map_lim <- c(15.5, 20.0, 39.8, 42.5)
    map_lim <- NA
    depth_lines <- c(200, 500, 800)
    buffer <- 0.5
    res <- 0.1
    save <- TRUE
    ta <- read.table(paste(wd, "input/TA/TA GSA18 2017-2020.csv", sep = "/"), sep = ";", header = T)
    tb <- read.table(paste(wd, "input/TB/TB GSA18 2017-2020.csv", sep = "/"), sep = ";", header = T)
    tc <- read.table(paste(wd, "input/TC/TC GSA18 2017-2020.csv", sep = "/"), sep = ";", header = T)
    wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"

    m <- read.table("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)\\output\\mergeTATB_MERLMER.csv", sep = ";", header = TRUE)

    hauls_position(mTATB = m, map_lim, depth_lines, buffer = 0.1, wd = wd, save = TRUE, verbose = TRUE)
  }

  if (!requireNamespace("mapproj", quietly = TRUE)) {
    stop("Package 'mapproj' needed for this function to work. Please install it.", call. = FALSE)
  }

  if (is.na(wd) & save) {
    save <- TRUE
    if (verbose) {
      message("Missing working directory. Results are not saved in the local folder.")
    }
  }


  countries <- unique(mTATB[!is.na(mTATB$COUNTRY), "COUNTRY"])
  l_country <- length(unique(mTATB$COUNTRY))

  if (l_country == 1) {
    check_country <- "Y"
    country_analysis <- countries
    if (verbose) {
      message(paste0("Analysis conducted for the following country: ", countries))
    }
  } else {
    if (any(country %in% "all")) {
      country_analysis <<- countries
      if (verbose) {
        message(paste0("Analysis conducted for the following countries: ", paste(countries, collapse = ", ")))
      }
    } else {
      country_analysis <<- country
      if (verbose) {
        message(paste0("Analysis conducted for the following countries: ", paste(countries, collapse = ", ")))
      }
    }
  }

  mTATB <- mTATB[mTATB$COUNTRY %in% country_analysis, ]

  metaDB <- mTATB

  GENERE <- as.character(unique(metaDB$GENUS)[!is.na(unique(metaDB$GENUS) != -1) & unique(metaDB$GENUS) != -1])
  SPECIE <- as.character(unique(metaDB$SPECIES)[!is.na(unique(metaDB$SPECIES) != -1) & unique(metaDB$SPECIES) != -1])
  sspp <- paste(GENERE, SPECIE, sep = "")
  GSA <- unique(metaDB$GSA)
  species <- sspp
  #### selection of map limits by file ####
  if (any(is.na(map_lim))) {
    map_lim[c(1:2)] <- range(metaDB$MEAN_LONGITUDE_DEC, na.rm = TRUE)
    map_lim[c(3, 4)] <- range(metaDB$MEAN_LATITUDE_DEC, na.rm = TRUE)
    lx <- map_lim[2] - map_lim[1]
    ly <- map_lim[4] - map_lim[3]
  } else {
    lx <- map_lim[2] - map_lim[1]
    ly <- map_lim[4] - map_lim[3]
  }


  x1 <- map_lim[1]
  x2 <- map_lim[2]
  y1 <- map_lim[3]
  y2 <- map_lim[4]

  map_ratio <- ly / lx

  ddd <- metaDB

  loc <- data.frame(id = ddd$id, Haul = ddd$HAUL_NUMBER, Year = as.factor(ddd$YEAR), lon = ddd$MEAN_LONGITUDE_DEC, lat = ddd$MEAN_LATITUDE_DEC)

  theme_opts <- list(theme(
    panel.grid.minor = element_blank(),
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill = "light blue", linetype = "solid", color = "black"),
    plot.background = element_rect(
      fill = "white",
      linewidth = 1, linetype = "solid", color = "black"
    ),
    axis.line = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    axis.title.x = element_blank(),
    axis.title.y = element_blank(),
    plot.title = element_text(size = 22)
  ))


  buff <- buffer
  dep_text <- "Hauls position"
  world <- map_data("world")
  xl <- c(x1, x2)
  yl <- c(y1, y2)
  x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
  y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))

  lon_1 <- min(xl) - 1 - buff
  lon_2 <- max(xl) + 1 + buff
  lat_1 <- min(yl) - 1 - buff
  lat_2 <- max(yl) + 1 + buff

  if (!is.na(res)) {
    bath <- suppressMessages(getNOAA.bathy(lon1 = min(xl) - 1, lon2 = max(xl) + 1, lat1 = min(yl) - 1 - buff, lat2 = max(yl) + 1 + buff, resolution = res))
    bat_xyz <- as.xyz(bath)
  } else {
    data("med_bathy", package = "BioIndex", envir = environment())
    bat_xyz <- as.xyz(med_bathy)
    bat_xyz <- bat_xyz[!is.na(bat_xyz[, 3]), ]
    colnames(bat_xyz) <- c("V1", "V2", "V3")
    bat_xyz <- subset(bat_xyz, V1 >= min(xl) & V1 <= max(xl) & V2 >= min(yl) - buff & V2 <= max(yl) + buff)
  }


  if (any(is.na(depth_lines))) {
    depth_lines <- c(-200, -500, -800)
  } else {
    depth_lines <- -1 * depth_lines
  }

  p <- suppressWarnings(suppressMessages(
    ggplot() +
      coord_sf(xlim = xl, ylim = yl, expand = TRUE) +
      geom_polygon(data = world, aes(long, lat, group = group), fill = "grey") +
      scale_x_continuous(breaks = x_breaks) +
      scale_y_continuous(breaks = y_breaks) +
      geom_contour(
        data = bat_xyz,
        aes(x = V1, y = V2, z = V3),
        breaks = depth_lines[3], color = "#1F618D", linewidth = 0.5
      ) +
      geom_contour(
        data = bat_xyz,
        aes(x = V1, y = V2, z = V3),
        breaks = depth_lines[2], color = "#5499C7", linewidth = 0.5
      ) +
      geom_contour(
        data = bat_xyz,
        aes(x = V1, y = V2, z = V3),
        breaks = depth_lines[1], color = "#4db5fa", linewidth = 0.5
      ) +
      geom_text(data = loc, aes(lon, lat, label = Haul)) +
      geom_point(data = loc, aes(lon, lat, color = Year, fill = Year)) +
      coord_map(projection = "mercator", xlim = c((x1 - buff), (x2 + buff)), ylim = c((y1 - buff), (y2 + buff))) +
      ggtitle(dep_text) +
      theme_opts
  ))
  # print(p) # rimosso per policy CRAN
  if (verbose) {
    message("Plot of Hauls position generated successfully.")
  }


  if (save) {
    if (is.na(wd)) {
      warning("\nNo working directory defined by the user. The plot was not saved in the local folder\n")
    } else if (!is.na(wd)) {
      ggsave(paste(wd, "/output/", sspp, "_GSA", GSA, " -Hauls position.jpg", sep = ""),
        width = 20, height = 20, units = "cm", dpi = 300
      )
      if (verbose) {
        message("Bubble plot of Hauls position correctly saved \n")
      }
    }
  }

  return(p)
}

#' Estimation of abundance indices for females
#'
#' @param mTATB data frame
#' @param GSA reference GSA for the analysis
#' @param country_analysis vector of reference countries for the analysis
#' @param depth_range range of depth strata to perform the analysis (min, max)
#' @param strata_scheme data frame of the stratification scheme
#' @param stratification data frame of strata surface area
#' @param wd working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @importFrom gridExtra grid.arrange
#' @importFrom reshape2 melt
#' @importFrom stringr str_split
#' @importFrom grDevices dev.off tiff
#' @importFrom graphics legend lines par
#' @return A \code{data.frame} containing the time series of indices specifically for females.
#' @examples
#' data(TA)
#' data(TB)
#' data(TC)
#' data(strata_scheme)
#' data(stratification)
#' m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
#'                   species = "MERLMER", country = "all", verbose = FALSE)
#' strata_scheme_g10 <- strata_scheme[strata_scheme$GSA == 10 & strata_scheme$COUNTRY == "ITA", ]
#' index_ts_F(m[[1]], GSA = 10, country_analysis = "all", depth_range = c(10, 800),
#'            strata_scheme = strata_scheme_g10, stratification = stratification, save = FALSE)
#' @export
index_ts_F <- function(mTATB, GSA, country_analysis, depth_range, strata_scheme, stratification, wd=NA, save=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))


    MEAN_DEPTH <- strata <- value <- year <- NULL

    merge_TATB <- mTATB

    merge_TATB$N_km2 <- merge_TATB$NB_OF_FEMALES / merge_TATB$SWEPT_AREA

    #-----------------------
    # Abundance INDICES
    #-----------------------

    #------------------------
    # Variable definitions #
    dependent <- "abundance"
    dep_text <-expression(paste("Abundance ", (n/km^2), sep=" "))
    varcol <- which(colnames(merge_TATB)=="N_km2")
    col_response <- which(colnames(merge_TATB)=="N_km2")
    positive_hauls<- 0
    total_hauls<- 0

    GENERE <- as.character(unique(merge_TATB$GENUS)[unique(merge_TATB$GENUS) != -1])
    SPECIE <- as.character(unique(merge_TATB$SPECIES)[unique(merge_TATB$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")
    GSA <- unique(merge_TATB$GSA)
    species <- sspp
    #------------------------


    #------------------------------------------
    # selection of depth range
    data <- merge_TATB
    depth <- data.frame(matrix(NA,ncol=4, nrow=length(strata_scheme$CODE)))
    colnames(depth) <- c("strata", "min", "max", "bul")
    depth$strata <- strata_scheme$CODE # c(1,2,3,4,5)
    depth$min <-strata_scheme$MIN_DEPTH  # c(10,50,100,200,500)
    depth$max <-strata_scheme$MAX_DEPTH  # c(50,100,200,500,800)
    # print("Select the depth range for the analysis")
    # depth_range <- dlgInput("Depth range for the analysis: ", default="10,100",Sys.info()[""])$res
    # depth_range <- data.frame(range = strsplit(depth_range, ",")); depth_range <- as.numeric(as.character(depth_range[,1]))
    if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
    data <-  data[data$MEAN_DEPTH>depth_range[1] & data$MEAN_DEPTH<=depth_range[2],]
    idx_from <- which(depth[,"min"] == depth_range[1])
    idx_to   <- which(depth[,"max"] == depth_range[2])

    if (length(idx_from) != 1 || length(idx_to) != 1) {
        stop(paste0("Error: depth range [", depth_range[1], ", ", depth_range[2],
                    "] does not match the strata boundaries defined for this GSA. ",
                    "Available min depths: ", paste(sort(unique(depth$min)), collapse=", "),
                    "; Available max depths: ", paste(sort(unique(depth$max)), collapse=", ")))
    }
    strata_range <- seq(idx_from, idx_to, 1)
    depth[depth$strata %in% strata_range, "bul"] <- T
    depth[!(depth$strata %in% strata_range), "bul"] <- F

    analysis_stratum1 <- NA
    analysis_stratum2 <- NA
    analysis_stratum3 <- NA
    analysis_stratum4 <- NA
    analysis_stratum5 <- NA
    analysis_stratum6 <- NA

    analysis_stratum1 <- depth[1,4]
    analysis_stratum2 <- depth[2,4]
    analysis_stratum3 <- depth[3,4]
    analysis_stratum4 <- depth[4,4]
    analysis_stratum5 <- depth[5,4]
    analysis_stratum6 <- depth[6,4]

    if (is.na(analysis_stratum1)){depth[1,4]<- F; analysis_stratum1 <- F}
    if (is.na(analysis_stratum2)){depth[2,4]<- F; analysis_stratum2 <- F}
    if (is.na(analysis_stratum3)){depth[3,4]<- F; analysis_stratum3 <- F}
    if (is.na(analysis_stratum4)){depth[4,4]<- F; analysis_stratum4 <- F}
    if (is.na(analysis_stratum5)){depth[5,4]<- F; analysis_stratum5 <- F}
    if (is.na(analysis_stratum6)){depth[6,4]<- F; analysis_stratum6 <- F}

    #------------------------------------------


    ddd <- merge_TATB
    year_range <- data.frame(year = sort(unique(merge_TATB$YEAR)))

    res_table_cala2 <- data.frame(year=sort(unique(merge_TATB$YEAR)))
    res_table_cala2$index_1 <- NA
    res_table_cala2$index_2 <- NA
    res_table_cala2$index_3 <- NA
    res_table_cala2$index_4 <- NA
    res_table_cala2$index_5 <- NA
    res_table_cala2$index_6 <- NA

    se_table2  <-  data.frame(year=sort(unique(merge_TATB$YEAR)))
    se_table2$sum_1 <- NA
    se_table2$sum_2 <- NA
    se_table2$sum_3 <- NA
    se_table2$sum_4 <- NA
    se_table2$sum_5 <- NA
    se_table2$sum_6 <- NA
    se_table2$sd <- NA

    # stratification <- read.table(paste(wd,"/scripts/utilities/stratification scheme.csv", sep=""), sep=";", header=T)
    stratification$SURF <- as.numeric(stratification$SURF)

    if (analysis_stratum1 == T){area_s1 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==1 & stratification$COUNTRY %in% country_analysis,5])} else {area_s1 <- 0}
    if (analysis_stratum2 == T){area_s2 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==2 & stratification$COUNTRY %in% country_analysis,5])} else {area_s2 <- 0}
    if (analysis_stratum3 == T){area_s3 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==3 & stratification$COUNTRY %in% country_analysis,5])} else {area_s3 <- 0}
    if (analysis_stratum4 == T){area_s4 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==4 & stratification$COUNTRY %in% country_analysis,5])} else {area_s4 <- 0}
    if (analysis_stratum5 == T){area_s5 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==5 & stratification$COUNTRY %in% country_analysis,5])} else {area_s5 <- 0}
    if (analysis_stratum6 == T){area_s6 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==6 & stratification$COUNTRY %in% country_analysis,5])} else {area_s6 <- 0}


    peso_s1 <- area_s1/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s2 <- area_s2/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s3 <- area_s3/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s4 <- area_s4/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s5 <- area_s5/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s6 <- area_s6/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)

    i=1
    for(i in 1:length(res_table_cala2[,1])) {

        data2 <- ddd[ddd$YEAR == res_table_cala2[i,1] , ]

        if (analysis_stratum1 == T) {s1 <- data2[floor(data2$MEAN_DEPTH) >= depth[1,2] & floor(data2$MEAN_DEPTH) <= depth[1,3], ]}
        if (analysis_stratum2 == T) {s2 <- data2[floor(data2$MEAN_DEPTH) > depth[2,2] & floor(data2$MEAN_DEPTH) <= depth[2,3], ]}
        if (analysis_stratum3 == T) {s3 <- data2[floor(data2$MEAN_DEPTH) > depth[3,2] & floor(data2$MEAN_DEPTH) <= depth[3,3], ]}
        if (analysis_stratum4 == T) {s4 <- data2[floor(data2$MEAN_DEPTH) > depth[4,2] & floor(data2$MEAN_DEPTH) <= depth[4,3], ]}
        if (analysis_stratum5 == T) {s5 <- data2[floor(data2$MEAN_DEPTH) > depth[5,2] & floor(data2$MEAN_DEPTH) <= depth[5,3], ]}
        if (analysis_stratum6 == T) {s6 <- data2[floor(data2$MEAN_DEPTH) > depth[6,2] & floor(data2$MEAN_DEPTH) <= depth[6,3], ]}

        if (analysis_stratum1 == T){
            # [ORIGINALE]:
            # s_n <- sum(s1[!is.na(s1$NB_OF_FEMALES),"NB_OF_FEMALES"])
            # s_a <- sum(s1$SWEPT_AREA)
            # res_table_cala2[i,2] <- s_n/s_a
            # mean_ind <- mean(s1[!is.na(s1$N_km2), "N_km2" ])
            # sq <- (s1[!is.na(s1$N_km2),"N_km2"] - mean_ind)^2
            # sqA <- sq*s1$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s1[,1])-1)
            # f1 <- s_a/area_s1
            # se_table2[i,2] <- (((peso_s1)^2 * sum_sqA)/ s_a)* (1-f1)

            # [MODIFICATO]:
            s_n <- sum(s1[!is.na(s1$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_a <- sum(s1$SWEPT_AREA)
            res_table_cala2[i,2] <- s_n/s_a
            s1_clean <- s1[!is.na(s1$N_km2), ]
            s_a_clean <- sum(s1_clean$SWEPT_AREA)
            if (nrow(s1_clean) > 1) {
                mean_ind <- mean(s1_clean$N_km2)
                sq <- (s1_clean$N_km2 - mean_ind)^2
                sqA <- sq*s1_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s1_clean)-1)
                f1 <- s_a_clean/area_s1
                se_table2[i,2] <- (((peso_s1)^2 * sum_sqA)/ s_a_clean)* (1-f1)
            } else {
                se_table2[i,2] <- 0
            }
        } else {area_s1 = 0}

        if (analysis_stratum2 == T){
            # [ORIGINALE]:
            # s_n <- sum(s2[!is.na(s2$NB_OF_FEMALES),"NB_OF_FEMALES"])
            # s_a <- sum(s2$SWEPT_AREA)
            # res_table_cala2[i,3] <- s_n/s_a
            # mean_ind <- mean(s2[!is.na(s2$N_km2), "N_km2" ])
            # sq <- (s2[!is.na(s2$N_km2),"N_km2"] - mean_ind)^2
            # sqA <- sq*s2$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s2[,1])-1)
            # f2 <- s_a/area_s2
            # se_table2[i,3] <- (((peso_s2)^2 * sum_sqA)/ s_a)* (1-f2)

            # [MODIFICATO]:
            s_n <- sum(s2[!is.na(s2$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_a <- sum(s2$SWEPT_AREA)
            res_table_cala2[i,3] <- s_n/s_a
            s2_clean <- s2[!is.na(s2$N_km2), ]
            s_a_clean <- sum(s2_clean$SWEPT_AREA)
            if (nrow(s2_clean) > 1) {
                mean_ind <- mean(s2_clean$N_km2)
                sq <- (s2_clean$N_km2 - mean_ind)^2
                sqA <- sq*s2_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s2_clean)-1)
                f2 <- s_a_clean/area_s2
                se_table2[i,3] <- (((peso_s2)^2 * sum_sqA)/ s_a_clean)* (1-f2)
            } else {
                se_table2[i,3] <- 0
            }
        } else {area_s2 = 0}

        if (analysis_stratum3 == T){
            # [ORIGINALE]:
            # s_n <- sum(s3[!is.na(s3$NB_OF_FEMALES),"NB_OF_FEMALES"])
            # s_a <- sum(s3$SWEPT_AREA)
            # res_table_cala2[i,4] <- s_n/s_a
            # mean_ind <- mean(s3[!is.na(s3$N_km2), "N_km2" ])
            # sq <- (s3[!is.na(s3$N_km2),"N_km2"] - mean_ind)^2
            # sqA <- sq*s3$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s3[,1])-1)
            # f3 <- s_a/area_s3
            # se_table2[i,4] <- (((peso_s3)^2 * sum_sqA)/ s_a)* (1-f3)

            # [MODIFICATO]:
            s_n <- sum(s3[!is.na(s3$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_a <- sum(s3$SWEPT_AREA)
            res_table_cala2[i,4] <- s_n/s_a
            s3_clean <- s3[!is.na(s3$N_km2), ]
            s_a_clean <- sum(s3_clean$SWEPT_AREA)
            if (nrow(s3_clean) > 1) {
                mean_ind <- mean(s3_clean$N_km2)
                sq <- (s3_clean$N_km2 - mean_ind)^2
                sqA <- sq*s3_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s3_clean)-1)
                f3 <- s_a_clean/area_s3
                se_table2[i,4] <- (((peso_s3)^2 * sum_sqA)/ s_a_clean)* (1-f3)
            } else {
                se_table2[i,4] <- 0
            }
        } else {area_s3 = 0}

        if (analysis_stratum4 == T){
            # [ORIGINALE]:
            # s_n <- sum(s4[!is.na(s4$NB_OF_FEMALES),"NB_OF_FEMALES"])
            # s_a <- sum(s4$SWEPT_AREA)
            # res_table_cala2[i,5] <- s_n/s_a
            # mean_ind <- mean(s4[!is.na(s4$N_km2), "N_km2" ])
            # sq <- (s4[!is.na(s4$N_km2),"N_km2"] - mean_ind)^2
            # sqA <- sq*s4$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s4[,1])-1)
            # f4 <- s_a/area_s4
            # se_table2[i,5] <- (((peso_s4)^2 * sum_sqA)/ s_a)* (1-f4)

            # [MODIFICATO]:
            s_n <- sum(s4[!is.na(s4$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_a <- sum(s4$SWEPT_AREA)
            res_table_cala2[i,5] <- s_n/s_a
            s4_clean <- s4[!is.na(s4$N_km2), ]
            s_a_clean <- sum(s4_clean$SWEPT_AREA)
            if (nrow(s4_clean) > 1) {
                mean_ind <- mean(s4_clean$N_km2)
                sq <- (s4_clean$N_km2 - mean_ind)^2
                sqA <- sq*s4_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s4_clean)-1)
                f4 <- s_a_clean/area_s4
                se_table2[i,5] <- (((peso_s4)^2 * sum_sqA)/ s_a_clean)* (1-f4)
            } else {
                se_table2[i,5] <- 0
            }
        } else {area_s4 = 0}

        if (analysis_stratum5 == T){
            # [ORIGINALE]:
            # s_n <- sum(s5[!is.na(s5$NB_OF_FEMALES),"NB_OF_FEMALES"])
            # s_a <- sum(s5$SWEPT_AREA)
            # res_table_cala2[i,6] <- s_n/s_a
            # mean_ind <- mean(s5[!is.na(s5$N_km2), "N_km2" ])
            # sq <- (s5[!is.na(s5$N_km2),"N_km2"] - mean_ind)^2
            # sqA <- sq*s5$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s5[,1])-1)
            # f5 <- s_a/area_s5
            # se_table2[i,6] <- (((peso_s5)^2 * sum_sqA)/ s_a)* (1-f5)

            # [MODIFICATO]:
            s_n <- sum(s5[!is.na(s5$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_a <- sum(s5$SWEPT_AREA)
            res_table_cala2[i,6] <- s_n/s_a
            s5_clean <- s5[!is.na(s5$N_km2), ]
            s_a_clean <- sum(s5_clean$SWEPT_AREA)
            if (nrow(s5_clean) > 1) {
                mean_ind <- mean(s5_clean$N_km2)
                sq <- (s5_clean$N_km2 - mean_ind)^2
                sqA <- sq*s5_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s5_clean)-1)
                f5 <- s_a_clean/area_s5
                se_table2[i,6] <- (((peso_s5)^2 * sum_sqA)/ s_a_clean)* (1-f5)
            } else {
                se_table2[i,6] <- 0
            }
        } else {area_s5 = 0}

        if (analysis_stratum6 == T){
            # [ORIGINALE]:
            # s_n <- sum(s6[!is.na(s6$NB_OF_FEMALES),"NB_OF_FEMALES"])
            # s_a <- sum(s6$SWEPT_AREA)
            # res_table_cala2[i,7] <- s_n/s_a
            # mean_ind <- mean(s6[!is.na(s6$N_km2), "N_km2" ])
            # sq <- (s6[!is.na(s6$N_km2),"N_km2"] - mean_ind)^2
            # sqA <- sq*s6$SWEPT_AREA
            # sum_sqA <- sum(sqA)/(length(s6[,1])-1)
            # f6 <- s_a/area_s6
            # se_table2[i,7] <- (((peso_s6)^2 * sum_sqA)/ s_a)* (1-f6)

            # [MODIFICATO]:
            s_n <- sum(s6[!is.na(s6$NB_OF_FEMALES),"NB_OF_FEMALES"])
            s_a <- sum(s6$SWEPT_AREA)
            res_table_cala2[i,7] <- s_n/s_a
            s6_clean <- s6[!is.na(s6$N_km2), ]
            s_a_clean <- sum(s6_clean$SWEPT_AREA)
            if (nrow(s6_clean) > 1) {
                mean_ind <- mean(s6_clean$N_km2)
                sq <- (s6_clean$N_km2 - mean_ind)^2
                sqA <- sq*s6_clean$SWEPT_AREA
                sum_sqA <- sum(sqA)/(nrow(s6_clean)-1)
                f6 <- s_a_clean/area_s6
                se_table2[i,7] <- (((peso_s6)^2 * sum_sqA)/ s_a_clean)* (1-f6)
            } else {
                se_table2[i,7] <- 0
            }
        } else {area_s6 = 0}

        # source(paste(wd, "/scripts/Index_estimation_cala.R", sep=""), encoding = 'UTF-8')
        positive_hauls[i] <- length(merge_TATB[merge_TATB$NB_OF_FEMALES>0 & merge_TATB$YEAR == res_table_cala2[i,1],"NB_OF_FEMALES"])
        total_hauls[i] <- length(merge_TATB[merge_TATB$YEAR == res_table_cala2[i,1],"HAUL_NUMBER"])

        sum_res_est <- c(res_table_cala2[i,2]*peso_s1,res_table_cala2[i,3]*peso_s2,res_table_cala2[i,4]*peso_s3,res_table_cala2[i,5]*peso_s4,res_table_cala2[i,6]*peso_s5,res_table_cala2[i,7]*peso_s6)
        res_table_cala2[i, 8]<- sum(sum_res_est[!is.na(sum_res_est)])# /sum(area_s1,area_s2,area_s3,area_s4,area_s5)
        colnames(res_table_cala2) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices")
        se_table2[i, "sd"] <-sqrt(rowSums(se_table2[i, 2:7], na.rm = T))
    }

    timeseries <- data.frame(year = res_table_cala2[,1], abundance = res_table_cala2[,8], sd= se_table2[, "sd"])
    timeseries$CV <- timeseries[,3] / timeseries[,2]
    timeseries$invCV <- 1/timeseries$CV
    timeseries$positive_hauls_perc <- positive_hauls / total_hauls * 100
    colnames(res_table_cala2) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices")

    # plot timeseries of mean indices
    main <- paste(sspp,"_GSA",GSA,"_(",dependent,")-FEMALES-Random_Stratified_Sampling_",depth_range[1],"-",depth_range[2], " m", sep="")
    main.lab <- paste(sspp," GSA",GSA," (",dependent,")-FEMALES-RSS ",depth_range[1],"-",depth_range[2], " m", sep="")
    max_index <- max(timeseries[,2]) + (max(timeseries[!is.na(timeseries$sd),"sd"])*1.2)
    if(save){
        write.table(timeseries, paste(wd,"/output/",main,"_Timeseries.csv", sep=""), sep=";", row.names = F)
    }

    if (save) {
    jpeg(paste(wd,"/output/",main,"_Timeseries.jpg",sep=""), res = 300, width = 8, height = 7, units = 'in')
    par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
    plot(timeseries[,1],  timeseries[,2], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab) # ylim=c(0,max_index*1.2)
    lines(timeseries[,1], (timeseries[,2]-1.96*timeseries[,3]), type="l",lty=2, col="red" )
    lines(timeseries[,1], (timeseries[,2]+1.96*timeseries[,3]), type="l",lty=2, col="red" )
    legend("topright", c("time series", "CI"), lty=c(1,1), pch=c(16, NA), col=c("black","red"))
    dev.off()
    } else {

        par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
        plot(timeseries[,1],  timeseries[,2], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab) # ylim=c(0,max_index*1.2)
        lines(timeseries[,1], (timeseries[,2]-1.96*timeseries[,3]), type="l",lty=2, col="red" )
        lines(timeseries[,1], (timeseries[,2]+1.96*timeseries[,3]), type="l",lty=2, col="red" )
        legend("topright", c("time series", "CI"), lty=c(1,1), pch=c(16, NA), col=c("black","red"))
    }













}

#' Conversion of MEDITS format coordinates in decimal degrees format
#' @description
#' Conversion of MEDITS format coordinates in decimal degrees format
#'
#' @param data data frame of the hauls data (TA, table A) in MEDITS format
#'
#' @return The function returns the data frame of the TA table with the coordinates expressed as decimal degrees
#' @examples
#' data(TA)
#' MEDITS.to.dd(TA)
#' @export
MEDITS.to.dd<-function(data)  {
  lat_start=data$SHOOTING_LATITUDE
  lon_start= data$SHOOTING_LONGITUDE
  lat_end=data$HAULING_LATITUDE
  lon_end= data$HAULING_LONGITUDE
  LatStartDeg = floor(floor(lat_start)/100);
  LonStartDeg = floor(floor(lon_start)/100);
  LatStartMin=(lat_start-LatStartDeg*100)/60
  LonStartMin=(lon_start-LonStartDeg*100)/60
  LatEndDeg = floor(floor(lat_end)/100);
  LonEndDeg = floor(floor(lon_end)/100);
  LatEndMin=(lat_end-LatEndDeg*100)/60
  LonEndMin=(lon_end-LonEndDeg*100)/60

  lat_start2= LatStartDeg + LatStartMin
  lon_start2 = LonStartDeg + LonStartMin
  lat_end2 = LatEndDeg + LatEndMin
  lon_end2 = LonEndDeg + LonEndMin
  data$SHOOTING_LATITUDE = lat_start2
  data$SHOOTING_LONGITUDE = lon_start2
  data$HAULING_LATITUDE = lat_end2
  data$HAULING_LONGITUDE = lon_end2
  k <- 1
  for (k in 1:length(data$SHOOTING_QUADRANT )){
    if (data$SHOOTING_QUADRANT[k] == 7){ data$SHOOTING_LONGITUDE[k] <- -1 * data$SHOOTING_LONGITUDE[k] }
    if (data$SHOOTING_QUADRANT[k] == 3){ data$SHOOTING_LATITUDE[k] <- -1 * data$SHOOTING_LATITUDE[k] }
    if (data$SHOOTING_QUADRANT[k] == 5){
      data$SHOOTING_LONGITUDE[k] <- -1 * data$SHOOTING_LONGITUDE[k]
      data$SHOOTING_LATITUDE[k] <- -1 * data$SHOOTING_LATITUDE[k]
    }


    if (data$HAULING_QUADRANT[k] == 7){ data$HAULING_LONGITUDE[k] <- -1 * data$HAULING_LONGITUDE[k] }
    if (data$HAULING_QUADRANT[k] == 3){ data$HAULING_LATITUDE[k] <- -1 * data$HAULING_LATITUDE[k] }
    if (data$HAULING_QUADRANT[k] == 5){
      data$SHOOTING_LONGITUDE[k] <- -1 * data$HAULING_LONGITUDE[k]
      data$SHOOTING_LATITUDE[k] <- -1 * data$HAULING_LATITUDE[k]
    }
  }

    return(data)

}

#' launches the embedded Shiny application included in the package.
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#'
#' @importFrom shiny runApp
#' @return \code{No return value, called for side effects} (launches the Shiny application).
#' @examples
#' if (interactive()) {
#'   run_BioIndex_app()
#' }
#' @export
run_BioIndex_app <- function() {
    app_dir <- system.file("shiny/BioIndexApp", package = "BioIndex")

    if (app_dir == "") {
        stop("Could not find the BioIndex Shiny app directory. Please reinstall the package.")
    }

    shiny::runApp(app_dir, display.mode = "normal")
}

#' LW (Length-Weight relationship)
#' @description
#' Estimates the length-weight relationship for the selected species,
#' a fundamental biological indicator to assess fish condition and convert
#' numerical length estimates into biomass equivalents.
#'
#' @param ta MEDITS or MEDITS-like TA table
#' @param te MEDITS or MEDITS-like TE table
#' @param sp species RUBIN code (MEDITS format, e.g. "MERLMER")
#' @param GSA reference GSA for the analysis
#' @param country reference country
#' @param nyears number of years of the time series to be considered in the analysis
#' @param wd path of the working directory
#' @param save boolean. If TRUE the outputs are saved in the local folder
#' @param verbose boolean. If TRUE messages are promted in the console
#' @return A \code{data.frame} containing the length-weight relationship parameters and statistics.
#' @import ggplot2
LW <- function(ta, te, sp, GSA, country = "all", nyears = NA, wd = NA, save = TRUE, verbose = FALSE) {
    if (is.na(wd)) {
        wd <- tempdir()
    }

    TA <- ta
    TE <- te

    if (verbose) {
        message("\n################################")
        message("Length-Weight relationship (LW)")
        message("################################")
        message("")
    }

    if (nrow(TE) > 0 & nrow(TA) > 0) {
        if (!dir.exists(file.path(wd, "output"))) {
            dir.create(file.path(wd, "output"), showWarnings = FALSE)
        }

        if (!dir.exists(file.path(wd, "output", "LW"))) {
            dir.create(file.path(wd, "output", "LW"), showWarnings = FALSE, recursive = TRUE)
        }


        GEAR <- unique(TA$GEAR)[1]
        YEARS <- sort(unique(TE$YEAR))
        nYEARS <- length(YEARS)

        if (is.na(nyears)) {
            n_years <- nYEARS
            if (verbose) {
                message(paste0("Only ", n_years, " have been used in the analysis."))
            }
        } else if (nyears >= nYEARS) {
            n_years <- nYEARS
            if (verbose) {
                message(paste0("The available time series is shorter than ", nYEARS, " years. Only ", n_years, " have been used in the analysis."))
            }
        }

        # YEARS
        if (!is.na(n_years) & n_years <= nYEARS & n_years > 0) {
            n_years <- n_years
        } else {
            n_years <- nYEARS
        }

        # SPECIES

        species <- substr(sp[1], 1, 4)
        species[2] <- substr(sp, 5, 7)
        sspp <- paste(species[1], species[2], sep = "")


        # COUNTRY

        #########################
        ###   filtro COUNTRY  ###
        #########################

        TA_country <- as.character(unique(TA$COUNTRY))
        TE_country <- as.character(unique(TE$COUNTRY))
        l_country <- length(TA_country)

        for (i in 1:length(TA_country)) {
            if (!(TA_country[i] %in% TE_country)) {
                warning(paste("The country ", TA_country[i], " is not present in the TE file.", sep = ""))
            }
        }

        if (l_country == 1) {
            check_country <- "Y"
            country_analysis <- TA_country
        } else {
            if (any(country %in% "all")) {
                country_analysis <- TA_country
            } else {
                country_analysis <- country
            }
        } # close -->if  length(l_country)==1

        TA <- TA[TA$COUNTRY %in% country_analysis, ]
        TE <- TE[TE$COUNTRY %in% country_analysis, ]

        TE <- TE[TE$AREA %in% GSA, ]
        if (nrow(TE) > 0) {
            LWf(TE = TE, sp = sspp, GEAR = GEAR, GSA = GSA, country = country_analysis, n_records = 10, wd = wd, save = save)
        }
    } # close (nrow(TE)>0 & nrow(TA)>0)
}

#' Estimation of haul distance
#'
#' @param data data frame containing the hauls data (TA, table A).
#' @param unit string value indicating the measure unit of the distance. Allowed values: "m" for meters, "km" for kilometers and "NM" for nautical miles.
#' @param verbose give verbose output reporting in the output the selected measure unit of the distance.
#' @return The function returns the vector of the distances expressed in the selected measure unit.
#' @examples
#' data(TA)
#' MEDITS.distance(TA, unit = "m", verbose = FALSE)
#' @export
MEDITS.distance<-function(data, unit = "m", verbose = TRUE)  {
  lat_start=data$SHOOTING_LATITUDE
  lon_start= data$SHOOTING_LONGITUDE
  lat_end=data$HAULING_LATITUDE
  lon_end= data$HAULING_LONGITUDE
  LatStartDeg = floor(floor(lat_start)/100);
  LonStartDeg = floor(floor(lon_start)/100);
  LatStartMin=(lat_start-LatStartDeg*100)/60
  LonStartMin=(lon_start-LonStartDeg*100)/60
  LatEndDeg = floor(floor(lat_end)/100);
  LonEndDeg = floor(floor(lon_end)/100);
  LatEndMin=(lat_end-LatEndDeg*100)/60
  LonEndMin=(lon_end-LonEndDeg*100)/60

  lat_start2= LatStartDeg + LatStartMin
  lon_start2 = LonStartDeg + LonStartMin
  lat_end2 = LatEndDeg + LatEndMin
  lon_end2 = LonEndDeg + LonEndMin
  data$SHOOTING_LATITUDE = lat_start2
  data$SHOOTING_LONGITUDE = lon_start2
  data$HAULING_LATITUDE = lat_end2
  data$HAULING_LONGITUDE = lon_end2

  N1  <- (((data$SHOOTING_LATITUDE/2)+45)*pi )/180
  N2  <- (((data$HAULING_LATITUDE/2)+45)*pi )/180
  N3  <- atan((pi*(data$HAULING_LONGITUDE-data$SHOOTING_LONGITUDE))/(180*(log(tan(N2))-log(tan(N1)))))
  dist <- abs(60*(data$HAULING_LATITUDE-data$SHOOTING_LATITUDE)/cos(N3))*1852

  if (unit =="m"){
    dist = dist
    if (verbose){
    message("meters")
      }
  }
  if (unit =="km"){
    dist = dist/1000
    if (verbose){
    message("kilometers")
    }
  }
  if (unit =="NM"){
    dist = dist/1852
    if (verbose){
    message("Nautical Miles")
    }
  }

  return(dist)
}

#' Estimation of L50 and L95
#'
#' @param lfd data frame of combined LFD
#' @param wd working directory
#' @param sspp MEDITS code for the selected species
#' @param GSA reference area for the analysis
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @param verbose boolean. If TRUE messages are reported in the console
#' @return A \code{data.frame} containing the length quantiles (e.g., L95) time series.
Lquant <- function(lfd, wd=NA, sspp, GSA, save=TRUE, verbose=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))

    if (FALSE) {
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        # ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        # tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        # tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)

        ta <- read.table(paste("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\B-USEFUL\\_MEDITS_Corrected_data_\\TA_GSA18_1999_2021_completo.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\B-USEFUL\\_MEDITS_Corrected_data_\\TB_GSA18_1999_2021_completo.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\B-USEFUL\\_MEDITS_Corrected_data_\\TC_GSA18_1999_2021_completo.csv", sep="/"), sep=";", header=T)
        # ta <- ta[ta$YEAR==2017, ]
        # tb <- tb[tb$YEAR==2017, ]
        # tc <- tc[tc$YEAR==2017, ]
        depth_range=c(10,800)
        country="all"
        sex <- "all"
        species <- "MERLMER"
        save=TRUE

        m <- merge_TATBTC(ta, tb, tc, species=species, country=country, wd=wd, verbose=TRUE)
        mTATC <- m[[2]]

        lfd <- LFD(mTATC, sex=sex, GSA=10, country=country, depth_range=c(10,800), strata_scheme, stratification, wd, save=TRUE)
        lfd <- lfd[[1]]

        Lquant(lfd,wd, save=TRUE)
    }

    if (is.na(wd) & save) {
        save=TRUE
        if (verbose){
            message("Missing working directory. Results are not saved in the local folder.")
        }
    }

    Dati=lfd

col_names <- colnames(Dati)
col_years <- col_names[-1]
# col_years <- as.numeric(substr(col_years,2,5))
years_tab <- col_years

table=data.frame(matrix(NA,nrow=6,ncol=(ncol(Dati)-1)))
i=2
for (i in c(2:ncol(Dati))){
    Dati_temp=cbind(Dati[,1],Dati[,i])
    table [1,i] = round(quant(Dati_temp, 0.50)[1],4)
    table [2,i] = round(quant(Dati_temp, 0.50)[2],4) #round(sqrt(sum(Dati_temp[,2]*(Dati_temp[,1]-table[6,i])^2)/sum(Dati_temp[,2])),2)
    table [3,i] = round(quant(Dati_temp, 0.50)[3],4)
    table [4,i] = round(quant(Dati_temp, 0.95)[1],4)
    table [5,i] = round(quant(Dati_temp, 0.95)[2],2)
    table [6,i] = round(quant(Dati_temp, 0.95)[3],4)
    Dati_temp[,] = 0
}
table[,1]=c("50th perc.","SD 50th perc.","CV 50th perc.","95th perc.","SD 95th perc.","CV 95th perc.")
colnames(table)=c("Indices", years_tab)
# [ORIGINALE]:
# if (save){
# write.table(table,paste(wd, "/output/",sspp,"_GSA",GSA,"_L50_L95_RSS.csv", sep=""),sep=";",row.names=F)
# }
# [MODIFICATO]:
if (save){
dir.create(file.path(wd, "output"), showWarnings = FALSE, recursive = TRUE)
write.table(table,paste(wd, "/output/",sspp,"_GSA",GSA,"_L50_L95_RSS.csv", sep=""),sep=";",row.names=F)
}

#formatting table for plots
# par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0)) # rimosso per policy CRAN
t <- data.frame(t(table))
headers <- as.character(t[1,])
t <- t[-1,]
years <- row.names(t)
t <- cbind(years, t)
colnames(t) <- c("years",headers)
row.names(t) <- seq(1,nrow(t),1)
for (i in 1:ncol(t)){
 t[,i] <- as.numeric(t[,i])
}
#plot
rL50 <- range(t$`50th perc.`, na.rm = TRUE)
if (!all(is.finite(rL50))) rL50 <- c(0, 1)
r1   <- (rL50[2]-rL50[1])*20/100
rL50[2] <- rL50[2]+r1
rL95 <- range(t$`95th perc.`, na.rm = TRUE)
if (!all(is.finite(rL95))) rL95 <- c(0, 1)
r2   <- (rL95[2]-rL95[1])*20/100
rL95[2] <- rL95[2]+r2

# plot(t[,1],  t[,2], type="b", col="black", pch=16, xlab="year", ylab="L50 (mm)", ylim=rL50) # rimosso per policy CRAN
# lines(t[,1], (t[,2]-1.96*t[,3]), type="l",lty=2, col="red" ) # rimosso per policy CRAN
# lines(t[,1], (t[,2]+1.96*t[,3]), type="l",lty=2, col="red" ) # rimosso per policy CRAN
# legend("topright", c("time series", "CI"), lty=c(1,1), pch=c(16, NA), col=c("black","red")) # rimosso per policy CRAN

# plot(t[,1],  t[,5], type="b", col="black", pch=16, xlab="year", ylab="L95 (mm)", ylim=rL95) # rimosso per policy CRAN
# lines(t[,1], (t[,5]-1.96*t[,6]), type="l",lty=2, col="red" ) # rimosso per policy CRAN
# lines(t[,1], (t[,5]+1.96*t[,6]), type="l",lty=2, col="red" ) # rimosso per policy CRAN
# legend("topright", c("time series", "CI"), lty=c(1,1), pch=c(16, NA), col=c("black","red")) # rimosso per policy CRAN

if (save){
jpeg(paste(wd,"/output/L50_Timeseries.jpg",sep=""), res = 300, width = 8, height = 7, units = 'in')
   plot(t[,1],  t[,2], type="b", col="black", pch=16, xlab="year", ylab="L50 (mm)", ylim=rL50) # ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab # ylim=c(0,max_index*1.2)
   lines(t[,1], (t[,2]-1.96*t[,3]), type="l",lty=2, col="red" )
   lines(t[,1], (t[,2]+1.96*t[,3]), type="l",lty=2, col="red" )
   legend("topright", c("L50 time series", "CI"), lty=c(1,1), pch=c(16, NA), col=c("black","red"))
dev.off()


jpeg(paste(wd,"/output/L95_Timeseries.jpg",sep=""), res = 300, width = 8, height = 7, units = 'in')
   plot(t[,1],  t[,5], type="b", col="black", pch=16, xlab="year", ylab="L95 (mm)", ylim=rL95) # ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab # ylim=c(0,max_index*1.2)
   lines(t[,1], (t[,5]-1.96*t[,6]), type="l",lty=2, col="red" )
   lines(t[,1], (t[,5]+1.96*t[,6]), type="l",lty=2, col="red" )
   legend("topright", c("l95 time series", "CI"), lty=c(1,1), pch=c(16, NA), col=c("black","red"))
dev.off()
}
return(t)

}

#' Generating maps of indexes
#' @description
#' Generates density and biomass maps based on the spatial grid,
#' allowing for the visual identification of resource concentration areas
#' or potential biological hotspots.
#'
#' @param mTATBsp spatial mTATB
#' @param stratum reference stratum range (allowed values: "10,200","10,800","200,800","5,35","5,45")
#' @param wd working directory
#' @param map_range range of coordinates for the map
#' @param threshold minimum number of individuals per haul
#' @param verbose boolean. If TRUE messages are promted in the console
#' @param save boolean. If TRUE the results are stored in the working directory
#' @return A \code{ggplot} object displaying the generated map of indices on the spatial grid.
#' @examples
#' data(TA)
#' data(TB)
#' data(TC)
#' m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
#'                   species = "MERLMER", country = "all", verbose = FALSE)
#' mTATBsp <- overlayGrid(m[[1]], m[[2]], GSA = 10, save = FALSE, verbose = FALSE)[[1]]
#' map_range <- c(9, 15, 39, 42)
#' index_on_grid(mTATBsp, stratum = "10,800", map_range = map_range, threshold = 5,
#'               save = FALSE, verbose = FALSE)
#' @export
#' @importFrom ggplot2 ggplot scale_fill_manual geom_polygon coord_sf labs xlab ylab map_data ggsave
#' @importFrom stringr str_split
#' @importFrom tidyterra geom_spatvector
#' @importFrom dplyr arrange
#' @importFrom stats quantile
index_on_grid <- function(mTATBsp, stratum, wd=NA, map_range, threshold = 30, verbose = FALSE, save=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }
  if (FALSE) {
    # library(stringr)
    # library(ggplot2)
    # library(tidyterra)
    verbose <- TRUE
    save <- TRUE
    map_range <- c(15, 21, 39.5, 42.5)
    wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\Test_BioIndex_package"
    ta <- read.table(paste(wd, "input/TA GSA18 2017-2020.csv", sep = "/"), sep = ";", header = T)
    tb <- read.table(paste(wd, "input/TB GSA18 2017-2020.csv", sep = "/"), sep = ";", header = T)
    tc <- read.table(paste(wd, "input/TC GSA18 2017-2020.csv", sep = "/"), sep = ";", header = T)
    country <- "all"
    species <- "MERLMER"

    m <- merge_TATBTC(ta, tb, tc, species = "MERLMER", country = "all", wd = wd, verbose = TRUE)

    mTATB <- m[[1]]
    mTATC <- m[[2]]

    mTATBsp <- overlayGrid(mTATB, mTATC, GSA = NA, wd = wd, save = TRUE, verbose = FALSE)[[1]]

    stratum <- "10,800"
    index_on_grid(mTATBsp, stratum, wd, map_range, threshold = 30, verbose = FALSE, save = TRUE)
  }

  level <- category <- long <- lat <- group <- NULL

  # threshold_to_ind_weight_calc <- 30

  # depthstr <- stratum
  depth_range <- as.numeric(str_split(stratum,",")[[1]])

  #----------------

  resolve_depthstr <- function(depth_range) {

    dr <- sort(as.numeric(depth_range))
    if (anyNA(dr)) stop("depth_range contains NA.")
    if (dr[1] >= dr[2]) stop("depth_range must be increasing (min < max).")

    lo <- dr[1]
    hi <- dr[2]

    # 2) Map to allowed depthstr values, ensuring coverage of [lo, hi]

    # Shallow set (mapped to 0_35 / 0_45 / 0_125 in your code)
    if (hi <= 35) {
      return("5,35")      # maps to BioIndex::stratum_0_35
    }
    if (hi <= 45) {
      return("5,45")      # maps to BioIndex::stratum_0_45
    }
    if (hi <= 125) {
      return("0,125")     # canonical, maps to BioIndex::stratum_0_125
    }

    # Deep set (mapped to 0_200 / 0_800 / 200_800)
    if (hi <= 200) {
      return("10,200")    # maps to BioIndex::stratum_0_200
    }
    if (hi <= 800) {
      if (lo >= 200) {
        return("200,800") # maps to BioIndex::stratum_200_800
      } else {
        return("10,800")  # maps to BioIndex::stratum_0_800
      }
    }
  }

  #------------------------------

  depthstr <- resolve_depthstr(depth_range)


  if (depthstr == "5,45") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_45)
  }

  if (depthstr == "5,35") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_35)
  }

  if (depthstr == "10,125") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_125)
  }

  if (depthstr == "5,125") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_125)
  }

  if (depthstr == "0,125") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_125)
  }

  if (depthstr == "10,200") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_200)
  }

  if (depthstr == "10,800") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_0_800)
  }

  if (depthstr == "200,800") {
    cgpmgrid <- terra::unwrap(BioIndex::stratum_200_800)
  }
  cgpmgrid_bkp <- cgpmgrid
  continent <- terra::unwrap(BioIndex::continent)

  metaDB <- mTATBsp
  metaDB <- metaDB[metaDB$MEAN_DEPTH > depth_range[1] & metaDB$MEAN_DEPTH <= depth_range[2], ]

  # }
  GENERE <- as.character(unique(metaDB$GENUS)[unique(metaDB$GENUS) != -1])
  SPECIE <- as.character(unique(metaDB$SPECIES)[unique(metaDB$SPECIES) != -1])
  sspp <- paste(GENERE, SPECIE, sep = "")

  years <- sort(unique(metaDB[, "YEAR"]))

  if (length(years) < 10) {
    n_years <- length(years)
    firstyear <- years[length(years) - (n_years - 1)]
    last_10year <- years[years >= firstyear]
  } else {
    firstyear <- years[length(years) - 9]
    last_10year <- years[years >= firstyear]
  }

  metaDB <- metaDB[metaDB$YEAR %in% last_10year, ]

  colnames(metaDB)[colnames(metaDB) == "cgpmgrid_id"] <- "CGPMgrid"
  colnames(metaDB)[colnames(metaDB) == "kg_km2"] <- "B_km2"
  colnames(metaDB)[colnames(metaDB) == "kg_h"] <- "B_h"

  # Create Mean Fish Weight by haul
  metaDB$MEAN_WEIGHT <- round(metaDB$TOTAL_WEIGHT_IN_THE_HAUL / metaDB$TOTAL_NUMBER_IN_THE_HAUL, 3) / 1000

  #----------------------------------------
  # CALCULATION OF THE ABUNDANCE and saving the tables
  #----------------------------------------
  nhauls.grid <- aggregate(metaDB$N_km2, by = list(metaDB$CGPMgrid, metaDB$x_center, metaDB$y_center), length) # [,4]
  colnames(nhauls.grid) <- c("CGPMgrid", "x_center", "y_center", "nhauls")

  cgpmgridlevel <- unique(metaDB$CGPMgrid)
  ntab <- data.frame(CGPMgrid = cgpmgridlevel, GSA = unique(metaDB$GSA))
  ntab$meanNkm2 <- NA
  i <- 1
  for (i in 1:length(cgpmgridlevel)) {
    ddd <- metaDB[metaDB$CGPMgrid == cgpmgridlevel[i], ]
    ntab[i, 3] <- mean(ddd[!is.na(ddd$N_km2), "N_km2"])
    ssd <- (ddd$N_km2 - ntab[i, 3])^2 # sum of squared differences
    variance <- sum(ssd) / (length(ddd$N_km2) - 1) # variance
    ntab$sdNkm2[i] <- sqrt(variance)
    ntab$cvNkm2[i] <- sqrt(variance) / ntab[i, 3]
    ntab$inverse_cvNkm2[i] <- 1 / ntab$cvNkm2[i]
    ntab$nhauls_in_square[i] <- length(ddd[, "HAUL_NUMBER"])
    ntab$nhauls_positive[i] <- length(ddd[ddd$N_km2 > 0, "HAUL_NUMBER"])
  }

  summary.grid <- merge(ntab, nhauls.grid[, 1:3])
  colnames(summary.grid) <- c("cgpmgridlevel", "GSA", "meanNkm2", "sdNkm2", "cvNkm2", "inverse_cvNkm2", "nhauls", "positive_hauls", "lon", "lat")

  abundance_grid <- summary.grid
  abundance_grid$sspp <- sspp
  if(verbose) message("Abundance indices for statistical squares correctly estimated")
  if(verbose) message("Inverse of CV of abundance indices for statistical squares correctly estimated \n")
  if (save) {
    write.table(abundance_grid, paste(wd, "/output/", sspp, " - GFCM GRID ABUNDANCE.csv", sep = ""), sep = ";", row.names = FALSE)

    if (verbose) {
      message(paste("CSV file of abundance indices for statistical squares saved in the output folder."))
    }
  }
  #----------------------------------------
  # CALCULATION OF THE BIOMASS and saving the tables
  #----------------------------------------
  nhauls.grid <- aggregate(metaDB$B_km2, by = list(metaDB$CGPMgrid, metaDB$x_center, metaDB$y_center), length) # [,4]
  colnames(nhauls.grid) <- c("CGPMgrid", "x_center", "y_center", "nhauls")

  cgpmgridlevel <- unique(metaDB$CGPMgrid)
  Btab <- data.frame(CGPMgrid = cgpmgridlevel, GSA = unique(metaDB$GSA))
  Btab$meanBkm2 <- NA
  i <- 1
  for (i in 1:length(cgpmgridlevel)) {
    ddd <- metaDB[metaDB$CGPMgrid == cgpmgridlevel[i], ]
    Btab[i, 3] <- mean(ddd[!is.na(ddd$B_km2), "B_km2"])
    ssd <- (ddd$B_km2 - Btab[i, 3])^2 # sum of squared differences
    variance <- sum(ssd) / (length(ddd$B_km2) - 1) # variance
    Btab$sdBkm2[i] <- sqrt(variance)
    Btab$cvBkm2[i] <- sqrt(variance) / Btab[i, 3]
    Btab$inverse_cvBkm2[i] <- 1 / Btab$cvBkm2[i]
    Btab$nhauls_in_square[i] <- length(ddd[, "HAUL_NUMBER"])
    Btab$nhauls_positive[i] <- length(ddd[ddd$B_km2 > 0, "HAUL_NUMBER"])
  }

  summary.grid <- merge(Btab, nhauls.grid[, 1:3])
  colnames(summary.grid) <- c("cgpmgridlevel", "GSA", "meanBkm2", "sdBkm2", "cvBkm2", "inverse_cvBkm2", "nhauls", "positive_hauls", "lon", "lat")

  biomass_grid <- summary.grid
  biomass_grid$sspp <- sspp

  if (verbose) message("Biomass indices for statistical squares correctly estimated.")
  if (save) {
    write.table(biomass_grid, paste(wd, "/output/", sspp, " - GFCM GRID BIOMASS.csv", sep = ""), sep = ";", row.names = FALSE)
    if (verbose) {
      message(paste("CSV file of Biomass indices for statistical squares saved in the output folder."))
    }
  }
  #----------------------------------------
  # Calculating mean weight
  #----------------------------------------

  nhauls.grid <- aggregate(metaDB$N_km2, by = list(metaDB$CGPMgrid, metaDB$x_center, metaDB$y_center), length) # [,4]
  colnames(nhauls.grid) <- c("CGPMgrid", "x_center", "y_center", "nhauls")

  cgpmgridlevel <- unique(metaDB$CGPMgrid)
  MIWtab <- data.frame(CGPMgrid = cgpmgridlevel, GSA = unique(metaDB$GSA))
  MIWtab$MIW <- NA
  i <- 1
  for (i in 1:length(cgpmgridlevel)) {
    ddd <- metaDB[metaDB$CGPMgrid == cgpmgridlevel[i], ]
    MIWtab[i, 3] <- sum(ddd[!is.na(ddd$MEAN_WEIGHT) & ddd$MEAN_WEIGHT > 0, "MEAN_WEIGHT"]) / length(ddd[!is.na(ddd$MEAN_WEIGHT) & ddd$MEAN_WEIGHT > 0, "MEAN_WEIGHT"])
    ssd <- (ddd[!is.na(ddd$MEAN_WEIGHT) & ddd$MEAN_WEIGHT > 0, "MEAN_WEIGHT"] - MIWtab[i, 3])^2 # sum of squared differences
    variance <- sum(ssd) / (length(ddd[!is.na(ddd$MEAN_WEIGHT) & ddd$MEAN_WEIGHT > 0, "MEAN_WEIGHT"]) - 1) # variance
    MIWtab$sdNkm2[i] <- sqrt(variance)
    MIWtab$cvNkm2[i] <- sqrt(variance) / MIWtab[i, 3]
    MIWtab$inverse_cvNkm2[i] <- 1 / MIWtab$cvNkm2[i]
    MIWtab$nhauls_in_square[i] <- length(ddd[, "HAUL_NUMBER"])
    MIWtab$nhauls_positive[i] <- length(ddd[!is.na(ddd$N_km2) & ddd$N_km2 > 0, "HAUL_NUMBER"])
  }

  summary.grid <- merge(MIWtab, nhauls.grid[, 1:3])
  colnames(summary.grid) <- c("cgpmgridlevel", "GSA", "MIW", "sdMIW", "cvMIW", "inverse_cvMIW", "nhauls", "positive_hauls", "lon", "lat")

  meanWEIGHT_grid <- summary.grid
  meanWEIGHT_grid$sspp <- sspp
  if (verbose) message("MIW for statistical squares correctly estimated.")
  if (save) {
    write.table(meanWEIGHT_grid, paste(wd, "/output/", sspp, " - GFCM GRID MIW.csv", sep = ""), sep = ";", row.names = FALSE)
    if (verbose) {
      message(paste("CSV file of MIW for statistical squares saved in the output folder."))
    }
  }

  #----------------------------------------
  # Plotting indices
  #----------------------------------------

  # plotGFCM(
  #   abundance_grid = abundance_grid,
  #   biomass_grid = biomass_grid,
  #   meanWEIGHT_grid = meanWEIGHT_grid,
  #   map_range = map_range,
  #   stratum = stratum
  # )



  sspp <- unique(abundance_grid$sspp)[1]
  names(map_range) <- c("xmin","xmax","ymin","ymax")
  summary.grid <- abundance_grid
  # metaDB <- mTATB

  x1 <- min(map_range[1])
  x2 <- max(map_range[2])
  y1 <- min(map_range[3])
  y2 <- max(map_range[4])

  #----------------------------------------
  # Saving maps of the ABUNDANCE index
  #----------------------------------------                                                                          # [summary.grid$meanNkm2>5]
  cat_lim <- as.numeric(quantile(summary.grid[!is.na(summary.grid$meanNkm2), "meanNkm2"], probs = seq(0, 1,0.20) ) )
  cate_label <-   round(cat_lim,1)
  cate_label_min <- cate_label
  cate_label_max <- cate_label[2:length(cate_label)]


  cate_label <- paste("(", cate_label_min, ", ", cate_label_max, "]", sep="")
  cate_label[1] <- paste("[", cate_label_min[1], ", ", cate_label_max[1], "]", sep="")
  cate_label <- cate_label[1: (length(cate_label)-1)]

  palette_cate <- heat.colors(length(cate_label))
  palette_cate <- palette_cate[length(palette_cate):1]

  # cgpmgrid <- terra::unwrap(cgpmgrid)
  # continent <- terra::unwrap(continent)
  cgpmgrid$abundance <- NA
  cgpmgrid$color <- "#FFFFFF"
  i=2
  for (i in 1:length(summary.grid[,1])){
    cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- summary.grid[i,"meanNkm2"]
    if(all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[2])  & all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] >= cat_lim[1])){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[1]
    }

    if(all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[3] ) & all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[2])){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[2]
    }

    if(all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[4])  & all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[3])){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[3]
    }

    if(all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[5])  & all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[4])){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[4]
    }

    if(all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[6])  & all(cgpmgrid$abundance[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[5])){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[5]
    }
  }


  lx =map_range["xmax"] - map_range["xmin"]
  ly =map_range["ymax"] - map_range["ymin"]
  ratio <- ly/lx
  img_width <- 30
  img_height <- img_width*ratio

  # par( mar=c(4, 5, 4, 2)) #c(bottom, left, top, right)
  # plot(1,1,type="n",xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]) ,  xlab=expression(paste("Longitude (",degree,"E)")), ylab=expression(paste("Latitude (",degree,"N)")), main=paste(paste("Abundance  (n/km^2)", sep=" "), sspp), asp=1)
  # plot(cgpmgrid,col=cgpmgrid$color, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=TRUE, border="light grey")                                                                                 #data = centroidi_coords,
  # if (stratum == "200,800" | stratum == 3){
  #   plot(mask,col="white", xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")
  # }
  # plot(continent, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), border="grey", col="light grey", add=T)
  # legend("topleft", cate_label, col=palette_cate, pch = 15, pt.cex=c(2), title = "Abundance (n/km^2)\n(classes based on percentiles)", bty = "n")
  # # legend(map_range[1,"xmin"],map_range[1,"ymax"], cate_label, col=palette_cate, pch = 15, pt.cex=c(2), title = "Abundance (n/km^2)\n(classes based on percentiles)", bty = "n")
  # box()
  # p1 <- recordPlot()
  #
  # jpeg(file=paste(wd, "/output/",sspp," - GFCM GRID ABUNDANCE.jpg", sep = ""), width=25, height=25, bg="white", units="cm",res=200)
  # replayPlot(p1)
  # dev.off()

 tab_color <- data.frame(category=cate_label,color=palette_cate,level=seq(1,length(cate_label),1))
 cgpmgrid <- terra::merge(cgpmgrid,tab_color,by.x="color",by.y="color")
 world <- map_data("world")
 xl <- c(x1,x2)
 yl <- c(y1,y2)
 x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
 y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))
 buff=0

 cols <- c("#FFFF80",
           "#FFFF00",
           "#FFAA00",
           "#FF5500",
           "#FF0000")

 names(cols) <- tab_color$category
 # cgpmgrid$level <- as.character( cgpmgrid$level)
 cgpmgrid$level <- is.factor(cgpmgrid$level)
 cgpmgrid$level <- factor(cgpmgrid$level, levels=seq(1,nrow(tab_color),1))
 cgpmgrid <- cgpmgrid %>% arrange(level)

 p1 <- ggplot(cgpmgrid)+
   geom_spatvector(aes(fill = category))+
   scale_fill_manual(values = cols, breaks=names(cols))+
   geom_polygon(data=world, aes(long,lat,group=group), fill="dark grey", colour="darkgrey")+
   coord_sf(xlim = xl, ylim = yl)+
   labs(fill="Abundance\n (n/km^2)")+
   xlab("longitude")+
   ylab("latitude")

  if(verbose) message("Plot of abundance on grid successfully created.")
  if (save){
    ggplot2::ggsave(filename=paste(wd, "/output/",sspp," - GFCM GRID ABUNDANCE.jpg", sep = ""), plot=p1, width=25, height=25, bg="white", units="cm", dpi=200)
    if(verbose) message("Plot of abundance on grid successfully saved")
  }
  #----------------------------------------
  # Saving maps of the inverse of CV of ABUNDANCE index
  #----------------------------------------

  cgpmgrid <- cgpmgrid_bkp
  cat_lim <- quantile(summary.grid$inverse_cvNkm2[!is.na(summary.grid$inverse_cvNkm2)], probs = seq(0, 1,0.20) )
  cate_label <-  round(cat_lim,3)

  cate_label_min <- cate_label
  cate_label_max <- cate_label[2:length(cate_label)]


  cate_label <- paste("(", cate_label_min, ", ", cate_label_max, "]", sep="")
  cate_label[1] <- paste("[", cate_label_min[1], ", ", cate_label_max[1], "]", sep="")

  #cate_label[length(cate_label)] <- paste(">", cate_label_max[length(cate_label_max)] )
  cate_label <- cate_label[1: (length(cate_label)-1)]

  palette_cate <- heat.colors(length(cate_label))
  palette_cate <- palette_cate[length(palette_cate):1]

  cgpmgrid$invCV <- NA
  cgpmgrid$color <- "#FFFFFF"

  i=3
  for (i in 1:length(summary.grid[,1])){
    cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- summary.grid[i,"inverse_cvNkm2"]

    if(all(is.na(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]])) ){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- "#FFFFFF"
    } else {

      if(all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[2])  & all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] >= cat_lim[1])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[1]
      }

      if(all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[3])  & all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[2])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[2]
      }

      if(all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[4])  & all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[3])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[3]
      }

      if(all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[5])  & all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[4])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[4]
      }

      if(all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[6])  & all(cgpmgrid$invCV[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[5])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[5]
      }
    }
  }

  lx =map_range["xmax"] - map_range["xmin"]
  ly =map_range["ymax"] - map_range["ymin"]
  ratio <- ly/lx
  img_width <- 30
  img_height <- img_width*ratio

  # par( mar=c(4, 5, 4, 2)) #c(bottom, left, top, right)
  # plot(1,1,type="n",xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), xlab=expression(paste("Longitude (",degree,"E)")), ylab=expression(paste("Latitude (",degree,"N)")), main=paste("Inverse of Abundance CV -", sspp) , asp=1)
  # plot(cgpmgrid,col=cgpmgrid$color, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")                                                                                 #data = centroidi_coords,
  # if (stratum == "200,800" | stratum == 3){
  #   plot(mask,col="white", xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")
  # }
  # plot(continent, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), border="grey", col="light grey", add=T)
  # legend("topleft", cate_label, col=palette_cate, pch = 15, pt.cex=c(2), title = "Inverse of Abundance CV\n(classes based on percentiles)", bty = "n")
  # box()
  # p2 <- recordPlot()

  tab_color <- data.frame(category=cate_label,color=palette_cate,level=seq(1,length(cate_label),1))
  cgpmgrid <- terra::merge(cgpmgrid,tab_color,by.x="color",by.y="color")
  world <- map_data("world")
  xl <- c(x1,x2)
  yl <- c(y1,y2)
  x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
  y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))
  buff=0

  cols <- c("#FFFF80",
            "#FFFF00",
            "#FFAA00",
            "#FF5500",
            "#FF0000")

  names(cols) <- tab_color$category
  # cgpmgrid$level <- as.character( cgpmgrid$level)
  cgpmgrid$level <- is.factor(cgpmgrid$level)
  cgpmgrid$level <- factor(cgpmgrid$level, levels=seq(1,nrow(tab_color),1))
  cgpmgrid <- cgpmgrid %>% arrange(level)

  p2 <- ggplot(cgpmgrid)+
    geom_spatvector(aes(fill = category))+
    scale_fill_manual(values = cols, breaks=names(cols))+
    geom_polygon(data=world, aes(long,lat,group=group), fill="dark grey", colour="darkgrey")+
    coord_sf(xlim = xl, ylim = yl)+
    labs(fill="Inverse of\n Abundance CV")+
    xlab("longitude")+
    ylab("latitude")

  if(verbose) message("Plot of abundance inverse CV on grid successfully created.")
  if(save){
    ggplot2::ggsave(filename=paste(wd, "/output/",sspp," - GFCM GRID ABUNDANCE Inverse CV.jpg", sep = ""), plot=p2, width=25, height=25, bg="white", units="cm", dpi=200)
    if(verbose) message("Plot of abundance inverse CV on grid successfully saved.")
  }


  ###################
  ###################
  ###             ###
  ###   BIOMASS   ###
  ###             ###
  ###################
  ###################

  cgpmgrid <- cgpmgrid_bkp
  summary.grid <- biomass_grid
  # metaDB <- mTATB

  #----------------------------------------
  # Saving maps of the BIOMASS index
  #----------------------------------------

  cat_lim <- quantile(summary.grid[!is.na(summary.grid$meanBkm2), "meanBkm2"], probs = seq(0, 1,0.20) )
  cate_label <-  round(cat_lim,3)

  cate_label_min <- cate_label
  # cate_label_min[1] <- 0
  cate_label_max <- cate_label[2:length(cate_label)]


  cate_label <- paste("(", cate_label_min, ", ", cate_label_max, "]", sep="")
  cate_label[1] <- paste("[", cate_label_min[1], ", ", cate_label_max[1], "]", sep="")

  cate_label <- cate_label[1: (length(cate_label)-1)]
  # cate_label[length(cate_label)] <- paste(">", cate_label_max[length(cate_label_max)] )


  palette_cate <- heat.colors(length(cate_label))
  palette_cate <- palette_cate[length(palette_cate):1]


  cgpmgrid$meanBkm2 <- NA
  cgpmgrid$color <- "#FFFFFF"

  i=3
  for (i in 1:length(summary.grid[,1])){
    cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- summary.grid[i,"meanBkm2"]

    if(all(is.na(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]])) ){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- "#FFFFFF"
    } else {

      if(all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[2] ) & all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] >= cat_lim[1])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[1]
      }

      if(all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[3])  & all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[2])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[2]
      }

      if(all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[4])  & all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[3])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[3]
      }

      if(all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[5])  & all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[4])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[4]
      }

      if(all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[6])  & all(cgpmgrid$meanBkm2[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[5])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[5]
      }
    }
  }


  lx =map_range["xmax"] - map_range["xmin"]
  ly =map_range["ymax"] - map_range["ymin"]
  ratio <- ly/lx
  img_width <- 30
  img_height <- img_width*ratio

  # par( mar=c(4, 5, 4, 2)) #c(bottom, left, top, right)
  # plot(1,1,type="n",xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), xlab=expression(paste("Longitude (",degree,"E)")), ylab=expression(paste("Latitude (",degree,"N)")), main=paste("Biomass (kg/km^2) -", sspp), asp=1 )
  # plot(cgpmgrid,col=cgpmgrid$color, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")                                                                                 #data = centroidi_coords,
  # if (stratum == "200,800" | stratum == 3){
  #   plot(mask,col="white", xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")
  # }
  # plot(continent, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), border="grey", col="light grey", add=T)
  #
  # legend("topleft", cate_label, col=palette_cate, pch = 15, pt.cex=c(2), title = "Biomass (kg/km^2)\n(classes based on percentiles)", bty = "n")
  # box()
  # p3 <- recordPlot()

  tab_color <- data.frame(category=cate_label,color=palette_cate,level=seq(1,length(cate_label),1))
  cgpmgrid <- terra::merge(cgpmgrid,tab_color,by.x="color",by.y="color")
  world <- map_data("world")
  xl <- c(x1,x2)
  yl <- c(y1,y2)
  x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
  y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))
  buff=0

  cols <- c("#FFFF80",
            "#FFFF00",
            "#FFAA00",
            "#FF5500",
            "#FF0000")

  names(cols) <- tab_color$category
  # cgpmgrid$level <- as.character( cgpmgrid$level)
  cgpmgrid$level <- is.factor(cgpmgrid$level)
  cgpmgrid$level <- factor(cgpmgrid$level, levels=seq(1,nrow(tab_color),1))
  cgpmgrid <- cgpmgrid %>% arrange(level)

  p3 <- ggplot(cgpmgrid)+
    geom_spatvector(aes(fill = category))+
    scale_fill_manual(values = cols, breaks=names(cols))+
    geom_polygon(data=world, aes(long,lat,group=group), fill="dark grey", colour="darkgrey")+
    coord_sf(xlim = xl, ylim = yl)+
    labs(fill="Biomass\n (kg/km^2)")+
    xlab("longitude")+
    ylab("latitude")

  if(verbose) message("Plot of biomass on grid successfully created.")
  if(save){
    ggplot2::ggsave(filename=paste(wd, "/output/",sspp," - GFCM GRID BIOMASS.jpg", sep = ""), plot=p3, width=25, height=25, bg="white", units="cm", dpi=200)
    if(verbose) message("Plot of biomass on grid successfully saved")
  }



  ###################
  ###################
  ###             ###
  ###     MIW     ###
  ###             ###
  ###################
  ###################

  cgpmgrid <- cgpmgrid_bkp
  summary.grid <- meanWEIGHT_grid
  # metaDB <- mTATB

  #----------------------------------------
  # Saving maps of the MIW
  #----------------------------------------                                                                          # [summary.grid$meanNkm2>5]
  cat_lim <- as.numeric(quantile(summary.grid[!is.na(summary.grid$MIW), "MIW"], probs = seq(0, 1,0.20) ) )
  #cat_lim[1] <- 0
  #  cate_label[len] <- ifelse( round(cat_lim[len],0) != 0, round(cat_lim[len],0), ifelse( round(cat_lim[len],1) != 0,  round(cat_lim[len],1),  round(cat_lim[len],2)))
  cate_label <-   round(cat_lim,3)
  cate_label_min <- cate_label
  #cate_label_min[1] <- 0
  cate_label_max <- cate_label[2:length(cate_label)]


  cate_label <- paste("(", cate_label_min, ", ", cate_label_max, "]", sep="")
  cate_label[1] <- paste("[", cate_label_min[1], ", ", cate_label_max[1], "]", sep="")

  #cate_label[length(cate_label)] <- paste(">", cate_label_max[length(cate_label_max)] )
  cate_label <- cate_label[1: (length(cate_label)-1)]

  palette_cate <- heat.colors(length(cate_label))
  palette_cate <- palette_cate[length(palette_cate):1]

  cgpmgrid$MIW <- NA
  cgpmgrid$color <- "#FFFFFF"

  i=3
  for (i in 1:length(summary.grid[,1])){
    cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- summary.grid[i,"MIW"]

    if(all(is.na(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]])) ){
      cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- "#FFFFFF"
    } else {

      if(all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[2])  & all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] >= cat_lim[1])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[1]
      }

      if(all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[3])  & all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[2])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[2]
      }

      if(all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[4])  & all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[3])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[3]
      }

      if(all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[5])  & all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[4])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[4]
      }

      if(all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <= cat_lim[6])  & all(cgpmgrid$MIW[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] > cat_lim[5])){
        cgpmgrid$color[cgpmgrid$GFCM_ID==summary.grid[i,"cgpmgridlevel"]] <- palette_cate[5]
      }
    }
  }


  lx =map_range["xmax"] - map_range["xmin"]
  ly =map_range["ymax"] - map_range["ymin"]
  ratio <- ly/lx
  img_width <- 30
  img_height <- img_width*ratio


  # par( mar=c(4, 5, 4, 2)) #c(bottom, left, top, right)
  # plot(1,1,type="n",xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]) ,  xlab=expression(paste("Longitude (",degree,"E)")), ylab=expression(paste("Latitude (",degree,"N)")), main=paste(paste("Mean Individual Weight  (kg)", sep=" "), sspp), asp=1)
  # plot(cgpmgrid, col=cgpmgrid$color, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")
  # if (stratum == "200,800" | stratum == 3){
  #   plot(mask,col="white", xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), add=T, border="light grey")
  # }
  # plot(continent, xlim=c(map_range["xmin"],map_range["xmax"]), ylim=c(map_range["ymin"],map_range["ymax"]), border="grey", col="light grey", add=T)
  #
  # legend("topleft", cate_label, col=palette_cate, pch = 15, pt.cex=c(2), title = "MIW (kg)\n(classes based on percentiles)", bty = "n")
  #
  # box()
  # p4 <- recordPlot()

  tab_color <- data.frame(category=cate_label,color=palette_cate,level=seq(1,length(cate_label),1))
  cgpmgrid <- terra::merge(cgpmgrid,tab_color,by.x="color",by.y="color")
  world <- map_data("world")
  xl <- c(x1,x2)
  yl <- c(y1,y2)
  x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
  y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))
  buff=0

  cols <- c("#FFFF80",
            "#FFFF00",
            "#FFAA00",
            "#FF5500",
            "#FF0000")

  names(cols) <- tab_color$category
  # cgpmgrid$level <- as.character( cgpmgrid$level)
  cgpmgrid$level <- is.factor(cgpmgrid$level)
  cgpmgrid$level <- factor(cgpmgrid$level, levels=seq(1,nrow(tab_color),1))
  cgpmgrid <- cgpmgrid %>% arrange(level)

  p4 <- ggplot(cgpmgrid)+
    geom_spatvector(aes(fill = category))+
    scale_fill_manual(values = cols, breaks=names(cols))+
    geom_polygon(data=world, aes(long,lat,group=group), fill="dark grey", colour="darkgrey")+
    coord_sf(xlim = xl, ylim = yl)+
    labs(fill="MIW (kg)")+
    xlab("longitude")+
    ylab("latitude")

  if(verbose) message("Plot of MIW on grid successfully created.")
  if(save){
    ggplot2::ggsave(filename=paste(wd, "/output/",sspp," - GFCM GRID MIW.jpg", sep = ""), plot=p4, width=25, height=25, bg="white", units="cm", dpi=200)
    if(verbose) message("Plot of MIW on grid successfully saved.")
  }


  return(list(abundance_grid, biomass_grid, meanWEIGHT_grid))
}

#' Length Frequency Distribution (LFD) Estimation and Plotting
#'
#' This function estimates and plots the length frequency distribution (LFD) by year and by stratum,
#' based on merged biological and haul data from MEDITS surveys. It computes raised numbers,
#' applies stratification weights, and generates publication-ready plots and CSV outputs.
#' Stratification weighting is based on swept area and surface area of the strata.
#'
#' @description Input parameters include biological and haul data, stratification schemes,
#' reference filters (e.g., sex, GSA, country, depth), and output control options (working directory, saving, verbosity).
#'
#' @param mTATC Data frame resulting from the merge of TA and TC datasets using `merge_TATBTC()`. It must include raised numbers.
#' @param sex Character. Target sex for the analysis. Allowed values: `"F"`, `"M"`, `"I"`, `"N"`, or `"all"` (default) for combined sexes.
#' @param GSA Numeric. The GSA (Geographical Sub-Area) code of reference for the analysis.
#' @param country Character vector. Reference country or countries for the analysis. Use `"all"` (default) to include all countries available in the data.
#' @param depth_range Numeric vector of length 2, specifying the minimum and maximum depths (in meters) to filter the hauls used in the analysis.
#' @param strata_scheme Data frame containing the stratification scheme. Must include `CODE`, `MIN_DEPTH`, `MAX_DEPTH`, `GSA`, `COUNTRY`.
#' @param stratification Data frame with surface areas per stratum. Must include columns: `GSA`, `CODE`, `COUNTRY`, `SURF`.
#' @param wd Character. Working directory used to save output plots and tables. Required if `save = TRUE`.
#' @param save Logical. If `TRUE` (default), the function saves output tables and plots to `wd/output/`. If `FALSE`, no files are saved.
#' @param verbose Logical. If `TRUE` (default), informative messages are printed to the console to trace the function steps.
#'
#' @return A named list with two elements:
#' \describe{
#'   \item{`LFD`}{A data frame of total LFD by year across all strata.}
#'   \item{`LFD by stratum`}{A data frame of LFD by year and by stratum.}
#' }
#'
#' @details
#' The function:
#' \itemize{
#'   \item Filters hauls by depth and country.
#'   \item Computes raised numbers per haul.
#'   \item Applies stratified weights based on the area per stratum.
#'   \item Outputs and/or plots the length frequency distribution by year and stratum.
#'   \item Handles sex-specific or combined-sex analyses.
#' }
#'
#' The plot outputs include:
#' \itemize{
#'   \item Combined LFD by year across all strata.
#'   \item LFD by stratum (faceted), stratified by year.
#' }
#'
#' Output files are saved as CSV and high-resolution JPEGs if `save = TRUE` and `wd` is defined.
#'
#' @importFrom dplyr summarise summarize group_by arrange
#' @importFrom reshape2 melt dcast
#' @importFrom ggplot2 ggplot geom_line xlab ylab ggtitle xlim ylim theme_bw aes facet_wrap
#' @return A \code{list} of \code{ggplot} objects representing the length-frequency distributions.
#' @examples
#' data(TA)
#' data(TC)
#' # Create a merged dataset for GSA 10
#' mTATC <- merge_TATC(TA[TA$AREA==10,], TC[TC$AREA==10,], "MERLMER")
#' LFD(mTATC, sex="all", GSA=10, country="all", depth_range=c(10,800),
#'     strata_scheme=BioIndex::strata_scheme,
#'     stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
#' @export
LFD <- function(mTATC, sex="all", GSA, country="all", depth_range, strata_scheme, stratification, wd=NA, save=TRUE, verbose=TRUE){
    if (is.na(wd)) { wd <- tempdir() }

    id <- NULL
    if (FALSE) {

        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        # ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        # tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        # tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)

        ta <- read.table(paste("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\B-USEFUL\\_MEDITS_Corrected_data_\\TA_GSA18_1999_2021_completo.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\B-USEFUL\\_MEDITS_Corrected_data_\\TB_GSA18_1999_2021_completo.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\B-USEFUL\\_MEDITS_Corrected_data_\\TC_GSA18_1999_2021_completo.csv", sep="/"), sep=";", header=T)
        # ta <- ta[ta$YEAR==2017, ]
        # tb <- tb[tb$YEAR==2017, ]
        # tc <- tc[tc$YEAR==2017, ]
        depth_range=c(10,800)
        country="all"
        sex <- "all"
        species <- "MERLMER"
        m <- merge_TATBTC(ta, tb, tc, species=species, country=country, wd=wd, verbose=TRUE)
        mTATC <- m[[2]]

        LFD(mTATC, sex=sex, GSA=10, country=country, depth_range=c(10,800), strata_scheme, stratification, wd, save=TRUE)

    }


    if (is.na(wd) & save) {
        save=TRUE
        if (verbose){
            message("Missing working directory. Results are not saved in the local folder.")
        }
    }

    Abundance <- LENGTH_CLASS <- STRATUM <- STRATUM_CODE <- SWEPT_AREA <- YEAR <-
    analysis_stratum1 <- analysis_stratum2 <- analysis_stratum3 <-
    analysis_stratum4 <- analysis_stratum5 <- analysis_stratum6 <- n_raised <-
    num_weighted <- NULL

    mTATC$n_raised <- mTATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE * mTATC$RAISING_FACTOR

    ddd <-  mTATC
    class_code <- unique(ddd[ddd$GENUS != -1, "LENGTH_CLASSES_CODE"])
    if (class_code == "m") {LC <- 1} else {
        if (class_code == "0") {LC <- 5} else {
            if (class_code == "1") {LC <- 10}
        }
    }


    GENERE <- as.character(unique(mTATC$GENUS)[unique(mTATC$GENUS) != -1])
    SPECIE <- as.character(unique(mTATC$SPECIES)[unique(mTATC$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")
    GSA <- unique(mTATC$GSA)
    species <- sspp



    countries <- unique(mTATC[!is.na(mTATC$COUNTRY),"COUNTRY"])
    l_country <- length(unique(mTATC$COUNTRY))

    if (l_country==1){
        check_country ="Y"
        country_analysis <- countries
    } else {
        if (any(country %in% "all")) {
            country_analysis <<- countries
        } else {
            country_analysis <<- country
        }
    }

    strata_scheme <- strata_scheme[strata_scheme$GSA == GSA & strata_scheme$COUNTRY %in% as.character(unique(country_analysis))[1], ]

    stratification$SURF <- as.numeric(stratification$SURF)

    depth <- data.frame(matrix(NA,ncol=4, nrow=length(strata_scheme$CODE)))
    colnames(depth) <- c("strata", "min", "max", "bul")
    depth$strata <- strata_scheme$CODE
    depth$min <-strata_scheme$MIN_DEPTH
    depth$max <-strata_scheme$MAX_DEPTH
    if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
    mTATC <-  mTATC[mTATC$MEAN_DEPTH>depth_range[1] & mTATC$MEAN_DEPTH<=depth_range[2],]
    ddd <-  ddd[ddd$MEAN_DEPTH>depth_range[1] & ddd$MEAN_DEPTH<=depth_range[2],]
    idx_from <- which(depth[,"min"] == depth_range[1])
    idx_to   <- which(depth[,"max"] == depth_range[2])

    if (length(idx_from) != 1 || length(idx_to) != 1) {
        stop(paste0("Error: depth range [", depth_range[1], ", ", depth_range[2],
                    "] does not match the strata boundaries defined for this GSA. ",
                    "Available min depths: ", paste(sort(unique(depth$min)), collapse=", "),
                    "; Available max depths: ", paste(sort(unique(depth$max)), collapse=", ")))
    }
    strata_range <- seq(idx_from, idx_to, 1)
    depth[depth$strata %in% strata_range, "bul"] <- T
    depth[!(depth$strata %in% strata_range), "bul"] <- F

    analysis_stratum1 <- NA
    analysis_stratum2 <- NA
    analysis_stratum3 <- NA
    analysis_stratum4 <- NA
    analysis_stratum5 <- NA
    analysis_stratum6 <- NA

    analysis_stratum1 <- depth[1,4]
    analysis_stratum2 <- depth[2,4]
    analysis_stratum3 <- depth[3,4]
    analysis_stratum4 <- depth[4,4]
    analysis_stratum5 <- depth[5,4]
    analysis_stratum6 <- depth[6,4]

    if (is.na(analysis_stratum1)){depth[1,4]<- F; analysis_stratum1 <- F}
    if (is.na(analysis_stratum2)){depth[2,4]<- F; analysis_stratum2 <- F}
    if (is.na(analysis_stratum3)){depth[3,4]<- F; analysis_stratum3 <- F}
    if (is.na(analysis_stratum4)){depth[4,4]<- F; analysis_stratum4 <- F}
    if (is.na(analysis_stratum5)){depth[5,4]<- F; analysis_stratum5 <- F}
    if (is.na(analysis_stratum6)){depth[6,4]<- F; analysis_stratum6 <- F}


    if (analysis_stratum1 == T){area_s1 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==1 & stratification$COUNTRY %in% country_analysis,5])} else {area_s1 <- 0}
    if (analysis_stratum2 == T){area_s2 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==2 & stratification$COUNTRY %in% country_analysis,5])} else {area_s2 <- 0}
    if (analysis_stratum3 == T){area_s3 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==3 & stratification$COUNTRY %in% country_analysis,5])} else {area_s3 <- 0}
    if (analysis_stratum4 == T){area_s4 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==4 & stratification$COUNTRY %in% country_analysis,5])} else {area_s4 <- 0}
    if (analysis_stratum5 == T){area_s5 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==5 & stratification$COUNTRY %in% country_analysis,5])} else {area_s5 <- 0}
    if (analysis_stratum6 == T){area_s6 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==6 & stratification$COUNTRY %in% country_analysis,5])} else {area_s6 <- 0}


    peso_s1 <- area_s1/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s2 <- area_s2/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s3 <- area_s3/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s4 <- area_s4/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s5 <- area_s5/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    peso_s6 <- area_s6/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
    strata_scheme$WEIGHT <- 0
    if ("1" %in% strata_scheme$CODE) {strata_scheme[strata_scheme$CODE =="1", "WEIGHT"] <- peso_s1}
    if ("2" %in% strata_scheme$CODE) {strata_scheme[strata_scheme$CODE =="2", "WEIGHT"] <- peso_s2}
    if ("3" %in% strata_scheme$CODE) {strata_scheme[strata_scheme$CODE =="3", "WEIGHT"] <- peso_s3}
    if ("4" %in% strata_scheme$CODE) {strata_scheme[strata_scheme$CODE =="4", "WEIGHT"] <- peso_s4}
    if ("5" %in% strata_scheme$CODE) {strata_scheme[strata_scheme$CODE =="5", "WEIGHT"] <- peso_s5}
    if ("6" %in% strata_scheme$CODE) {strata_scheme[strata_scheme$CODE =="6", "WEIGHT"] <- peso_s6}


    ddd <- mTATC[mTATC$LENGTH_CLASS != -1,]
    if (!sex %in% c("I","N","M","F","all")) {
        stop("Unexpected sex selection for the analysis.\n")
    } else if (sex=="I") {
        ddd <- ddd[ddd$SEX == "I", ]
        # mTATC <- mTATC[mTATC$SEX == "I", ]
    } else if (sex=="N") {
        ddd <- ddd[ddd$SEX == "N", ]
        # mTATC <- mTATC[mTATC$SEX == "N", ]
    } else if (sex=="M") {
        ddd <- ddd[ddd$SEX == "M", ]
        # mTATC <- mTATC[mTATC$SEX == "M", ]
    } else if (sex=="F") {
        ddd <- ddd[ddd$SEX == "F", ]
        # mTATC <- mTATC[mTATC$SEX == "F", ]
    }

    if (nrow(ddd) != 0) {
        ddd <- ddd[ddd$GSA %in% GSA & ddd$COUNTRY %in% as.character(country_analysis), ]
        mTATC <- mTATC[mTATC$GSA %in% GSA & mTATC$COUNTRY %in% as.character(country_analysis), ]
    }

    if (nrow(ddd) != 0 & nrow(mTATC) != 0) {

        df1 <- suppressMessages( data.frame(ddd %>% group_by(YEAR, STRATUM_CODE,LENGTH_CLASS) %>% summarise(numb_per_stratum = sum(n_raised))))
        df1$id <- paste(df1$YEAR,df1$STRATUM_CODE,sep="_")

        df2t <-  suppressMessages(data.frame(mTATC %>% group_by(id, YEAR, STRATUM_CODE) %>% summarise(SWEPT_AREA=mean(SWEPT_AREA))))
        df2t$id <- paste(df2t$YEAR,df2t$STRATUM_CODE,sep="_")
        df2 <- suppressMessages(data.frame(df2t %>% group_by(id, YEAR, STRATUM_CODE) %>% summarise(SWEPT_AREA=sum(SWEPT_AREA))))

        mdf <- merge(df1,df2[, c("id", "SWEPT_AREA")], by.x="id",by.y="id",all=TRUE)
        mdf$n_km <- mdf$numb_per_stratum / mdf$SWEPT_AREA
        mw <- merge(mdf,strata_scheme[, c("CODE","WEIGHT")],by.x="STRATUM_CODE", by.y="CODE",all.x=TRUE)
        mw$num_weighted <- mw$n_km * mw$WEIGHT
        mw <- mw[!is.na(mw$STRATUM_CODE),]


        df <- dcast(mw, LENGTH_CLASS + YEAR ~ STRATUM_CODE, value.var = "n_km")
        strata <- colnames(df[3:ncol(df)])
        strata <- paste("STRATUM",strata,sep=" ")
        colnames(df) <- c("LC","YEAR", strata)
        LC_range <- range(mw$LENGTH_CLASS)
        LCs <- seq(LC_range[1],LC_range[2],LC)
        LCs <- data.frame(LC = LCs)
        years <- sort(unique(df$YEAR))
        msc <- list()
        y=1
        for (y in 1:length(years)) {
            temp <- merge(LCs,df[df$YEAR == years[y], ], by.x="LC",by.y="LC",all.x=TRUE, all.y=TRUE)
            temp$YEAR <- years[[y]]
            temp[is.na(temp)] <- 0
            msc[[y]] <- temp
        }

        mst <- do.call(rbind,msc)
        mst$id <- paste(mst$YEAR,mst$LC,sep="_")
        df_all <- suppressMessages(data.frame(mw %>% group_by(YEAR,LENGTH_CLASS) %>% summarise(`ALL STRATA` = sum(num_weighted))))
        df_all$id <- paste(df_all$YEAR,df_all$LENGTH_CLASS,sep="_")
        m <- merge(mst,df_all[, c("ALL.STRATA","id")], by.x="id",by.y="id", all.x=TRUE, all.y=TRUE)
        m[is.na(m)] <- 0

        m2 <- m[, -c(which(colnames(m)=="id"))]
        m2 <- m2 %>% arrange(YEAR,LC)
        m2all <- m2[, c("LC","YEAR","ALL.STRATA")]
        m2all <- dcast(m2all, LC ~ YEAR, value.var = "ALL.STRATA")

        m2s <-  m2[, -(which(colnames(m2)=="ALL.STRATA"))]
        m2s <- melt(m2s, id.vars=c("LC","YEAR"),variable.name = "STRATUM",value.name = "Abundance")
        m2s <- dcast(m2s, LC + STRATUM ~ YEAR, value.var = "Abundance")
        m2s <- m2s %>% arrange(STRATUM, LC)
        ss <- strsplit(as.character(m2s$STRATUM)," ")
        ss <- do.call(rbind,ss)[,2]
        m2s$STRATUM <- ss

        if (verbose) message("LFD successfully estimated.")

     if (save) {
        if (sex=="all"){
        write.table(m2s,paste(wd, "/output/",sspp,"_GSA",GSA,"_LFD_(Combined_by_stratum).csv", sep=""),sep=";",row.names=F)
        write.table(m2all,paste(wd, "/output/",sspp,"_GSA",GSA,"_LFD_(Combined).csv", sep=""),sep=";",row.names=F)
        if (verbose) message("LFD tables saved in output folder.")
        } else {
        write.table(m2s,paste(wd, "/output/",sspp,"_GSA",GSA,"_LFD_(",sex,"_by_stratum).csv", sep=""),sep=";",row.names=F)
        write.table(m2all,paste(wd, "/output/",sspp,"_GSA",GSA,"_LFD_(",sex,").csv", sep=""),sep=";",row.names=F)
        if (verbose) message("LFD tables saved in output folder.")
            }
     }

        m <- melt(m, id.vars=c("id","LC","YEAR"),variable.name = "STRATUM",value.name = "Abundance")
        m$YEAR <- as.factor(m$YEAR)

# PLOT by year
        if (sex=="all"){
            main <- paste(sspp,"_GSA",GSA,"_(LFD_by_year_Combained)_",depth_range[1],"-",depth_range[2], " m", sep="")
            main.lab <- paste(sspp," GSA",GSA," (LFD by year, Combained) ",depth_range[1],"-",depth_range[2], " m", sep="")
            dep_text <-expression(paste("LFD", sep=" "))
            min_lc <- min(m[m$STRATUM == "ALL.STRATA", "LC"])
            max_lc <- max(m[m$STRATUM == "ALL.STRATA", "LC"])
            max_freq <- max(m[m$STRATUM == "ALL.STRATA", "Abundance"])
            if (!is.finite(max_freq) || max_freq == 0) max_freq <- 1

            p <- ggplot(data=m[m$STRATUM == "ALL.STRATA", ], aes(x=LC, y=Abundance, group=YEAR, colour=YEAR)) +
                geom_line() +
                xlab("LC") + ylab(expression(paste("Abundance ", (n/km^2), sep=" "))) +
                ggtitle(main.lab) +
                xlim(min_lc, max_lc) +
                ylim(0,max_freq) +
                theme_bw()
            # print(p) # rimosso per policy CRAN
            if(save){
                if(save) ggsave(paste(wd,"/output/",main,".jpg",sep=""), dpi=300 , width=6, height=5, plot = p) #
                }
        } else {
            main <- paste(sspp,"_GSA",GSA,"_(LFD_by_year_",toupper(sex),")_",depth_range[1],"-",depth_range[2], " m", sep="")
            main.lab <- paste(sspp," GSA",GSA," (LFD by year, ",toupper(sex),") ",depth_range[1],"-",depth_range[2], " m", sep="")
            dep_text <-expression(paste("LFD", sep=" "))
            min_lc <- min(m[m$STRATUM == "ALL.STRATA", "LC"])
            max_lc <- max(m[m$STRATUM == "ALL.STRATA", "LC"])
            max_freq <- max(m[m$STRATUM == "ALL.STRATA", "Abundance"])
            if (!is.finite(max_freq) || max_freq == 0) max_freq <- 1

            p <- ggplot(data=m[m$STRATUM == "ALL.STRATA", ], aes(x=LC, y=Abundance, group=YEAR, colour=YEAR)) +
                geom_line() +
                xlab("LC") + ylab(expression(paste("Abundance ", (n/km^2), sep=" "))) +
                ggtitle(main.lab) +
                xlim(min_lc, max_lc) +
                ylim(0,max_freq) +
                theme_bw()
            # print(p) # rimosso per policy CRAN
            if (save){
                if(save) ggsave(paste(wd,"/output/",main,".jpg",sep=""), dpi=300 , width=6, height=5, plot = p)
                }
        }



        # plot by strata

        if (sex=="all"){
            main <- paste(sspp,"_GSA",GSA,"_(LFD_by_stratum_Combained)_",depth_range[1],"-",depth_range[2], " m", sep="")
            main.lab <- paste(sspp," GSA",GSA," (LFD by stratum, Combained) ",depth_range[1],"-",depth_range[2], " m", sep="")
            dep_text <-expression(paste("LFD", sep=" "))
            min_lc <- min(m[m$STRATUM != "ALL.STRATA", "LC"])
            max_lc <- max(m[m$STRATUM != "ALL.STRATA", "LC"])
            max_freq <- max(m[m$STRATUM != "ALL.STRATA", "Abundance"])
            if (!is.finite(max_freq) || max_freq == 0) max_freq <- 1

            p2 <- ggplot(data=m[m$STRATUM != "ALL.STRATA", ], aes(x=LC, y=Abundance, group=YEAR, colour=YEAR)) +
                geom_line(linewidth=0.3) +
                xlab("LC") + ylab(expression(paste("Abundance ", (n/km^2), sep=" "))) +
                ggtitle(main.lab) +
                xlim(min_lc, max_lc) +
                ylim(0,max_freq) +
                theme_bw()+
                facet_wrap(~ STRATUM)  # , ncol=cols.graph
            # print(p2) # rimosso per policy CRAN
            if (save) {
                if(save) ggsave(paste(wd,"/output/",main,".jpg",sep=""), dpi=300 , width=6, height=5, plot = p2) #
            }
        } else {
            main <- paste(sspp,"_GSA",GSA,"_(LFD_by_stratum_",toupper(sex),")_",depth_range[1],"-",depth_range[2], " m", sep="")
            main.lab <- paste(sspp," GSA",GSA," (LFD by stratum, ",toupper(sex),") ",depth_range[1],"-",depth_range[2], " m", sep="")
            dep_text <-expression(paste("LFD", sep=" "))
            min_lc <- min(m[m$STRATUM != "ALL.STRATA", "LC"])
            max_lc <- max(m[m$STRATUM != "ALL.STRATA", "LC"])
            max_freq <- max(m[m$STRATUM != "ALL.STRATA", "Abundance"])
            if (!is.finite(max_freq) || max_freq == 0) max_freq <- 1

            p2 <- ggplot(data=m[m$STRATUM != "ALL.STRATA", ], aes(x=LC, y=Abundance, group=YEAR, colour=YEAR)) +
                geom_line(linewidth=0.3) +
                xlab("LC") + ylab(expression(paste("Abundance ", (n/km^2), sep=" "))) +
                ggtitle(main.lab) +
                xlim(min_lc, max_lc) +
                ylim(0,max_freq) +
                theme_bw()+
                facet_wrap(~ STRATUM)  # , ncol=cols.graph
            # print(p2) # rimosso per policy CRAN
            if(save){
                if(save) ggsave(paste(wd,"/output/",main,".jpg",sep=""), dpi=300 , width=6, height=5, plot = p2)
            }
        }

return(list("LFD"=m2all,"LFD by stratum"=m2s))
    } else {
        if (verbose) message("Not enough data for LFD estimation.")
    }

}

#' Check hauls TB TA (RoME)
#' @description
#' Check if all the hauls in TB are in TA.
#'
#' @param DataTA MEDITS TA table
#' @param DataTB MEDITS TB table
#' @param year reference year for the analysis
#' @param wd working directory
#' @param suffix name of the log file
#' @note
#' This function is an internal routine based on \strong{RoME version 0.2.3}.
#' It is provided within \code{BioIndex} to ensure the package remains
#' functional and self-sufficient for data validation.
#' @return \code{TRUE} if the validation passes, \code{FALSE} otherwise.
#' @examples
#' data(TA)
#' data(TB)
#' check_hauls_TBTA(TA, TB, year = 2007)
#' @export
check_hauls_TBTA <- function(DataTA, DataTB, year, wd=NA, suffix=NA) {

  if (is.na(wd)) {
    wd <- tempdir()
  }

  if (!file.exists(file.path(wd, "Logfiles"))) {
    dir.create(file.path(wd, "Logfiles"), recursive = TRUE, showWarnings = FALSE)
  }
  
  if (missing(suffix) || is.na(suffix)) {
    suffix <- paste0(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"))
  }
  
  Errors <- file.path(wd, "Logfiles", paste0("Logfile_", suffix, ".dat"))
  if (!file.exists(Errors)) {
    file.create(Errors)
  }

  if (length(year) != 1 || is.na(year)) {
    stop("Argument 'year' must be a single non-NA value")
  }
  
  DataTA <- DataTA[DataTA$YEAR == year, ]
  DataTB <- DataTB[DataTB$YEAR == year, ]

  if (nrow(DataTB) == 0) {
    write(paste("\n----------- check presence in TA of TB hauls - ", year), file = Errors, append = TRUE)
    write("No data found for this year in TB", file = Errors, append = TRUE)
    return(TRUE)
  }

  write(paste("\n----------- check presence in TA of TB hauls - ", year), file = Errors, append = TRUE)

  numberError <- 0
  unique_hauls_TB <- unique(DataTB$HAUL_NUMBER)
  
  for (h in unique_hauls_TB) {
    if (!(h %in% DataTA$HAUL_NUMBER)) {
      write(paste("No haul", h, "in TA"), file = Errors, append = TRUE)
      numberError <- numberError + 1
    }
  }

  if (numberError == 0) {
    write("No error occurred", file = Errors, append = TRUE)
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' Estimation of ALK
#'
#' @param te MEDITS or MEDITS-like TE table
#' @param sp species RUBIN code (MEDITS format, e.g. "MERLMER")
#' @param GEAR type of gear reported in the corresponding TA file
#' @param GSA reference GSA for the analysis
#' @param country reference country
#' @param years number of years to be considered in the analysis
#' @importFrom stats predict AIC nls
#' @param wd path of the working directory
#' @param save boolean. If TRUE the outputs are saved in the local folder
#' @return A \code{data.frame} representing the Age-Length Key for females.
#' 

ALKf <- function(te, sp, GEAR, GSA, country = NA, years = 5, wd = NA, save=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }

    if (FALSE) {
        TE <- te #[te$AGE <30, ] # read.table("D:\\OneDrive - Coispa Tecnologia & Ricerca S.C.A.R.L\\BLACK SEA\\2023 - Data preparation - RAPA\\-DATA-\\BGR\\Spring 2022\\TE.csv", sep = ";", header = TRUE)
        # TE <- read.table("D:\\Documents and Settings\\Utente\\Documenti\\__ DATI MEDITS AGGIORNATI __\\_____MEDITS DATA\\GSA18\\TE 2012-20 con letture GSA18.csv", sep = ";", header = TRUE)
        # TE$SEX <- "N"
        sp <- "RAPAVEN"
        GSA <- 29
        country <- "TUE"
        years <- 5
        GEAR <- "BEAMT" # "TBB", "GOC73", "TRAWL"

        sp <- "MULLSUR"
        GSA <- 18
        country <- NA
        years <- 5
        GEAR <- "GOC73" # "BEAMT", "TBB", "GOC73", "TRAWL"


        ALK(TE, GEAR = "BEAMT", GSA = 29, country = "BGR", sp = "RAPAVEN", years = 15)

        ALK(TE, GEAR = "GOC73", GSA = 18, country = NA, sp = "MULLBAR", years = 15)

        ALK(TE, GEAR = "GOC73", GSA = 18, country = NA, sp = "MULLSUR", years = 5)

        ALK(TE, GEAR = "GOC73", GSA = 18, country = NA, sp = "MERLMER", years = 15)
    }

    TE <- te

    vbf3 <- function(t, Linf, k, t0) {
        Linf * (1 - exp(-k * (t - t0)))
    }

    if (is.na(wd)) {
        wd <- tempdir()
    }
    sspp <- sp
    genus <- substr(unique(sspp), 1, 4)
    species <- substr(unique(sspp), 5, 7)

    TE$SEX <- toupper(TE$SEX)

    TE$COUNTRY <- as.character(TE$COUNTRY)
    if (all(is.na(country))) {
        country <- as.character(unique(TE$COUNTRY))
        TE <- TE[TE$AREA == GSA & TE$COUNTRY %in% country, ]
    } else {
        TE <- TE[TE$AREA == GSA & TE$COUNTRY %in% country, ]
    }
    data_C <- TE[TE$GENUS == genus & TE$SPECIES == species &
                     !is.na(TE$AGE) &
                     as.character(TE$AGE) != " " &
                     as.character(TE$AGE) != "" &
                     as.character(TE$AGE) != "UR" &
                     as.character(TE$AGE) != "NR" &
                     as.character(TE$AGE) != "-1" &
                     TE$SEX %in% c("F", "M", "N"), ]
    data_C$SEX <- "C"

    data <- TE[TE$GENUS == genus & TE$SPECIES == species &
                   !is.na(TE$AGE) &
                   as.character(TE$AGE) != " " &
                   as.character(TE$AGE) != "" &
                   as.character(TE$AGE) != "UR" &
                   as.character(TE$AGE) != "NR" &
                   as.character(TE$AGE) != "-1" &
                   TE$SEX %in% c("F", "M", "I", "N"), ]

    last_year <- max(TE$YEAR)
    years <- c((last_year - (years-1)):last_year)
    data <- rbind(data, data_C)
    data <- data[data$YEAR %in% years, ]
    data$AGE <- as.numeric(data$AGE)
    data <- data[!is.na(data$AGE), ]

    if (nrow(data) > 1) {
        sexes <- unique(data$SEX)
        # sexes <- sexes[sexes != "I"]
        s <- 3
        params <- list()
        for (s in 1:length(sexes)) {
            df <- data[data$SEX %in% c(sexes[s], "I"), ]
            if (nrow(df) > 0) {
                if (unique(GEAR)[1] %in% c("BEAMT", "TBB")) {
                    # ss <- suppressWarnings(vbStarts(SHELL_WIDTH~AGE,data=df,plot=FALSE))
                    ss <- list()
                    ss$Linf <- max(df$LENGTH_CLASS) / 0.95
                    if (length(df[df$AGE == 1, "LENGTH_CLASS"])>0){
                        ss$K <- mean(df[df$AGE == 1, "LENGTH_CLASS"]) / ss$Linf
                    } else {
                        ss$K <- 0.310 # (SaÄŸlam, N., SaÄŸlam, C., & SaÄŸlam, Y. (2015))
                    }
                    ss$t0 <- -0.5
                    error <- try(mod3 <- nls(LENGTH_CLASS ~ vbf3(AGE, Linf, k, t0), start = c(Linf = abs(ss$Linf), k = abs(ss$K), t0 = ss$t0), data = df, algorithm = "default"),silent=TRUE)
                    if (!is(error, "try-error")) {
                        mod3 <- nls(LENGTH_CLASS ~ vbf3(AGE, Linf, k, t0), start = c(Linf = abs(ss$Linf), k = abs(ss$K), t0 = ss$t0), data = df, algorithm = "default")
                        sum <- summary(mod3)
                        sum
                        AIC(mod3)
                        Linf <- sum$parameters[1]
                        k <- sum$parameters[2]
                        t0 <- sum$parameters[3]
                        params[[s]] <- data.frame(species = sp, sex = sexes[s], Linf = as.numeric(round(Linf, 4)), k = as.numeric(round(k, 4)), t0 = as.numeric(round(t0, 4)), notes = "")
                        pred <- data.frame(AGE = seq(0, max(df$AGE), 0.1))
                        pred$LENGTH <- predict(mod3, newdata = data.frame(AGE = pred$AGE))
                        p <- ggplot() +
                            geom_point(data = df, aes(x = AGE, y = LENGTH_CLASS), colour = "red", stat = "identity") +
                            geom_line(data = pred, aes(x = AGE, y = LENGTH), colour = "blue",size=1) +
                            ggtitle(paste("ALK", sexes[s], sspp, sep = " - ")) +
                            labs(col = "Year") +
                            xlab("Age") +
                            theme(legend.position = "none") +
                            ylab(paste("Length (mm)"))
                        if(save) ggsave(file.path(wd, "output","ALK", paste0("ALK_", sspp, "_", sexes[s], ".jpg")), dpi = 300, width = 9, height = 9, units = "in")
                    } else {
                        params[[s]] <- data.frame(species = sp, sex = sexes[s], Linf = NA, k = NA, t0 = NA, notes = "model not converged")
                        p <- ggplot() +
                            geom_point(data = df, aes(x = AGE, y = LENGTH_CLASS), colour = "red", stat = "identity") +
                            ggtitle(paste("ALK", sexes[s], sspp, sep = " - ")) +
                            labs(col = "Year") +
                            xlab("Age") +
                            theme(legend.position = "none") +
                            ylab(paste("Length (mm)"))
                        if(save) ggsave(file.path(wd, "output","ALK", paste0("ALK_", sspp, "_", sexes[s], ".jpg")), dpi = 300, width = 9, height = 9, units = "in")
                    }
                }

                if (unique(GEAR)[1] %in% c("GOC73", "TRAWL")) {{ ss <- list()
                ss$Linf <- max(df$LENGTH_CLASS) / 0.95
                ss$K <- mean(df[df$AGE == 1, "LENGTH_CLASS"]) / ss$Linf
                ss$t0 <- -0.5
                # ss <- suppressWarnings(vbStarts(LENGTH_CLASS~AGE,data=df,plot=TRUE))

                error <- try(mod3 <- nls(LENGTH_CLASS ~ vbf3(AGE, Linf, k, t0), start = c(Linf = abs(ss$Linf), k = abs(ss$K), t0 = ss$t0), data = df, algorithm = "default"), silent = TRUE)
                if (!is(error, "try-error")) {
                    mod3 <- nls(LENGTH_CLASS ~ vbf3(AGE, Linf, k, t0), start = c(Linf = abs(ss$Linf), k = abs(ss$K), t0 = ss$t0), data = df, algorithm = "default")
                    sum <- summary(mod3)
                    sum
                    AIC(mod3)
                    Linf <- sum$parameters[1]
                    k <- sum$parameters[2]
                    t0 <- sum$parameters[3]
                    params[[s]] <- data.frame(species = sp, sex = sexes[s], Linf = as.numeric(round(Linf, 4)), k = as.numeric(round(k, 4)), t0 = as.numeric(round(t0, 4)), notes = "")
                    pred <- data.frame(AGE = seq(0, max(df$AGE), 0.1))
                    pred$LENGTH <- predict(mod3, newdata = data.frame(AGE = pred$AGE))
                    p <- ggplot() +
                        geom_point(data = df, aes(x = AGE, y = LENGTH_CLASS), colour = "red", stat = "identity") +
                        geom_line(data = pred, aes(x = AGE, y = LENGTH), colour = "blue",size=1) +
                        ggtitle(paste("ALK", sexes[s], sspp, sep = " - ")) +
                        labs(col = "Year") +
                        xlab("Age") +
                        theme(legend.position = "none") +
                        ylab(paste("Length (mm)"))
                    if(save) ggsave(file.path(wd, "output","ALK", paste0("ALK_", sspp, "_", sexes[s], ".jpg")), dpi = 300, width = 9, height = 9, units = "in")
                } else {
                    params[[s]] <- data.frame(species = sp, sex = sexes[s], Linf = NA, k = NA, t0 = NA, notes = "model not converged")
                    p <- ggplot() +
                        geom_point(data = df, aes(x = AGE, y = LENGTH_CLASS), colour = "red", stat = "identity") +
                        ggtitle(paste("ALK", sexes[s], sspp, sep = " - ")) +
                        labs(col = "Year") +
                        xlab("Age") +
                        theme(legend.position = "none") +
                        ylab(paste("Length (mm)"))
                    if(save) ggsave(file.path(wd, "output","ALK", paste0("ALK_", sspp, "_", sexes[s], ".jpg")), dpi = 300, width = 9, height = 9, units = "in")
                } }}
            } else {
                message(paste0("Not age data for '", sexes[s], "' sex"))
            }
        }
        df.params <- do.call(rbind, params)
        if(save) write.table(df.params, file.path(wd, "output","ALK", paste0("ALK_", sspp, "_summary_table.csv")), sep=";", row.names=FALSE)
        return(df.params)
    } else {
        message("Not enougth data to plot ALK")
    }
} ### close function ALK

#' Bubbleplot of abundance indices for recruits and spawners
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @description
#' The function generates bubbleplots of abundance indices for recruits and spawners
#' If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
#'
#' @param mTATC mTATC table
#' @param map_range range of coordinates for the map
#' @param thresh_rec threshold value to select recruits data from mTATC table (reported in mm)
#' @param thresh_spaw threshold value to select spawners data from mTATC table (reported in mm)
#' @param depths three reference bathymetric lines to be plotted in the maps
#' @param res resolution of the depth lines
#' @param buffer buffer around the map
#' @param wd working directory
#' @param save boolean. If TRUE the outputs are saved in the local folder
#' @param verbose boolean. If TRUE messages are prompted in the console
# [ORIGINALE]:
# #' @return A \code{ggplot} object representing the bubble plot of sex ratio by hauls.
# [MODIFICATO]:
#' @return A \code{list} containing two \code{ggplot} objects: the bubble plot of recruits (\code{pr}) and spawners (\code{ps}) abundance indices.
#' @examples
#' \donttest{
#' # Create a merged dataset for GSA 10
#' mTATC <- merge_TATC(TA[TA$AREA==10,], TC[TC$AREA==10,], "MERLMER")
#' bubbleplot_RS_by_hauls(mTATC, map_range=c(15,21,39,43), thresh_rec=200, thresh_spaw=250)
#' }
#' @export
#' @importFrom marmap getNOAA.bathy as.xyz
#' @importFrom utils data
#' @importFrom ggplot2 coord_sf geom_polygon scale_x_continuous scale_y_continuous geom_contour geom_point scale_size coord_map ggtitle theme ggsave element_blank element_rect element_text map_data aes labs
#' @importFrom stats aggregate
bubbleplot_RS_by_hauls <- function(mTATC, map_range, thresh_rec, thresh_spaw, depths = c(50, 200, 800), res=NA, buffer=0.1,wd=NA, save=TRUE, verbose = FALSE) {
    # [MODIFICATO]:
    pr <- NULL
    ps <- NULL
    if (is.na(wd)) { wd <- tempdir() }
  if (FALSE) {
    thresh_rec <- 20
    thresh_spaw <- 21
    map_range <- c(15.0, 21.0, 39.5, 42.5)
    depths = c(50, 200, 800)
    res=1
    buffer=0.1
    save <- TRUE
    verbose <- TRUE
    bubbleplot_RS_by_hauls(mTATC, map_range, thresh_rec, thresh_spaw, depths = c(50, 200, 800))
  }

  if (!requireNamespace("mapproj", quietly = TRUE)) {
    stop("Package 'mapproj' needed for this function to work. Please install it.", call. = FALSE)
  }

  long <- lat <- group <- V1 <- V2 <- V3 <- lon <- indices <- NULL

  buff <- buffer

  merge_TATC <- mTATC
  GENERE <- as.character(unique(merge_TATC$GENUS)[unique(merge_TATC$GENUS) != -1])
  SPECIE <- as.character(unique(merge_TATC$SPECIES)[unique(merge_TATC$SPECIES) != -1])
  sspp <- paste(GENERE, SPECIE, sep = "")

  depths <- -1 * depths


  names(map_range) <- c("xmin", "xmax", "ymin", "ymax")
  x1 <- min(map_range[1])
  x2 <- max(map_range[2])
  y1 <- min(map_range[3])
  y2 <- max(map_range[4])

  xl <- c(x1, x2)
  yl <- c(y1, y2)

  map_lim <- map_range
  lx <- map_range["xmax"] - map_range["xmin"]
  ly <- map_range["ymax"] - map_range["ymin"]
  map_ratio <- ly / lx

  limits <- data.frame(matrix(NA, ncol = 2, nrow = 2))
  colnames(limits) <- c("X", "Y")
  limits[1, 1] <- map_lim[1]
  limits[1, 2] <- map_lim[3]
  limits[2, 1] <- map_lim[2]
  limits[2, 2] <- map_lim[4]

  GSA <- unique(merge_TATC$GSA)[[1]]
  threshold <- thresh_rec

  ddd_r <- NA
  ddd_r <- merge_TATC[merge_TATC$LENGTH_CLASS <= threshold & merge_TATC$LENGTH_CLASS != -1, ]

  # load("data/med_bathy.rda")
  if (!is.na(res)){
    bath <- suppressMessages(suppressWarnings(getNOAA.bathy(lon1 = min(xl) - 1, lon2 = max(xl) + 1, lat1 = min(yl) - 1 - buff, lat2 = max(yl) + 1 + buff, resolution = res)))
    bat_xyz <- as.xyz(bath)
  } else {
  data("med_bathy", package = "BioIndex", envir = environment())
  bat_xyz <- as.xyz(med_bathy)
  bat_xyz <- bat_xyz[!is.na(bat_xyz[,3]), ]
  colnames(bat_xyz) <- c("V1","V2","V3")
  bat_xyz <- subset(bat_xyz, V1 >= min(xl) & V1 <= max(xl) & V2 >= min(yl) - buff & V2 <= max(yl) + buff)
  }

  if (nrow(ddd_r) != 0) {
    pivot_r <- aggregate(as.numeric(as.character(ddd_r$N_km2)), by = list(ddd_r$id, ddd_r$MEAN_LONGITUDE_DEC, ddd_r$MEAN_LATITUDE_DEC), sum)
    colnames(pivot_r) <- c("id", "lon", "lat", "indices")
    loc <- pivot_r
    br <- as.numeric(summary(loc$indices)[-4])
    labels <- as.character(round(as.numeric(summary(loc$indices))[-4], 3))

    theme_opts <- list(theme(
      panel.grid.minor = element_blank(),
      panel.grid.major = element_blank(),
      panel.background = element_rect(fill = "light blue", linetype = "solid", color = "black"),
      axis.line = element_blank(),
      axis.text.x = element_blank(),
      axis.text.y = element_blank(),
      axis.ticks = element_blank(),
      axis.title.x = element_blank(),
      axis.title.y = element_blank(),
      plot.title = element_text(size = 22)
    ))


    dep_text <- expression(paste("Abundance of recruits ", (n / km^2), sep = " "))

    world <- map_data("world")
    x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
    y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))


    suppressWarnings(suppressMessages(
      pr <- ggplot() +
        coord_sf(xlim = xl, ylim = yl, expand = TRUE) +
        geom_polygon(data = world, aes(long, lat, group = group), fill = "grey") +
        scale_x_continuous(breaks = x_breaks) +
        scale_y_continuous(breaks = y_breaks) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depths[3], color = "#1F618D", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depths[2], color = "#5499C7", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depths[1], color = "#4db5fa", linewidth = 0.5
        ) +
        geom_point(
          data = loc, aes(lon, lat, group = NULL, fill = NULL, size = indices), shape = 21,
          color = "black", fill = "blue", alpha = I(4 / 10)
        ) +
        scale_size(range = c(1, 14), breaks = br, labels = labels, guide = "legend", labs(size = "n/km^2")) +
        coord_map(projection = "mercator", xlim = c((x1 - buff), (x2 + buff)), ylim = c((y1 - buff), (y2 + buff))) +
        ggtitle(dep_text) +
        theme_opts
    ))
        # print(pr) # rimosso per policy CRAN
    if (save) {
      ggsave(paste(wd, "/output/", sspp, "_GSA", GSA, " -indices of RECRUITS.jpg", sep = ""),
        width = 20, height = 20, units = "cm", dpi = 300, plot = pr
      )
    }
    if (verbose) {
      message("Bubble plot of recruits correctly saved ")
    }
  } else {
    if (verbose) {
      message("Not enough data for recruits to be plotted")
      message("Bubble plots - indices of recruits skipped\n")
    }
  }

  ##################
  ##################
  ##################

  merge_TATC <- mTATC
  threshold <- thresh_spaw
  ddd_s <- NA
  ddd_s <- merge_TATC[merge_TATC$LENGTH_CLASS >= threshold & merge_TATC$LENGTH_CLASS != -1 & merge_TATC$SEX == "F", ]

  if (nrow(ddd_s) != 0) {
    pivot_s <- aggregate(as.numeric(as.character(ddd_s$N_km2)), by = list(ddd_s$id, ddd_s$MEAN_LONGITUDE_DEC, ddd_s$MEAN_LATITUDE_DEC), sum)
    colnames(pivot_s) <- c("id", "lon", "lat", "indices")
    loc <- pivot_s
    br <- as.numeric(summary(loc$indices)[-4])
    labels <- as.character(round(as.numeric(summary(loc$indices)[-4]), 3))

    theme_opts <- list(theme(
      panel.grid.minor = element_blank(),
      panel.grid.major = element_blank(),
      panel.background = element_rect(fill = "light blue", linetype = "solid", color = "black"),
      axis.line = element_blank(),
      axis.text.x = element_blank(),
      axis.text.y = element_blank(),
      axis.ticks = element_blank(),
      axis.title.x = element_blank(),
      axis.title.y = element_blank(),
      plot.title = element_text(size = 22)
    ))

    dep_text <- expression(paste("Abundance of spawners ", (n / km^2), sep = " "))

    # res <- 0.1
    world <- map_data("world")
    x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
    y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))

    # bath <- suppressMessages(getNOAA.bathy(lon1 = min(xl) - 1, lon2 = max(xl) + 1, lat1 = min(yl) - 1 - buff, lat2 = max(yl) + 1 + buff, resolution = res))
    # bat_xyz <- as.xyz(bath)

    suppressWarnings(suppressMessages(
      ps <- ggplot() +
        coord_sf(xlim = xl, ylim = yl, expand = TRUE) +
        geom_polygon(data = world, aes(long, lat, group = group), fill = "grey") +
        scale_x_continuous(breaks = x_breaks) +
        scale_y_continuous(breaks = y_breaks) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depths[3], color = "#1F618D", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depths[2], color = "#5499C7", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depths[1], color = "#4db5fa", linewidth = 0.5
        ) +
        geom_point(
          data = loc, aes(lon, lat, group = NULL, fill = NULL, size = indices), shape = 21,
          color = "black", fill = "blue", alpha = I(4 / 10)
        ) +
        scale_size(range = c(1, 14), breaks = br, labels = labels, guide = "legend", labs(size = "n/km^2")) +
        coord_map(projection = "mercator", xlim = c((x1 - buff), (x2 + buff)), ylim = c((y1 - buff), (y2 + buff))) +
        ggtitle(dep_text) +
        theme_opts
    ))
        # print(ps) # rimosso per policy CRAN
    if (save) {
      ggsave(paste(wd, "/output/", sspp, "_GSA", GSA, " -indices of SPAWNERS.jpg", sep = ""),
        width = 20, height = 20, units = "cm", dpi = 300, plot = ps
      )
    }
    if (verbose) {
      message("Bubble plot of spawners correctly saved ")
    }
  } else {
    if (verbose) {
      message("Not enough data for spawners to be plotted")
      message("Bubble plots - indices of spawners skipped")
    }
  }
  # [MODIFICATO]:
  return(list(pr, ps))
}

#' Merge TA and TB tables (haul-level catches)
#'
#' @description
#' Combines haul-level operational data (TA) with species catch data (TB),
#' providing the essential foundation for calculating stratified abundance (N)
#' and biomass (kg) indices.
#' The function keeps only valid hauls, performs quality checks (times,
#' positions, wing opening, haul/date consistency) using internal validation
#' functions (based on \code{RoME} version 0.2.3), computes spatial means,
#' swept area, depth stratum, biomass/density indicators, andâ€”optionallyâ€”writes
#' the merged data to disk.
#'
#' @param ta   A `data.frame`/`data.table` containing a full TA dataset
#'             (columns listed in `BioIndex::TA_cols`).
#' @param tb   A `data.frame`/`data.table` containing a full TB dataset
#'             (columns listed in `BioIndex::TB_cols`).
#' @param species Character, MEDITS 7-letter code (e.g. `"ARISFOL"`).
#'                The first 4 characters are interpreted as *GENUS* and
#'                the last 3 as *SPECIES*.
#' @param country Character vector of ISO-3 country codes to keep; use
#'                `"all"` (default) to keep every country present in `ta`.
#' @param strata  Depth-stratum reference table; default is
#'                `BioIndex::strata_scheme`.
#' @param wd      Working directory.  If `NA` (the default when
#'                `save     = FALSE`) no files are written.
#' @param save    Logical.  If `TRUE`, writes
#'                `"mergeTATB_<GENUS><SPECIES>.csv"` to
#'                `file.path(wd, "output")`.
#' @param verbose Logical; if `TRUE` (default) prints progress messages.
#'
#' @return A `data.frame` with one row per haul and the following groups
#'         of variables:
#'         * **TA** metadata (haul position, times, depths, etc.)
#'         * **TB** catch totals (numbers and weight)
#'         * Calculated fields: mean lat/lon, swept area, depth stratum,
#'           density (`N_h`, `N_km2`) and biomass (`kg_h`, `kg_km2`)
#' @details
#' The implementation mirrors the original BioIndex routine.
#'
#' \strong{Data Validation:}
#' The function performs syntactic data validation using internal
#' implementations of the validation routines (based on \code{RoME} v0.2.3).
#'
#' \strong{Optimisations:}
#' Includes two speed-ups: (1) vectorised replacement of missing `"NA NA"` records,
#' (2) a single loop over depth strata instead of a nested haul Ã— stratum
#' loop.  Results are identical to the reference routine.
#'
#' @note
#' This version of \code{BioIndex} uses internal validation functions based on
#' \strong{RoME version 0.2.3}. The package is fully self-sufficient and does
#' not require external validation packages to be installed.
#'
#' @seealso [BioIndex] package documentation.
#' @importFrom hms hms
#' @importFrom utils write.table
#' @examples
#' # Use internal data
#' data("TA", package = "BioIndex")
#' data("TB", package = "BioIndex")
#' m_tb <- merge_TATB(
#'   ta       = TA,
#'   tb       = TB,
#'   species  = "ARISFOL",
#'   country  = "ITA",
#'   wd       = tempdir(),
#'   save     = FALSE
#' )
#' head(m_tb)
#' @export
merge_TATB <- function(ta, tb,
                       species,
                       country = "all",
                       strata  = BioIndex::strata_scheme,
                       wd      = NA,
                       save     = FALSE,
                       verbose = TRUE) {

    if (is.na(wd)) { wd <- tempdir() }
    ## Working directory check (unchanged) ------------------------------- ##
    if (is.na(wd) & save) {
        save <- FALSE
        if (verbose)
            message("Missing working directory. Results will not be saved locally.")
    }

    strata_scheme <- strata           # local alias

    ## Column templates (unchanged) -------------------------------------- ##
    TA_cols <- BioIndex::TA_cols
    TB_cols <- BioIndex::TB_cols

    ## Species code parsing (unchanged) ---------------------------------- ##
    species[2] <- substr(species, 5, 7)
    species[1] <- substr(species[1], 1, 4)

    ## Input data (unchanged) -------------------------------------------- ##
    TA <- ta
    TB <- tb

    ## COUNTRY filter (unchanged, minus TC notes) ------------------------ ##
    TA_country <- sort(unique(TA$COUNTRY))
    TB_country <- sort(unique(TB$COUNTRY))
    for (cn in TA_country)
        if (!(cn %in% TB_country))
            warning("Country ", cn, " not present in TB file.")

    country_analysis <- if (length(TA_country) == 1) TA_country else
        if (any(country %in% "all")) TA_country else country
    TA <- TA[TA$COUNTRY %in% country_analysis, ]
    TB <- TB[TB$COUNTRY %in% country_analysis, ]

    ## Unique haul IDs (unchanged) --------------------------------------- ##
    uid <- function(df) {
        if (nrow(df) == 0) return(character(0))
        paste(df$AREA, df$COUNTRY, df$YEAR, "_",
              df$VESSEL, df$MONTH, df$DAY, "_",
              df$HAUL_NUMBER, sep = "")
    }
    TA$id <- uid(TA); TB$id <- uid(TB)
    invalid_ids <- TA$id[TA$VALIDITY == "I"]

    ## Data validation: local BioIndex routines (based on RoME 0.2.3)     ##
    suffix <- paste(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"), sep = "")
    for (yy in sort(unique(TA$YEAR))) {
        TAy <- TA[TA$YEAR == yy, ]; TBy <- TB[TB$YEAR == yy, ]
        if (!check_dictionary(TAy,"SHOOTING_TIME",0:2400,yy,paste0(wd,"/output/"),suffix))
            stop("SHOOTING_TIME out of range.")
        if (!check_dictionary(TAy,"HAULING_TIME",0:2400,yy,paste0(wd,"/output/"),suffix))
            stop("HAULING_TIME out of range.")
        if (!check_dictionary(TAy,"WING_OPENING",c(30,50:250),yy,paste0(wd,"/output/"),suffix))
            stop("WING_OPENING out of range.")
        if (!check_numeric_range(TAy,"SHOOTING_LATITUDE",c(3020,4730),yy,paste0(wd,"/output/"),suffix))
            stop("SHOOTING_LATITUDE out of range.")
        if (!check_numeric_range(TAy,"HAULING_LATITUDE",c(3020,4730),yy,paste0(wd,"/output/"),suffix))
            stop("HAULING_LATITUDE out of range.")
        if (!check_numeric_range(TAy,"SHOOTING_LONGITUDE",c(0,4200),yy,paste0(wd,"/output/"),suffix))
            stop("SHOOTING_LONGITUDE out of range.")
        if (!check_numeric_range(TAy,"HAULING_LONGITUDE",c(0,4200),yy,paste0(wd,"/output/"),suffix))
            stop("HAULING_LONGITUDE out of range.")
        if (!check_date_haul(TAy,TBy,yy,paste0(wd,"/output/"),suffix))
            stop("Date in TB not consistent with TA.")
        if (!check_hauls_TBTA(TAy,TBy,yy,paste0(wd,"/output/"),suffix))
            stop("Hauls in TB not consistent with TA.")
    }

    ## Preparation for merge (unchanged) --------------------------------- ##
    names(TA)[names(TA)=="AREA"] <- "GSA"
    names(TB)[names(TB)=="AREA"] <- "GSA"

    TA_ <- TA[!TA$id %in% invalid_ids, TA_cols]
    TB_ <- TB[!TB$id %in% invalid_ids &
                  TB$GENUS == species[1] & TB$SPECIES == species[2], TB_cols]

    ## MERGE TA â€“ TB (unchanged logic, incl. optimisations) -------------- ##
    if (verbose) message("- Merging TA-TB files")
    merge_TATB <- merge(TA_, TB_, by = "id", all.x = TRUE)
    merge_TATB$MEDITS_CODE <- paste(merge_TATB$GENUS, merge_TATB$SPECIES)

    bad <- merge_TATB$MEDITS_CODE == "NA NA"
    if (any(bad)) {
        merge_TATB$MEDITS_CODE[bad]            <- "NA"
        merge_TATB$GENUS[bad]                  <- -1
        merge_TATB$SPECIES[bad]                <- -1
        merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL[bad]  <- 0
        merge_TATB$TOTAL_NUMBER_IN_THE_HAUL[bad]  <- 0
        merge_TATB$NB_OF_FEMALES[bad]             <- 0
        merge_TATB$NB_OF_MALES[bad]               <- 0
        merge_TATB$NB_OF_UNDETERMINED[bad]        <- 0
    }

    ## MERGE coordinates (RoME-based) ------------------------------------ ##
    coord <- convert_coordinates(merge_TATB)
    merge_TATB$MEAN_LATITUDE     <- (merge_TATB$SHOOTING_LATITUDE +
                                         merge_TATB$HAULING_LATITUDE) / 2
    merge_TATB$MEAN_LATITUDE_DEC <- (coord$lat_start + coord$lat_end) / 2
    merge_TATB$MEAN_LONGITUDE    <- (merge_TATB$SHOOTING_LONGITUDE +
                                         merge_TATB$HAULING_LONGITUDE) / 2
    merge_TATB$MEAN_LONGITUDE_DEC<- (coord$lon_start + coord$lon_end) / 2
    merge_TATB$MEAN_DEPTH        <- (merge_TATB$SHOOTING_DEPTH +
                                         merge_TATB$HAULING_DEPTH) / 2
    merge_TATB$SWEPT_AREA        <- merge_TATB$DISTANCE *
        merge_TATB$WING_OPENING / 10000000

    ## MERGE Stratum (optimised single loop) ----------------------------- ##
    subset_strata <- strata_scheme[strata_scheme$GSA == unique(merge_TATB$GSA) &
                                       strata_scheme$COUNTRY %in% as.character(unique(merge_TATB$COUNTRY)), ]
    merge_TATB$STRATUM_CODE <- NA
    for (j in seq_len(nrow(subset_strata))) {
        low <- subset_strata[j, 4]; up <- subset_strata[j, 5]; co <- subset_strata[j, 3]
        if (co == 1) {
            idx <- floor(merge_TATB$MEAN_DEPTH) >= low &
                floor(merge_TATB$MEAN_DEPTH) <= up
        } else {
            idx <- floor(merge_TATB$MEAN_DEPTH) >  low &
                floor(merge_TATB$MEAN_DEPTH) <= up
        }
        merge_TATB$STRATUM_CODE[idx] <- co
    }

    ## Duration and Final Indicators ------------------------------------- ##
    h_s <- ifelse(nchar(merge_TATB$SHOOTING_TIME) == 4,
                  substr(merge_TATB$SHOOTING_TIME, 1, 2),
                  substr(merge_TATB$SHOOTING_TIME, 1, 1))
    m_s <- ifelse(nchar(merge_TATB$SHOOTING_TIME) == 4,
                  substr(merge_TATB$SHOOTING_TIME, 3, 4),
                  substr(merge_TATB$SHOOTING_TIME, 2, 3))
    h_h <- ifelse(nchar(merge_TATB$HAULING_TIME) == 4,
                  substr(merge_TATB$HAULING_TIME, 1, 2),
                  substr(merge_TATB$HAULING_TIME, 1, 1))
    m_h <- ifelse(nchar(merge_TATB$HAULING_TIME) == 4,
                  substr(merge_TATB$HAULING_TIME, 3, 4),
                  substr(merge_TATB$HAULING_TIME, 2, 3))

    hms_s    <- hms(rep(0, length(h_s)), as.numeric(m_s), as.numeric(h_s))
    hms_h    <- hms(rep(0, length(h_h)), as.numeric(m_h), as.numeric(h_h))
    dur      <- as.numeric(hms_h - hms_s) / 3600
    merge_TATB$SHOOTING_TIME <- hms_s
    merge_TATB$HAULING_TIME  <- hms_h
    merge_TATB$N_h    <- merge_TATB$TOTAL_NUMBER_IN_THE_HAUL / dur
    merge_TATB$N_km2  <- merge_TATB$TOTAL_NUMBER_IN_THE_HAUL /
        merge_TATB$SWEPT_AREA
    merge_TATB$kg_h   <- (merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL / 1000) / dur
    merge_TATB$kg_km2 <- (merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL / 1000) /
        merge_TATB$SWEPT_AREA

    if (save) {
        if (!dir.exists(wd)) dir.create(wd, recursive = TRUE)
        out <- file.path(wd, "output")
        if (!dir.exists(out)) dir.create(out, recursive = TRUE)
        write.table(merge_TATB, file.path(out, paste0("mergeTATB_", species[1], species[2], ".csv")),
                    sep = ";", row.names = FALSE)
    }

    return(merge_TATB)
}

#' TA table headings
#'
#' @name TA_cols
#' @docType data
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @keywords TA
"TA_cols"

#' TB table headings
#'
#' @name TB_cols
#' @docType data
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @keywords TB
"TB_cols"

#' TC table headings
#'
#' @name TC_cols
#' @docType data
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @keywords TC
"TC_cols"

#' stratification scheme
#'
#' @name strata_scheme
#' @docType data
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @keywords stratification
#' @description
#' Stratification scheme adopted in the bottom trawl demersal survey (e.g. MEDITS survey).
#'
"strata_scheme"


#' stratification
#'
#' @name stratification
#' @docType data
#' @keywords stratification MEDITS
#' @description
#' Data frame containing the surface area relative to the depth strata adopted in the stratification scheme (strata_scheme) of the demersal surveys (e.g. MEDITS survey).
#'
"stratification"

#' stratification scheme (rapa whelk)
#'
#' @name strata_scheme_rapana
#' @docType data
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @keywords stratification rapa whelk Black Sea
#' @description
#' Stratification scheme adopted in the rapa whelk survey (e.g. Black Sea beam trawl survey).
#'
"strata_scheme_rapana"


#' stratification (rapa whelk)
#'
#' @name stratification_rapana
#' @docType data
#' @keywords stratification rapa whelk Black Sea
#' @description
#' Data frame containing the surface area relative to the depth strata adopted in the stratification scheme (strata_scheme) of the rapa whelk surveys (e.g. Black Sea beam trawl survey).
#'
"stratification_rapana"

















#' TA table example
#'
#' @name TA
#' @docType data
#' @keywords TA MEDITS
"TA"

#' TB table example
#'
#' @name TB
#' @docType data
#' @keywords TB MEDITS
"TB"

#' TC table example
#'
#' @name TC
#' @docType data
#' @keywords TC MEDITS
"TC"


#' centroidi
#'
#' @name centroidi
#' @docType data
#' @keywords centroidi
"centroidi"

#' cgpmgrid
#'
#' @name cgpmgrid
#' @docType data
#' @keywords cgpmgrid
"cgpmgrid"

#' continent
#'
#' @name continent
#' @docType data
#' @keywords continent
"continent"


#' stratum_0_125
#'
#' @name stratum_0_125
#' @docType data
#' @keywords stratum_0_125
"stratum_0_125"

#' stratum_0_200
#'
#' @name stratum_0_200
#' @docType data
#' @keywords stratum_0_200
"stratum_0_200"


#' stratum_0_35
#'
#' @name stratum_0_35
#' @docType data
#' @keywords stratum_0_35
"stratum_0_35"


#' stratum_0_45
#'
#' @name stratum_0_45
#' @docType data
#' @keywords stratum_0_45
"stratum_0_45"


#' stratum_0_800
#'
#' @name stratum_0_800
#' @docType data
#' @keywords stratum_0_800
"stratum_0_800"


#' stratum_200_800
#'
#' @name stratum_200_800
#' @docType data
#' @keywords stratum_200_800
"stratum_200_800"

#' Mediterranean and Black Sea bathymetry (0â€“1000 m, bathy object)
#'
#' A precomputed \code{bathy} object containing bathymetric data for the Mediterranean Sea
#' and the Black Sea. Depth values are restricted between 0 and -1000 meters. This dataset
#' was downloaded using \code{\link[marmap]{getNOAA.bathy}} with a resolution of 1 arc-minute,
#' and filtered to remove deeper areas.
#'
#' The object can be used directly with functions from the \pkg{marmap} package, such as
#' \code{plot.bathy()} and \code{get.depth()}.
#'
#' @format An object of class \code{bathy} (a matrix with longitude and latitude as axes, and depth in meters)
#'
#' @details
#' The spatial extent includes:
#' \itemize{
#'   \item Longitude: from -6Â° to 42Â°
#'   \item Latitude: from 30Â° to 47Â°
#'   \item Depth: from 0 to -1000 meters
#' }
#' Only marine cells within this depth range are retained. Land and deeper areas are set to \code{NA}.
#'
#' @usage data(med_bathy)
#'
#' @source NOAA ETOPO1 via \code{marmap::getNOAA.bathy()}
#' @seealso \code{\link[marmap]{getNOAA.bathy}}, \code{\link[marmap]{plot.bathy}}, \code{\link{bubbleplot_RS_by_hauls}}
#' @keywords datasets bathymetry mediterranean blacksea
"med_bathy"

#' Bubble plot of abundance and biomass indices by haul
#'
#' @author Walter Zupa \email{zupa@fondazionecoispa.org}
#' @description
#' The function generates bubble plot of abundance and biomass indices by haul.
#' If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
#'
#' @param mTATB data frame
#' @param map_lim coordinates limits for the plotted map
#' @param depth_lines vector of three depth bathymetrical lines to be plotted
#' @param buffer buffer to the coordinate limits in map units
#' @param res resolution of the bathymetrical lines
#' @param wd working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @param verbose boolean. If TRUE a message is printed
#' @importFrom marmap getNOAA.bathy as.xyz
#' @importFrom utils data
#' @importFrom ggplot2 coord_sf geom_polygon scale_x_continuous scale_y_continuous geom_contour geom_point scale_size coord_map ggtitle theme ggsave element_blank element_rect element_text map_data aes labs
#' @return A \code{ggplot} object representing the bubble plot of abundance or biomass by hauls.
#' @examples
#' \donttest{
#' # Use internal data
#' data(TA)
#' data(TB)
#' # Create a merged dataset for GSA 10 and year 2016
#' mTATB <- merge_TATB(TA[TA$AREA == 10 & TA$YEAR == 2016, ],
#'                     TB[TB$AREA == 10 & TB$YEAR == 2016, ],
#'                     "MERLMER")
#' bubble_plot_by_haul_indexes(mTATB, map_lim = NA, depth_lines = c(200, 500, 800), save = FALSE)
#' }
#' @export

bubble_plot_by_haul_indexes <- function(mTATB, map_lim, depth_lines, buffer = 0, res = NA, wd = NA, save = TRUE, verbose = TRUE) {
  if (is.na(wd)) {
    wd <- tempdir()
  }

  if (FALSE) {
    map_lim <- c(15.5, 20.0, 39.8, 42.5)
    # map_lim <- NA
    depth_lines <- c(200, 500, 800)
    buffer <- 0.5
    res <- NA
    save <- TRUE
    verbose <- TRUE
    # ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
    # tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
    # tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
    wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"

    mTATB <- read.table("D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)\\output\\mergeTATB_MERLMER.csv", sep = ";", header = TRUE)

    bubble_plot_by_haul_indexes(mTATB, map_lim, depth_lines, buffer = 0, wd = wd, save = TRUE, verbose = TRUE)
  }

  if (!requireNamespace("mapproj", quietly = TRUE)) {
    stop("Package 'mapproj' needed for coord_map() in ggplot2. Please install it.", call. = FALSE)
  }

  if (is.na(wd) & save) {
    if (verbose) {
      message("Missing working directory. Results are not saved in the local folder.")
    }
  }

  N_km2 <- V1 <- V2 <- V3 <- group <- kg_km2 <- lat <- lon <- long <- NULL

  dep_text_N <- expression(paste("Abundance by haul ", (n / km^2), sep = " ")) # ," - ",depth_range[1],"-",depth_range[2]," m"
  dep_text_kg <- expression(paste("Biomass by haul ", (kg / km^2), sep = " ")) # ," - ",depth_range[1],"-",depth_range[2]," m"

  metaDB <- mTATB
  GENERE <- as.character(unique(metaDB$GENUS)[!is.na(unique(metaDB$GENUS) != -1) & unique(metaDB$GENUS) != -1])
  SPECIE <- as.character(unique(metaDB$SPECIES)[!is.na(unique(metaDB$SPECIES) != -1) & unique(metaDB$SPECIES) != -1])
  sspp <- paste(GENERE, SPECIE, sep = "")
  GSA <- unique(metaDB$GSA)
  species <- sspp


  if (any(is.na(map_lim))) {
    map_lim[c(1:2)] <- range(metaDB$MEAN_LONGITUDE_DEC, na.rm = TRUE)
    map_lim[c(3, 4)] <- range(metaDB$MEAN_LATITUDE_DEC, na.rm = TRUE)
    lx <- map_lim[2] - map_lim[1]
    ly <- map_lim[4] - map_lim[3]
  } else {
    lx <- map_lim[2] - map_lim[1]
    ly <- map_lim[4] - map_lim[3]
  }

  x1 <- map_lim[1]
  x2 <- map_lim[2]
  y1 <- map_lim[3]
  y2 <- map_lim[4]

  map_ratio <- ly / lx
  theme_opts <- list(theme(
    panel.grid.minor = element_blank(),
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill = "light blue", linetype = "solid", color = "black"),
    plot.background = element_rect(
      fill = "white",
      linewidth = 1, linetype = "solid", color = "black"
    ),
    axis.line = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    axis.title.x = element_blank(),
    axis.title.y = element_blank(),
    plot.title = element_text(size = 22)
  ))


  buff <- buffer


  world <- map_data("world")
  xl <- c(x1, x2)
  yl <- c(y1, y2)
  x_breaks <- c(round(xl[1], 0), round(xl[1], 0) + round((xl[2] - xl[1]) / 2, 0), round(xl[1], 0) + 2 * round((xl[2] - xl[1]) / 2, 0))
  y_breaks <- c(round(yl[1], 0), round(yl[1], 0) + round((yl[2] - yl[1]) / 2, 0), round(yl[1], 0) + 2 * round((yl[2] - yl[1]) / 2, 0))


  # bath <- suppressMessages(getNOAA.bathy (lon1 = min(xl)-1, lon2 = max(xl)+1, lat1 = min(yl)-1-buff,lat2 = max(yl)+1+buff, resolution = res))
  # bat_xyz <- as.xyz(bath)

  if (!is.na(res)) {
    bath <- suppressMessages(getNOAA.bathy(lon1 = min(xl) - 1, lon2 = max(xl) + 1, lat1 = min(yl) - 1 - buff, lat2 = max(yl) + 1 + buff, resolution = res))
    bat_xyz <- as.xyz(bath)
  } else {
    data("med_bathy", package = "BioIndex", envir = environment())
    bat_xyz <- as.xyz(med_bathy)
    bat_xyz <- bat_xyz[!is.na(bat_xyz[, 3]), ]
    colnames(bat_xyz) <- c("V1", "V2", "V3")
    bat_xyz <- subset(bat_xyz, V1 >= min(xl) & V1 <= max(xl) & V2 >= min(yl) - buff & V2 <= max(yl) + buff)
  }

  if (any(is.na(depth_lines))) {
    depth_lines <- c(-200, -500, -800)
  } else {
    depth_lines <- -1 * depth_lines
  }


  if (nrow(metaDB) != 0) {
    loc <- data.frame(id = metaDB$id, COUNTRY = metaDB$COUNTRY, GSA = metaDB$GSA, YEAR = metaDB$YEAR, lon = metaDB$MEAN_LONGITUDE_DEC, lat = metaDB$MEAN_LATITUDE_DEC, N_km2 = metaDB$N_km2, kg_km2 = metaDB$kg_km2)

    loc1 <- loc[loc$N_km2 != 0, ]
    br_n <- c(0, as.numeric(summary(loc1$N_km2)[-4]))
    br_kg <- c(0, as.numeric(summary(loc1$kg_km2)[-4]))
    labels_n <- c("0", as.character(round(as.numeric(summary(loc1$N_km2))[-4], 3)))
    labels_kg <- c("0", as.character(round(as.numeric(summary(loc1$kg_km2))[-4], 3)))

    # coordinates(loc)<-c("lon","lat")
    # proj4string(loc) <- CRS("+proj=longlat")
    # loc_laea<-spTransform(loc, CRS("+proj=longlat"))
    # loc_laea_df<-data.frame(loc_laea)
    loc[loc$N_km2 == 0, "colore"] <- "red"
    loc[loc$N_km2 != 0, "colore"] <- "blue"

    limits <- data.frame(matrix(NA, ncol = 2, nrow = 2))
    colnames(limits) <- c("X", "Y")
    limits[1, 1] <- x1
    limits[1, 2] <- y1
    limits[2, 1] <- x2
    limits[2, 2] <- y2
    # coordinates(limits) <- ~ X+Y
    # proj4string(limits) <- CRS("+proj=longlat")
    # lim_laea<-as.data.frame(spTransform(limits, CRS("+proj=longlat")))
    xmin <- x1
    xmax <- x2
    ymin <- y1
    ymax <- y2


    # ABUNDANCE OUTPUT
    suppressWarnings(suppressMessages(
      p_n <- ggplot() +
        # coord_map() requires the mapproj package to be installed.
        coord_map(projection = "mercator", xlim = c((xmin - buff), (xmax + buff)), ylim = c((ymin - buff), (ymax + buff))) +
        geom_polygon(data = world, aes(long, lat, group = group), fill = "light grey", colour = "darkgrey") +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depth_lines[3], color = "#1F618D", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depth_lines[2], color = "#5499C7", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depth_lines[1], color = "#4db5fa", linewidth = 0.5
        ) +
        geom_point(
          data = loc, aes(lon, lat, group = NA, fill = NA, size = N_km2), shape = 21,
          colour = "black", fill = loc$colore, alpha = I(3 / 10)
        ) +
        scale_size(range = c(1, 14), breaks = br_n, labels = labels_n, guide = "legend", labs(size = "n/km^2")) +
        # theme(aspect.ratio=map_ratio*1.5)+
        ggtitle(dep_text_N) +
        theme_opts
    ))

    # print(p_n) # rimosso per policy CRAN
    if (verbose) {
      message("Plot of abundance by haul generated successfully.")
    }
    if (save) {
      if (is.na(wd)) {
        warning("\nNo working directory defined by the user. The plot was not saved in the local folder\n")
      } else if (!is.na(wd)) {
        ggsave(paste(wd, "/output/", sspp, "_GSA", GSA, "-Abundance_by_haul.jpg", sep = ""),
          plot = p_n, width = 20, height = 20, units = "cm", dpi = 300
        )
        if (verbose) {
          message("Plot of abundance by haul saved successfully")
        }
      }
    }

    # BIOMASS OUTPUT
    suppressWarnings(suppressMessages(
      p_k <- ggplot() +
        coord_map(projection = "mercator", xlim = c((xmin - buff), (xmax + buff)), ylim = c((ymin - buff), (ymax + buff))) +
        geom_polygon(data = world, aes(long, lat, group = group), fill = "light grey", colour = "darkgrey") +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depth_lines[3], color = "#1F618D", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depth_lines[2], color = "#5499C7", linewidth = 0.5
        ) +
        geom_contour(
          data = bat_xyz,
          aes(x = V1, y = V2, z = V3),
          breaks = depth_lines[1], color = "#4db5fa", linewidth = 0.5
        ) +
        geom_point(
          data = loc, aes(lon, lat, group = NA, fill = NA, size = kg_km2), shape = 21,
          colour = "black", fill = loc$colore, alpha = I(3 / 10)
        ) +
        scale_size(range = c(1, 14), breaks = br_n, labels = labels_n, guide = "legend", labs(size = "kg/km^2")) +
        # theme(aspect.ratio=map_ratio*1.5)+
        ggtitle(dep_text_kg) +
        theme_opts
    ))

    # print(p_k) # rimosso per policy CRAN
    if (verbose) {
      message("Plot of biomass by haul generated successfully.")
    }
    if (save) {
      if (is.na(wd)) {
        warning("\nNo working directory defined by the user. The plot was not saved in the local folder\n")
      } else if (!is.na(wd)) {
        ggsave(paste(wd, "/output/", sspp, "_GSA", GSA, "-Biomass_by_haul.jpg", sep = ""),
          plot = p_k, width = 20, height = 20, units = "cm", dpi = 300
        )
        if (verbose) {
          message("Plot of biomass by haul saved successfully")
        }
      }
    }

    return(list(p_n, p_k))
  } else {
    if (verbose) {
      message("Not enough data to be plotted\n")
      message("Bubble plots - indices  by haul skipped\n\n")
    }
  }
}

#' Quantile estimation
#'
#' @param weighted LFD data.frame
#' @param qlin reference quantile for the analysis
#' @return A \code{numeric} value or vector containing the requested quantiles.
quant <- function(weighted,qlin=0.95){
    weighted <- data.frame(weighted)
    weighted <- weighted[weighted$X2 != 0 , ]
    w <- aggregate(weighted[,2],list(G=weighted[,1]),FUN=sum,na.rm=T)
    x <- as.numeric(as.character(w$G))
    w <- w$x
    if(qlin<0.5) {
        x <- -x
        ql <- 1-qlin
    } else {
        ql <- qlin
    }
    tot <- sum(w)
    frac <- tot*(1-ql)
    w <- w[order(-x)]
    x <- x[order(-x)]
    somme <- w[1]
    i <- 2
    while(somme < frac){
        somme <- somme + w[i]
        i <- i+1
    }
    lq <- round(x[i] + (x[i-1]-x[i])/w[i-1]*(somme-frac),1)
    f1 <- w[i]/tot
    vlq <- ql*(1-ql)/(f1^2)/tot
    SD <- sqrt(vlq)
    cv <- sqrt(vlq)/ifelse(qlin<0.5,-lq,lq)
    c(quantile=ifelse(qlin<0.5,-lq,lq),SD=SD,cv=cv)
}

#' Check numeric range (RoME)
#' @description
#' The function checks whether the values contained in specific fields are consistent with the allowed ranges.
#'
#' @param ResultData data frame in MEDITS tables
#' @param Field field of the table to be checked
#' @param Values vector of the allowed values
#' @param year reference year for the analysis
#' @param wd working directory
#' @param suffix name of the log file
#' @param verbose boolean. If TRUE messages are prompted in the console
#' @note
#' This function is an internal routine based on \strong{RoME version 0.2.3}.
#' It is provided within \code{BioIndex} to ensure the package remains
#' functional and self-sufficient for data validation.
#' @return \code{TRUE} if the validation passes, \code{FALSE} otherwise.
#' @examples
#' data(TA)
#' check_numeric_range(TA, Field = "HAUL_DURATION", Values = c(10, 120), year = 2007)
#' @export
check_numeric_range <- function(ResultData, Field, Values, year, wd=NA, suffix=NA, verbose=FALSE) {

  if (is.na(wd)) {
    wd <- tempdir()
    if (verbose) message(paste("Missing working directory. Results are saved in the temporary folder: ", wd))
  }

  if (!file.exists(file.path(wd, "Logfiles"))) {
    dir.create(file.path(wd, "Logfiles"), recursive = TRUE, showWarnings = FALSE)
  }
  
  if (missing(suffix) || is.na(suffix)) {
    suffix <- paste0(Sys.Date(), format(Sys.time(), "_time_h%Hm%Ms%OS0"))
  }
  
  Errors <- file.path(wd, "Logfiles", paste0("Logfile_", suffix, ".dat"))
  if (!file.exists(Errors)) {
    file.create(Errors)
  }

  # Filtering data for the selected year
  if (length(year) != 1 || is.na(year)) {
    stop("Argument 'year' must be a single non-NA value")
  }
  df <- ResultData[ResultData$YEAR == year, , drop = FALSE]
  if (nrow(df) == 0) {
    write(paste("\n----------- check range of values for field:", Field, "-", year), file = Errors, append = TRUE)
    write(sprintf("No data found for year %s", year), file = Errors, append = TRUE)
    return(TRUE)
  }
  
  tf <- df$TYPE_OF_FILE[1]
  write(paste("\n----------- check range of values for field:", Field, "-", year), file = Errors, append = TRUE)

  indexcol <- which(names(df) == Field)
  Valuesf <- as.numeric(Values)
  lrange <- length(Valuesf)

  # Remove NAs for range check
  df_sub <- df[!is.na(df[[Field]]), ]
  numberError <- 0

  if (nrow(df_sub) > 0) {
    if (lrange == 2) {
      for (k in 1:nrow(df_sub)) {
        val <- df_sub[k, indexcol]
        if (!(val >= Valuesf[1] & val <= Valuesf[2])) {
          write(paste("Haul", df_sub$HAUL_NUMBER[k], ": value (", val, ") out of allowed range for", Field, "in", tf), file = Errors, append = TRUE)
          numberError <- numberError + 1
        }
      }
    } else if (lrange > 2) {
      for (k in 1:nrow(df_sub)) {
        val <- df_sub[k, indexcol]
        if (!((val >= Valuesf[1] & val <= Valuesf[2]) | val %in% Valuesf[3:lrange])) {
          write(paste("Haul", df_sub$HAUL_NUMBER[k], ": value (", val, ") out of allowed range for", Field, "in", tf), file = Errors, append = TRUE)
          numberError <- numberError + 1
        }
      }
    }
  }

  if (numberError == 0) {
    write(paste("No error occurred for field", Field, "in", tf), file = Errors, append = TRUE)
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' Estimation of abundance indices for spawners (females)
#' @description
#' Calculates abundance and biomass indices focused on the adult fraction
#' (spawners, using the user-defined cutoff value), an essential indicator
#' for evaluating the self-renewal capacity of the population.
#'
#' @param mTATB data frame
#' @param mTATC data frame
#' @param GSA reference GSA for the analysis
#' @param country vector of reference countries for the analysis
#' @param depth_range range of depth strata to perform the analysis (min, max)
#' @param cutoff cutoff value for splitting spawner portion of population (reported in mm)
#' @param stratification data frame of strata surface area
#' @param wd working directory
#' @param save boolean. If TRUE the plot is saved in the user defined working directory (wd)
#' @importFrom dplyr group_by summarise summarize
#' @importFrom magrittr %>%
#' @return A \code{data.frame} containing the abundance and biomass indices for the spawner fraction.
#' @examples
#' data(TA)
#' data(TB)
#' data(TC)
#' data(stratification)
#' m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
#'                   species = "MERLMER", country = "all", verbose = FALSE)
#' index_spawn(m[[1]], m[[2]], GSA = 10, country = "all", depth_range = c(10, 800),
#'             cutoff = 200, stratification = stratification, save = FALSE)
#' @export
index_spawn <- function(mTATB,mTATC, GSA, country, depth_range, cutoff, stratification, wd=NA, save=TRUE) {
    if (is.na(wd)) { wd <- tempdir() }
    oldpar <- par(no.readonly = TRUE)
    oldpar$new <- NULL
    on.exit(suppressWarnings(par(oldpar)))

    if (FALSE) {
        verbose=TRUE
        wd <- "D:\\Documents and Settings\\Utente\\Documenti\\GitHub\\BioIndex\\R_BioIndex_3.3_(in update)"
        GSA=10
        save=TRUE
        depth_range <- c(10,800)
        cutoff <- 30
        ta <- read.table(paste(wd,"input/TA/TA GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tb <- read.table(paste(wd,"input/TB/TB GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        tc <- read.table(paste(wd,"input/TC/TC GSA18 2017-2020.csv", sep="/"), sep=";", header=T)
        country="all"
        species <- "MERLMER"
        m <- merge_TATBTC(ta, tb, tc, species="MERLMER", country="all", wd=wd, verbose=TRUE)
        mTATB <- m[[1]]
        mTATC <- m[[2]]
        index_spawn(mTATB,mTATC, GSA, country, depth_range, cutoff, stratification, wd, save=TRUE)
    }

    if (is.na(wd) & save) {
        save=TRUE
        if (verbose){
            message("Missing working directory. Results are not saved in the local folder.")
        }
    }

    N_km2 <- id <- n_raised <- NULL

    merge_TATB <- mTATB
    merge_TATC <- mTATC

    dependent <- "abundance"
    dep_text <-expression(paste(Abundance, (n/km^2), sep=" "))
    varcol <- which(colnames(merge_TATC)=="N_km2")
    col_response <- which(colnames(merge_TATC)=="N_km2")

    if (!is.na(cutoff)){
       threshold <- cutoff
    } else {
        stop("Missing cutoff value")
    }

    sex <- "F"

    GENERE <- as.character(unique(merge_TATC$GENUS)[unique(merge_TATC$GENUS) != -1])
    SPECIE <- as.character(unique(merge_TATC$SPECIES)[unique(merge_TATC$SPECIES) != -1])
    sspp <- paste(GENERE,SPECIE, sep="")
    GSA <- unique(merge_TATC$GSA)
    species <- sspp

    countries <- unique(mTATB[!is.na(mTATB$COUNTRY),"COUNTRY"])
    l_country <- length(unique(mTATB$COUNTRY))

    if (l_country==1){
        check_country ="Y"
        country_analysis <- countries
    } else {
        if (any(country %in% "all")) {
            country_analysis <<- countries
        } else {
            country_analysis <<- country
        }
    }

    merge_TATC$n_raised <- merge_TATC$NUMBER_OF_INDIVIDUALS_IN_THE_LENGTH_CLASS_AND_MATURITY_STAGE * merge_TATC$RAISING_FACTOR
    ddd <- merge_TATC[merge_TATC$LENGTH_CLASS >= threshold  & merge_TATC$SEX ==sex  ,] # & merge_TATC$LENGTH_CLASS != -1

    if (nrow(ddd) != 0) {

        ddd_n <- as.data.frame(ddd %>% group_by(id) %>% summarise(sum_N = sum(n_raised, na.rm=TRUE)))
        ddd_nkm2 <-  as.data.frame(ddd %>% group_by(id) %>% summarise(sum_nkm2 = sum(N_km2, na.rm=TRUE)))
        merge_N <- merge(ddd_n, ddd_nkm2, by.x="id")
        ddd <- merge(merge_TATB[,which(colnames(merge_TATB)%in%c("id","GSA","YEAR","HAUL_NUMBER","GENUS","SPECIES","MEAN_LATITUDE_DEC","MEAN_LONGITUDE_DEC","MEAN_DEPTH","SWEPT_AREA"))], merge_N, by.y="id",all.x =T) #c(1,3,4,7,23,24,35,36) c("id","GSA","YEAR","HAUL_NUMBER","GENUS","SPECIES","MEAN_DEPTH","SWEPT_AREA")
        ddd <- ddd[ddd$MEAN_DEPTH >=depth_range[1] &ddd$MEAN_DEPTH < depth_range[2],]

        for (i in 1:length(ddd$sum_N)){
            if (is.na(ddd[i,"sum_N" ])) {ddd[i,"sum_N" ] <- 0} else {ddd[i,"sum_N" ] <- ddd[i,"sum_N" ]}
            if (is.na(ddd[i,"sum_nkm2" ])) {ddd[i,"sum_nkm2" ] <- 0} else {ddd[i,"sum_nkm2" ] <- ddd[i,"sum_nkm2" ]}
        }

        main <- paste(sspp,"_GSA",GSA,"_(abundance of spawners)-Random_Stratified_Sampling_",depth_range[1],"-",depth_range[2], " m", sep="")

        if (save){
            write.table(ddd, paste(wd,"/output/",main,"_Hauls.csv", sep=""), sep=";", row.names = F)
        }

        strata_scheme <- strata_scheme[strata_scheme$GSA == GSA & strata_scheme$COUNTRY %in% as.character(unique(country_analysis))[1], ]

        depth <- data.frame(matrix(NA,ncol=4, nrow=length(strata_scheme$CODE)))
        colnames(depth) <- c("strata", "min", "max", "bul")
        depth$strata <- strata_scheme$CODE # c(1,2,3,4,5)
        depth$min <-strata_scheme$MIN_DEPTH  # c(10,50,100,200,500)
        depth$max <-strata_scheme$MAX_DEPTH  # c(50,100,200,500,800)
        if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
        data <-  ddd[ddd$MEAN_DEPTH>depth_range[1] & ddd$MEAN_DEPTH<=depth_range[2],]
        idx_from <- which(depth[,"min"] == depth_range[1])
        idx_to   <- which(depth[,"max"] == depth_range[2])

        if (length(idx_from) != 1 || length(idx_to) != 1) {
            stop(paste0("Error: depth range [", depth_range[1], ", ", depth_range[2],
                        "] does not match the strata boundaries defined for this GSA. ",
                        "Available min depths: ", paste(sort(unique(depth$min)), collapse=", "),
                        "; Available max depths: ", paste(sort(unique(depth$max)), collapse=", ")))
        }
        strata_range <- seq(idx_from, idx_to, 1)
        depth[depth$strata %in% strata_range, "bul"] <- T
        depth[!(depth$strata %in% strata_range), "bul"] <- F

        year_range <- data.frame(year = sort(unique(merge_TATC$YEAR)))

        res_table_cala2 <- data.frame(year=sort(unique(merge_TATC$YEAR)))
        res_table_cala2$index_1 <- NA
        res_table_cala2$index_2 <- NA
        res_table_cala2$index_3 <- NA
        res_table_cala2$index_4 <- NA
        res_table_cala2$index_5 <- NA
        res_table_cala2$index_6 <- NA

        se_table2  <-  data.frame(year=sort(unique(merge_TATC$YEAR)))
        se_table2$sum_1 <- NA
        se_table2$sum_2 <- NA
        se_table2$sum_3 <- NA
        se_table2$sum_4 <- NA
        se_table2$sum_5 <- NA
        se_table2$sum_6 <- NA
        se_table2$sd <- NA

        se_table_a  <-  data.frame(year=sort(unique(merge_TATC$YEAR)))
        se_table_a$sum_1 <- NA
        se_table_a$sum_2 <- NA
        se_table_a$sum_3 <- NA
        se_table_a$sum_4 <- NA
        se_table_a$sum_5 <- NA
        se_table_a$sum_6 <- NA
        se_table_a$sd <- NA

        # stratification <- read.table(paste(wd,"/scripts/utilities/stratification scheme.csv", sep=""), sep=";", header=T)
        stratification$SURF <- as.numeric(stratification$SURF)

        analysis_stratum1 <- NA
        analysis_stratum2 <- NA
        analysis_stratum3 <- NA
        analysis_stratum4 <- NA
        analysis_stratum5 <- NA
        analysis_stratum6 <- NA

        analysis_stratum1 <- depth[1,4]
        analysis_stratum2 <- depth[2,4]
        analysis_stratum3 <- depth[3,4]
        analysis_stratum4 <- depth[4,4]
        analysis_stratum5 <- depth[5,4]
        analysis_stratum6 <- depth[6,4]

        if (is.na(analysis_stratum1)){depth[1,4]<- F; analysis_stratum1 <- F}
        if (is.na(analysis_stratum2)){depth[2,4]<- F; analysis_stratum2 <- F}
        if (is.na(analysis_stratum3)){depth[3,4]<- F; analysis_stratum3 <- F}
        if (is.na(analysis_stratum4)){depth[4,4]<- F; analysis_stratum4 <- F}
        if (is.na(analysis_stratum5)){depth[5,4]<- F; analysis_stratum5 <- F}
        if (is.na(analysis_stratum6)){depth[6,4]<- F; analysis_stratum6 <- F}

        if (analysis_stratum1 == T){area_s1 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==1 & stratification$COUNTRY %in% country_analysis,5])} else {area_s1 <- 0}
        if (analysis_stratum2 == T){area_s2 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==2 & stratification$COUNTRY %in% country_analysis,5])} else {area_s2 <- 0}
        if (analysis_stratum3 == T){area_s3 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==3 & stratification$COUNTRY %in% country_analysis,5])} else {area_s3 <- 0}
        if (analysis_stratum4 == T){area_s4 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==4 & stratification$COUNTRY %in% country_analysis,5])} else {area_s4 <- 0}
        if (analysis_stratum5 == T){area_s5 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==5 & stratification$COUNTRY %in% country_analysis,5])} else {area_s5 <- 0}
        if (analysis_stratum6 == T){area_s6 <- sum(stratification[stratification$GSA == GSA & stratification$CODE==6 & stratification$COUNTRY %in% country_analysis,5])} else {area_s6 <- 0}

        peso_s1 <- area_s1/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
        peso_s2 <- area_s2/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
        peso_s3 <- area_s3/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
        peso_s4 <- area_s4/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
        peso_s5 <- area_s5/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)
        peso_s6 <- area_s6/sum(area_s1,area_s2,area_s3,area_s4,area_s5,area_s6)

        i=1
        for(i in 1:length(res_table_cala2[,1])) {


            data2 <- ddd[ddd$YEAR == res_table_cala2[i,1] , ]

            if (analysis_stratum1 == T) {s1 <- data2[floor(data2$MEAN_DEPTH) >= depth[1,2] & floor(data2$MEAN_DEPTH) <= depth[1,3], ]}
            if (analysis_stratum2 == T) {s2 <- data2[floor(data2$MEAN_DEPTH) > depth[2,2] & floor(data2$MEAN_DEPTH) <= depth[2,3], ]}
            if (analysis_stratum3 == T) {s3 <- data2[floor(data2$MEAN_DEPTH) > depth[3,2] & floor(data2$MEAN_DEPTH) <= depth[3,3], ]}
            if (analysis_stratum4 == T) {s4 <- data2[floor(data2$MEAN_DEPTH) > depth[4,2] & floor(data2$MEAN_DEPTH) <= depth[4,3], ]}
            if (analysis_stratum5 == T) {s5 <- data2[floor(data2$MEAN_DEPTH) > depth[5,2] & floor(data2$MEAN_DEPTH) <= depth[5,3], ]}
            if (analysis_stratum6 == T) {s6 <- data2[floor(data2$MEAN_DEPTH) > depth[6,2] & floor(data2$MEAN_DEPTH) <= depth[6,3], ]}

            if (analysis_stratum1 == T){
                # [ORIGINALE]:
                # s_n <- sum(s1[!is.na(s1$sum_N),"sum_N"])
                # s_a <- sum(s1$SWEPT_AREA)
                # res_table_cala2[i,2] <- s_n/s_a
                # mean_ind <- mean(s1[!is.na(s1$sum_nkm2), "sum_nkm2" ])
                # sq <- (s1[!is.na(s1$sum_nkm2),"sum_nkm2"] - mean_ind)^2
                # sqA <- sq*s1$SWEPT_AREA
                # sum_sqA <- sum(sqA)/(length(s1[,1])-1)
                # f1 <- s_a/area_s1
                # se_table2[i,2] <- (((peso_s1)^2 * sum_sqA)/ s_a)* (1-f1)

                # [MODIFICATO]:
                s_n <- sum(s1[!is.na(s1$sum_N),"sum_N"])
                s_a <- sum(s1$SWEPT_AREA)
                res_table_cala2[i,2] <- s_n/s_a
                s1_clean <- s1[!is.na(s1$sum_nkm2), ]
                s_a_clean <- sum(s1_clean$SWEPT_AREA)
                if (nrow(s1_clean) > 1) {
                    mean_ind <- mean(s1_clean$sum_nkm2)
                    sq <- (s1_clean$sum_nkm2 - mean_ind)^2
                    sqA <- sq*s1_clean$SWEPT_AREA
                    sum_sqA <- sum(sqA)/(nrow(s1_clean)-1)
                    f1 <- s_a_clean/area_s1
                    se_table2[i,2] <- (((peso_s1)^2 * sum_sqA)/ s_a_clean)* (1-f1)
                } else {
                    se_table2[i,2] <- 0
                }
            } else {area_s1 = 0}

            if (analysis_stratum2 == T){
                # [ORIGINALE]:
                # s_n <- sum(s2[!is.na(s2$sum_N),"sum_N"])
                # s_a <- sum(s2$SWEPT_AREA)
                # res_table_cala2[i,3] <- s_n/s_a
                # mean_ind <- mean(s2[!is.na(s2$sum_nkm2), "sum_nkm2" ])
                # sq <- (s2[!is.na(s2$sum_nkm2),"sum_nkm2"] - mean_ind)^2
                # sqA <- sq*s2$SWEPT_AREA
                # sum_sqA <- sum(sqA)/(length(s2[,1])-1)
                # f2 <- s_a/area_s2
                # se_table2[i,3] <- (((peso_s2)^2 * sum_sqA)/ s_a)* (1-f2)

                # [MODIFICATO]:
                s_n <- sum(s2[!is.na(s2$sum_N),"sum_N"])
                s_a <- sum(s2$SWEPT_AREA)
                res_table_cala2[i,3] <- s_n/s_a
                s2_clean <- s2[!is.na(s2$sum_nkm2), ]
                s_a_clean <- sum(s2_clean$SWEPT_AREA)
                if (nrow(s2_clean) > 1) {
                    mean_ind <- mean(s2_clean$sum_nkm2)
                    sq <- (s2_clean$sum_nkm2 - mean_ind)^2
                    sqA <- sq*s2_clean$SWEPT_AREA
                    sum_sqA <- sum(sqA)/(nrow(s2_clean)-1)
                    f2 <- s_a_clean/area_s2
                    se_table2[i,3] <- (((peso_s2)^2 * sum_sqA)/ s_a_clean)* (1-f2)
                } else {
                    se_table2[i,3] <- 0
                }
            } else {area_s2 = 0}

            if (analysis_stratum3 == T){
                # [ORIGINALE]:
                # s_n <- sum(s3[!is.na(s3$sum_N),"sum_N"])
                # s_a <- sum(s3$SWEPT_AREA)
                # res_table_cala2[i,4] <- s_n/s_a
                # mean_ind <- mean(s3[!is.na(s3$sum_nkm2), "sum_nkm2" ])
                # sq <- (s3[!is.na(s3$sum_nkm2),"sum_nkm2"] - mean_ind)^2
                # sqA <- sq*s3$SWEPT_AREA
                # sum_sqA <- sum(sqA)/(length(s3[,1])-1)
                # f3 <- s_a/area_s3
                # se_table2[i,4] <- (((peso_s3)^2 * sum_sqA)/ s_a)* (1-f3)

                # [MODIFICATO]:
                s_n <- sum(s3[!is.na(s3$sum_N),"sum_N"])
                s_a <- sum(s3$SWEPT_AREA)
                res_table_cala2[i,4] <- s_n/s_a
                s3_clean <- s3[!is.na(s3$sum_nkm2), ]
                s_a_clean <- sum(s3_clean$SWEPT_AREA)
                if (nrow(s3_clean) > 1) {
                    mean_ind <- mean(s3_clean$sum_nkm2)
                    sq <- (s3_clean$sum_nkm2 - mean_ind)^2
                    sqA <- sq*s3_clean$SWEPT_AREA
                    sum_sqA <- sum(sqA)/(nrow(s3_clean)-1)
                    f3 <- s_a_clean/area_s3
                    se_table2[i,4] <- (((peso_s3)^2 * sum_sqA)/ s_a_clean)* (1-f3)
                } else {
                    se_table2[i,4] <- 0
                }
            } else {area_s3 = 0}

            if (analysis_stratum4 == T){
                # [ORIGINALE]:
                # s_n <- sum(s4[!is.na(s4$sum_N),"sum_N"])
                # s_a <- sum(s4$SWEPT_AREA)
                # res_table_cala2[i,5] <- s_n/s_a
                # mean_ind <- mean(s4[!is.na(s4$sum_nkm2), "sum_nkm2" ])
                # sq <- (s4[!is.na(s4$sum_nkm2),"sum_nkm2"] - mean_ind)^2
                # sqA <- sq*s4$SWEPT_AREA
                # sum_sqA <- sum(sqA)/(length(s4[,1])-1)
                # f4 <- s_a/area_s4
                # se_table2[i,5] <- (((peso_s4)^2 * sum_sqA)/ s_a)* (1-f4)

                # [MODIFICATO]:
                s_n <- sum(s4[!is.na(s4$sum_N),"sum_N"])
                s_a <- sum(s4$SWEPT_AREA)
                res_table_cala2[i,5] <- s_n/s_a
                s4_clean <- s4[!is.na(s4$sum_nkm2), ]
                s_a_clean <- sum(s4_clean$SWEPT_AREA)
                if (nrow(s4_clean) > 1) {
                    mean_ind <- mean(s4_clean$sum_nkm2)
                    sq <- (s4_clean$sum_nkm2 - mean_ind)^2
                    sqA <- sq*s4_clean$SWEPT_AREA
                    sum_sqA <- sum(sqA)/(nrow(s4_clean)-1)
                    f4 <- s_a_clean/area_s4
                    se_table2[i,5] <- (((peso_s4)^2 * sum_sqA)/ s_a_clean)* (1-f4)
                } else {
                    se_table2[i,5] <- 0
                }
            } else {area_s4 = 0}

            if (analysis_stratum5 == T){
                # [ORIGINALE]:
                # s_n <- sum(s5[!is.na(s5$sum_N),"sum_N"])
                # s_a <- sum(s5$SWEPT_AREA)
                # res_table_cala2[i,6] <- s_n/s_a
                # mean_ind <- mean(s5[!is.na(s5$sum_nkm2), "sum_nkm2" ])
                # sq <- (s5[!is.na(s5$sum_nkm2),"sum_nkm2"] - mean_ind)^2
                # sqA <- sq*s5$SWEPT_AREA
                # sum_sqA <- sum(sqA)/(length(s5[,1])-1)
                # f5 <- s_a/area_s5
                # se_table2[i,6] <- (((peso_s5)^2 * sum_sqA)/ s_a)* (1-f5)

                # [MODIFICATO]:
                s_n <- sum(s5[!is.na(s5$sum_N),"sum_N"])
                s_a <- sum(s5$SWEPT_AREA)
                res_table_cala2[i,6] <- s_n/s_a
                s5_clean <- s5[!is.na(s5$sum_nkm2), ]
                s_a_clean <- sum(s5_clean$SWEPT_AREA)
                if (nrow(s5_clean) > 1) {
                    mean_ind <- mean(s5_clean$sum_nkm2)
                    sq <- (s5_clean$sum_nkm2 - mean_ind)^2
                    sqA <- sq*s5_clean$SWEPT_AREA
                    sum_sqA <- sum(sqA)/(nrow(s5_clean)-1)
                    f5 <- s_a_clean/area_s5
                    se_table2[i,6] <- (((peso_s5)^2 * sum_sqA)/ s_a_clean)* (1-f5)
                } else {
                    se_table2[i,6] <- 0
                }
            } else {area_s5 = 0}

            if (analysis_stratum6 == T){
                # [ORIGINALE]:
                # s_n <- sum(s6[!is.na(s6$sum_N),"sum_N"])
                # s_a <- sum(s6$SWEPT_AREA)
                # res_table_cala2[i,7] <- s_n/s_a
                # mean_ind <- mean(s6[!is.na(s6$sum_nkm2), "sum_nkm2" ])
                # sq <- (s6[!is.na(s6$sum_nkm2),"sum_nkm2"] - mean_ind)^2
                # sqA <- sq*s6$SWEPT_AREA
                # sum_sqA <- sum(sqA)/(length(s6[,1])-1)
                # f6 <- s_a/area_s6
                # se_table2[i,7] <- (((peso_s6)^2 * sum_sqA)/ s_a)* (1-f6)

                # [MODIFICATO]:
                s_n <- sum(s6[!is.na(s6$sum_N),"sum_N"])
                s_a <- sum(s6$SWEPT_AREA)
                res_table_cala2[i,7] <- s_n/s_a
                s6_clean <- s6[!is.na(s6$sum_nkm2), ]
                s_a_clean <- sum(s6_clean$SWEPT_AREA)
                if (nrow(s6_clean) > 1) {
                    mean_ind <- mean(s6_clean$sum_nkm2)
                    sq <- (s6_clean$sum_nkm2 - mean_ind)^2
                    sqA <- sq*s6_clean$SWEPT_AREA
                    sum_sqA <- sum(sqA)/(nrow(s6_clean)-1)
                    f6 <- s_a_clean/area_s6
                    se_table2[i,7] <- (((peso_s6)^2 * sum_sqA)/ s_a_clean)* (1-f6)
                } else {
                    se_table2[i,7] <- 0
                }
            } else {area_s6 = 0}

            # source(paste(wd, "/scripts/Index_estimation_cala_recruits.R", sep=""), encoding = 'UTF-8')
            sum_res_est <- c(res_table_cala2[i,2]*peso_s1,res_table_cala2[i,3]*peso_s2,res_table_cala2[i,4]*peso_s3,res_table_cala2[i,5]*peso_s4,res_table_cala2[i,6]*peso_s5,res_table_cala2[i,7]*peso_s6)
            res_table_cala2[i, 8]<- sum(sum_res_est[!is.na(sum_res_est)])# /sum(area_s1,area_s2,area_s3,area_s4,area_s5)
            colnames(res_table_cala2) <- c("year", "stratum 1","stratum 2", "stratum 3", "stratum 4", "stratum 5", "stratum 6", "Indices")
            se_table2[i, "sd"] <-sqrt(rowSums(se_table2[i, 2:7], na.rm = T))
        }

        timeseries <- data.frame(year = res_table_cala2[,1], abundance = res_table_cala2[,8], sd= se_table2[, "sd"])

        # plot timeseries of mean spawners indices
        main <- paste(sspp,"_GSA",GSA,"_(abundance of spawners)-Random_Stratified_Sampling_",depth_range[1],"-",depth_range[2], " m", sep="")
        main.lab <- paste(sspp," GSA",GSA," (abundance of spawners)-RSS ",depth_range[1],"-",depth_range[2], " m", sep="")
        max_index <- max(timeseries[,"abundance"]) + max(timeseries[!is.na(timeseries$sd) & timeseries$sd != Inf,"sd"])*1.2
        if (save) {
        write.table(timeseries, paste(wd,"/output/",main,"_Timeseries.csv", sep=""), sep=";", row.names = F)

            jpeg(paste(wd,"/output/",main,"_Timeseries.jpg",sep=""), res = 300, width = 8, height = 7, units = 'in')
            par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
            plot(timeseries[,"year"],  timeseries[,"abundance"], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab) # ylim=c(0,max_index*1.2)
            lines(timeseries[,"year"], (timeseries[,"abundance"]-1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
            lines(timeseries[,"year"], (timeseries[,"abundance"]+1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
            legend("topright", c("time series", "CI"), lty=c(1,2), pch=c(16, NA), col=c("black","red"))
            dev.off()

        } else {
            par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0))
            plot(timeseries[,"year"],  timeseries[,"abundance"], type="b", col="black", pch=16, xlab="year", ylim=c(0,max_index*1.2), ylab=dep_text, main=main.lab) # ylim=c(0,max_index*1.2)
            lines(timeseries[,"year"], (timeseries[,"abundance"]-1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
            lines(timeseries[,"year"], (timeseries[,"abundance"]+1.96*timeseries[,"sd"]), type="l",lty=2, col="red" )
            legend("topright", c("time series", "CI"), lty=c(1,2), pch=c(16, NA), col=c("black","red"))
            }

    return(timeseries)
    } else {
        message("No data for female individuals in TC available for the estimation of spawners' abundance indices.")
    }

}

---
title: "BioIndex"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BioIndex}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Description

**BioIndex** supports the standardized analysis of MEDITS trawl survey data and the calculation of biological indicators for selected species and population components. The package provides functions to estimate abundance and biomass indices, analyse size structure and length frequency distributions, derive sex ratio and maturity related metrics, explore spatial patterns, and assess temporal trends across surveys. 

Developed for integration within the RDBFIS framework, it is intended to work on quality checked input data (via the **RoME** package) and to produce reproducible outputs that can support monitoring, comparative analyses among GSAs and countries, and fishery management.

The package allows to perform the analysis at GSA level or country level, estimating the time series and the spatial distribution of a wide pool of population state-indicators. It also offers statistical trend analysis of both abundance and biomass time series based on the Hotelling-Pabst test (Cotter, 2009) and spatial analysis using the 0.5*0.5° GFCM grid or bubble plots.

# Installation

Install the latest BioIndex release from CRAN (when available):

```r
install.packages("BioIndex")
```

# Use

The main function `BioIndex()` can be used to launch a complete analysis:

```{r, eval=FALSE, fig.height=8, fig.width=8}
BioIndex::BioIndex(ta=BioIndex::TA, tb=BioIndex::TB, tc=BioIndex::TC, 
                   sspp="MERLMER",rec_threshold=200,
                   spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10,
                   country="all", map_lim=c(13.3,15.2,39.9,41.3),
                   depth_lines=c(50,200,800), strata=BioIndex::strata_scheme,
                   stratification_tab = BioIndex::stratification, resolution=1,
                   buffer=0.1, wd=tempdir(), zip=FALSE, save=FALSE, verbose=TRUE)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dd.distance.R
\name{dd.distance}
\alias{dd.distance}
\title{Estimate hauls distances (decimal degrees)}
\usage{
dd.distance(data, unit = "m", verbose = TRUE)
}
\arguments{
\item{data}{data frame of the hauls data (TA, table A) with coordinates reported as decimal degrees}

\item{unit}{string value indicating the measure unit of the distance. Allowed values: "m" for meters, "km" for kilometers and "NM" for nautical miles.}

\item{verbose}{give verbose output reporting in the output the selected measure unit of the distance.}
}
\value{
The function returns the vector of the distances expressed in the selected measure unit.
}
\description{
Function to estimate the hauls length using TA (table A, hauls data) with coordinates in the decimal degrees format (dd.ddd). The distances could be returned expressed in meters, kilometers and nautical miles.
}
\examples{
data(TA)
ta_dd <- MEDITS.to.dd(TA)
dd.distance(ta_dd, unit = "km", verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ALK.R
\name{ALK}
\alias{ALK}
\title{ALK (Age Length Key)}
\usage{
ALK(
  ta,
  te,
  sp,
  GSA,
  country = "all",
  nyears = NA,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{ta}{MEDITS or MEDITS-like TA table}

\item{te}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{nyears}{number of years of the time series to be considered in the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are promted in the console}
}
\value{
A \code{data.frame} representing the Age-Length Key.
}
\description{
Computes the Age-Length Key (ALK) from biological samples, a crucial
parameter for converting length distributions into age classes and
analyzing stock growth dynamics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_TATC.r
\name{merge_TATC}
\alias{merge_TATC}
\title{Merge TA and TC tables (haul-level lengthâ€“frequency data)}
\usage{
merge_TATC(
  ta,
  tc,
  species,
  country = "all",
  strata = BioIndex::strata_scheme,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{ta}{A `data.frame`/`data.table` containing a full TA dataset
(columns listed in `BioIndex::TA_cols`).}

\item{tc}{A `data.frame`/`data.table` containing a full TC dataset
(columns listed in `BioIndex::TC_cols`).}

\item{species}{Character, MEDITS 7-letter code (e.g. `"ARISFOL"`).
The first 4 characters are interpreted as *GENUS* and
the last 3 as *SPECIES*.}

\item{country}{Character vector of ISO-3 country codes to keep; use
`"all"` (default) to keep every country present in `ta`.}

\item{strata}{Depth-stratum reference table; default is
`BioIndex::strata_scheme`.}

\item{wd}{Working directory.  If `NA` (the default when
`save=FALSE`) no files are written.}

\item{save}{Logical.  If `TRUE`, writes
`"mergeTATC_<GENUS><SPECIES>.csv"` to
`file.path(wd, "output")`.}

\item{verbose}{Logical; if `TRUE` (default) prints progress messages.}
}
\value{
A `data.frame` in which each row represents one length-class
        (and sex/maturity) sampled in a haul, including:
        * **TA** metadata
        * **TC** length-frequency counts and weights
        * Calculated fields: mean lat/lon, swept area, depth stratum,
          raising factor, density (`N_h`, `N_km2`) and biomass
          (`kg_h`, `kg_km2`)
}
\description{
Links haul metadata (TA) with length-frequency samples (TC),
enabling the analysis of population demographic structure and the
reconstruction of length-frequency distributions (LFD).
The function keeps only valid hauls, runs all relevant quality checks
(times, positions, wing opening, haul/date consistency) using internal
validation functions (based on \code{RoME} version 0.2.3),
computes swept area, depth stratum, raising factors,
abundance/biomass indicators, andâ€”optionallyâ€”writes the merged data.
}
\details{
The code is identical to the official BioIndex routine.

\strong{Data Validation:}
The function performs syntactic data validation using internal
implementations of the validation routines (based on \code{RoME} v0.2.3).

\strong{Optimisations:}
Includes two micro-optimisations: vectorised handling of missing `"NA NA"` entries
and a single depth-stratum loop. Numerical results remain unchanged.
}
\note{
This version of \code{BioIndex} uses internal validation functions based on
\strong{RoME version 0.2.3}. The package is fully self-sufficient and does
not require external validation packages to be installed.
}
\examples{
# Use internal data
data("TA", package = "BioIndex")
data("TC", package = "BioIndex")
m_tc <- merge_TATC(
  ta       = TA,
  tc       = TC,
  species  = "ARISFOL",
  country  = "ESP",
  wd       = tempdir(),
  save=FALSE
)
head(m_tc)
}
\seealso{
[BioIndex] package documentation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{stratum_0_45}
\alias{stratum_0_45}
\title{stratum_0_45}
\format{
An object of class \code{PackedSpatVector} of length 1.
}
\usage{
stratum_0_45
}
\description{
stratum_0_45
}
\keyword{stratum_0_45}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Lquant.r
\name{Lquant}
\alias{Lquant}
\title{Estimation of L50 and L95}
\usage{
Lquant(lfd, wd = NA, sspp, GSA, save = TRUE, verbose = TRUE)
}
\arguments{
\item{lfd}{data frame of combined LFD}

\item{wd}{working directory}

\item{sspp}{MEDITS code for the selected species}

\item{GSA}{reference area for the analysis}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are reported in the console}
}
\value{
A \code{data.frame} containing the length quantiles (e.g., L95) time series.
}
\description{
Estimation of L50 and L95
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spearman.R
\name{spearman}
\alias{spearman}
\title{Spearman test}
\usage{
spearman(
  abundance = NA,
  biomass = NA,
  years,
  sspp = NA,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{abundance}{data frame of abundance indices}

\item{biomass}{data frame of biomass indices}

\item{years}{reference years for the analysis}

\item{sspp}{reference species for the analysis}

\item{wd}{path of working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
A \code{data.frame} or \code{list} with the results of the Spearman rank correlation test.
}
\description{
Spearman test
}
\examples{
abundance <- data.frame(year = 2010:2020, index = runif(11, 1, 10))
biomass <- data.frame(year = 2010:2020, index = runif(11, 5, 20))
spearman(abundance=abundance, biomass=biomass, years=c(2010,2020), 
         sspp="MERLMER", wd=tempdir(), save=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{strata_scheme}
\alias{strata_scheme}
\title{stratification scheme}
\format{
An object of class \code{data.frame} with 126 rows and 5 columns.
}
\usage{
strata_scheme
}
\description{
Stratification scheme adopted in the bottom trawl demersal survey (e.g. MEDITS survey).
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
\keyword{stratification}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index_spawn.r
\name{index_spawn}
\alias{index_spawn}
\title{Estimation of abundance indices for spawners (females)}
\usage{
index_spawn(
  mTATB,
  mTATC,
  GSA,
  country,
  depth_range,
  cutoff,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{mTATC}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country}{vector of reference countries for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{cutoff}{cutoff value for splitting spawner portion of population (reported in mm)}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{data.frame} containing the abundance and biomass indices for the spawner fraction.
}
\description{
Calculates abundance and biomass indices focused on the adult fraction
(spawners, using the user-defined cutoff value), an essential indicator
for evaluating the self-renewal capacity of the population.
}
\examples{
data(TA)
data(TB)
data(TC)
data(stratification)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
index_spawn(m[[1]], m[[2]], GSA = 10, country = "all", depth_range = c(10, 800),
            cutoff = 200, stratification = stratification, save = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index_on_grid.r
\name{index_on_grid}
\alias{index_on_grid}
\title{Generating maps of indexes}
\usage{
index_on_grid(
  mTATBsp,
  stratum,
  wd = NA,
  map_range,
  threshold = 30,
  verbose = FALSE,
  save = TRUE
)
}
\arguments{
\item{mTATBsp}{spatial mTATB}

\item{stratum}{reference stratum range (allowed values: "10,200","10,800","200,800","5,35","5,45")}

\item{wd}{working directory}

\item{map_range}{range of coordinates for the map}

\item{threshold}{minimum number of individuals per haul}

\item{verbose}{boolean. If TRUE messages are promted in the console}

\item{save}{boolean. If TRUE the results are stored in the working directory}
}
\value{
A \code{ggplot} object displaying the generated map of indices on the spatial grid.
}
\description{
Generates density and biomass maps based on the spatial grid,
allowing for the visual identification of resource concentration areas
or potential biological hotspots.
}
\examples{
data(TA)
data(TB)
data(TC)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
mTATBsp <- overlayGrid(m[[1]], m[[2]], GSA = 10, save = FALSE, verbose = FALSE)[[1]]
map_range <- c(9, 15, 39, 42)
index_on_grid(mTATBsp, stratum = "10,800", map_range = map_range, threshold = 5,
              save = FALSE, verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LFD.r
\name{LFD}
\alias{LFD}
\title{Length Frequency Distribution (LFD) Estimation and Plotting}
\usage{
LFD(
  mTATC,
  sex = "all",
  GSA,
  country = "all",
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATC}{Data frame resulting from the merge of TA and TC datasets using `merge_TATBTC()`. It must include raised numbers.}

\item{sex}{Character. Target sex for the analysis. Allowed values: `"F"`, `"M"`, `"I"`, `"N"`, or `"all"` (default) for combined sexes.}

\item{GSA}{Numeric. The GSA (Geographical Sub-Area) code of reference for the analysis.}

\item{country}{Character vector. Reference country or countries for the analysis. Use `"all"` (default) to include all countries available in the data.}

\item{depth_range}{Numeric vector of length 2, specifying the minimum and maximum depths (in meters) to filter the hauls used in the analysis.}

\item{strata_scheme}{Data frame containing the stratification scheme. Must include `CODE`, `MIN_DEPTH`, `MAX_DEPTH`, `GSA`, `COUNTRY`.}

\item{stratification}{Data frame with surface areas per stratum. Must include columns: `GSA`, `CODE`, `COUNTRY`, `SURF`.}

\item{wd}{Character. Working directory used to save output plots and tables. Required if `save = TRUE`.}

\item{save}{Logical. If `TRUE` (default), the function saves output tables and plots to `wd/output/`. If `FALSE`, no files are saved.}

\item{verbose}{Logical. If `TRUE` (default), informative messages are printed to the console to trace the function steps.}
}
\value{
A named list with two elements:
\describe{
  \item{`LFD`}{A data frame of total LFD by year across all strata.}
  \item{`LFD by stratum`}{A data frame of LFD by year and by stratum.}
}

A \code{list} of \code{ggplot} objects representing the length-frequency distributions.
}
\description{
Input parameters include biological and haul data, stratification schemes,
reference filters (e.g., sex, GSA, country, depth), and output control options (working directory, saving, verbosity).
}
\details{
This function estimates and plots the length frequency distribution (LFD) by year and by stratum,
based on merged biological and haul data from MEDITS surveys. It computes raised numbers,
applies stratification weights, and generates publication-ready plots and CSV outputs.
Stratification weighting is based on swept area and surface area of the strata.


The function:
\itemize{
  \item Filters hauls by depth and country.
  \item Computes raised numbers per haul.
  \item Applies stratified weights based on the area per stratum.
  \item Outputs and/or plots the length frequency distribution by year and stratum.
  \item Handles sex-specific or combined-sex analyses.
}

The plot outputs include:
\itemize{
  \item Combined LFD by year across all strata.
  \item LFD by stratum (faceted), stratified by year.
}

Output files are saved as CSV and high-resolution JPEGs if `save = TRUE` and `wd` is defined.
}
\examples{
data(TA)
data(TC)
# Create a merged dataset for GSA 10
mTATC <- merge_TATC(TA[TA$AREA==10,], TC[TC$AREA==10,], "MERLMER")
LFD(mTATC, sex="all", GSA=10, country="all", depth_range=c(10,800),
    strata_scheme=BioIndex::strata_scheme,
    stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hauls_position.r
\name{hauls_position}
\alias{hauls_position}
\title{Plot of hauls time series}
\usage{
hauls_position(
  mTATB,
  country = "all",
  map_lim,
  depth_lines,
  buffer = 0,
  res = NA,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{country}{country code as reported in MEDITS format. "all" code to perform the analysis on all the countries of the same GSA}

\item{map_lim}{coordinates limits for the plotted map}

\item{depth_lines}{vactor of three depth bathymetrical lines to be plotted}

\item{buffer}{buffer to the coordinate limits in map units}

\item{res}{resolution of the bathymetrical lines}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are reported in the console}
}
\value{
A \code{ggplot} object representing the spatial distribution of hauls.
}
\description{
This function generates a spatial plot of haul positions over time, displaying haul labels and bathymetric lines over a customizable map extent.
If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
}
\examples{
\donttest{
# Use internal data
data(TA)
data(TB)
# Create a merged dataset for GSA 10 and year 2016
mTATB <- merge_TATB(TA[TA$AREA == 10 & TA$YEAR == 2016, ],
                    TB[TB$AREA == 10 & TB$YEAR == 2016, ],
                    "MERLMER")
hauls_position(mTATB, country = "all", map_lim = NA, depth_lines = c(200, 500, 800), save = FALSE)
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{med_bathy}
\alias{med_bathy}
\title{Mediterranean and Black Sea bathymetry (0â€“1000 m, bathy object)}
\format{
An object of class \code{bathy} (a matrix with longitude and latitude as axes, and depth in meters)
}
\source{
NOAA ETOPO1 via \code{marmap::getNOAA.bathy()}
}
\usage{
data(med_bathy)
}
\description{
A precomputed \code{bathy} object containing bathymetric data for the Mediterranean Sea
and the Black Sea. Depth values are restricted between 0 and -1000 meters. This dataset
was downloaded using \code{\link[marmap]{getNOAA.bathy}} with a resolution of 1 arc-minute,
and filtered to remove deeper areas.
}
\details{
The object can be used directly with functions from the \pkg{marmap} package, such as
\code{plot.bathy()} and \code{get.depth()}.


The spatial extent includes:
\itemize{
  \item Longitude: from -6Â° to 42Â°
  \item Latitude: from 30Â° to 47Â°
  \item Depth: from 0 to -1000 meters
}
Only marine cells within this depth range are retained. Land and deeper areas are set to \code{NA}.
}
\seealso{
\code{\link[marmap]{getNOAA.bathy}}, \code{\link[marmap]{plot.bathy}}, \code{\link{bubbleplot_RS_by_hauls}}
}
\keyword{bathymetry}
\keyword{blacksea}
\keyword{datasets}
\keyword{mediterranean}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MIW.r
\name{MIW}
\alias{MIW}
\title{Estimation of Mean Individual Weight (MIW) time series}
\usage{
MIW(
  mTATB,
  GSA,
  country = "all",
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATB}{data frame of the merged TA and TB}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference countries in the GSA for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{strata_scheme}{data frame of the stratification scheme}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are reported in the console}
}
\value{
A \code{data.frame} containing the Mean Individual Weight (MIW) time series.
}
\description{
Calculates the Mean Individual Weight (MIW) time series, providing a
summary indicator of the average size within the captured population
and potential shifts due to fishing pressure.
}
\examples{
data(TA)
data(TB)
# Create a merged dataset for GSA 10
mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
MIW(mTATB, GSA=10, country="all", depth_range=c(10,800),
    strata_scheme=BioIndex::strata_scheme,
    stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TC}
\alias{TC}
\title{TC table example}
\format{
An object of class \code{data.frame} with 11185 rows and 22 columns.
}
\usage{
TC
}
\description{
TC table example
}
\keyword{MEDITS}
\keyword{TC}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IUT.r
\name{IUT}
\alias{IUT}
\title{Interception Union Tets}
\usage{
IUT(abundance, biomass, species, lastn = 5, GSA = 10, wd = NA, save = TRUE)
}
\arguments{
\item{abundance}{dataframe of abundance time series as produced by indices_ts function}

\item{biomass}{dataframe of biomass time series as produced by indices_ts function}

\item{species}{reference species for the analysis (MEDITS code)}

\item{lastn}{number of recent years for diagnosis of change}

\item{GSA}{reference GSA for the analysis (default is 10)}

\item{wd}{working directory (default is NA, uses tempdir())}

\item{save}{boolean. If TRUE results are saved in the output folder}
}
\value{
A \code{data.frame} containing the Indicator of Unfished Trends (IUT) results.
}
\description{
Interception Union Tets
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TB}
\alias{TB}
\title{TB table example}
\format{
An object of class \code{data.frame} with 3059 rows and 19 columns.
}
\usage{
TB
}
\description{
TB table example
}
\keyword{MEDITS}
\keyword{TB}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spear.r
\name{spear}
\alias{spear}
\title{Spearman test for timeseries}
\usage{
spear(x)
}
\arguments{
\item{x}{time series}
}
\value{
A \code{data.frame} representing the Spearman correlation coefficient and related statistics.
}
\description{
Spearman test for timeseries
}
\examples{
x <- runif(10, 1, 100)
spear(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TA}
\alias{TA}
\title{TA table example}
\format{
An object of class \code{data.frame} with 100 rows and 43 columns.
}
\usage{
TA
}
\description{
TA table example
}
\keyword{MEDITS}
\keyword{TA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_TATBTC.r
\name{merge_TATBTC}
\alias{merge_TATBTC}
\title{Merge TAâ€“TB and TAâ€“TC tables (MEDITS protocol)}
\usage{
merge_TATBTC(
  ta,
  tb,
  tc,
  species,
  country = "all",
  strata = BioIndex::strata_scheme,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{ta}{A MEDITS or MEDITS-like \strong{TA} table
(columns listed in \code{BioIndex::TA_cols}).}

\item{tb}{A MEDITS or MEDITS-like \strong{TB} table
(columns listed in \code{BioIndex::TB_cols}).}

\item{tc}{A MEDITS or MEDITS-like \strong{TC} table
(columns listed in \code{BioIndex::TC_cols}).}

\item{species}{Character. MEDITS 7-letter RUBIN code (e.g. \code{"MERLMER"}):
the first 4 letters are used as \emph{GENUS}, the last 3 as
\emph{SPECIES}.}

\item{country}{Character vector of MEDITS country codes to keep.
Use \code{"all"} (default) to include every country present
in \code{ta}.}

\item{strata}{A data-frame with the depth-stratification scheme adopted
by the MEDITS survey. Defaults to
\code{BioIndex::strata_scheme}.}

\item{wd}{Working directory. When \code{save = TRUE}, the merged
tables are written to \code{file.path(wd, "output")}.}

\item{save}{Logical. If \code{TRUE} (default) the function writes
\code{"mergeTATB_<GENUS><SPECIES>.csv"} and
\code{"mergeTATC_<GENUS><SPECIES>.csv"}.}

\item{verbose}{Logical (default \code{TRUE}); prints progress messages.}
}
\value{
A list of two \code{data.frame}s:
  \describe{
    \item{\code{merge_TA_TB}}{One row per haul with TA metadata, TB catch
          totals, depth stratum, densities and biomasses.}
    \item{\code{merge_TA_TC}}{One row per haul/length-class/sex/maturity
          with TA metadata, TC counts, depth stratum, raising factor,
          densities and biomasses.}
  }

A \code{list} containing two data frames: the first with merged TA-TB-TC data at the haul level, and the second with the length-frequency distribution.
}
\description{
This function is the primary entry point for integrating MEDITS survey data
(e.g. TA, TB, and TC tables), producing unified datasets essential for
population analysis and the estimation of demographic indicators.
The routine:
\itemize{
  \item filters hauls by validity and by country;
  \item performs a full set of quality checks (times, positions, wing opening,
        haul/date consistency) using internal validation functions
        (based on \code{RoME} version 0.2.3);
  \item merges TA with TB and TA with TC, respectively;
  \item computes swept area, mean positions, depth stratum, raising
        factors, densities and biomasses;
  \item (optionally) writes the two merged tables to
        \code{file.path(wd, "output")}.
}
}
\details{
The implementation reproduces the official BioIndex workflow.

\strong{Data Validation:}
The function automatically performs syntactic data validation using internal
routines based on \code{RoME} version 0.2.3 logic to ensure data integrity
and standardisation.

\strong{Optimisations:}
Two micro-optimisations are included:
\enumerate{
  \item vectorised replacement of missing \code{"NA NA"} entries
        in both merges;
  \item a single loop over depth strata (instead of haul Ã— stratum) to
        assign \code{STRATUM_CODE}.
}
Numerical outputs are identical to the reference routine.
}
\note{
This version of \code{BioIndex} uses internal validation functions based on
\strong{RoME version 0.2.3}. The package is fully self-sufficient and does
not require external validation packages to be installed.
}
\examples{
# Use internal data

data("TA", package = "BioIndex")
data("TB", package = "BioIndex")
data("TC", package = "BioIndex")
m <- merge_TATBTC(
  ta      = TA,
  tb      = TB,
  tc      = TC,
  species = "MERLMER",
  country = "all",
  wd      = tempdir(),
  verbose = TRUE
)
mTATB <- m[[1]]   # TAâ€“TB merged table
mTATC <- m[[2]]   # TAâ€“TC merged table

}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indices_ts.R
\name{indices_ts}
\alias{indices_ts}
\title{Estimation of abundance and biomass indices}
\usage{
indices_ts(
  mTATB,
  GSA,
  country = "all",
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference countries in the GSA for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{strata_scheme}{data frame of the stratification scheme}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{list} of \code{ggplot} objects representing the time series of calculated indices.
}
\description{
Computes time series of stratified abundance and biomass indices,
serving as the primary tool for monitoring temporal trends in stock status.
}
\examples{
data(TA)
data(TB)
# Create a merged dataset for GSA 10
mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
# Run indices_ts
indices_ts(mTATB, GSA=10, country="all", depth_range=c(10,800),
           strata_scheme=BioIndex::strata_scheme,
           stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_coordinates.r
\name{convert_coordinates}
\alias{convert_coordinates}
\title{MEDITS coordinates in decimal degrees}
\usage{
convert_coordinates(Data)
}
\arguments{
\item{Data}{data frame of TA table}
}
\value{
the function return the same data frame with the coordinates converted in the decimal degrees format
}
\description{
The function returns the data frame of the TA table with the coordinates expressed as decimal degrees.
}
\examples{
data(TA)
convert_coordinates(TA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quant.r
\name{quant}
\alias{quant}
\title{Quantile estimation}
\usage{
quant(weighted, qlin = 0.95)
}
\arguments{
\item{weighted}{LFD data.frame}

\item{qlin}{reference quantile for the analysis}
}
\value{
A \code{numeric} value or vector containing the requested quantiles.
}
\description{
Quantile estimation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TA_cols}
\alias{TA_cols}
\title{TA table headings}
\format{
An object of class \code{character} of length 22.
}
\usage{
TA_cols
}
\description{
TA table headings
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
\keyword{TA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_dictionary.r
\name{check_dictionary}
\alias{check_dictionary}
\title{Check dictionary (RoME)}
\usage{
check_dictionary(
  ResultData,
  Field,
  Values,
  year,
  wd = NA,
  suffix = NA,
  verbose = FALSE
)
}
\arguments{
\item{ResultData}{data frame in MEDITS tables}

\item{Field}{field of the table to be checked}

\item{Values}{vector of the allowed values}

\item{year}{reference year for the analysis}

\item{wd}{working directory}

\item{suffix}{name of the log file}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
\code{TRUE} if the validation passes, \code{FALSE} otherwise.
}
\description{
The function checks whether the values contained in specific fields are consistent with the allowed values of the dictionaries.
}
\note{
This function is an internal routine based on \strong{RoME version 0.2.3}.
It is provided within \code{BioIndex} to ensure the package remains
functional and self-sufficient for data validation.
}
\examples{
data(TA)
check_dictionary(TA, Field = "COUNTRY", Values = c("ITA"), year = 2007)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LWf.R
\name{LWf}
\alias{LWf}
\title{Estimation of LW relationship}
\usage{
LWf(TE, sp, GEAR, GSA, country = NA, n_records = 10, wd = NA, save = TRUE)
}
\arguments{
\item{TE}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GEAR}{type of gear reported in the corresponding TA file}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{n_records}{minimum number of records to perform the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}
}
\value{
A \code{data.frame} containing the length-weight relationship parameters specifically for females.
}
\description{
Estimation of LW relationship
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Bubble_plot_by_haul_indexes.r
\name{bubble_plot_by_haul_indexes}
\alias{bubble_plot_by_haul_indexes}
\title{Bubble plot of abundance and biomass indices by haul}
\usage{
bubble_plot_by_haul_indexes(
  mTATB,
  map_lim,
  depth_lines,
  buffer = 0,
  res = NA,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{map_lim}{coordinates limits for the plotted map}

\item{depth_lines}{vector of three depth bathymetrical lines to be plotted}

\item{buffer}{buffer to the coordinate limits in map units}

\item{res}{resolution of the bathymetrical lines}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE a message is printed}
}
\value{
A \code{ggplot} object representing the bubble plot of abundance or biomass by hauls.
}
\description{
The function generates bubble plot of abundance and biomass indices by haul.
If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
}
\examples{
\donttest{
# Use internal data
data(TA)
data(TB)
# Create a merged dataset for GSA 10 and year 2016
mTATB <- merge_TATB(TA[TA$AREA == 10 & TA$YEAR == 2016, ],
                    TB[TB$AREA == 10 & TB$YEAR == 2016, ],
                    "MERLMER")
bubble_plot_by_haul_indexes(mTATB, map_lim = NA, depth_lines = c(200, 500, 800), save = FALSE)
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_hauls_TBTA.r
\name{check_hauls_TBTA}
\alias{check_hauls_TBTA}
\title{Check hauls TB TA (RoME)}
\usage{
check_hauls_TBTA(DataTA, DataTB, year, wd = NA, suffix = NA)
}
\arguments{
\item{DataTA}{MEDITS TA table}

\item{DataTB}{MEDITS TB table}

\item{year}{reference year for the analysis}

\item{wd}{working directory}

\item{suffix}{name of the log file}
}
\value{
\code{TRUE} if the validation passes, \code{FALSE} otherwise.
}
\description{
Check if all the hauls in TB are in TA.
}
\note{
This function is an internal routine based on \strong{RoME version 0.2.3}.
It is provided within \code{BioIndex} to ensure the package remains
functional and self-sufficient for data validation.
}
\examples{
data(TA)
data(TB)
check_hauls_TBTA(TA, TB, year = 2007)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_ratio_on_grid.r
\name{sex_ratio_on_grid}
\alias{sex_ratio_on_grid}
\title{Plot sex ratio spatial distribution}
\usage{
sex_ratio_on_grid(
  mTATBsp,
  depth,
  wd = NA,
  map_range,
  threshold = 30,
  verbose = FALSE,
  save = TRUE
)
}
\arguments{
\item{mTATBsp}{A spatial version of the merged TA-TB dataset, processed through `overlayGrid()`, containing sex information.}

\item{depth}{Character string specifying the depth range used for filtering (e.g., `"10,800"`). Must correspond to available internal GFCM grid masks.}

\item{wd}{Working directory. Used to save output files if `save = TRUE`.}

\item{map_range}{numeric vector with coordinates defining the map extent (`xmin`, `xmax`, `ymin`, `ymax`).}

\item{threshold}{Minimum number of individuals required per haul to be included in the sex ratio estimation. Default is 30.}

\item{verbose}{Logical. If `TRUE`, informative messages are printed to the console.}

\item{save}{Logical. If `TRUE`, the plot and sex ratio table are saved to the working directory.}
}
\value{
A `data.frame` containing sex ratio (`ratio`), standard deviation (`sd`), and coefficient of variation (`CV`) for each GFCM grid cell.
}
\description{
This function calculates and plots the spatial distribution of sex ratio (F / (F + M)) over the GFCM grid for a given species. It uses data from merged MEDITS datasets (TA, TB, TC), previously spatialized with `overlayGrid()`. The function filters hauls by total abundance threshold and depth range, calculates sex ratio per grid cell, and plots the output using color-coded categories based on quantiles. The result is a choropleth map of sex ratio patterns, useful to highlight spatial differences in sex structure. Raw sex ratio estimates per grid cell can also be exported.
}
\examples{
data(TA)
data(TB)
data(TC)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
mTATBsp <- overlayGrid(m[[1]], m[[2]], GSA = 10, save = FALSE, verbose = FALSE)[[1]]
map_range <- c(9, 15, 39, 42)
sex_ratio_on_grid(mTATBsp, depth = "10,800", map_range = map_range, threshold = 5,
                  save = FALSE, verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ALKf.R
\name{ALKf}
\alias{ALKf}
\title{Estimation of ALK}
\usage{
ALKf(te, sp, GEAR, GSA, country = NA, years = 5, wd = NA, save = TRUE)
}
\arguments{
\item{te}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GEAR}{type of gear reported in the corresponding TA file}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{years}{number of years to be considered in the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}
}
\value{
A \code{data.frame} representing the Age-Length Key for females.
}
\description{
Estimation of ALK
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_ratio.r
\name{sex_ratio}
\alias{sex_ratio}
\title{Sex ratio}
\usage{
sex_ratio(
  mTATB,
  GSA,
  country,
  depth_range,
  stratas,
  stratification,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{mTATB}{data frame of the merged TA and TB}

\item{GSA}{reference GSA for the analysis}

\item{country}{vector of reference countries for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{stratas}{data frame of the reference strata for the study area}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE a message is printed}
}
\value{
A \code{data.frame} containing the sex ratio statistics by year and stratum.
}
\description{
Assesses the proportion of sexes within the population, providing
critical information on demographic structure, reproductive potential,
and stock resilience.
}
\examples{
data(TA)
data(TB)
# Create a merged dataset for GSA 10
mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
sex_ratio(mTATB, GSA=10, country="all", depth_range=c(10,800),
          stratas=BioIndex::strata_scheme,
          stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{stratification}
\alias{stratification}
\title{stratification}
\format{
An object of class \code{data.frame} with 276 rows and 6 columns.
}
\usage{
stratification
}
\description{
Data frame containing the surface area relative to the depth strata adopted in the stratification scheme (strata_scheme) of the demersal surveys (e.g. MEDITS survey).
}
\keyword{MEDITS}
\keyword{stratification}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate_gsas.r
\name{aggregate_gsas}
\alias{aggregate_gsas}
\title{Aggregate MEDITS data across multiple GSAs}
\usage{
aggregate_gsas(
  ta,
  tb,
  tc,
  gsas,
  strata_scheme = BioIndex::strata_scheme,
  stratification = BioIndex::stratification
)
}
\arguments{
\item{ta}{A data frame containing the MEDITS \code{TA} table.}

\item{tb}{A data frame containing the MEDITS \code{TB} table.}

\item{tc}{A data frame containing the MEDITS \code{TC} table.}

\item{gsas}{A numeric or character vector of GSA codes to be aggregated.}

\item{strata_scheme}{A data frame describing the depth-strata scheme for the
selected GSAs. Defaults to \code{BioIndex::strata_scheme}.}

\item{stratification}{A data frame containing the stratification information
for the selected GSAs. Defaults to \code{BioIndex::stratification}.}
}
\value{
A list with five elements:
\describe{
  \item{[[1]]}{The filtered and aggregated \code{TA} table.}
  \item{[[2]]}{The filtered and aggregated \code{TB} table.}
  \item{[[3]]}{The filtered and aggregated \code{TC} table.}
  \item{[[4]]}{The filtered and updated \code{stratification} table.}
  \item{[[5]]}{The filtered and updated \code{strata_scheme} table.}
}
}
\description{
Aggregate TA, TB, and TC MEDITS tables across a user-defined set of
Geographical Sub-Areas (GSAs), and update the associated stratification
objects accordingly.
}
\details{
This function filters the input TA, TB, and TC tables to retain only the
selected GSAs, concatenates the original \code{AREA} and \code{HAUL_NUMBER}
values to create unique haul identifiers across GSAs, and replaces the
\code{AREA} field with a single aggregated GSA code obtained by collapsing
the selected GSA values into one string. The same aggregation is applied to
the \code{stratification} and \code{strata_scheme} tables.


The function performs three main operations:
\enumerate{
  \item Checks that all requested GSAs are present in the \code{AREA} column
  of the input \code{ta}, \code{tb}, and \code{tc} tables.
  \item Filters the three MEDITS tables to the selected GSAs and updates
  \code{HAUL_NUMBER} by prefixing it with the original \code{AREA} value,
  ensuring uniqueness after aggregation.
  \item Filters and updates the \code{stratification} and
  \code{strata_scheme} tables so that all selected GSAs are represented under
  a single aggregated GSA code.
}

The aggregated GSA code is created using \code{paste(gsas, collapse = "")}.
For example, \code{c(17, 18)} becomes \code{"1718"}.
}
\examples{
# Use internal data
data("strata_scheme", package = "BioIndex")
data("stratification", package = "BioIndex")
data("TA", package = "BioIndex")
data("TB", package = "BioIndex")
data("TC", package = "BioIndex")

d <- aggregate_gsas(
  ta = TA,
  tb = TB,
  tc = TC,
  gsas = 10,
  strata_scheme = BioIndex::strata_scheme,
  stratification = BioIndex::stratification
)

ta_agg <- d[[1]]
tb_agg <- d[[2]]
tc_agg <- d[[3]]
stratification_agg <- d[[4]]
strata_scheme_agg <- d[[5]]

}
\seealso{
\code{\link[BioIndex:BioIndex]{strata_scheme}},
\code{\link[BioIndex:BioIndex]{stratification}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BioIndex.R
\name{BioIndex}
\alias{BioIndex}
\title{Main function to perform BioIndex analysis}
\usage{
BioIndex(
  ta,
  tb,
  tc,
  sspp,
  rec_threshold,
  spaw_threshold,
  haul_threshold = 30,
  sexes = "all",
  depth,
  GSA,
  country = "all",
  map_lim,
  depth_lines = c(10, 200, 800),
  strata = BioIndex::strata_scheme,
  stratification_tab = BioIndex::stratification,
  resolution = NA,
  buffer = 0.1,
  wd = NA,
  zip = TRUE,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{ta}{data frame of the TA table in the MEDITS format}

\item{tb}{data frame of the TB table in the MEDITS format}

\item{tc}{data frame of the TC table in the MEDITS format}

\item{sspp}{reference species for the analysis}

\item{rec_threshold}{cutoff threshold for recruits (reported in mm)}

\item{spaw_threshold}{cutoff threshold for spawners  (reported in mm)}

\item{haul_threshold}{minimum number of individuals to be used in estimation of the spatial indicators}

\item{sexes}{reference sex for the analysis}

\item{depth}{reference depth range}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{map_lim}{coordinates limits for the maps}

\item{depth_lines}{depth contours to be plotted in the maps (3 values allowed, e.g c(50,200,800))}

\item{strata}{data frame of the reference strata for the study area}

\item{stratification_tab}{data frame of the stratification scheme}

\item{resolution}{resolution of the depth line}

\item{buffer}{buffer around the map}

\item{wd}{path of the working directory}

\item{zip}{boolean. If TRUE the results are stored in a zip file into the working directory}

\item{save}{boolean. If TRUE the results are stored in the working directory}

\item{verbose}{boolean. If TRUE messages are promted in the console}
}
\value{
A \code{list} containing the results of the BioIndex workflow, including data frames and plot objects.
}
\description{
BioIndex is an R package designed to support the standardized analysis of MEDITS trawl survey data and the calculation of biological indicators for selected species and population components.
}
\examples{
\donttest{
BioIndex(ta=TA[TA$YEAR \%in\% c(2015, 2016), ], tb=TB[TB$YEAR \%in\% c(2015, 2016), ],
tc=TC[TC$YEAR \%in\% c(2015, 2016), ], sspp="MERLMER",rec_threshold=200,
spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10, country="all",
map_lim=c(13.3,15.2,39.9,41.3),depth_lines=c(50,200,800),
strata=BioIndex::strata_scheme, stratification_tab =
BioIndex::stratification, resolution=NA, buffer=0.1, wd=tempdir(),
zip=FALSE, save=TRUE, verbose=TRUE)
}

}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Bubbleplot_RS_by_hauls.r
\name{bubbleplot_RS_by_hauls}
\alias{bubbleplot_RS_by_hauls}
\title{Bubbleplot of abundance indices for recruits and spawners}
\usage{
bubbleplot_RS_by_hauls(
  mTATC,
  map_range,
  thresh_rec,
  thresh_spaw,
  depths = c(50, 200, 800),
  res = NA,
  buffer = 0.1,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{mTATC}{mTATC table}

\item{map_range}{range of coordinates for the map}

\item{thresh_rec}{threshold value to select recruits data from mTATC table (reported in mm)}

\item{thresh_spaw}{threshold value to select spawners data from mTATC table (reported in mm)}

\item{depths}{three reference bathymetric lines to be plotted in the maps}

\item{res}{resolution of the depth lines}

\item{buffer}{buffer around the map}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
A \code{list} containing two \code{ggplot} objects: the bubble plot of recruits (\code{pr}) and spawners (\code{ps}) abundance indices.
}
\description{
The function generates bubbleplots of abundance indices for recruits and spawners
If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
}
\examples{
\donttest{
# Create a merged dataset for GSA 10
mTATC <- merge_TATC(TA[TA$AREA==10,], TC[TC$AREA==10,], "MERLMER")
bubbleplot_RS_by_hauls(mTATC, map_range=c(15,21,39,43), thresh_rec=200, thresh_spaw=250)
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index_recr.r
\name{index_recr}
\alias{index_recr}
\title{Estimation of abundance indices for recruits}
\usage{
index_recr(
  mTATB,
  mTATC,
  GSA,
  country,
  depth_range,
  cutoff,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{mTATC}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country}{vector of reference countries for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{cutoff}{cutoff value for splitting recruits portion of population (reported in mm)}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{data.frame} containing the abundance indices and density for the recruitment fraction.
}
\description{
Estimates specific abundance indices for the recruitment phase (using the
user-defined cutoff value), allowing for the assessment of reproductive
success and the prediction of future stock productivity.
}
\examples{
data(TA)
data(TB)
data(TC)
data(stratification)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
index_recr(m[[1]], m[[2]], GSA = 10, country = "all", depth_range = c(10, 800),
           cutoff = 200, stratification = stratification, save = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_BioIndex_app.r
\name{run_BioIndex_app}
\alias{run_BioIndex_app}
\title{launches the embedded Shiny application included in the package.}
\usage{
run_BioIndex_app()
}
\value{
\code{No return value, called for side effects} (launches the Shiny application).
}
\description{
launches the embedded Shiny application included in the package.
}
\examples{
if (interactive()) {
  run_BioIndex_app()
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{stratification_rapana}
\alias{stratification_rapana}
\title{stratification (rapa whelk)}
\format{
An object of class \code{data.frame} with 7 rows and 6 columns.
}
\usage{
stratification_rapana
}
\description{
Data frame containing the surface area relative to the depth strata adopted in the stratification scheme (strata_scheme) of the rapa whelk surveys (e.g. Black Sea beam trawl survey).
}
\keyword{Black}
\keyword{Sea}
\keyword{rapa}
\keyword{stratification}
\keyword{whelk}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LW.R
\name{LW}
\alias{LW}
\title{LW (Length-Weight relationship)}
\usage{
LW(
  ta,
  te,
  sp,
  GSA,
  country = "all",
  nyears = NA,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{ta}{MEDITS or MEDITS-like TA table}

\item{te}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{nyears}{number of years of the time series to be considered in the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are promted in the console}
}
\value{
A \code{data.frame} containing the length-weight relationship parameters and statistics.
}
\description{
Estimates the length-weight relationship for the selected species,
a fundamental biological indicator to assess fish condition and convert
numerical length estimates into biomass equivalents.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MEDITS.distance.R
\name{MEDITS.distance}
\alias{MEDITS.distance}
\title{Estimation of haul distance}
\usage{
MEDITS.distance(data, unit = "m", verbose = TRUE)
}
\arguments{
\item{data}{data frame containing the hauls data (TA, table A).}

\item{unit}{string value indicating the measure unit of the distance. Allowed values: "m" for meters, "km" for kilometers and "NM" for nautical miles.}

\item{verbose}{give verbose output reporting in the output the selected measure unit of the distance.}
}
\value{
The function returns the vector of the distances expressed in the selected measure unit.
}
\description{
Estimation of haul distance
}
\examples{
data(TA)
MEDITS.distance(TA, unit = "m", verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_TATB.r
\name{merge_TATB}
\alias{merge_TATB}
\title{Merge TA and TB tables (haul-level catches)}
\usage{
merge_TATB(
  ta,
  tb,
  species,
  country = "all",
  strata = BioIndex::strata_scheme,
  wd = NA,
  save = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{ta}{A `data.frame`/`data.table` containing a full TA dataset
(columns listed in `BioIndex::TA_cols`).}

\item{tb}{A `data.frame`/`data.table` containing a full TB dataset
(columns listed in `BioIndex::TB_cols`).}

\item{species}{Character, MEDITS 7-letter code (e.g. `"ARISFOL"`).
The first 4 characters are interpreted as *GENUS* and
the last 3 as *SPECIES*.}

\item{country}{Character vector of ISO-3 country codes to keep; use
`"all"` (default) to keep every country present in `ta`.}

\item{strata}{Depth-stratum reference table; default is
`BioIndex::strata_scheme`.}

\item{wd}{Working directory.  If `NA` (the default when
`save     = FALSE`) no files are written.}

\item{save}{Logical.  If `TRUE`, writes
`"mergeTATB_<GENUS><SPECIES>.csv"` to
`file.path(wd, "output")`.}

\item{verbose}{Logical; if `TRUE` (default) prints progress messages.}
}
\value{
A `data.frame` with one row per haul and the following groups
        of variables:
        * **TA** metadata (haul position, times, depths, etc.)
        * **TB** catch totals (numbers and weight)
        * Calculated fields: mean lat/lon, swept area, depth stratum,
          density (`N_h`, `N_km2`) and biomass (`kg_h`, `kg_km2`)
}
\description{
Combines haul-level operational data (TA) with species catch data (TB),
providing the essential foundation for calculating stratified abundance (N)
and biomass (kg) indices.
The function keeps only valid hauls, performs quality checks (times,
positions, wing opening, haul/date consistency) using internal validation
functions (based on \code{RoME} version 0.2.3), computes spatial means,
swept area, depth stratum, biomass/density indicators, andâ€”optionallyâ€”writes
the merged data to disk.
}
\details{
The implementation mirrors the original BioIndex routine.

\strong{Data Validation:}
The function performs syntactic data validation using internal
implementations of the validation routines (based on \code{RoME} v0.2.3).

\strong{Optimisations:}
Includes two speed-ups: (1) vectorised replacement of missing `"NA NA"` records,
(2) a single loop over depth strata instead of a nested haul Ã— stratum
loop.  Results are identical to the reference routine.
}
\note{
This version of \code{BioIndex} uses internal validation functions based on
\strong{RoME version 0.2.3}. The package is fully self-sufficient and does
not require external validation packages to be installed.
}
\examples{
# Use internal data
data("TA", package = "BioIndex")
data("TB", package = "BioIndex")
m_tb <- merge_TATB(
  ta       = TA,
  tb       = TB,
  species  = "ARISFOL",
  country  = "ITA",
  wd       = tempdir(),
  save     = FALSE
)
head(m_tb)
}
\seealso{
[BioIndex] package documentation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MEDITS.to.dd.R
\name{MEDITS.to.dd}
\alias{MEDITS.to.dd}
\title{Conversion of MEDITS format coordinates in decimal degrees format}
\usage{
MEDITS.to.dd(data)
}
\arguments{
\item{data}{data frame of the hauls data (TA, table A) in MEDITS format}
}
\value{
The function returns the data frame of the TA table with the coordinates expressed as decimal degrees
}
\description{
Conversion of MEDITS format coordinates in decimal degrees format
}
\examples{
data(TA)
MEDITS.to.dd(TA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlayGrid.r
\name{overlayGrid}
\alias{overlayGrid}
\title{Overlay mTATB and mTATC on GFCM spatial grid}
\usage{
overlayGrid(
  mTATB,
  mTATC,
  GSA = NA,
  country = "all",
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{mTATB}{data frame of the merged TA and TB}

\item{mTATC}{data frame of the merged TA and TC}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference countries for the analysis}

\item{wd}{working directory used to save results}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
A \code{list} or \code{data.frame} containing the merged biological data mapped to the spatial grid.
}
\description{
Overlays survey haul data onto a standard spatial grid (e.g., GFCM grid),
harmonizing biological information with a consistent geographic structure
for standardized spatial analysis.
}
\examples{
# Use internal data
data(TA)
data(TB)
data(TC)
# Create a merged dataset for GSA 10
m <- merge_TATBTC(TA[TA$AREA==10,], TB[TB$AREA==10,], TC[TC$AREA==10,], species="MERLMER")
mTATB <- m[[1]]
mTATC <- m[[2]]
overlayGrid(mTATB, mTATC, GSA=10, save=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index_ts_M.r
\name{index_ts_M}
\alias{index_ts_M}
\title{Estimation of abundance indices for males}
\usage{
index_ts_M(
  mTATB,
  GSA,
  country_analysis,
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country_analysis}{vector of reference countries for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{strata_scheme}{data frame of the stratification scheme}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{data.frame} containing the time series of indices specifically for males.
}
\description{
Estimation of abundance indices for males
}
\examples{
data(TA)
data(TB)
data(TC)
data(strata_scheme)
data(stratification)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
strata_scheme_g10 <- strata_scheme[strata_scheme$GSA == 10 & strata_scheme$COUNTRY == "ITA", ]
index_ts_M(m[[1]], GSA = 10, country_analysis = "all", depth_range = c(10, 800),
           strata_scheme = strata_scheme_g10, stratification = stratification, save = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dd.to.MEDITS.R
\name{dd.to.MEDITS}
\alias{dd.to.MEDITS}
\title{Conversion of decimal degrees coordinates in MEDITS format}
\usage{
dd.to.MEDITS(data)
}
\arguments{
\item{data}{data frame of the hauls data (TA, table A) in MEDITS format}
}
\value{
The function returns the data frame of the TA (table A) reporting the coordinates in MEDITS format.
}
\description{
Conversion of decimal degrees coordinates in MEDITS format
}
\examples{
data(TA)
ta_dd <- MEDITS.to.dd(TA)
dd.to.MEDITS(ta_dd)
}
